/*
 * linux/arch/arm/mach-tcc92x/board-tcc9200s-gpio.c
 *
 * Copyright (C) 2009, 2010 Telechips, Inc.
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/io.h>
#include <linux/irq.h>
#include <asm/mach-types.h>
#include <mach/gpio.h>

#include <mach/bsp.h>

struct board_gpio_irq_config {
	unsigned gpio;
	unsigned irq;
};

struct board_gpio_irq_config tcc92xx_gpio_irqs[] = {
	{ GPIO_PORTA|2, INT_EI4 },
	{ GPIO_PORTA|3, INT_EI5 },
	{ GPIO_PORTA|12, INT_EI6 },
};

int gpio_to_irq(unsigned gpio)
{
#if 0	/* XXX: disable GPIO interrupt for now */
	struct board_gpio_irq_config *gpio_irq;
	int i;

	for (i = 0; i < ARRAY_SIZE(tcc92xx_gpio_irqs); i++) {
		gpio_irq = &tcc92xx_gpio_irqs[i];
		if (gpio_irq->gpio == gpio)
			return gpio_irq->irq;
	}
#endif
	return -EINVAL;
}
EXPORT_SYMBOL(gpio_to_irq);

int tcc9200s_init_gpio(void)
{
	if (!machine_is_tcc9200s())
		return 0;

#if 0	/* XXX: disable GPIO interrupt for now */
	/* Initialize GPIO external interrupts */
	writel(HwEINTSEL1_EINT4(SEL_GPIOA2)
	       | HwEINTSEL1_EINT5(SEL_GPIOA3)
	       | HwEINTSEL1_EINT6(SEL_GPIOA12),
	       &HwEINTSEL->EINTSEL1);

	/* Set level-triggered mode */
	writel(readl(&HwPIC->MODE0)
	       | ((1 << INT_EI6) | (1 << INT_EI5) | (1 << INT_EI4)),
	       &HwPIC->MODE0);

	/* Set interrupt polarity to active low */
	writel(readl(&HwPIC->POL0)
	       | ((1 << INT_EI6) | (1 << INT_EI5) | (1 << INT_EI4)),
	       &HwPIC->POL0);
#endif

	printk(KERN_INFO "TCC9200s GPIO initialized\n");
	return 0;
}
postcore_initcall(tcc9200s_init_gpio);
