/*
 * linux/arch/arm/mach-tcc93x/gpio.c
 *
 * Copyright (C) 2009, 2010 Telechips, Inc.
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <linux/init.h>
#include <linux/module.h>
#include <linux/interrupt.h>
#include <linux/sysdev.h>
#include <linux/err.h>
#include <linux/clk.h>
#include <linux/io.h>

#include <mach/hardware.h>
#include <asm/irq.h>
#include <mach/irqs.h>
#include <mach/gpio.h>
#include <asm/mach/irq.h>

#include <mach/reg_physical.h>

#define GPIO_REG(reg)	IO_ADDRESS(reg)

static inline GPION *tcc93xx_gpio_to_reg(struct gpio_chip *chip)
{
	if (chip->base == GPIO_PORTA)
		return (GPION *) GPIO_REG(TCC_GPA_PHYS_BASE);
	if (chip->base == GPIO_PORTB)
		return (GPION *) GPIO_REG(TCC_GPB_PHYS_BASE);
	if (chip->base == GPIO_PORTC)
		return (GPION *) GPIO_REG(TCC_GPC_PHYS_BASE);
	if (chip->base == GPIO_PORTD)
		return (GPION *) GPIO_REG(TCC_GPD_PHYS_BASE);
	if (chip->base == GPIO_PORTE)
		return (GPION *) GPIO_REG(TCC_GPE_PHYS_BASE);
	if (chip->base == GPIO_PORTF)
		return (GPION *) GPIO_REG(TCC_GPF_PHYS_BASE);
	if (chip->base == GPIO_PORTG)
		return (GPION *) GPIO_REG(TCC_GPG_PHYS_BASE);
    return 0;
}

static int tcc93xx_gpio_get(struct gpio_chip *chip, unsigned offset)
{
	GPION *reg = tcc93xx_gpio_to_reg(chip);

	return (readl(&reg->GPDAT) >> offset) & 1;
}

static void tcc93xx_gpio_set(struct gpio_chip *chip, unsigned offset, int value)
{
	GPION *reg = tcc93xx_gpio_to_reg(chip);

	if (value)
		writel(readl(&reg->GPDAT) | (1 << offset), &reg->GPDAT);
	else
		writel(readl(&reg->GPDAT) & ~(1 << offset), &reg->GPDAT);
}

static int tcc93xx_gpio_direction_input(struct gpio_chip *chip, unsigned offset)
{
	GPION *reg = tcc93xx_gpio_to_reg(chip);

	writel(readl(&reg->GPEN) & ~(1 << offset), &reg->GPEN);
	return 0;
}

static int tcc93xx_gpio_direction_output(struct gpio_chip *chip, unsigned offset, int value)
{
	GPION *reg = tcc93xx_gpio_to_reg(chip);

	writel(readl(&reg->GPEN) | (1 << offset), &reg->GPEN);
	tcc93xx_gpio_set(chip, offset, value);
	return 0;
}

static struct gpio_chip tcc93xx_gpio_chips[] = {
	{
		.label = "gpio_a",
		.base = GPIO_PORTA,
		.ngpio = 20,
	},
	{
		.label = "gpio_b",
		.base = GPIO_PORTB,
		.ngpio = 32,
	},
	{
		.label = "gpio_c",
		.base = GPIO_PORTC,
		.ngpio = 32,
	},
	{
		.label = "gpio_d",
		.base = GPIO_PORTD,
		.ngpio = 26,
	},
	{
		.label = "gpio_e",
		.base = GPIO_PORTE,
		.ngpio = 32,
	},
	{
		.label = "gpio_f",
		.base = GPIO_PORTF,
		.ngpio = 30,
	},
	{
		.label = "gpio_g",
		.base = GPIO_PORTG,
		.ngpio = 30,
	},
};

int __init tcc93xx_gpio_init(void)
{
	struct gpio_chip *chip;
	int i;

	for (i = 0; i < ARRAY_SIZE(tcc93xx_gpio_chips); i++) {
		chip = &tcc93xx_gpio_chips[i];

		chip->direction_input = tcc93xx_gpio_direction_input;
		chip->direction_output = tcc93xx_gpio_direction_output;
		chip->set = tcc93xx_gpio_set;
		chip->get = tcc93xx_gpio_get;

		gpiochip_add(chip);
	}
	return 0;
}

postcore_initcall(tcc93xx_gpio_init);
