/*
 *       TITLE: LINUX SATA DEVICE DRIVER
 *   CORE TYPE: TCCXXX
 * DRIVER TYPE: kernel module
 *        NAME: tccXXX_sata_device.ko
 *
 * @Copyright 2010 Telechips Inc. <linux@telechips.com>
 *
 */

#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/init.h>

#include <linux/kernel.h>
#include <linux/blkdev.h>
#include <linux/delay.h>


#include <mach/tca_ckc.h>
//#include <mach/tcc_pca953x.h>
#include <mach/gpio.h>
#include <mach/clock.h>

#include  "bsp.h"
#include "DWC_ahsata_vtb_defs.h"

#define MAX_SATA_DEV_NUM	1

#define SATA_DEVICE_VER		"v1.00"
#define SATA_DEV_NAME		"tcc_sata_host"
#define SATA_DEV_MAJOR 		241
#define SDD_MAX_PARTITIONS	8

//#define j_print(f, a...)  printk("== sata-debug == " f, ##a)
#define j_print(f, a...)

extern struct clk *clk_get(struct device *dev, const char *id);
extern int clk_enable(struct clk *clk);
extern int clk_set_rate(struct clk *clk, unsigned long rate);

//=============================================================================
//*
//*
//*                           [ CONST DATA DEFINE ]
//*
//*
//=============================================================================
extern char f3Gbps_60MHz[317][3];

static void set_hsio_clock(void)
{
	j_print("[%s:%d]\n", __func__, __LINE__);

	tca_ckc_setpmupwroff(PMU_SATAPHY, 1);

	tca_ckc_setcommonhsiobus(HSIOBUS_SATA, 1);		

	//======================================
	// Rxoob Clock 					: 150MHz
	// HSATA1, DSATA RefCore Clock 	: 60MHz
	//======================================
	/* tca_ckc_setperi(PERI_SATA_REF, 1, 600000, PCDIRECTPLL5); */
	/* tca_ckc_setperi(PERI_SATA, 1, 1500000, PCDIRECTPLL5); */
	tca_ckc_setperi(PERI_SATA, 1, 1500000, PCDIRECTPLL2);

	//printk("[DSATA]Set Clock ON\n");
}

static void enable_ahsata(void)
{
	struct clk *pHsioClk, *pSataHostClk;
	//volatile PMU *p_pmu = (volatile PMU *)tcc_p2v(HwPMU_BASE);

	j_print("[%s:%d]\n", __func__, __LINE__);
	
	pHsioClk = clk_get(NULL, "hsio");
	if (pHsioClk == NULL) 
		printk("***** [%s] HSIO Clock Enable Failed.*****\n", __func__);	
	clk_enable(pHsioClk);
	
	set_hsio_clock(); //set IOCLK

#if 0
	// SATA PHY Isolation Off Setting
	BITSET(p_pmu->PWROFF, Hw4);

	mdelay(200);
	tca_ckc_sethsiobus(HSIO_SATAH, ON);
	tca_ckc_sethsiobusswreset(HSIO_SATAH, ON);
	tca_ckc_sethsiobusswreset(HSIO_SATAH, OFF);
#else
	tca_ckc_sethsiobusswreset(HSIO_SATAH, ON);
	tca_ckc_sethsiobusswreset(HSIO_SATAH, OFF);
#endif /* 0 */

	pSataHostClk = clk_get(NULL, "satah");
	if (pSataHostClk == NULL) 
		printk("***** [%s] HSUSB Clock Enable Failed.*****\n", __func__);	
	clk_enable(pSataHostClk);	
}

static void enable_ahsata_android(void)
{
/***********************************************
High Speed IO Bus Configuration
***********************************************/

	/* SATA PHY ON : SP[4] */
	tca_ckc_setpmupwroff(PMU_SATAPHY, 1);

	/* High Speed IO Bus : HSB[22:20] */
	tca_ckc_setpmupwroff(PMU_HSBUSPRE, 0);
	tca_ckc_setpmupwroff(PMU_HSBUS, 0);
	tca_ckc_setpmupwroff(PMU_HSBUSISOLATION, 1);

	/* Software Reset for High Speed IO Bus  */
	tca_ckc_setswreset(RESET_HSBUS, 0);

	/* HSIO enable */
	tca_ckc_enable(CLKCTRL8, 1);

/***********************************************
SATA HOST Configuration
***********************************************/

	tca_ckc_setperi(PERI_SATA, 1, 1500000, PCDIRECTPLL2);

	/* SATA Host SWReset */
	tca_ckc_sethsiobusswreset(HSIO_SATAH, ON);
	tca_ckc_sethsiobusswreset(HSIO_SATAH, OFF);
	
	/* SATA Host PClk */
	tca_ckc_pclk_enable(PERI_SATA, 1);
	tca_ckc_pclk_enable(PERI_SATA_REF, 1);
	tca_ckc_sethsiobus(HSIO_SATAH, 1);
}

static void set_cmu_cdn_c(char cmu)
{
	char tmp;

	j_print("[%s:%d]\n", __func__, __LINE__);
	ahb_read(0x0, tmp);

	tmp |= cmu;
	ahb_write(0x0, tmp);                           // set_cmu_cdn
}

static void ahb_cfg_c(void)
{
	int i, j;
	int model = 0, addr = 0, ahb_addr = 0;
	char value = 0, i_value = 0, ahb_data = 0;

	j_print("[%s:%d]\n", __func__, __LINE__);
	for (i = 0; i < 317; i++)                          // f3Gbps_60MHz[317][3]
	{
		model = (int)f3Gbps_60MHz[i][0];
		addr  = (int)f3Gbps_60MHz[i][1];
		value =      f3Gbps_60MHz[i][2];

		ahb_data = 0;
		ahb_addr = ((model << 9) | addr);        // {model[11:9], addr[8:0]}

		for (j = 0; j < 8; j++)
		{
			i_value = (value & (0x80 >> j));

			if (j < 4)
			{
				i_value = (i_value >> (7 - (2 * j)));
			}
			else
			{
				i_value = (i_value << ((2 * j) - 7));
			}
			ahb_data |= i_value;
		}

		ahb_write(ahb_addr, ahb_data);
	}
}

static void sata_cfg(void)
{
	volatile HSIOBUSCFG *p_hsio_bus = (volatile HSIOBUSCFG *)tcc_p2v(HwHSIOBUSCFG_BASE);

	/* fast_sim = 0 */
	BITCLR(p_hsio_bus->SATA_PCTRL, Hw17);
	/* SATA PHY: Gen2 3.0G */
	BITCLR(p_hsio_bus->SATA_LCTRL, Hw18);
	// SATA PHY UnReset
	BITSET(p_hsio_bus->SATA_PCTRL, Hw8);

	ahb_cfg_c();
	set_cmu_cdn_c(0x1);

	/* cmu_reset_i (enable after por to norm) */
	BITSET(p_hsio_bus->SATA_PCTRL, Hw10);
	/* cmu_power down */
	BITCLR(p_hsio_bus->SATA_PCTRL, Hw16);

	j_print("[%s:%d]\n", __func__, __LINE__);
}

static void set_lane_cdn_c(char common, char lane)
{
	char tmp;

	j_print("[%s:%d]\n", __func__, __LINE__);
	ahb_read(0xa00, tmp);
	tmp |= common;
	ahb_write(0xa00, tmp);                         // set_lc_master_cdn

	lane = (lane & 1) << 1;
	ahb_read(0xa00, tmp);
	tmp |= lane;
	ahb_write(0xa00, tmp);                         // set_l0_master_cdn
}

static int sata_enable(void)
{
	int count, ok_cmu = 0, ok_lane = 0;

	volatile HSIOBUSCFG *p_hsio_bus = (volatile HSIOBUSCFG *)tcc_p2v(HwHSIOBUSCFG_BASE);

	for (count = 0; count < 3000; count++)
	{
		if (p_hsio_bus->SATA_PSTS & Hw18)
		{
			ok_cmu = 1;
			break;
		}
		barrier();
	}

	if (ok_cmu)
	{
		set_lane_cdn_c(0, 0xf);

		// SATA PHY Lane UnReset
		BITSET(p_hsio_bus->SATA_PCTRL, Hw9);

		// Wait Lane OK of PHY ( ln0_stat_o[12] ) wait about 144,000ns
		mdelay(200);

		for (count = 0; count < 3000; count++)
		{
			if (p_hsio_bus->SATA_PSTS & Hw16)
			{
				ok_lane = 1;
				break;
			}
			barrier();
		}

		if (ok_lane)
		{
			// SATA Link : phy_calibrated
			BITSET(p_hsio_bus->SATA_LCTRL, Hw17);

			// SATA Link UnReset
			BITSET(p_hsio_bus->SATA_LCTRL, Hw5);
			BITSET(p_hsio_bus->SATA_LCTRL, Hw6);
			BITSET(p_hsio_bus->SATA_LCTRL, Hw7);
			BITSET(p_hsio_bus->SATA_LCTRL, Hw8);
			return 0;
		}
		else
		{
			printk("[%s:%d] LANE_OK Timeout !!!\n", __func__, __LINE__);
		}
	}
	else
	{
		printk("[%s:%d] CMU_OK Timeout !!!\n", __func__, __LINE__);
	}
	
	return -1;

}

static void hba_reset(void)
{
	j_print("[%s:%d]\n", __func__, __LINE__);

	/* Write GHC to reset HBA */
	ahb_write_single(NON_PORT,
					 HDATA_WIDTH, // AHB write size
					 HBA_BASE_ADDRESS + ATB_GCSR_GHC, // Global host control register
					 0x1);
}

int enable_tcc_sata_phy(void)
{
	enable_ahsata();
	sata_cfg();
	if (sata_enable())
	{
		printk("[%s:%d] cannot enable SATA !!!\n", __func__, __LINE__);
		return -1;
	}
	hba_reset();
	return 0;
}

EXPORT_SYMBOL(enable_tcc_sata_phy);


