#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/delay.h>
#include <linux/fb.h>
#include <linux/init.h>
#include <linux/dma-mapping.h>
#include <linux/interrupt.h>
#include <linux/workqueue.h>
#include <linux/wait.h>
#include <linux/platform_device.h>
#include <linux/clk.h>

#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/div64.h>
#include <asm/mach/map.h>
#ifdef CONFIG_PM
#include <linux/pm.h>
#endif

#include <mach/bsp.h>
#include <mach/tca_ckc.h>



#ifdef CONFIG_FB_TCC92XX_WVGA_DX08D11VM0AAA
#include <mach/dev_gpsb.h>
#include <mach/TCC_LCD_Interface.h>
#include <mach/TCC_LCD_DriverCtrl.h>

#if 0
#define dprintk(msg...)	 { printk( "DX08D11VM0AAA: " msg); }
#else
#define dprintk(msg...)	 
#endif

#define WAIT_FLAG	0xFFFF
#define END_FLAG	0xFFFE

unsigned short gLCDInit_Table[][2]=
{
		{0x002, 0x0000}, 
		{0x003, 0x0000},
		{0x004, 0x0000},
		{0x010, 0x0210}, 
		{0x020, 0x0000},
		{0x021, 0x2831},
		{0x022, 0x003E},
		{0x023, 0x7400},
		{0x024, 0x7400},
		{0x025, 0x6A06},
		{0x026, 0x7400},
		{0x027, 0x1C06},
		{0x028, 0x1C28},
		{0x029, 0x1C4A},
		{0x02A, 0x0666},
		{0x100, 0x0533},
		{0x101, 0x0003},
		{0x102, 0x3700},
#if 1		
		{0x300, 0x7777}, 
		{0x301, 0x0717},
		{0x302, 0xC114},
		{0x303, 0x0071}, 
		{0x304, 0x7110}, 
		{0x305, 0x01D4},
#else
		{0x300, 0x5757}, //0x7777
		{0x301, 0x0537}, //0x0717
		{0x302, 0xC114},
		{0x303, 0x0053}, //0x0071
		{0x304, 0x5130}, //0x7110
		{0x305, 0xC317}, //0x01D4
#endif
		{0x402, 0x0000},
		{0x501, 0xFFFF},
		{0x502, 0xFFFF},
		{0x503, 0xFFFF},
		{0x504, 0x00FF},
		{0x002, 0x0200},
		{WAIT_FLAG, 2},//wait 10ms
		{0x001, 0x0001},
		{WAIT_FLAG, 1},//wait 2ms
		{0x002, 0x8210},
		{WAIT_FLAG, 4},//wait 20ms
		{0x002, 0x8310},
		{WAIT_FLAG, 4},//wait 20ms
		{0x002, 0x0710},
		{WAIT_FLAG, 4},//wait 20ms
		{0x002, 0x1730},
		{WAIT_FLAG, 4},//wait 20ms
		{0x001, 0x0012},
		{0x001, 0x0032},
		{0x023, 0x0000},
		{WAIT_FLAG, 4},//wait 20ms
		{0x001, 0x0033},
		{0x023, 0x7400},
		{END_FLAG, 0x0}
};

unsigned short gLCDOff_Table[][2]=
{
		{0x203, 0x0000},
		{WAIT_FLAG, 4},//wait 20ms
		{0x001 ,0x032},
		{0x023 ,0x7400},
		{WAIT_FLAG, 4},//wait 20ms
		{0x023, 0x0000},
		{WAIT_FLAG, 4},//wait 20ms
		{0x001 ,0x0012},
		{WAIT_FLAG, 4},//wait 20ms
		{0x001 ,0x0000},
		{WAIT_FLAG, 1},//wait 1ms
		{0x002 ,0x0730},
		{0x100 ,0x0000},
		{0x002 ,0x0000},
		{WAIT_FLAG, 1},//wait 1ms
		{0x002 ,0x0001},
		{WAIT_FLAG, 1},//wait 1ms
		{END_FLAG, 0x0}
};


volatile unsigned int gReg = 0x100000;
volatile unsigned int gWriteData = 0x120000;
volatile unsigned int gReadData = 0x130000;

void DX08D11VM0AAA_Set_Data(unsigned int Reg, unsigned int Data)
{
	unsigned int value;
	
	value = gReg  | Reg;
	DEV_LCD_SPIO_TxRxData(&value, 0, 1);

	value = gWriteData | Data;
	DEV_LCD_SPIO_TxRxData(&value, 0, 1);
}

void Module_device_init(void)
{
	int i = 0;
	DEV_LCD_SPIO_Config_type lcd_spio; 

	lcd_spio.clock = 60000;
	lcd_spio.data_length = 24;
	lcd_spio.port = LCD_GPSB_PORT9;

	DEV_LCD_SPIO_Config(&lcd_spio);

	while(gLCDInit_Table[i][0] != END_FLAG)
	{
		if(gLCDInit_Table[i][0] == WAIT_FLAG)
			LCD_Driver_mdelay(gLCDInit_Table[i][1]);
		else
			DX08D11VM0AAA_Set_Data(gLCDInit_Table[i][0], gLCDInit_Table[i][1]);

		i++;			
	}

}



void Module_device_Off(void)
{
	int i = 0;
	
	DEV_LCD_SPIO_Config_type lcd_spio; 
	
	lcd_spio.clock = 60000;
	lcd_spio.data_length = 24;
	
	lcd_spio.port = LCD_GPSB_PORT9;

	DEV_LCD_SPIO_Config(&lcd_spio);

	while(gLCDOff_Table[i][0] != END_FLAG)
	{
		if(gLCDOff_Table[i][0] == WAIT_FLAG)
			LCD_Driver_mdelay(gLCDOff_Table[i][1]);
		else
			DX08D11VM0AAA_Set_Data(gLCDOff_Table[i][0], gLCDOff_Table[i][1]);

		i++;			
	}

}

// Unit 100hz
#define LCDC_CLK		(230000)
#define LCD_DIVIDER		2

#define LCD_BUS_WIDTH	24

#define LPW		4	// line pulse width
#define LPC		480	// line pulse count (active horizontal pixel - 1)
#define LSWC	8	// line start wait clock (the number of dummy pixel clock - 1)
#define LEWC	5	// line end wait clock (the number of dummy pixel clock - 1)
#define VDB		0	// Back porch Vsync delay
#define VDF		0 	// front porch of Vsync delay
#define FPW1	4	// TFT/TV : Frame pulse width is the pulse width of frmae clock
#define FLC1	800	// frmae line count is the number of lines in each frmae on the screen
#define FSWC1	2	// frmae start wait cycle is the number of lines to insert at the end each frame
#define FEWC1	11	// frame start wait cycle is the number of lines to insert at the begining each frame
#define FPW2	4	// TFT/TV : Frame pulse width is the pulse width of frmae clock
#define FLC2	800	// frmae line count is the number of lines in each frmae on the screen
#define FSWC2	2	// frmae start wait cycle is the number of lines to insert at the end each frame
#define FEWC2 	11	// frame start wait cycle is the number of lines to insert at the begining each frame

#define SYNC_INVET (ID_INVERT_EN|IV_INVERT_EN|IH_INVERT_EN|IP_INVERT_EN)
	
void DEV_LCD_MODULE_SPEC(LCD_MODULE_SPEC_TYPE* spec)
{
	spec->lcdc_clk = LCDC_CLK;
	spec->lcdc_clk_div = LCD_DIVIDER;
	spec->lcd_bus_width = LCD_BUS_WIDTH;
	
	spec->lcd_width = 480;
	spec->lcd_height = 800;
	spec->lcd_LPW = LPW;		// line pulse width
	spec->lcd_LPC = LPC;		// line pulse count (active horizontal pixel - 1)
	spec->lcd_LSWC = LSWC;		// line start wait clock (the number of dummy pixel clock - 1)
	spec->lcd_LEWC = LEWC;		// line end wait clock (the number of dummy pixel clock - 1)
	spec->lcd_VDB = VDB;		// Back porch Vsync delay
	spec->lcd_VDF = VDF;		// front porch of Vsync delay

	spec->lcd_FPW1 = FPW1;		// TFT/TV : Frame pulse width is the pulse width of frmae clock
	spec->lcd_FLC1 = FLC1;		// frmae line count is the number of lines in each frmae on the screen
	spec->lcd_FSWC1 = FSWC1;		// frmae start wait cycle is the number of lines to insert at the end each frame
	spec->lcd_FEWC1 = FEWC1;		// frame start wait cycle is the number of lines to insert at the begining each frame

	spec->lcd_FPW2 = FPW2;		// TFT/TV : Frame pulse width is the pulse width of frmae clock
	spec->lcd_FLC2 = FLC2;		// frmae line count is the number of lines in each frmae on the screen
	spec->lcd_FSWC2 = FSWC2;		// frmae start wait cycle is the number of lines to insert at the end each frame
	spec->lcd_FEWC2 = FEWC2;		// frame start wait cycle is the number of lines to insert at the begining each frame
	spec->SyncSignal_Invert = SYNC_INVET;
}

void GPSB_Port_disable(void)
{
	BITCSET(HwGPIOF->GPFN3, HwPORTCFG_GPFN0_MASK | HwPORTCFG_GPFN2_MASK | HwPORTCFG_GPFN3_MASK,
							 HwPORTCFG_GPFN0(0)|HwPORTCFG_GPFN2(0)| HwPORTCFG_GPFN3(0));

	BITCSET(HwGPIOF->GPEN, Hw24|Hw26|Hw27, Hw24|Hw26|Hw27);
	BITCLR(HwGPIOF->GPDAT, Hw24|Hw26|Hw27);
}

 void DEV_LCD_Module (LCD_MODULE_CTRL uiControl)
{

	dprintk(KERN_INFO "%s :  uiControl: %d\n", __FUNCTION__, uiControl);
	switch(uiControl)
	{
		case LCD_M_INITIALIZE:
			Module_device_init();
			break;
		case LCD_M_ON:
			Module_device_init();
			break;
		case LCD_M_OFF:
			Module_device_Off();
			break;
		default:
			break;
	}
	GPSB_Port_disable();
}


#endif//FB_TCC92XX_WVGA_DX08D11VM0AAA
