

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/delay.h>
#include <linux/fb.h>
#include <linux/init.h>
#include <linux/dma-mapping.h>
#include <linux/interrupt.h>
#include <linux/workqueue.h>
#include <linux/wait.h>
#include <linux/platform_device.h>
#include <linux/clk.h>

#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/div64.h>
#include <asm/mach/map.h>
#ifdef CONFIG_PM
#include <linux/pm.h>
#endif

#include <mach/bsp.h>
#include <mach/tca_ckc.h>


#ifdef __KERNEL__
#	if defined(_TCC79x_)
#		include <mach/tcc79x.h>
#	elif defined(_TCC83x_)
#		include <mach/tcc83x_virt.h>
#	endif
#else
#	if defined(_TCC79x_)
#		include "TCC79x_Virtual.h"
#	elif defined(_TCC83x_)
#		include "TCC83x_Virtual.h"
#	endif
#endif

#include "tca_lcdc.h"

void DEV_LCDSI_DATA(uint32 uiCS, uint32 usData)
{

	if (uiCS & LCDSI_CS00)
		HwLCDSI->DATA0[0] = usData;
	else if (uiCS & LCDSI_CS01)
		HwLCDSI->DATA1[0] = usData;
	else if (uiCS & LCDSI_CS10)
		HwLCDSI->DATA2[0] = usData;
	else 
		HwLCDSI->DATA3[0] = usData;
}

void DEV_LCDSI_BW_Set(unsigned int bus_width)
{
	uint32 uiBw;
	
	if(bus_width == 16) {
		uiBw = Hw15;
	}
	else {
		uiBw = Hw15| Hw31;
	}

	BITCSET(HwLCDSI->CTRL0, HwLCDSICTRL_BW, uiBw);		
	BITCSET(HwLCDSI->CTRL1, HwLCDSICTRL_BW, uiBw);		
	BITCSET(HwLCDSI->CTRL2, HwLCDSICTRL_BW, uiBw);		
	BITCSET(HwLCDSI->CTRL3, HwLCDSICTRL_BW, uiBw);		
}

void DEV_LCDSI_CTRL_SET(TCCLCDC_type lcdc, uint32 LcdSi_reg)
{
	volatile unsigned int LCDSI_CTRL;

	if(lcdc == TCC_LCDC0)
		LCDSI_CTRL = (uint32)&HwLCDSI->L0CTRL;
	else
		LCDSI_CTRL = (uint32)&HwLCDSI->L1CTRL;
	

	BITCSET(LCDSI_CTRL, 0xFFFFFFFF, LcdSi_reg);
}

uint32 DEV_LCDSI_CTRL (TCCLCDC_type lcdc, uint32 uiFlag, uint32 uiOM, uint32 uiIM, uint32 uiFmt, uint32 uiRsp, uint32 uiCs, uint32 uiIa, uint32 uiIvs)
{
	uint32 uiReadSI = 0;
	volatile unsigned int *LCDSI_CTRL;

	
	if(lcdc == TCC_LCDC0)
		LCDSI_CTRL = (volatile unsigned int *)&HwLCDSI->L0CTRL;
	else
		LCDSI_CTRL = (volatile unsigned int *)&HwLCDSI->L1CTRL;
	
	
	if(uiFlag & READ_LCDSI_MODE)
	{
		uiReadSI = (unsigned int)*LCDSI_CTRL;
	}

	if(uiFlag & SET_LCDSI_OUTPUTMODE)
	{
		BITCSET(HwLCDSI->MODE, HwLCDSICTRL0_OM, (uiOM << 0));
	}

	if(uiFlag & SET_LCDSI_INPUTMODE)
	{
		BITCSET(*LCDSI_CTRL, HwLCDSICTRL_EN, uiIM<<8);
	}

	if(uiFlag & SET_LCDSI_FORMAT)
	{
		BITCSET(*LCDSI_CTRL, HwLCDSICTRL_FMT, (uiFmt << 2));
	}

	if(uiFlag & SET_LCDSI_RESPONCE)
	{
		BITCSET(*LCDSI_CTRL, HwLCDSICTRL_RSP, (uiRsp << 6));
	}

	if(uiFlag & SET_LCDSI_CS)
	{
		BITCSET(*LCDSI_CTRL, HwLCDSICTRL_CS, (uiCs << 7));
	}

	if(uiFlag & SET_LCDSI_IA)
	{
		BITCSET(*LCDSI_CTRL, HwLCDSICTRL_IA, (uiIa << 15));
	}

	if(uiFlag & SET_LCDSI_IVS)
	{
		BITCSET(*LCDSI_CTRL, HwLCDSICTRL_IVS, (uiIvs << 14));
	}

	return uiReadSI;
}




void DEV_InitLCDSI_Time(uint32 uiCS, uint32 uiRW, uint32 uiStp, uint32 uiPw, uint32 uiHld)
{

	if (uiCS & LCDSI_CS00)
	{
		if (uiRW & LCDSI_READ)
		{
			BITCSET(HwLCDSI->CTRL0, 0x00007000, (uiStp << 12));
			BITCSET(HwLCDSI->CTRL0, 0x00000FF8, (uiPw << 3));
			BITCSET(HwLCDSI->CTRL0, 0x00000007, uiHld);
		} 

		if(uiRW & LCDSI_WRITE) 
		{
			BITCSET(HwLCDSI->CTRL0, 0x70000000, (uiStp << 28));
			BITCSET(HwLCDSI->CTRL0, 0x0FF80000, (uiPw << 19));
			BITCSET(HwLCDSI->CTRL0, 0x00070000, (uiHld << 16));
		}
	}

	if (uiCS & LCDSI_CS01)
	{
		if (uiRW & LCDSI_READ)
		{
			BITCSET(HwLCDSI->CTRL1, 0x00007000, (uiStp << 12));
			BITCSET(HwLCDSI->CTRL1, 0x00000FF8, (uiPw << 3));
			BITCSET(HwLCDSI->CTRL1, 0x00000007, uiHld);
		}

		if(uiRW & LCDSI_WRITE)
		{
			BITCSET(HwLCDSI->CTRL1, 0x70000000, (uiStp << 28));
			BITCSET(HwLCDSI->CTRL1, 0x0FF80000, (uiPw << 19));
			BITCSET(HwLCDSI->CTRL1, 0x00070000, (uiHld << 16));
		}
	}

	if (uiCS & LCDSI_CS10)
	{

		if (uiRW & LCDSI_READ)
		{
			BITCSET(HwLCDSI->CTRL2, 0x00007000, (uiStp << 12));
			BITCSET(HwLCDSI->CTRL2, 0x00000FF8, (uiPw << 3));
			BITCSET(HwLCDSI->CTRL2, 0x00000007, uiHld);
		}

		if(uiRW & LCDSI_WRITE)
		{
			BITCSET(HwLCDSI->CTRL2, 0x70000000, (uiStp << 28));
			BITCSET(HwLCDSI->CTRL2, 0x0FF80000, (uiPw << 19));
			BITCSET(HwLCDSI->CTRL2, 0x00070000, (uiHld << 16));
		}
	}

	if (uiCS & LCDSI_CS11)
	{

		if (uiRW & LCDSI_READ)
		{
			BITCSET(HwLCDSI->CTRL3, 0x00007000, (uiStp << 12));
			BITCSET(HwLCDSI->CTRL3, 0x00000FF8, (uiPw << 3));
			BITCSET(HwLCDSI->CTRL3, 0x00000007, uiHld);
		}

		if(uiRW & LCDSI_WRITE)
		{
			BITCSET(HwLCDSI->CTRL3, 0x70000000, (uiStp << 28));
			BITCSET(HwLCDSI->CTRL3, 0x0FF80000, (uiPw << 19));
			BITCSET(HwLCDSI->CTRL3, 0x00070000, (uiHld << 16));
		}
	}
}
void DEV_LCDSI_Time(unsigned int system_clk, unsigned int stp_ns, unsigned int pw_ns, unsigned int hld_ns)
{
	#define TIMING(time, clock) ((time) / (1000/clock) + 1)
	
	unsigned int setup_time, plus_width, hold_time;

	setup_time = TIMING(stp_ns, system_clk/10000);
	plus_width = TIMING(pw_ns,  system_clk/10000);
	hold_time  = TIMING(hld_ns, system_clk/10000);

	DEV_InitLCDSI_Time(LCDSI_CS00,LCDSI_READ, setup_time*2, plus_width*2, hold_time*2);
	DEV_InitLCDSI_Time(LCDSI_CS00,LCDSI_WRITE, setup_time, plus_width, hold_time);
	DEV_InitLCDSI_Time(LCDSI_CS01,LCDSI_READ, setup_time*2, plus_width*2, hold_time*2);
	DEV_InitLCDSI_Time(LCDSI_CS01,LCDSI_WRITE, setup_time, plus_width, hold_time);

}


