//-------------------------------------------------------------------
// Copyright SAMSUNG Electronics Co., Ltd
// All right reserved.
//
// This software is the confidential and proprietary information
// of Samsung Electronics, Inc. ("Confidential Information").  You
// shall not disclose such Confidential Information and shall use
// it only in accordance with the terms of the license agreement
// you entered into with Samsung Electronics.
//-------------------------------------------------------------------
/**
 * @file key-public.h
 * @brief This file has encrypted HDCP A1 test key.
 *
 * @author   Digital IP Development Team (mrkim@samsung.com) @n
 *           SystemLSI, Samsung Electronics
 * @version  1.0
 *
 * 09-30-2008 1.0 Initial version
 */
#ifndef _HDMI_KEY_
#define _HDMI_KEY_

/** Size of HDCP device private key */
#define HDCP_KEY_SIZE   288

/**
 * @brief   AES-encrypted HDCP device private keys\n
 * User should set their own AES-encrypted HDCP device private keys.
 */
static const unsigned char HDCP_Test_key[HDCP_KEY_SIZE] =
{
    // Encrypted Transmitter A1 key
    // (See HDCP v1.3 Specification Appendix A)
    // AES key value used for HDCP key encryption
    //    uint8 key[16]=
    //{
    //    0x02,0x0a,0x26,0xe4,0x08,0x01,0x21,0x12,0x08,0x04,
    //    0xc0,0x89,0xff,0xff,0xff,0xff
    //};
    0x48,0xf8,0x11,0xb6,0x85,0x66,0x9b,0x65,0x0b,0x9f,
    0x5a,0x01,0xb4,0x43,0xaf,0xd7,0x34,0xeb,0xbe,0xe0,
    0x52,0xfb,0x85,0xfe,0xfa,0xb1,0x2f,0xe4,0xc3,0xce,
    0xa9,0x27,0x33,0x74,0x97,0xd8,0xfc,0x62,0xb8,0x92,
    0x4a,0xb6,0xce,0x7b,0xb8,0xda,0x67,0xbf,0xda,0xea,
    0xbf,0xa9,0xc0,0x2a,0xc8,0xf6,0x44,0x41,0x5a,0x10,
    0x59,0x88,0x54,0xcf,0x51,0x91,0x12,0xd5,0xa8,0x41,
    0x3a,0x8a,0x88,0xd1,0x5a,0x9a,0x55,0xc1,0xbb,0x5e,
    0x8a,0xa0,0x84,0x1b,0xa8,0xea,0x31,0x59,0xea,0x71,
    0x0c,0xcf,0x59,0xf5,0xa8,0x32,0x57,0xbb,0xd4,0xa0,
    0x5b,0x88,0x44,0x66,0xd6,0x80,0xfa,0xe9,0x18,0xe0,
    0x50,0x73,0x92,0x63,0xe1,0x5c,0x13,0xbf,0x7d,0x0d,
    0x70,0x0b,0xf8,0x25,0x4a,0x3b,0x9c,0x17,0x56,0xb3,
    0x71,0x2b,0xfe,0x3c,0xcb,0x7c,0x19,0x28,0x53,0xa7,
    0x5c,0x57,0x47,0xe3,0xe1,0x4c,0x76,0x62,0x0a,0x40,
    0x30,0xcf,0xbe,0x51,0xaf,0x0d,0x11,0x73,0xd6,0x6a,
    0xc2,0xbf,0x4f,0xc1,0x88,0x8d,0x14,0xa6,0xd1,0x92,
    0x6c,0xf7,0x8a,0xe6,0x9c,0x96,0xc5,0xc4,0x5c,0x36,
    0xf6,0xfb,0x39,0xf4,0x79,0x3f,0x7a,0x30,0x71,0x5e,
    0x3e,0xfe,0xf3,0x4d,0x0c,0x02,0x55,0xeb,0x08,0x24,
    0x5f,0x64,0xd7,0xcf,0xf3,0x48,0x35,0x03,0xc4,0xc8,
    0x29,0xf7,0x9d,0xcf,0x21,0xb8,0x67,0x05,0xc6,0x47,
    0x05,0x1b,0x5f,0xf3,0xa7,0xbc,0x23,0xf0,0x09,0xc4,
    0x90,0x44,0x5d,0x3f,0xf9,0x79,0x74,0xea,0x7b,0x42,
    0x57,0x88,0xce,0x32,0x43,0xa5,0xf4,0x4e,0x05,0xc9,
    0x73,0xc2,0x49,0x94,0x85,0x5c,0xa2,0x11,0x91,0x1f,
    0x9e,0xe3,0x21,0xbe,0xe9,0x36,0x52,0xec,0x4b,0xa6,
    0x7d,0xf6,0x8a,0x85,0xb9,0xe1,0xc7,0x6e,0x6b,0x08,
    0x9d,0xf2,0xee,0x7d,0x28,0xbd,0xf0,0x9d
};

#endif // _HDMI_KEY_
