//-------------------------------------------------------------------
// Copyright TELECHIPS Co., Ltd
// All right reserved.
//
//-------------------------------------------------------------------
/**
 * @file  lcdc.c
 * @brief This file contains an implementation of lcdc device driver.
 *
 * @author  WinCE/Linux BSP Team (nuekii@telechips.com) \n
 *          System Group 4, Telechips
 *
 * @version 1.1
 *
 * @remarks 04-26-2009 1.0 Initial version \n
 *          xx-xx-2009 1.1 xxxx\n
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/interrupt.h>
#include <linux/fs.h>
#include <linux/miscdevice.h>
#include <linux/errno.h>
#include <linux/wait.h>
#include <linux/poll.h>
#include <linux/i2c.h>
#include <linux/clk.h>



#include <asm/io.h>
#include <asm/mach-types.h>
#include <asm/uaccess.h>

#if defined(CONFIG_ARCH_TCC92X)
#include <mach/bsp.h>
#elif defined(CONFIG_ARCH_TCC79X)
#include <mach/tcc79x.h>
#endif
#include <asm/delay.h>
#include <linux/delay.h>
#include <mach/tca_ckc.h>



#include <mach/audio.h>
#include <mach/hdmi.h>
#include <mach/tcc92xxfb_ioctrl.h>


// composite, component ߰ >>>
#include <mach/tcc_composite_ioctl.h>
#include <mach/tcc_component_ioctl.h>
#include <mach/tca_fb_output.h>

extern char TCC_FB_LCDC_NumSet(char NumLcdc);
extern void TCC_OUTPUT_LCDC_OnOff(char output_type, char output_lcdc_num, char onoff);
extern void tcc_composite_start(TCC_COMPOSITE_MODE_TYPE mode);
extern void tcc_composite_clock_onoff(char OnOff);

extern void tcc_component_start(TCC_COMPONENT_MODE_TYPE mode);
extern void tcc_component_clock_onoff(char OnOff);
// composite, component ߰ << 

/* Debugging stuff */
static int debug = 1;
#define DPRINTF(msg...)	if (debug) { printk( "external display: " msg); }


extern void tcc92xxfb_hdmi_starter(char hdmi_lcdc_num, struct lcdc_timimg_parms_t *lcdc_timing);

extern int hdmi_set_color_space(enum ColorSpace);
extern int hdmi_set_color_depth(enum ColorDepth);
extern void hdmi_set_video_mode(struct device_video_params mode);
extern int hdmi_set_pixel_limit(enum PixelLimit);
extern int hdmi_set_hdmimode(int mode);

extern int hdmi_set_pixel_aspect_ratio(enum PixelAspectRatio ratio);

extern void tcc_hdmi_power_on(void);
extern void hdmi_start(void);

extern struct display_platform_data tcc_display_data;

enum PixelFreq {
    /** 25.2 MHz pixel frequency */
    PIXEL_FREQ_25_200 = 2520,
    /** 25.175 MHz pixel frequency */
    PIXEL_FREQ_25_175 = 2517,
    /** 27 MHz pixel frequency */
    PIXEL_FREQ_27 = 2700,
    /** 27.027 MHz pixel frequency */
    PIXEL_FREQ_27_027 = 2702,
    /** 54 MHz pixel frequency */
    PIXEL_FREQ_54 = 5400,
    /** 54.054 MHz pixel frequency */
    PIXEL_FREQ_54_054 = 5405,
    /** 74.25 MHz pixel frequency */
    PIXEL_FREQ_74_250 = 7425,
    /** 74.176 MHz pixel frequency */
    PIXEL_FREQ_74_176 = 7417,
    /** 148.5 MHz pixel frequency */
    PIXEL_FREQ_148_500 = 14850,
    /** 148.352 MHz pixel frequency */
    PIXEL_FREQ_148_352 = 14835,
    /** 108.108 MHz pixel frequency */
    PIXEL_FREQ_108_108 = 10810,
    /** 72 MHz pixel frequency */
    PIXEL_FREQ_72 = 7200,
    /** 25 MHz pixel frequency */
    PIXEL_FREQ_25 = 2500,
    /** 65 MHz pixel frequency */
    PIXEL_FREQ_65 = 6500,
    /** 108 MHz pixel frequency */
    PIXEL_FREQ_108 = 10800,
    /** 162 MHz pixel frequency */
    PIXEL_FREQ_162 = 16200,
};

//! Structure for video timing parameters
static const struct hdmi_video_params
{
    /** [H Blank] */
    unsigned int    HBlank;

    /** [V Blank] */
    unsigned int    VBlank;

    /**
     * [H Total : V Total] @n
     * For more information, refer HDMI register map.
     */
    unsigned int    HVLine;

    /**
     * [H Sync polarity : H Sync end point : H Sync start point]@n
     * For more information, refer HDMI register map.
     */
    unsigned int    HSYNCGEN;

    /**
     * [V Sync start line num + V Sync end line num] @n
     * For more information, refer HDMI register map.
     */
    unsigned int    VSYNCGEN;

    /** CEA VIC */
    unsigned char   AVI_VIC;
    /** CEA VIC for 16:9 pixel ratio */
    unsigned char   AVI_VIC_16_9;

    /** 0 - progresive, 1 - interlaced */
    unsigned char   interlaced;

    /** Pixel repetition if double, set 1 */
    unsigned char   repetition;

    /** V Sync polarity */
    unsigned char   polarity;

    /**
     * In case of interlaced mode, @n
     * [end point of bottom field's active region : start point of that]@n
     * For more information, refer HDMI register map.
     */
    unsigned int    VBLANK_F;

    /**
     * In case of interlaced mode, @n
     * [start line of bottom field's V Sync : end line of that]@n
     * For more information, refer HDMI register map.
     */
    unsigned int    VSYNCGEN2;

    /**
     * In case of interlaced mode, @n
     * [start transition point of bottom field's V Sync : end transition of that]@n
     * For more information, refer HDMI register map.
     */
    unsigned int    VSYNCGEN3;

    /** Pixel frequency */
    enum PixelFreq PixelClock; // pixel clock
} HDMIVideoParams[] =
{

  //{ 0x140, 0x13326, 0x540326, 0x1070A0, 0x2008 , 1 , 1 , 0, 0, 0, 0       , 0       , 0       , PIXEL_FREQ_65, },  // v1024x768p_60Hz
  //{ 0x140, 0x13326, 0x540326, 0x1701C , 0x3009 , 1 , 1 , 0, 0, 0, 0       , 0       , 0       , PIXEL_FREQ_65, },  // v1024x768p_60Hz
    { 0xA0 , 0x16A0D, 0x32020D, 0x11B80E, 0xA00C , 1 , 1 , 0, 0, 1, 0       , 0       , 0       , PIXEL_FREQ_25_200,  },  // v640x480p_60Hz
    { 0x8A , 0x16A0D, 0x35A20D, 0x11300E, 0x900F , 2 , 3 , 0, 0, 1, 0       , 0       , 0       , PIXEL_FREQ_27_027,  },  // v720x480p_60Hz
    { 0x172, 0xF2EE , 0x6722EE, 0x2506C , 0x500A , 4 , 4 , 0, 0, 0, 0       , 0       , 0       , PIXEL_FREQ_74_250,  },  // v1280x720p_60Hz
    { 0x118, 0xB232 , 0x898465, 0x20856 , 0x2007 , 5 , 5 , 1, 0, 0, 0x232A49, 0x234239, 0x4A44A4, PIXEL_FREQ_74_250,  },  // v1920x1080i_60Hz
    { 0x114, 0xB106 , 0x6B420D, 0x128024, 0x4007 , 6 , 7 , 1, 1, 1, 0x10691D, 0x10A10D, 0x380380, PIXEL_FREQ_27_027,  },  // v720x480i_60Hz
    { 0x114, 0xB106 , 0x6B4106, 0x128024, 0x4007 , 8 , 9 , 0, 1, 1, 0       , 0       , 0       , PIXEL_FREQ_27_027,  },  // v720x240p_60Hz
    { 0x228, 0xB106 , 0xD6820D, 0x15084A, 0x4007 , 10, 11, 1, 1, 1, 0x10691D, 0x10A10D, 0x700700, PIXEL_FREQ_54_054,  },  // v2880x480i_60Hz
    { 0x228, 0xB106 , 0x6B4106, 0x15084A, 0x4007 , 12, 13, 0, 1, 1, 0       , 0       , 0       , PIXEL_FREQ_54_054,  },  // v2880x240p_60Hz
    { 0x114, 0x16A0D, 0x6B420D, 0x12681E, 0x900F , 14, 15, 0, 1, 1, 0       , 0       , 0       , PIXEL_FREQ_54_054,  },  // v1440x480p_60Hz
    { 0x118, 0x16C65, 0x898465, 0x20856 , 0x4009 , 16, 16, 0, 0, 0, 0       , 0       , 0       , PIXEL_FREQ_148_500, },  // v1920x1080p_60Hz
    { 0x90 , 0x18A71, 0x360271, 0x11280A, 0x500A , 17, 18, 0, 0, 1, 0       , 0       , 0       , PIXEL_FREQ_27,      },  // v720x576p_50Hz
    { 0x2BC, 0xF2EE , 0x7BC2EE, 0x779B6 , 0x500A , 19, 19, 0, 0, 0, 0       , 0       , 0       , PIXEL_FREQ_74_250,  },  // v1280x720p_50Hz
    { 0x2D0, 0xB232 , 0xA50465, 0x8EA0E , 0x2007 , 20, 20, 1, 0, 0, 0x232A49, 0x234239, 0x738738, PIXEL_FREQ_74_250,  },  // v1920x1080i_50Hz
    { 0x120, 0xC138 , 0x6C0271, 0x125016, 0x2005 , 21, 22, 1, 1, 1, 0x138951, 0x13A13D, 0x378378, PIXEL_FREQ_27,      },  // v720x576i_50Hz
    { 0x120, 0xC138 , 0x6C0138, 0x125016, 0x3006 , 23, 24, 0, 1, 1, 0       , 0       , 0       , PIXEL_FREQ_27,      },  // v720x288p_50Hz
    { 0x240, 0xC138 , 0xD80271, 0x14A82E, 0x2005 , 25, 26, 1, 1, 1, 0x138951, 0x13A13D, 0x6F06F0, PIXEL_FREQ_54,      },  // v2880x576i_50Hz
    { 0x240, 0xC138 , 0xD80138, 0x14A82E, 0x2005 , 27, 28, 0, 1, 1, 0       , 0       , 0       , PIXEL_FREQ_54,      },  // v2880x288p_50Hz
    { 0x120, 0x18A71, 0x6C0271, 0x125816, 0x500A , 29, 30, 0, 1, 1, 0       , 0       , 0       , PIXEL_FREQ_54,      },  // v1440x576p_50Hz
    { 0x2D0, 0x16C65, 0xA50465, 0x8EA0E , 0x4009 , 31, 31, 0, 0, 0, 0       , 0       , 0       , PIXEL_FREQ_148_500, },  // v1920x1080p_50Hz
    { 0x33E, 0x16C65, 0xABE465, 0xAA27C , 0x4009 , 32, 32, 0, 0, 0, 0       , 0       , 0       , PIXEL_FREQ_74_250,  },  // v1920x1080p_24Hz
    { 0x2D0, 0x16C65, 0xA50465, 0x8EA0E , 0x4009 , 33, 33, 0, 0, 0, 0       , 0       , 0       , PIXEL_FREQ_74_250,  },  // v1920x1080p_25Hz
    { 0x2D0, 0x16C65, 0xA50465, 0x8EA0E , 0x4009 , 34, 34, 0, 0, 0, 0       , 0       , 0       , PIXEL_FREQ_74_250,  },  // v1920x1080p_30Hz
    { 0x228, 0x16A0D, 0xD6820D, 0x14D83E, 0x900F , 35, 36, 0, 0, 1, 0       , 0       , 0       , PIXEL_FREQ_108_108, },  // v2880x480p_60Hz
    { 0x240, 0x18A71, 0xD80271, 0x14B82E, 0x500A , 37, 38, 0, 1, 1, 0       , 0       , 0       , PIXEL_FREQ_108,     },  // v2880x576p_50Hz
    { 0x180, 0x2AA71, 0x9004E2, 0x3181E , 0x1701C, 39, 39, 0, 0, 0, 0x2712C6, 0x28728F, 0x4a44a4, PIXEL_FREQ_72,      },  // v1920x1080i_50Hz(1250)
    { 0x2D0, 0xB232 , 0xA50465, 0x8EA0E , 0x2007 , 40, 40, 1, 0, 0, 0x232A49, 0x234239, 0x738738, PIXEL_FREQ_148_500, },  // v1920x1080i_100Hz
    { 0x2BC, 0xF2EE , 0x7BC2EE, 0x779B6 , 0x500A , 41, 41, 0, 0, 0, 0       , 0       , 0       , PIXEL_FREQ_148_500, },  // v1280x720p_100Hz
    { 0x90 , 0x18A71, 0x360271, 0x11280A, 0x500A , 42, 43, 0, 0, 1, 0       , 0       , 0       , PIXEL_FREQ_54,      },  // v720x576p_100Hz
    { 0x120, 0xC138 , 0x6C0271, 0x125016, 0x2005 , 44, 45, 1, 1, 1, 0x138951, 0x13A13D, 0x378378, PIXEL_FREQ_54,      },  // v720x576i_100Hz
    { 0x118, 0xB232 , 0x898465, 0x20856 , 0x2007 , 46, 46, 1, 0, 0, 0x232A49, 0x234239, 0x4A44A4, PIXEL_FREQ_148_500, },  // v1920x1080i_120Hz
    { 0x172, 0xF2EE , 0x6722EE, 0x2506C , 0x500A , 47, 47, 0, 0, 0, 0       , 0       , 0       , PIXEL_FREQ_148_500, },  // v1280x720p_120Hz
    { 0x8A , 0x16A0D, 0x35A20D, 0x11300E, 0x900F , 48, 49, 0, 0, 1, 0       , 0       , 0       , PIXEL_FREQ_54_054,  },  // v720x480p_120Hz
    { 0x114, 0xB106 , 0x6B420D, 0x128024, 0x4007 , 50, 51, 1, 1, 1, 0x10691D, 0x10A10D, 0x380380, PIXEL_FREQ_54_054,  },  // v720x480i_120Hz
    { 0x90 , 0x18A71, 0x360271, 0x11280A, 0x500A , 52, 53, 0, 0, 1, 0       , 0       , 0       , PIXEL_FREQ_108,     },  // v720x576p_200Hz
    { 0x120, 0xC138 , 0x6C0271, 0x125016, 0x2005 , 54, 55, 1, 1, 1, 0x138951, 0x13A13D, 0x378378, PIXEL_FREQ_108,     },  // v720x576i_200Hz
    { 0x8A , 0x16A0D, 0x35A20D, 0x11300E, 0x900F , 56, 57, 0, 0, 1, 0       , 0       , 0       , PIXEL_FREQ_108_108, },  // v720x480p_240Hz
    { 0x114, 0xB106 , 0x6B420D, 0x128024, 0x4007 , 58, 59, 1, 1, 1, 0x10691D, 0x10A10D, 0x380380, PIXEL_FREQ_108_108, },  // v720x480i_240Hz
};


#if defined(CONFIG_MACH_TCC9200) || defined(CONFIG_MACH_TCC9201) || defined(CONFIG_MACH_TCC8900)
static const unsigned char phy_config[][3][40] = {
	// for internal PLL
	// freq = 25.200 MHz
	{
		{ 0x85, 0x88, 0x20, 0x50, 0x00, 0x0C, 0x00, 0x06, 0xD8, 0xC2, 0xA4, 0x46, 0x08, 0x0A, 0xBB, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x0C, 0x03, 0x23, 0x38, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0x24, 0x96, 0x6D, 0x02, 0x00, 0x80 },
		{ 0x85, 0x88, 0x20, 0x50, 0x00, 0x0C, 0x00, 0x06, 0xD8, 0xC2, 0xA4, 0x56, 0x08, 0x0A, 0xBB, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x0C, 0x04, 0x23, 0x46, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0x24, 0x96, 0x57, 0x02, 0x00, 0x80 },
		{ 0x85, 0x88, 0x20, 0x50, 0x00, 0x0C, 0x00, 0x06, 0xD8, 0xC2, 0xA4, 0x56, 0x08, 0x0A, 0xBB, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x08, 0x02, 0x23, 0x2A, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0x24, 0x16, 0x92, 0x02, 0x00, 0x80 },
	},
	// freq = 25.175 MHz
	{
		{ 0x85, 0x88, 0x20, 0x50, 0x00, 0x0C, 0x00, 0x06, 0xD8, 0xC2, 0xA4, 0x46, 0x07, 0x0A, 0xBB, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x0C, 0x03, 0x23, 0x38, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0x24, 0x96, 0x6D, 0x02, 0x00, 0x80 },
		{ 0x85, 0x88, 0x20, 0x50, 0x00, 0x0C, 0x00, 0x06, 0xD8, 0xC2, 0xA4, 0x56, 0x07, 0x0A, 0xBB, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x0C, 0x04, 0x23, 0x46, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0x24, 0x96, 0x57, 0x02, 0x00, 0x80 },
		{ 0x85, 0x88, 0x20, 0x50, 0x00, 0x0C, 0x00, 0x06, 0xD8, 0xC2, 0xA4, 0x56, 0x07, 0x0A, 0xBB, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x08, 0x02, 0x23, 0x2A, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0x24, 0x56, 0x92, 0x02, 0x00, 0x80 },
	},
	// freq = 27 MHz
	{
		{ 0x85, 0x88, 0x00, 0xA0, 0x34, 0x07, 0x0D, 0x0B, 0xD8, 0xC2, 0xA4, 0x06, 0xD8, 0x5B, 0xE2, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x1C, 0x03, 0x23, 0x3C, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0xFF, 0x55, 0x66, 0x02, 0x00, 0x80 },
		{ 0x85, 0x88, 0x00, 0xA0, 0x34, 0x07, 0x0D, 0x0B, 0xD8, 0xC2, 0xA4, 0x16, 0xD8, 0x5B, 0xE2, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x1C, 0x04, 0x23, 0x4B, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0xFF, 0xD5, 0x51, 0x02, 0x00, 0x80 },
		{ 0x85, 0x88, 0x00, 0xA0, 0x34, 0x07, 0x0D, 0x0B, 0xD8, 0xC2, 0xA4, 0x16, 0xD8, 0x5B, 0xE2, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x18, 0x02, 0x23, 0x2D, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0xFF, 0x55, 0x88, 0x02, 0x00, 0x80 },
	},
	// freq = 27.027 MHz
	{
		{ 0x85, 0x88, 0x00, 0xA0, 0x34, 0x07, 0x0D, 0x0B, 0xD8, 0xC2, 0xA4, 0x46, 0x28, 0x4D, 0xE3, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x1C, 0x03, 0x23, 0x3C, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0xFE, 0x15, 0x66, 0x02, 0x00, 0x80 },
		{ 0x85, 0x88, 0x00, 0xA0, 0x34, 0x07, 0x0D, 0x0B, 0xD8, 0xC2, 0xA4, 0x56, 0x28, 0x4D, 0xE3, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x1C, 0x04, 0x23, 0x4B, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0xFE, 0x95, 0x51, 0x02, 0x00, 0x80 },
		{ 0x85, 0x88, 0x00, 0xA0, 0x34, 0x07, 0x0D, 0x0B, 0xD8, 0xC2, 0xA4, 0x56, 0x28, 0x4D, 0xE3, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x18, 0x02, 0x23, 0x2D, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0xFE, 0x55, 0x88, 0x02, 0x00, 0x80 },
	},
	// freq = 54 MHz
	{
		{ 0x85, 0x88, 0x00, 0xA0, 0x34, 0x07, 0x0D, 0x07, 0xD8, 0xC2, 0xA4, 0x06, 0xD8, 0x5B, 0xE2, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x18, 0x03, 0x23, 0x3C, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0xFF, 0x54, 0x66, 0x02, 0x00, 0x80 },
		{ 0x85, 0x88, 0x00, 0xA0, 0x34, 0x07, 0x0D, 0x07, 0xD8, 0xC2, 0xA4, 0x16, 0xD8, 0x5B, 0xE2, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x18, 0x04, 0x23, 0x4B, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0xFF, 0xD4, 0x51, 0x02, 0x00, 0x80 },
		{ 0x85, 0x88, 0x00, 0xA0, 0x34, 0x07, 0x0D, 0x07, 0xD8, 0xC2, 0xA4, 0x16, 0xD8, 0x5B, 0xE2, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x14, 0x02, 0x23, 0x2D, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0xFF, 0x54, 0x88, 0x02, 0x00, 0x80 },
	},
	// freq = 54.054 MHz
	{
		{ 0x85, 0x88, 0x00, 0xA0, 0x34, 0x07, 0x0D, 0x07, 0xD8, 0xC2, 0xA4, 0x46, 0x28, 0x4D, 0xE3, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x18, 0x03, 0x23, 0x3C, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0xFF, 0x14, 0x66, 0x02, 0x00, 0x80 },
		{ 0x85, 0x88, 0x00, 0xA0, 0x34, 0x07, 0x0D, 0x07, 0xD8, 0xC2, 0xA4, 0x56, 0x28, 0x4D, 0xE3, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x18, 0x04, 0x23, 0x4B, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0xFF, 0x94, 0x51, 0x02, 0x00, 0x80 },
		{ 0x85, 0x88, 0x00, 0xA0, 0x34, 0x07, 0x0D, 0x07, 0xD8, 0xC2, 0xA4, 0x56, 0x28, 0x4D, 0xE3, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x14, 0x02, 0x23, 0x2D, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0xFF, 0x54, 0x88, 0x02, 0x00, 0x80 },
	},
	// freq = 74.25 MHz
	{
		{ 0x85, 0x88, 0x00, 0xA0, 0x2C, 0x07, 0x00, 0x06, 0xD8, 0xC2, 0xA4, 0x06, 0xD8, 0x5B, 0x22, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x18, 0x03, 0xA3, 0x52, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0xB9, 0x54, 0x4A, 0x02, 0x00, 0x80 },
		{ 0x85, 0x88, 0x00, 0xA0, 0x2C, 0x07, 0x00, 0x06, 0xD8, 0xC2, 0xA4, 0x16, 0xD8, 0x5B, 0x22, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x14, 0x01, 0xA3, 0x33, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0xB9, 0x14, 0x77, 0x02, 0x00, 0x80 },
		{ 0x85, 0x88, 0x00, 0xA0, 0x2C, 0x07, 0x00, 0x06, 0xD8, 0xC2, 0xA4, 0x16, 0xD8, 0x5B, 0x22, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x14, 0x02, 0x23, 0x3E, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0xB9, 0x14, 0x63, 0x02, 0x00, 0x80 },
	},
	// freq = 74.176 MHz
	{
		{ 0x85, 0x88, 0x00, 0xA0, 0x2C, 0x07, 0x00, 0x06, 0xD8, 0xC2, 0xA4, 0x46, 0xD8, 0x5B, 0x23, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x18, 0x03, 0xA3, 0x52, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0xB9, 0x54, 0x4A, 0x02, 0x00, 0x80 },
		{ 0x85, 0x88, 0x00, 0xA0, 0x2C, 0x07, 0x00, 0x06, 0xD8, 0xC2, 0xA4, 0x56, 0xD8, 0x5B, 0x23, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x14, 0x01, 0xA3, 0x33, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0xB9, 0x14, 0x77, 0x02, 0x00, 0x80 },
		{ 0x85, 0x88, 0x00, 0xA0, 0x2C, 0x07, 0x00, 0x06, 0xD8, 0xC2, 0xA4, 0x56, 0xD8, 0x5B, 0x23, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x14, 0x02, 0xE3, 0x3D, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0xB9, 0x54, 0x63, 0x02, 0x00, 0x80 },
	},
	// freq = 148.5 MHz
	{
		{ 0x85, 0x88, 0x00, 0xA0, 0x2C, 0x07, 0x00, 0x02, 0xD8, 0xC2, 0xA4, 0x06, 0xD8, 0x5B, 0x22, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x14, 0x03, 0xA3, 0x52, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0x5C, 0x54, 0x4A, 0x02, 0x00, 0x80 },
		{ 0x85, 0x88, 0x00, 0xA0, 0x2C, 0x07, 0x00, 0x02, 0xD8, 0xC2, 0xA4, 0x16, 0xD8, 0x5B, 0x22, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x10, 0x01, 0xA3, 0x33, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0x5C, 0x14, 0x77, 0x02, 0x00, 0x80 },
		{ 0x85, 0x88, 0x00, 0xA0, 0x2C, 0x07, 0x00, 0x02, 0xD8, 0xC2, 0xA4, 0x16, 0xD8, 0x5B, 0x22, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x10, 0x02, 0x23, 0x3E, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0x5C, 0x14, 0x63, 0x02, 0x00, 0x80 },
	},
	// freq = 148.352 MHz
	{
		{ 0x85, 0x88, 0x00, 0xA0, 0x2C, 0x07, 0x00, 0x02, 0xD8, 0xC2, 0xA4, 0x46, 0xD8, 0x5B, 0x23, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x14, 0x03, 0xA3, 0x52, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0x5C, 0x54, 0x4A, 0x02, 0x00, 0x80 },
		{ 0x85, 0x88, 0x00, 0xA0, 0x2C, 0x07, 0x00, 0x02, 0xD8, 0xC2, 0xA4, 0x56, 0xD8, 0x5B, 0x23, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x10, 0x01, 0xA3, 0x33, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0x5C, 0x14, 0x77, 0x02, 0x00, 0x80 },
		{ 0x85, 0x88, 0x00, 0xA0, 0x2C, 0x07, 0x00, 0x02, 0xD8, 0xC2, 0xA4, 0x56, 0xD8, 0x5B, 0x23, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x10, 0x02, 0xE3, 0x3D, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0x5C, 0x54, 0x63, 0x02, 0x00, 0x80 },
	},
	// freq = 108.108 MHz
	{
		{ 0x85, 0x88, 0x00, 0xA0, 0x34, 0x07, 0x0D, 0x03, 0xD8, 0xC2, 0xA4, 0x46, 0x28, 0x4D, 0xE3, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x14, 0x03, 0x23, 0x3C, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0x7F, 0x14, 0x66, 0x02, 0x00, 0x80 },
		{ 0x85, 0x88, 0x00, 0xA0, 0x34, 0x07, 0x0D, 0x03, 0xD8, 0xC2, 0xA4, 0x56, 0x28, 0x4D, 0xE3, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x14, 0x04, 0x23, 0x4B, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0x7F, 0x94, 0x51, 0x02, 0x00, 0x80 },
		{ 0x85, 0x88, 0x00, 0xA0, 0x34, 0x07, 0x0D, 0x03, 0xD8, 0xC2, 0xA4, 0x56, 0x28, 0x4D, 0xE3, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x10, 0x02, 0x23, 0x2D, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0x7F, 0x54, 0x88, 0x02, 0x00, 0x80 },
	},
	// freq = 72 MHz
	{
		{ 0x85, 0x88, 0x00, 0xA0, 0x30, 0x0B, 0x00, 0x02, 0xD8, 0xC2, 0xA4, 0x06, 0xD8, 0x5B, 0x82, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x08, 0x03, 0x23, 0x50, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0xBF, 0x94, 0x4C, 0x02, 0x00, 0x80 },
		{ 0x85, 0x88, 0x00, 0xA0, 0x30, 0x0B, 0x00, 0x02, 0xD8, 0xC2, 0xA4, 0x16, 0xD8, 0x5B, 0x82, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x04, 0x01, 0x23, 0x32, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0xBF, 0xD4, 0x7A, 0x02, 0x00, 0x80 },
		{ 0x85, 0x88, 0x00, 0xA0, 0x30, 0x0B, 0x00, 0x02, 0xD8, 0xC2, 0xA4, 0x16, 0xD8, 0x5B, 0x82, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x04, 0x02, 0x23, 0x3C, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0xBF, 0x54, 0x66, 0x02, 0x00, 0x80 },
	},
	// freq = 25 MHz
	{
		{ 0x85, 0x88, 0x00, 0x30, 0x00, 0x0C, 0x00, 0x06, 0xD8, 0xC2, 0xA4, 0x06, 0xD8, 0x5B, 0xB2, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x0C, 0x03, 0xA3, 0x37, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0x28, 0x56, 0x6E, 0x02, 0x00, 0x80 },
		{ 0x85, 0x88, 0x00, 0x30, 0x00, 0x0C, 0x00, 0x06, 0xD8, 0xC2, 0xA4, 0x16, 0xD8, 0x5B, 0xB2, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x0C, 0x04, 0xA3, 0x45, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0x28, 0x56, 0x58, 0x02, 0x00, 0x80 },
		{ 0x85, 0x88, 0x00, 0x30, 0x00, 0x0C, 0x00, 0x06, 0xD8, 0xC2, 0xA4, 0x16, 0xD8, 0x5B, 0xB2, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x08, 0x02, 0xE3, 0x29, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0x28, 0x56, 0x93, 0x02, 0x00, 0x80 },
	},
	// freq = 65 MHz
	{
		{ 0x85, 0x88, 0x40, 0xA0, 0x82, 0x0B, 0x00, 0x02, 0xD8, 0xC2, 0xA4, 0x06, 0xD8, 0x5B, 0x12, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x08, 0x03, 0x63, 0x48, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0xD4, 0xD4, 0x54, 0x02, 0x00, 0x80 },
		{ 0x85, 0x88, 0x40, 0xA0, 0x82, 0x0B, 0x00, 0x02, 0xD8, 0xC2, 0xA4, 0x16, 0xD8, 0x5B, 0x12, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x04, 0x01, 0x63, 0x2D, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0xD4, 0xD4, 0x87, 0x02, 0x00, 0x80 },
		{ 0x85, 0x88, 0x40, 0xA0, 0x82, 0x0B, 0x00, 0x02, 0xD8, 0xC2, 0xA4, 0x16, 0xD8, 0x5B, 0x12, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x04, 0x02, 0x63, 0x36, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0xD4, 0x54, 0x71, 0x02, 0x00, 0x80 },
	},
	// freq = 108 MHz
	{
		{ 0x85, 0x88, 0x00, 0xA0, 0x34, 0x07, 0x0D, 0x03, 0xD8, 0xC2, 0xA4, 0x06, 0xD8, 0x5B, 0xE2, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x14, 0x03, 0x23, 0x3C, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0x7F, 0x54, 0x66, 0x02, 0x00, 0x80 },
		{ 0x85, 0x88, 0x00, 0xA0, 0x34, 0x07, 0x0D, 0x03, 0xD8, 0xC2, 0xA4, 0x16, 0xD8, 0x5B, 0xE2, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x14, 0x04, 0x23, 0x4B, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0x7F, 0xD4, 0x51, 0x02, 0x00, 0x80 },
		{ 0x85, 0x88, 0x00, 0xA0, 0x34, 0x07, 0x0D, 0x03, 0xD8, 0xC2, 0xA4, 0x16, 0xD8, 0x5B, 0xE2, 0x84, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x10, 0x02, 0x23, 0x2D, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0x7F, 0x54, 0x88, 0x02, 0x00, 0x80 },
	},
	// freq = 162 MHz
	{
		{ 0x85, 0x88, 0x00, 0x10, 0x00, 0x06, 0x00, 0x00, 0xD8, 0xC2, 0xA4, 0x06, 0xD8, 0x5B, 0xF2, 0x83, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x10, 0x00, 0x23, 0x2D, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0x54, 0x54, 0x88, 0x02, 0x00, 0x80 },
		{ 0x85, 0x88, 0x00, 0x10, 0x00, 0x06, 0x00, 0x00, 0xD8, 0xC2, 0xA4, 0x16, 0xD8, 0x5B, 0xF2, 0x83, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x10, 0x01, 0x63, 0x38, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0x54, 0x14, 0x6D, 0x02, 0x00, 0x80 },
		{ 0x85, 0x88, 0x00, 0x10, 0x00, 0x06, 0x00, 0x00, 0xD8, 0xC2, 0xA4, 0x16, 0xD8, 0x5B, 0xF2, 0x83, 0x03, 0x82, 0x55/*0x04*/, 0xE0, 0x00, 0x10, 0x02, 0xA3, 0x43, 0x38, 0x40, 0x88, 0x7C, 0x09, 0x13, 0x06, 0x00, 0x54, 0x54, 0xD4, 0x5A, 0x02, 0x00, 0x80 },
	}
};
#elif defined(CONFIG_MACH_TCC9300) || defined(CONFIG_MACH_TCC9300CM) || defined(CONFIG_MACH_TCC9300ST)
static const unsigned char phy_config[][3][31] = {// TCC9300 HDMI PHY Setting
        //25.200
    {
        {0x05, 0x00, 0xD8, 0x10, 0x1C, 0x30, 0x40, 0x6B, 0x50, 0x11, 0x54, 0x58, 0x20, 0x5B, 0xA8, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0x11, 0x27, 0x00, 0x00, 0x00, 0x80},
        {0x05, 0x00, 0xD8, 0x10, 0x1C, 0x30, 0x40, 0x6B, 0x50, 0x11, 0x54, 0x58, 0x20, 0x5B, 0xA8, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0x11, 0x27, 0x00, 0x00, 0x00, 0x80},
        {0x05, 0x00, 0xD8, 0x10, 0x1C, 0x30, 0x40, 0x6B, 0x50, 0x11, 0x54, 0x58, 0x20, 0x5B, 0xA8, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0x11, 0x27, 0x00, 0x00, 0x00, 0x80},
    },
        //25.175
    {
        {0x05, 0x00, 0xD8, 0x10, 0x9C, 0xFC, 0x48, 0x6B, 0x50, 0x11, 0x54, 0x58, 0x20, 0x5B, 0xA8, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0x12, 0x27, 0x00, 0x00, 0x00, 0x80},
        {0x05, 0x00, 0xD8, 0x10, 0x9C, 0xFC, 0x48, 0x6B, 0x50, 0x11, 0x54, 0x58, 0x20, 0x5B, 0xA8, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0x12, 0x27, 0x00, 0x00, 0x00, 0x80},
        {0x05, 0x00, 0xD8, 0x10, 0x9C, 0xFC, 0x48, 0x6B, 0x50, 0x11, 0x54, 0x58, 0x20, 0x5B, 0xA8, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0x12, 0x27, 0x00, 0x00, 0x00, 0x80},
    },
        //27.000
    {
        {0x05, 0x00, 0xD8, 0x10, 0x1C, 0x30, 0x40, 0x6B, 0x10, 0x02, 0x54, 0x38, 0x20, 0x5B, 0x78, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xFF, 0x26, 0x00, 0x00, 0x00, 0x80},
        {0x05, 0x00, 0xD8, 0x10, 0x1C, 0x30, 0x40, 0x6B, 0x10, 0x02, 0x54, 0x38, 0x20, 0x5B, 0x78, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xFF, 0x26, 0x00, 0x00, 0x00, 0x80},
        {0x05, 0x00, 0xD8, 0x10, 0x1C, 0x30, 0x40, 0x6B, 0x10, 0x02, 0x54, 0x38, 0x20, 0x5B, 0x78, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xFF, 0x26, 0x00, 0x00, 0x00, 0x80},
    },
        //27.027
    {
        {0x05, 0x00, 0xD8, 0x10, 0x9C, 0x02, 0x32, 0x6B, 0x10, 0x02, 0x54, 0x38, 0x20, 0x5B, 0x78, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xFF, 0x26, 0x00, 0x00, 0x00, 0x80},
        {0x05, 0x00, 0xD8, 0x10, 0x9C, 0x02, 0x32, 0x6B, 0x10, 0x02, 0x54, 0x38, 0x20, 0x5B, 0x78, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xFF, 0x26, 0x00, 0x00, 0x00, 0x80},
        {0x05, 0x00, 0xD8, 0x10, 0x9C, 0x02, 0x32, 0x6B, 0x10, 0x02, 0x54, 0x38, 0x20, 0x5B, 0x78, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xFF, 0x26, 0x00, 0x00, 0x00, 0x80},
    },
        //54.000
    {
        {0x05, 0x00, 0xD8, 0x10, 0x1C, 0x30, 0x40, 0x6B, 0x10, 0x01, 0x54, 0x38, 0x20, 0x5B, 0x78, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xFF, 0x26, 0x01, 0x00, 0x00, 0x80},
        {0x05, 0x00, 0xD8, 0x10, 0x1C, 0x30, 0x40, 0x6B, 0x10, 0x01, 0x54, 0x38, 0x20, 0x5B, 0x78, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xFF, 0x26, 0x01, 0x00, 0x00, 0x80},
        {0x05, 0x00, 0xD8, 0x10, 0x1C, 0x30, 0x40, 0x6B, 0x10, 0x01, 0x54, 0x38, 0x20, 0x5B, 0x78, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xFF, 0x26, 0x01, 0x00, 0x00, 0x80},
    },
        //54.054
    {
        {0x05, 0x00, 0xD8, 0x10, 0x1C, 0x30, 0x40, 0x6B, 0x10, 0x01, 0x54, 0x38, 0x20, 0x5B, 0x78, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xFF, 0x26, 0x01, 0x00, 0x00, 0x80},
        {0x05, 0x00, 0xD8, 0x10, 0x1C, 0x30, 0x40, 0x6B, 0x10, 0x01, 0x54, 0x38, 0x20, 0x5B, 0x78, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xFF, 0x26, 0x01, 0x00, 0x00, 0x80},
        {0x05, 0x00, 0xD8, 0x10, 0x1C, 0x30, 0x40, 0x6B, 0x10, 0x01, 0x54, 0x38, 0x20, 0x5B, 0x78, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xFF, 0x26, 0x01, 0x00, 0x00, 0x80},
    },
        //74.250
    {
        {0x05, 0x00, 0xD8, 0x10, 0x1C, 0x30, 0x40, 0x6B, 0x10, 0x01, 0x54, 0x98, 0x21, 0x5B, 0xA5, 0x84, 0x00, 0x20, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xB9, 0x26, 0x01, 0x00, 0x00, 0x80},
        {0x05, 0x00, 0xD8, 0x10, 0x1C, 0x30, 0x40, 0x6B, 0x10, 0x01, 0x54, 0x98, 0x21, 0x5B, 0xA5, 0x84, 0x00, 0x20, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xB9, 0x26, 0x01, 0x00, 0x00, 0x80},
        {0x05, 0x00, 0xD8, 0x10, 0x1C, 0x30, 0x40, 0x6B, 0x10, 0x01, 0x54, 0x98, 0x21, 0x5B, 0xA5, 0x84, 0x00, 0x20, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xB9, 0x26, 0x01, 0x00, 0x00, 0x80},
    },
        //74.1
    {
        {0x05, 0x00, 0xD8, 0x10, 0x9C, 0x56, 0x5B, 0x6B, 0x10, 0x01, 0x54, 0x18, 0x25, 0x5B, 0xA5, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xB9, 0x26, 0x01, 0x00, 0x00, 0x80},
        {0x05, 0x00, 0xD8, 0x10, 0x9C, 0x56, 0x5B, 0x6B, 0x10, 0x01, 0x54, 0x18, 0x25, 0x5B, 0xA5, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xB9, 0x26, 0x01, 0x00, 0x00, 0x80},
        {0x05, 0x00, 0xD8, 0x10, 0x9C, 0x56, 0x5B, 0x6B, 0x10, 0x01, 0x54, 0x18, 0x25, 0x5B, 0xA5, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xB9, 0x26, 0x01, 0x00, 0x00, 0x80},
    },
        //148.500
    {
        {0x05, 0x00, 0xD8, 0x10, 0x1C, 0x30, 0x40, 0x6B, 0x18, 0x01, 0x54, 0x98, 0x21, 0x5B, 0xA5, 0x84, 0x00, 0x20, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xB9, 0x26, 0x02, 0x00, 0x00, 0x80},
        {0x05, 0x00, 0xD8, 0x10, 0x1C, 0x30, 0x40, 0x6B, 0x18, 0x01, 0x54, 0x98, 0x21, 0x5B, 0xA5, 0x84, 0x00, 0x20, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xB9, 0x26, 0x02, 0x00, 0x00, 0x80},
        {0x05, 0x00, 0xD8, 0x10, 0x1C, 0x30, 0x40, 0x6B, 0x18, 0x01, 0x54, 0x98, 0x21, 0x5B, 0xA5, 0x84, 0x00, 0x20, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xB9, 0x26, 0x02, 0x00, 0x00, 0x80},
    },
        //148.352
    {
        {0x05, 0x00, 0xD8, 0x10, 0x9C, 0x56, 0x5B, 0x6B, 0x18, 0x01, 0x54, 0x18, 0x25, 0x5B, 0xA5, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xB9, 0x26, 0x02, 0x00, 0x00, 0x80},
        {0x05, 0x00, 0xD8, 0x10, 0x9C, 0x56, 0x5B, 0x6B, 0x18, 0x01, 0x54, 0x18, 0x25, 0x5B, 0xA5, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xB9, 0x26, 0x02, 0x00, 0x00, 0x80},
        {0x05, 0x00, 0xD8, 0x10, 0x9C, 0x56, 0x5B, 0x6B, 0x18, 0x01, 0x54, 0x18, 0x25, 0x5B, 0xA5, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xB9, 0x26, 0x02, 0x00, 0x00, 0x80},
    },
        //108.108
    {
        {0x05, 0x00, 0xD8, 0x10, 0x9C, 0x02, 0x32, 0x6B, 0x18, 0x01, 0x54, 0x38, 0x20, 0x5B, 0x78, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xFF, 0x26, 0x02, 0x00, 0x00, 0x80},
        {0x05, 0x00, 0xD8, 0x10, 0x9C, 0x02, 0x32, 0x6B, 0x18, 0x01, 0x54, 0x38, 0x20, 0x5B, 0x78, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xFF, 0x26, 0x02, 0x00, 0x00, 0x80},
        {0x05, 0x00, 0xD8, 0x10, 0x9C, 0x02, 0x32, 0x6B, 0x18, 0x01, 0x54, 0x38, 0x20, 0x5B, 0x78, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xFF, 0x26, 0x02, 0x00, 0x00, 0x80},
    },
        //72.000
    {
        {0x05, 0x00, 0xD8, 0x10, 0x1C, 0x30, 0x40, 0x6B, 0x18, 0x01, 0x54, 0x68, 0x21, 0x5B, 0x50, 0x84, 0x00, 0x21, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xBF, 0x26, 0x01, 0x00, 0x00, 0x80},
        {0x05, 0x00, 0xD8, 0x10, 0x1C, 0x30, 0x40, 0x6B, 0x18, 0x01, 0x54, 0x68, 0x21, 0x5B, 0x50, 0x84, 0x00, 0x21, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xBF, 0x26, 0x01, 0x00, 0x00, 0x80},
        {0x05, 0x00, 0xD8, 0x10, 0x1C, 0x30, 0x40, 0x6B, 0x18, 0x01, 0x54, 0x68, 0x21, 0x5B, 0x50, 0x84, 0x00, 0x21, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xBF, 0x26, 0x01, 0x00, 0x00, 0x80},
    },
        //25.000
    {
        {0x05, 0x00, 0xD8, 0x10, 0x9C, 0xE8, 0x36, 0x6B, 0x50, 0x11, 0x54, 0x58, 0x20, 0x5B, 0xA7, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0x13, 0x27, 0x00, 0x00, 0x00, 0x80},
        {0x05, 0x00, 0xD8, 0x10, 0x9C, 0xE8, 0x36, 0x6B, 0x50, 0x11, 0x54, 0x58, 0x20, 0x5B, 0xA7, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0x13, 0x27, 0x00, 0x00, 0x00, 0x80},
        {0x05, 0x00, 0xD8, 0x10, 0x9C, 0xE8, 0x36, 0x6B, 0x50, 0x11, 0x54, 0x58, 0x20, 0x5B, 0xA7, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0x13, 0x27, 0x00, 0x00, 0x00, 0x80},
    },
        //65.000
    {
        {0x05, 0x00, 0xD8, 0x10, 0x9C, 0x08, 0x36, 0x6B, 0x10, 0x01, 0x54, 0x48, 0x20, 0x5B, 0x90, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xD4, 0x26, 0x01, 0x00, 0x00, 0x80},
        {0x05, 0x00, 0xD8, 0x10, 0x9C, 0x08, 0x36, 0x6B, 0x10, 0x01, 0x54, 0x48, 0x20, 0x5B, 0x90, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xD4, 0x26, 0x01, 0x00, 0x00, 0x80},
        {0x05, 0x00, 0xD8, 0x10, 0x9C, 0x08, 0x36, 0x6B, 0x10, 0x01, 0x54, 0x48, 0x20, 0x5B, 0x90, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xD4, 0x26, 0x01, 0x00, 0x00, 0x80},
    },
        //108.000
    {
        {0x05, 0x00, 0xD8, 0x10, 0x1C, 0x30, 0x40, 0x6B, 0x18, 0x01, 0x54, 0x38, 0x20, 0x5B, 0x78, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xFF, 0x26, 0x02, 0x00, 0x00, 0x80},
        {0x05, 0x00, 0xD8, 0x10, 0x1C, 0x30, 0x40, 0x6B, 0x18, 0x01, 0x54, 0x38, 0x20, 0x5B, 0x78, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xFF, 0x26, 0x02, 0x00, 0x00, 0x80},
        {0x05, 0x00, 0xD8, 0x10, 0x1C, 0x30, 0x40, 0x6B, 0x18, 0x01, 0x54, 0x38, 0x20, 0x5B, 0x78, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xFF, 0x26, 0x02, 0x00, 0x00, 0x80},
    },
        //162.000
    {
        {0x05, 0x00, 0xD8, 0x10, 0x1C, 0x30, 0x40, 0x6B, 0x18, 0x01, 0x54, 0x48, 0x21, 0x5B, 0xB4, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xAA, 0x26, 0x02, 0x00, 0x00, 0x80},
        {0x05, 0x00, 0xD8, 0x10, 0x1C, 0x30, 0x40, 0x6B, 0x18, 0x01, 0x54, 0x48, 0x21, 0x5B, 0xB4, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xAA, 0x26, 0x02, 0x00, 0x00, 0x80},
        {0x05, 0x00, 0xD8, 0x10, 0x1C, 0x30, 0x40, 0x6B, 0x18, 0x01, 0x54, 0x48, 0x21, 0x5B, 0xB4, 0x84, 0x00, 0x00, 0x38, 0x00, 0x08, 0x10, 0xE0, 0x22, 0x40, 0xAA, 0x26, 0x02, 0x00, 0x00, 0x80},
    }
};
#else
#error not defined chip
#endif


#if (0)
#define DEFAULT_HDMI_LCDC_TIMING	{ 0,  1,  1,  0,  0,  1,  0,  1,  0,  61 ,    639 ,   59 ,     37 ,    0,  0,  5,  479 ,   29,  8,   5,   479 ,    29,  8, /*640x480p @ 60Hz       25.200MHz   */ }
#else
#define DEFAULT_HDMI_LCDC_TIMING	{ 0,  0,  0,  0,  0,  1,  0,  1,  0,  43 ,    1919,   147,     87 ,    0,  0,  4,  1079,   35,  3,   4,   1079,    35,  3, /*1920x1080p @ 60Hz     148.5MHz    */ }
#endif


static const struct lcdc_timimg_parms_t LCDCTimimgParams[] = 
{                                                  /*  ID  IV  IH  IP  DP  NI  TV TFT STN  LPW       LPC   LSWC     LEWC   VDB VDF FPW    FLC  FSWC FEWC FPW2   FLC2  FSWC2 FEWC2 */
  /* v640x480p_60Hz         PIXEL_FREQ_25_200,  */    { 0,  1,  1,  0,  0,  1,  0,  1,  0,  61 ,    639 ,   59 ,     37 ,    0,  0,  5,  479 ,   29,  8,   5,   479 ,    29,  8, /*640x480p @ 60Hz       25.200MHz   */ },
  /* v720x480p_60Hz         PIXEL_FREQ_27_027,  */    { 0,  1,  1,  0,  0,  1,  0,  1,  0,  61 ,    719 ,   59 ,     15 ,    0,  0,  5,  479 ,   29,  8,   5,   479 ,    29,  8, /*720x480p @ 60Hz       27.027MHz   */ },
  /* v1280x720p_60Hz        PIXEL_FREQ_74_250,  */    { 0,  0,  0,  0,  0,  1,  0,  1,  0,  39 ,    1279,   219,     109,    0,  0,  4,  719 ,   19,  4,   4,   719 ,    19,  4, /*1280x720p @ 60Hz      74.25MHz    */ },
  /* v1920x1080i_60Hz       PIXEL_FREQ_74_250,  */    { 0,  0,  0,  0,  0,  0,  1,  0,  0,  43 ,    1919,   147,     87 ,    0,  0,  9,  1079,   29,  3,   9,   1079,    31,  3, /*1920x1080i @ 60Hz     74.25MHz    */ },
  /* v720x480i_60Hz         PIXEL_FREQ_27_027,  */    { 0,  1,  1,  0,  1,  0,  1,  0,  0,  123,    1439,   113,     37 ,    0,  0,  5,  479 ,   29,  7,   5,   479 ,    31,  7, /*720x480i @ 60Hz       27.027MHz   */ },
  /* v720x240p_60Hz         PIXEL_FREQ_27_027,  */    DEFAULT_HDMI_LCDC_TIMING,
  /* v2880x480i_60Hz        PIXEL_FREQ_54_054,  */    DEFAULT_HDMI_LCDC_TIMING,
  /* v2880x240p_60Hz        PIXEL_FREQ_54_054,  */    DEFAULT_HDMI_LCDC_TIMING,
  /* v1440x480p_60Hz        PIXEL_FREQ_54_054,  */    {0, 0, 0, 0, 1, 1, 0, 1, 0, 123, 1439, 120, 30, 0, 0, 5, 479, 32, 5, 5, 479, 32, 5, /*1440x480p @ 60Hz 54.054MHz*/ },
  /* v1920x1080p_60Hz       PIXEL_FREQ_148_500, */    { 0,  0,  0,  0,  0,  1,  0,  1,  0,  43 ,    1919,   147,     87 ,    0,  0,  4,  1079,   35,  3,   4,   1079,    35,  3, /*1920x1080p @ 60Hz     148.5MHz    */ },
  /* v720x576p_50Hz         PIXEL_FREQ_27,      */    { 0,  1,  1,  0,  0,  1,  0,  1,  0,  63 ,    719 ,   67 ,     11 ,    0,  0,  4,  575 ,   38,  4,   4,   575 ,    38,  4, /*720x576p @ 50Hz       27MHz       */ },
  /* v1280x720p_50Hz        PIXEL_FREQ_74_250,  */    { 0,  0,  0,  0,  0,  1,  0,  1,  0,  39 ,    1279,   219,     439,    0,  0,  4,  719 ,   19,  4,   4,   719 ,    19,  4, /*1280x720p @ 50Hz      74.25MHz    */ },
  /* v1920x1080i_50Hz       PIXEL_FREQ_74_250,  */    { 0,  0,  0,  0,  0,  0,  1,  0,  0,  43 ,    1919,   147,     527,    0,  0,  9,  1079,   29,  3,   9,   1079,    31,  3, /*1920x1080i @ 50Hz     74.25MHz    */ },
  /* v720x576i_50Hz         PIXEL_FREQ_27,      */    { 0,  1,  1,  0,  1,  0,  1,  0,  0,  125,    1439,   137,     23 ,    0,  0,  5,  575 ,   37,  3,   5,   575 ,    39,  3, /*720x576i @ 50Hz       27MHz       */ },
  /* v720x288p_50Hz         PIXEL_FREQ_27,      */    DEFAULT_HDMI_LCDC_TIMING,
  /* v2880x576i_50Hz        PIXEL_FREQ_54,      */    DEFAULT_HDMI_LCDC_TIMING,
  /* v2880x288p_50Hz        PIXEL_FREQ_54,      */    DEFAULT_HDMI_LCDC_TIMING,
  /* v1440x576p_50Hz        PIXEL_FREQ_54,      */    {0, 0, 1, 0, 1, 1, 0, 1, 0, 125, 1439, 137, 23, 0, 0, 4, 575, 38, 4, 4, 575, 38, 4, /*1440x576p @ 50Hz 54MHz    */ },
  /* v1920x1080p_50Hz       PIXEL_FREQ_148_500, */    { 0,  0,  0,  0,  0,  1,  0,  1,  0,  43 ,    1919,   147,     87 ,    0,  0,  4,  1079,   35,  3,   4,   1079,    35,  3, /*1920x1080p @ 50Hz     148.5MHz    */ },
  /* v1920x1080p_24Hz       PIXEL_FREQ_74_250,  */    DEFAULT_HDMI_LCDC_TIMING,
  /* v1920x1080p_25Hz       PIXEL_FREQ_74_250,  */    DEFAULT_HDMI_LCDC_TIMING,
  /* v1920x1080p_30Hz       PIXEL_FREQ_74_250,  */    DEFAULT_HDMI_LCDC_TIMING,
  /* v2880x480p_60Hz        PIXEL_FREQ_108_108, */    DEFAULT_HDMI_LCDC_TIMING,
  /* v2880x576p_50Hz        PIXEL_FREQ_108,     */    DEFAULT_HDMI_LCDC_TIMING,
  /* v1920x1080i_50Hz(1250) PIXEL_FREQ_72,      */    DEFAULT_HDMI_LCDC_TIMING,
  /* v1920x1080i_100Hz      PIXEL_FREQ_148_500, */    DEFAULT_HDMI_LCDC_TIMING,
  /* v1280x720p_100Hz       PIXEL_FREQ_148_500, */    DEFAULT_HDMI_LCDC_TIMING,
  /* v720x576p_100Hz        PIXEL_FREQ_54,      */    DEFAULT_HDMI_LCDC_TIMING,
  /* v720x576i_100Hz        PIXEL_FREQ_54,      */    DEFAULT_HDMI_LCDC_TIMING,
  /* v1920x1080i_120Hz      PIXEL_FREQ_148_500, */    DEFAULT_HDMI_LCDC_TIMING,
  /* v1280x720p_120Hz       PIXEL_FREQ_148_500, */    DEFAULT_HDMI_LCDC_TIMING,
  /* v720x480p_120Hz        PIXEL_FREQ_54_054,  */    DEFAULT_HDMI_LCDC_TIMING,
  /* v720x480i_120Hz        PIXEL_FREQ_54_054,  */    DEFAULT_HDMI_LCDC_TIMING,
  /* v720x576p_200Hz        PIXEL_FREQ_108,     */    DEFAULT_HDMI_LCDC_TIMING,
  /* v720x576i_200Hz        PIXEL_FREQ_108,     */    DEFAULT_HDMI_LCDC_TIMING,
  /* v720x480p_240Hz        PIXEL_FREQ_108_108, */    DEFAULT_HDMI_LCDC_TIMING,
  /* v720x480i_240Hz        PIXEL_FREQ_108_108, */    DEFAULT_HDMI_LCDC_TIMING,
};



#define PHY_I2C_ADDRESS             0x70
#define PHY_REG_MODE_SET_DONE       0x28

#define PHY_CONFIG_START_OFFSET     0x01

static unsigned short probe_i2c[] = {3, PHY_I2C_ADDRESS, I2C_CLIENT_END};	/* { i2c_num, i2c_addr } */
static unsigned short dummy[] = {I2C_CLIENT_END};

static struct i2c_client_address_data addr_data = {
	.normal_i2c = dummy,
	.probe = probe_i2c,
	.ignore = dummy,
};

static struct i2c_driver HDMIPHY_i2c_driver;
struct i2c_client *HDMIPHY_i2c_client = NULL;
//EXPORT_SYMBOL(pca953x_i2c_client);

static int HDMIPHY_i2c_probe(struct i2c_adapter *adap, int addr, int kind)
{
	struct i2c_client *i2c;

	i2c = kzalloc(sizeof(*i2c), GFP_KERNEL);
	if (!i2c)
		return -ENOMEM;

	strcpy(i2c->name, "HDMIPHY");
	i2c->flags = 0;
	i2c->addr = addr;
	i2c->adapter = adap;
	i2c->driver = &HDMIPHY_i2c_driver;

	HDMIPHY_i2c_client = i2c;

	return i2c_attach_client(i2c);
}

static int HDMIPHY_i2c_detach(struct i2c_client *client)
{
	i2c_detach_client(client);
	kfree(HDMIPHY_i2c_client);
	return 0;
}

static int HDMIPHY_i2c_attach(struct i2c_adapter *adap)
{
	return i2c_probe(adap, &addr_data, HDMIPHY_i2c_probe);
}

static struct i2c_driver HDMIPHY_i2c_driver = {
	.driver = {
		.name = "HDMIPHY",
		.owner = THIS_MODULE,
	},
	.id = 1,
	.attach_adapter = HDMIPHY_i2c_attach,
	.detach_client = HDMIPHY_i2c_detach,
};

static int __init HDMIPHY_i2c_init(void)
{
	return i2c_add_driver(&HDMIPHY_i2c_driver);
}

static void __exit HDMIPHY_i2c_exit(void)
{
	i2c_del_driver(&HDMIPHY_i2c_driver);
}


int HDMIPHY_extWrite(unsigned short reg, unsigned char *val, unsigned short len)
{

	unsigned char data[132];
	unsigned int cnt;
	unsigned char bytes;


	data[0]= (char)reg&0xff;
	bytes = 1;
	
	for( cnt = 0; cnt<len ; cnt++)
	{
		data[bytes + cnt] = val[cnt];
	}

	bytes += len;

	if (i2c_master_send(HDMIPHY_i2c_client, data, bytes) != bytes)
	{
		printk("write error!!!! \n");
		return -EIO; 
	}
	return 0;
}




int PHYConfig(const enum PixelFreq clk, const enum ColorDepth depth)
{
    int index, freq;
    int size;
    unsigned char *buffer;
    unsigned char reg;

    // get depth index
    switch (depth)
    {
        case HDMI_CD_24:
            index = 0;
            break;
        case HDMI_CD_30:
            index = 1;
            break;
        case HDMI_CD_36:
            index = 2;
            break;
        default:
            printk("not available depth arg = %d\n", (int)depth);
            return 0;
    }

    // get clk freq index
    switch (clk)
    {
        case PIXEL_FREQ_25_200 :
            freq = 0;
            break;
        case PIXEL_FREQ_25_175 :
            freq = 1;
            break;
        case PIXEL_FREQ_27 :
            freq = 2;
            break;
        case PIXEL_FREQ_27_027 :
            freq = 3;
            break;
        case PIXEL_FREQ_54 :
            freq = 4;
            break;
        case PIXEL_FREQ_54_054 :
            freq = 5;
            break;
        case PIXEL_FREQ_74_250 :
            freq = 6;
            break;
        case PIXEL_FREQ_74_176 :
            freq = 7;
            break;
        case PIXEL_FREQ_148_500 :
            freq = 8;
            break;
        case PIXEL_FREQ_148_352 :
            freq = 9;
            break;
        case PIXEL_FREQ_108_108 :
            freq = 10;
            break;
        case PIXEL_FREQ_72 :
            freq = 11;
            break;
        case PIXEL_FREQ_25 :
            freq = 12;
            break;
        case PIXEL_FREQ_65 :
            freq = 13;
            break;
        case PIXEL_FREQ_108 :
            freq = 14;
            break;
        case PIXEL_FREQ_162 :
            freq = 15;
            break;
        default:
            printk("not availlable clk arg = %d\n",(int)clk);
            return 0;
    }

    // start to reconfig after that phy_ready goes down
    reg = 0x00;
#if 0	
    if (!PHYWrite(PHY_I2C_ADDRESS,PHY_REG_MODE_SET_DONE, 1, &reg))
    {
        DPRINTF("fail to write reconfig 0x02%x\n",PHY_REG_MODE_SET_DONE);
        return 0;
    }
#else
	HDMIPHY_extWrite(PHY_REG_MODE_SET_DONE, &reg, 1);
#endif//

	{volatile int ttt;for(ttt=0;ttt<500;ttt++);}

    size = sizeof(phy_config[freq][index]) / sizeof(phy_config[freq][index][0]);
    buffer = (unsigned char *) phy_config[freq][index];

//    if (!PHYWrite(PHY_I2C_ADDRESS, PHY_CONFIG_START_OFFSET, size, buffer))
//    {
//        return 0;
//    }

	HDMIPHY_extWrite(PHY_CONFIG_START_OFFSET, buffer, size);

    //for ( idx = 0 ; idx < 1000 ; idx ++ );            // delay

    return 1;
}


int hdmi_set_phy_config(const struct HDMIVideoParameter * const hdmi_video_mode)
{
    unsigned char phy_ready = 0;
    int i = 0;


    if (!PHYConfig(HDMIVideoParams[hdmi_video_mode->resolution].PixelClock,
          hdmi_video_mode->colorDepth))
    {
        DPRINTF("phy config failed!\n");
        return 0;
    }

#if 0
    do {
		phy_ready = readb(HDMI_PHY_STATUS);
        if (i++ == 1000) break;
    } while (!phy_ready);
#endif//
    if (!phy_ready)
    {
        DPRINTF("phy is not ready!!!\n");
    }
    else
    {
        DPRINTF("phy configured\n");
    }

    return 1;
}


// [2] A
int hdmi_set_lcdc_config(enum VideoFormat hdmi_video_format)
{
    struct lcdc_timimg_parms_t device;

    // set lcdc
    memcpy((void*)&device,(const void*)&(LCDCTimimgParams[hdmi_video_format]),sizeof(device));

	tcc92xxfb_hdmi_starter(1, &device);

    return 1;
}



int hdmi_starter_set_video_mode(const struct HDMIVideoParameter * const hdmi_video_mode)
{
    struct device_video_params device;
	enum PixelLimit pxl_lmt = HDMI_FULL_RANGE;

	DPRINTF(" %s \n", __func__);

    if (!hdmi_video_mode)
    {
        DPRINTF("bad args: hdmi_video_mode\n");
        return 0;
    }

    // parsing video parameters
    memcpy((void*)&device,(const void*)&(HDMIVideoParams[hdmi_video_mode->resolution]),sizeof(device));

	// set lcdc
	hdmi_set_lcdc_config(hdmi_video_mode->resolution);
	

    // set pixel aspect ratio
    // !! must be setting before 'HDMI_IOC_SET_VIDEOMODE'
    hdmi_set_pixel_aspect_ratio(hdmi_video_mode->pixelAspectRatio);

    // set video parameters
//    hdmi_set_video_timing(device);
	hdmi_set_video_mode(device);

    // set video format information
	//gHdmiVideoParms.resolution = hdmi_video_mode->resolution;

    // set color space
    if ( !hdmi_set_color_space(hdmi_video_mode->colorSpace) )
    {
        DPRINTF("bad args: hdmi_video_mode->colorSpace : Not Correct Arg = %d\n", hdmi_video_mode->colorSpace);
        //return -EFAULT;
    }
	//gHdmiVideoParms.colorSpace = hdmi_video_mode->colorSpace;

    // set color depth
    if ( !hdmi_set_color_depth(hdmi_video_mode->colorDepth) )
    {
        DPRINTF("bad args: hdmi_video_mode->colorDepth : Not Correct Arg = %d\n", hdmi_video_mode->colorDepth);
        //return -EFAULT;
    }
	//gHdmiVideoParms.colorDepth = hdmi_video_mode->colorDepth;

	// set pixel limitation.
	switch(hdmi_video_mode->colorSpace) 
	{
		case HDMI_CS_RGB:		/** RGB color space */
			if (hdmi_video_mode->resolution == v640x480p_60Hz)
				pxl_lmt = HDMI_FULL_RANGE;
			else
				pxl_lmt = HDMI_RGB_LIMIT_RANGE;
			break;
		case HDMI_CS_YCBCR444:	/** YCbCr 4:4:4 color space */
		case HDMI_CS_YCBCR422:	/** YCbCr 4:2:2 color space */
			pxl_lmt = HDMI_YCBCR_LIMIT_RANGE;
			break;
	}

    if (!hdmi_set_pixel_limit(pxl_lmt))
    {
		DPRINTF("bad args: hdmi_video_mode->colorDepth : Not Correct Arg = %d\n", pxl_lmt);
        //return -EFAULT;
    }
	//gPixelLimit = pxl_lmt;

    // set phy
    if (!hdmi_set_phy_config(hdmi_video_mode))
    {
        DPRINTF("fail to config PHY!\n");
        //return 0;
    }

    return 1;
}



extern struct HDMIVideoParameter gHdmiVideoParms;

void hdmi_initialize_start(void)
{
	tcc_hdmi_power_on();

	gHdmiVideoParms.mode = HDMI;
	gHdmiVideoParms.resolution = v1280x720p_60Hz;
	gHdmiVideoParms.colorSpace = HDMI_CS_RGB;
	gHdmiVideoParms.colorDepth = HDMI_CD_24;
	gHdmiVideoParms.colorimetry = HDMI_COLORIMETRY_NO_DATA;
	gHdmiVideoParms.pixelAspectRatio = HDMI_PIXEL_RATIO_16_9;
	

	// video mode HDMI/DVI
	hdmi_set_hdmimode(gHdmiVideoParms.mode);

	hdmi_starter_set_video_mode(&gHdmiVideoParms);
#if  0
	if (gHdmiVideoParms.mode == HDMI)
		hdmi_set_audio_mode(&audio);
#endif//

	hdmi_start();



}



static int tcc_hdmi_starter_ioctl(struct inode *inode, struct file *file, unsigned int cmd, void *arg)
{
	switch(cmd)
	{

		default:
			DPRINTF(" Unsupported IOCTL!!!\n");      
			break;			
	}
	return 0;
}




static int tcc_hdmi_starter_release(struct inode *inode, struct file *file)
{
	return 0;
}

static int tcc_hdmi_starter_open(struct inode *inode, struct file *file)
{

	return 0;
}

static struct file_operations tcc_hdmi_starter_fops = 
{
	.owner			= THIS_MODULE,
	.write			= NULL,
	.read			= NULL,
	.poll 			= NULL,
	.ioctl			= tcc_hdmi_starter_ioctl,
	.mmap			= NULL,
	.open			= tcc_hdmi_starter_open,
	.release		= tcc_hdmi_starter_release,
};


#define DEVICE_NAME			"hdmi starter"
#define MAJOR_ID			207
#define MINOR_ID			1

static struct class *hdmi_starter_class;


void __exit
tcc_hdmi_starter_cleanup(void)
{
	unregister_chrdev(MAJOR_ID, DEVICE_NAME);

	i2c_del_driver(&HDMIPHY_i2c_driver);
	
	if(HDMIPHY_i2c_client)
	{
		kfree(HDMIPHY_i2c_client);
		HDMIPHY_i2c_client = NULL;
	}

	return;
}

static char banner[] __initdata = KERN_INFO "TCC HDMI driver starter \n";
static struct class *tcc_hdmi_starter_class;


// external display device use lcdc  0, 1
#define EXTERNAL_DD_LCDC 	1
void ExternalDisplay_update(TCC_OUTPUT_TYPE DDtype)
{
	unsigned int regl, fb_address, fb_format;
	PLCDC pLCDC0, pLCDC1;
	unsigned int img_width = 0, img_height = 0, img_offset = 0,  lcd_width = 0, lcd_height = 0, lcd_w_pos = 0, lcd_h_pos = 0;
	PLCDC_CHANNEL pLCDC_OUTPUT_FB_CH;


	pLCDC0 = (volatile PLCDC)tcc_p2v(HwLCDC0_BASE);
	pLCDC1 = (volatile PLCDC)tcc_p2v(HwLCDC1_BASE);


	if(EXTERNAL_DD_LCDC)
	{
		pLCDC_OUTPUT_FB_CH = (volatile PLCDC_CHANNEL)tcc_p2v(pLCDC1->LI2C);

		pLCDC1->LCTRL &= (~HwLCTRL_LEN );

		// current framebuffer size
		regl = pLCDC0->LI0S; // get image size 
		img_width = (regl & 0xFFFF);
		img_height = ((regl>>16) & 0xFFFF);
		fb_format = pLCDC0->LI0C & HwLIC_FMT;
		fb_address = pLCDC0->LI0BA0;
		img_offset = pLCDC0->LI0O;

		
		regl = pLCDC1->LDS; // get LCD size 
	}
	else
	{
		pLCDC_OUTPUT_FB_CH = (volatile PLCDC_CHANNEL)tcc_p2v(pLCDC1->LI0C);
		pLCDC0->LCTRL &= (~HwLCTRL_LEN );

		// current framebuffer size
		regl = pLCDC1->LI2S; // get image size 
		img_width = (regl & 0xFFFF);
		img_height = ((regl>>16) & 0xFFFF);		
		fb_format = pLCDC1->LI0C & HwLIC_FMT;
		fb_address = pLCDC1->LI2BA0;
		img_offset = pLCDC1->LI2O;

		regl = pLCDC0->LDS; // get LCD size 
	}

	lcd_width = (regl & 0xFFFF);
	lcd_height = ((regl>>16) & 0xFFFF);


	DPRINTF("Image Info: lcd[%d ,%d] imgage[%d ,%d] fmt:%d offset:0x%x \n", lcd_width,lcd_height, img_width, img_height, fb_format, img_offset); 	 

	if((!lcd_width) || (!lcd_height))
		return;


// lcd image offset
	pLCDC_OUTPUT_FB_CH->LIO = img_offset;

	if(lcd_width > img_width)
		lcd_w_pos = (lcd_width - img_width)/2;

	if(lcd_height > img_height)
		lcd_h_pos = (lcd_height - img_height)/2;
// position
	pLCDC_OUTPUT_FB_CH->LIP = ((lcd_h_pos << 16) | (lcd_w_pos));

// scale option
	pLCDC_OUTPUT_FB_CH->LISR =((0<<4) | 0);

// size
	if(lcd_width < img_width)
		pLCDC_OUTPUT_FB_CH->LIS =((img_height << 16) | (lcd_width));
	else
		pLCDC_OUTPUT_FB_CH->LIS =((img_height << 16) | (img_width));
		
// image address
	pLCDC_OUTPUT_FB_CH->LIBA0 = fb_address;


	if(DDtype == TCC_OUTPUT_COMPOSITE)
	{
		pLCDC_OUTPUT_FB_CH->LIC = HwLIC_INTL | HwLIC_IEN | fb_format;
	}

	if(EXTERNAL_DD_LCDC)
	{
		pLCDC1->LCTRL |= (HwLCTRL_LEN);
	}
	else
	{
		pLCDC0->LCTRL |= (HwLCTRL_LEN );

	}

}



int __init 
tcc_hdmi_starter_init(void)
{
	int ret;

	printk(banner);

	register_chrdev(MAJOR_ID, DEVICE_NAME, &tcc_hdmi_starter_fops);

	hdmi_starter_class = class_create(THIS_MODULE, DEVICE_NAME);
	device_create(hdmi_starter_class,NULL,MKDEV(MAJOR_ID, MINOR_ID), NULL, DEVICE_NAME);

	printk("HDMI_STARTER_I2C_Initialize In \n");

	if(tcc_display_data.output == 0 || tcc_display_data.output == 1)
	{
		ret = i2c_add_driver(&HDMIPHY_i2c_driver);

		hdmi_initialize_start();
	}
	else if(tcc_display_data.output == 2)
	{
		tcc_composite_clock_onoff(1);

		TCC_FB_LCDC_NumSet(0);
		TCC_OUTPUT_LCDC_OnOff(TCC_OUTPUT_COMPOSITE, 1, 1);
		tcc_composite_start(COMPOSITE_NTST_M);
		ExternalDisplay_update(TCC_OUTPUT_COMPOSITE);
	}
	else if(tcc_display_data.output == 3)
	{
		tcc_component_clock_onoff(1);

		TCC_FB_LCDC_NumSet(0);
		TCC_OUTPUT_LCDC_OnOff(TCC_OUTPUT_COMPONENT, 1, 1);
		tcc_component_start(COMPONENT_NTST_M);
		ExternalDisplay_update(TCC_OUTPUT_COMPONENT);
	}

	return 0;
}





MODULE_AUTHOR("TELECHIPS");
MODULE_LICENSE("GPL");

module_init(tcc_hdmi_starter_init);
module_exit(tcc_hdmi_starter_cleanup);
