
/****************************************************************************
 *   FileName    : tca_backlight.c
 *   Description : 
 ****************************************************************************
 *
 *   TCC Version 1.0
 *   Copyright (c) Telechips, Inc.
 *   ALL RIGHTS RESERVED
 *
 ****************************************************************************/
/*****************************************************************************
*
* Header Files Include
*
******************************************************************************/
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/delay.h>
#include <linux/fb.h>
#include <linux/init.h>
#include <linux/dma-mapping.h>
#include <linux/interrupt.h>
#include <linux/workqueue.h>
#include <linux/wait.h>
#include <linux/platform_device.h>
#include <linux/clk.h>

#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/div64.h>
#include <asm/mach/map.h>
#ifdef CONFIG_PM
#include <linux/pm.h>
#endif

#include <mach/bsp.h>
#include <mach/tca_ckc.h>


#ifdef __KERNEL__
#	if defined(_TCC79x_)
#		include <mach/tcc79x.h>
#	elif defined(_TCC83x_)
#		include <mach/tcc83x_virt.h>
#	endif
#else
#	if defined(_TCC79x_)
#		include "TCC79x_Virtual.h"
#	elif defined(_TCC83x_)
#		include "TCC83x_Virtual.h"
#	endif
#endif


#include <mach/dev_gpsb.h>
#include <mach/tca_backlight.h>
#include <mach/TCC_LCD_Interface.h>
#include <mach/TCC_LCD_DriverCtrl.h>
#include "../input/touchscreen/tca_tsc2003tchctl.h"
#include "../input/touchscreen/tca_tchhwctl.h"

#if 0
#define dprintk(msg...)	 { printk( "Tcc92_bl: " msg); }
#else
#define dprintk(msg...)	 
#endif

unsigned int tcc_lcd_fb_on = 1;
static unsigned int bl_value;

/*****************************************************************************
*
* Defines
*
******************************************************************************/

/*****************************************************************************
*
* structures
*
******************************************************************************/

/*****************************************************************************
*
* Variables
*
******************************************************************************/

/*****************************************************************************
*
* Functions
*
******************************************************************************/
#define NOT_USE_DIMMING
static volatile PTIMER pTIMER;

static PLCDC pLCDC;
static char current_lcdc;
static char current_lcdc_FbCh;


static LCDC_CHANNEL LCDC_CH0;
static LCDC_CHANNEL LCDC_CH1;
static LCDC_CHANNEL LCDC_CH2;

void lcdc_channel_restore(int ch_num, int backup)
{
	switch(ch_num)
	{
		case 0:
			if(backup)
				memcpy((char *)&LCDC_CH0, (char *)&pLCDC->LI0C, sizeof(LCDC_CHANNEL)); 
			else
				memcpy((char *)&pLCDC->LI0C, (char *)&LCDC_CH0, sizeof(LCDC_CHANNEL)); 				

			break;

		case 1:
			if(backup)
				memcpy((char *)&LCDC_CH1, (char *)&pLCDC->LI1C, sizeof(LCDC_CHANNEL)); 
			else
				memcpy((char *)&pLCDC->LI1C, (char *)&LCDC_CH1, sizeof(LCDC_CHANNEL)); 				

			break;

		case 2:
			if(backup)
				memcpy((char *)&LCDC_CH2, (char *)&pLCDC->LI2C, sizeof(LCDC_CHANNEL)); 
			else
				memcpy((char *)&pLCDC->LI2C, (char *)&LCDC_CH2, sizeof(LCDC_CHANNEL)); 				

			break;

	}

}

void lcdc_initialize(LCD_MODULE_SPEC_TYPE *lcd_spec)
{
	unsigned int lcd_reg;

	lcd_reg = 0;

	if(lcd_spec->lcd_bus_width == 24)
		lcd_reg |= (0xC<<16);
	else if(lcd_spec->lcd_bus_width == 18)
		lcd_reg |= (0x5<<16);
	else
		lcd_reg |= (0x3<<16);

	if(lcd_spec->SyncSignal_Invert & ID_INVERT_EN)
		lcd_reg |= Hw15;

	if(lcd_spec->SyncSignal_Invert & IV_INVERT_EN)
		lcd_reg |= Hw14;

	if(lcd_spec->SyncSignal_Invert & IH_INVERT_EN)
		lcd_reg |= Hw13;

	if(lcd_spec->SyncSignal_Invert & IP_INVERT_EN)
		lcd_reg |= Hw12;

#ifdef CONFIG_LCD_CPU_INTERFACE
	lcd_reg |= Hw23; // clock gating enable 
#endif//

	lcd_reg |=	Hw8 |			// Non-interlace
				(0x2<<5)|		// TFT-LCD mode: STN(Hw5), TFT(Hw6), TV(Hw7)
				//Hw4 | 		// Master Select for IMG0
				(0x5<<1)|		// OVP[3:1] - 5 : Image2 > Image1 > Image0
				//Hw0 | 		// LCD Controller Enable
				0;			// End Of Value

	pLCDC->LCTRL = lcd_reg;

// Set LCD clock
	pLCDC->LCLKDIV = ( (1 << 16) | lcd_spec->lcdc_clk_div/2); //((PCK_LCD+1)/2)/((LCD_DEVIDE)*2)

// Background color
	pLCDC->LBC = 0x80000000;

//	Horizontal timing
	pLCDC->LHTIME1 = ((lcd_spec->lcd_LPW) << 16) | (lcd_spec->lcd_width - 1);
	pLCDC->LHTIME2 = ((lcd_spec->lcd_LSWC) << 16) | (lcd_spec->lcd_LEWC);

//	Vertical timing
	pLCDC->LVTIME1 = ((lcd_spec->lcd_FPW1) << 16) | (lcd_spec->lcd_height- 1);
	pLCDC->LVTIME2 = ((lcd_spec->lcd_FSWC1) << 16) | (lcd_spec->lcd_FEWC2);
	pLCDC->LVTIME3 = ((lcd_spec->lcd_FPW1) << 16) | (lcd_spec->lcd_height - 1);
	pLCDC->LVTIME4 = ((lcd_spec->lcd_FSWC1) << 16) | (lcd_spec->lcd_FEWC2);

//	Display Size
	pLCDC->LDS = (lcd_spec->lcd_height << 16) | lcd_spec->lcd_width;

//	Interrupt mask
	pLCDC->LIM &= 0xFFFFFFE6;
	pLCDC->LIM |= ((1 << 4) | 1);
	
}



#ifdef CONFIG_MACH_TCC9200S
//E3 : LCD_BL_EN, C29: LCD_RESET	C30:LCD_DISP  C31:LCD_PWRCTL
#define LCD_PROT_INT() 		BITCSET(HwGPIOC->GPEN, Hw29 | Hw30| Hw31, Hw29 | Hw30| Hw31)
							
#ifdef CONFIG_LCD_CPU_INTERFACE	
#define LCD_POWER_ON()		BITCSET(HwGPIOC->GPDAT, Hw31, Hw31);	\
							BITCSET(HwGPIOC->GPDAT, Hw30, Hw30)

#define LCD_POWER_OFF()		BITCSET(HwGPIOC->GPDAT, Hw31, 0);	\
							BITCSET(HwGPIOC->GPDAT, Hw30, 0)
#define LCD_DISPLAYON()
#define LCD_DISPLAYOFF()

#else
#define LCD_POWER_ON()		BITCSET(HwGPIOC->GPDAT, Hw31, Hw31)
#define LCD_POWER_OFF()		BITCSET(HwGPIOC->GPDAT, Hw31, 0)
#define LCD_DISPLAYON()		BITCSET(HwGPIOC->GPDAT, Hw30, Hw30)
#define LCD_DISPLAYOFF()	BITCSET(HwGPIOC->GPDAT, Hw30, 0)
#endif//
#define LCD_RESET_H()		BITCSET(HwGPIOC->GPDAT, Hw29, Hw29)
#define LCD_RESET_L()		BITCSET(HwGPIOC->GPDAT, Hw29, 0)

#elif defined(CONFIG_MACH_TCC9200) || defined(CONFIG_MACH_TCC9201) || defined(CONFIG_MACH_TCC8900)
#include <mach/tcc_pca953x.h>

//  A7 : LCD_bl_en ,  D22 : LCD_RST,   C28:LCD_DISP ,  extension: LCD_PWRCTL
#define LCD_PROT_INT() 		BITCSET(HwGPIOA->GPEN, Hw7, Hw7); 	\
							BITCSET(HwGPIOD->GPEN, Hw22, Hw22); \
							BITCSET(HwGPIOC->GPEN, Hw28 , Hw28); \
					 		BITCSET(HwGPIOA->GPDAT, Hw7, 0);  \
							BITCSET(HwGPIOD->GPDAT, Hw22, 0); \
							BITCSET(HwGPIOC->GPDAT, Hw28 , 0); 

#define LCD_POWER_ON()		tcc_pca953x_setup(PCA9539_U3_SLAVE_ADDR, LCD, OUTPUT, HIGH, SET_DIRECTION | SET_VALUE);
#define LCD_POWER_OFF()		tcc_pca953x_setup(PCA9539_U3_SLAVE_ADDR, LCD, OUTPUT, LOW, SET_DIRECTION | SET_VALUE);

#define LCD_DISPLAYON()		BITCSET(HwGPIOC->GPDAT, Hw28 ,Hw28)
#define LCD_DISPLAYOFF()	BITCSET(HwGPIOC->GPDAT, Hw28, 0)
#define LCD_RESET_H()		BITCSET(HwGPIOD->GPDAT, Hw22 ,Hw22)
#define LCD_RESET_L()		BITCSET(HwGPIOD->GPDAT, Hw22, 0)
#endif//


//  0 : off   ~ (1: 99): dim ~  level 100: on 
// GPIO A6

typedef struct{
		char reg;
		char value;
}BD6083GUL_CTRL_TYPE;
#define I2C_BD6083GUL_SLAVE 0xEC


//N-1) Power ON change to Normal mode
BD6083GUL_CTRL_TYPE BD6083_init[] = {{0x00, 0x01}, {0x01, 0x0E}, {0x03, 0x63},
											{0x09, 0x00},{0x02, 0x01}};

BD6083GUL_CTRL_TYPE BD6083_NormaleToALCD[] = {{0x01, 0x0F}, {0x09, 0x87},	{0x0A, 0x01},
													{0x0B, 0x01}, {0x0D, 0x13},	{0x0E, 0x63}, {0x01, 0x0F} };

BD6083GUL_CTRL_TYPE BD6083_ALCToNormal[] = {{0x01, 0x0E},	{0x09, 0x00},{0x02, 0x01}};

BD6083GUL_CTRL_TYPE BD6083_NormalToDiming[] = {{0x03, 0x04}};

BD6083GUL_CTRL_TYPE BD6083_DimingToSleep[] = {{0x02, 0x00}};

BD6083GUL_CTRL_TYPE BD6083_SleepToNomal[] = {{0x06, 0x63}, {0x02, 0x01}};

BD6083GUL_CTRL_TYPE BD6083_DimingToNomal[] = {{0x03, 0x63}};


void DX08D11VM0AAA_BL_I2C_Write(BD6083GUL_CTRL_TYPE *BL_ctrl, char size)
{
 	int i;
	char i2c_write_buf[2];
	
	for(i = 0; i < size; i++)
	{
		if(BL_ctrl[i].reg == 0xFF)
			LCD_Driver_mdelay(BL_ctrl[i].value);
		else
		{
			i2c_write_buf[0] = BL_ctrl[i].reg;
			i2c_write_buf[1] = BL_ctrl[i].value;
			i2c_xfer(I2C_BD6083GUL_SLAVE, 2, i2c_write_buf, 0, 0, I2C_CH0);
		}

	}
}

static char BD6083_state = 1;
void BD6083_BackLight (unsigned int BLlevel)
{
	if(BLlevel)
	{
		if(!BD6083_state)
			DX08D11VM0AAA_BL_I2C_Write(BD6083_init, sizeof(BD6083_init)/sizeof(BD6083GUL_CTRL_TYPE));
		else
		{
			BD6083_NormalToDiming[0].value = (BLlevel>>1);
			DX08D11VM0AAA_BL_I2C_Write(BD6083_NormalToDiming, sizeof(BD6083_NormalToDiming)/sizeof(BD6083GUL_CTRL_TYPE));
		}
		
		BD6083_state = 1;
	}
	else
	{
		BD6083_state = 0;
		DX08D11VM0AAA_BL_I2C_Write(BD6083_DimingToSleep, sizeof(BD6083_DimingToSleep)/sizeof(BD6083GUL_CTRL_TYPE));
	}

}


void backlight_ctrl(unsigned int level)
{
#ifdef NOT_USE_DIMMING
	#ifdef CONFIG_MACH_TCC9200S
	BITCSET(HwGPIOE->GPFN0, HwPORTCFG_GPFN3_MASK, HwPORTCFG_GPFN3(0)); 
	BITCSET(HwGPIOE->GPEN, Hw3, Hw3);

	if(level)	{
		BITCSET(HwGPIOE->GPDAT, Hw3, Hw3);
	}
	else	{
		BITCSET(HwGPIOE->GPDAT, Hw3, 0); // BL OFF	
	}
	#elif defined(CONFIG_MACH_TCC9200) || defined(CONFIG_MACH_TCC9201) || defined(CONFIG_MACH_TCC8900)
	BITCSET(HwGPIOA->GPEN, Hw7, Hw7);
	if(level)	{
		BITCSET(HwGPIOA->GPDAT, Hw7, Hw7);
	}
	else	{
		BITCSET(HwGPIOA->GPDAT, Hw7, 0); // BL OFF	
	}
	#endif//
	
	#ifdef CONFIG_FB_TCC92XX_WVGA_DX08D11VM0AAA
	BD6083_BackLight(level);
	#endif//

#else
	if(level)
	{
	#ifdef CONFIG_BOARD_89XX_CAR
		BITCSET(HwGPIOA->GPFN0, HwPORTCFG_GPFN7_MASK, HwPORTCFG_GPFN7(2)); 
		pTIMER->TMREF3 = (0x34*level)/100;
	#else
		BITCSET(HwGPIOA->GPFN0, HwPORTCFG_GPFN6_MASK, HwPORTCFG_GPFN6(2)); 
		pTIMER->TMREF2 = (0x34*level)/100;
	#endif
	}
	else
	{
		BITCSET(HwGPIOA->GPFN0, HwPORTCFG_GPFN6_MASK, HwPORTCFG_GPFN6(0)); 
		BITCSET(HwGPIOA->GPEN, Hw6, Hw6);
		BITCSET(HwGPIOA->GPDAT, Hw6, 0);
	}
#endif//


}

void tca_lcdc_port_setting(char Nlcdc)
{
	LCD_MODULE_SPEC_TYPE LCDspec;
		
	DEV_LCD_MODULE_SPEC(&LCDspec);
	LCDC_IO_Set(Nlcdc, LCDspec.lcd_bus_width);

}

/*****************************************************************************
* Function Name : tca_bkl_init()
******************************************************************************/
void tca_bkl_init(char lcdctrl_num)
{
	dprintk(KERN_INFO "%s\n", __FUNCTION__);

	current_lcdc = lcdctrl_num;

	if(lcdctrl_num)	{
		pLCDC	= (volatile PLCDC)tcc_p2v(HwLCDC1_BASE);
		current_lcdc_FbCh = 2;
	}
	else {
		pLCDC	= (volatile PLCDC)tcc_p2v(HwLCDC0_BASE);
		current_lcdc_FbCh = 0;
	}

	#ifndef NOT_USE_DIMMING
	pTIMER	= (volatile PTIMER)tcc_p2v(HwTMR_BASE);
	pTIMER->TCFG2 = 0x0114;
	pTIMER->TREF2 = 0x0034;
	pTIMER->TMREF2= 17;	// Level of Backlight (0 to 0x34), Max is 0x34 
	pTIMER->TCFG2	= 0x115;
	#endif//

}

#ifdef CONFIG_FB_TCC92XX_HVGA_LMS350DF01
static int tsc2003ts_suspend_d(void)
{
	tca_tchhw_disableirq();	
	tca_tsc2003_poweroff();
    return 0;
}

static int tsc2003ts_resume_d(void)
{
	tca_tchhw_enableirq();	
	tca_tsc2003_poweroff();
    return 0;
}
#endif//CONFIG_FB_TCC92XX_HVGA_LMS350DF01
/*****************************************************************************
* Function Name : tca_bkl_powerup()
******************************************************************************/
void tca_bkl_powerup(unsigned int last_buf)
{
	unsigned int lcdctrl;
	LCD_MODULE_SPEC_TYPE LCDspec;

	
	unsigned int lclk, lcd_div;
	
   	dprintk(KERN_INFO "%s bl levelL%d\n", __FUNCTION__, bl_value);

#ifdef CONFIG_FB_TCC92XX_HVGA_LMS350DF01
	tsc2003ts_resume_d();
#endif//CONFIG_FB_TCC92XX_HVGA_LMS350DF01

	DEV_LCD_MODULE_SPEC(&LCDspec);
	
	lcd_div = LCDspec.lcdc_clk_div / 2;
	
#ifdef LCDC0_USE
	lclk  = tca_ckc_getperi(PERI_LCD0);
#else
	lclk  = tca_ckc_getperi(PERI_LCD1);
#endif//LCDC0_USE

    dprintk("LCD pclk:%d lcdc:%d div:%d  \n",lclk, LCDspec.lcdc_clk, lcd_div);

#if !defined(CONFIG_FB_TCC92XX_HVGA_LMS350DF01) && !defined(CONFIG_FB_TCC92XX_WVGA_DX08D11VM0AAA)
	LCD_POWER_ON();
	LCD_Driver_mdelay(10);
#endif//
	LCD_RESET_H();
	LCD_Driver_udelay(10);

	LCDC_IO_Set(current_lcdc, LCDspec.lcd_bus_width);

	DEV_LCD_Module(LCD_M_INITIALIZE);

	lcdc_initialize(&LCDspec);

	lcdc_channel_restore(current_lcdc_FbCh, 0);


	//last pandisplay buffer apply
	if(current_lcdc_FbCh)	{
		printk("LCD ctrl:0x%x ch_2:0x%x :0x%x: 0x%x \n",pLCDC->LCTRL, pLCDC->LI2C,pLCDC->LI1C, pLCDC->LI0C);
		
		pLCDC->LI2BA0 = last_buf;
		pLCDC->LI2C |= HwLIC_IEN; //  image enable;
	}
	else {
		printk("LCD ctrl:0x%x ch_0:0x%x  \n",pLCDC->LCTRL, pLCDC->LI0C);

		pLCDC->LI0BA0 = last_buf;
		pLCDC->LI0C |= HwLIC_IEN; //  image enable ;
	}
	// lcdc enable
	lcdctrl = pLCDC->LCTRL;
	pLCDC->LCTRL = lcdctrl | (0x1);

	LCD_Driver_mdelay(16);
	
	LCD_DISPLAYON();

	mdelay(5);

	tcc_lcd_fb_on = 1;

}
EXPORT_SYMBOL(tca_bkl_powerup);

/*****************************************************************************
* Function Name : tca_bkl_powerdown()
******************************************************************************/
void tca_bkl_powerdown(void)
{
	unsigned int lcdctrl;
	LCD_MODULE_SPEC_TYPE LCDspec;
	
	tcc_lcd_fb_on = 0;

	LCD_DISPLAYOFF();
	
	LCD_Driver_mdelay(20);
	// lcdc enable
	lcdctrl = pLCDC->LCTRL;
	pLCDC->LCTRL = lcdctrl & (~(0x1));

	if(current_lcdc_FbCh)
		pLCDC->LI2C &= (~(HwLIC_IEN));
	else
		pLCDC->LI0C &= (~(HwLIC_IEN));

	lcdc_channel_restore(current_lcdc_FbCh, 1);

	DEV_LCD_MODULE_SPEC(&LCDspec);

	
#ifdef CONFIG_FB_TCC92XX_HVGA_LMS350DF01
	tsc2003ts_suspend_d();
#endif//CONFIG_FB_TCC92XX_HVGA_LMS350DF01

	DEV_LCD_Module(LCD_M_OFF);
	LCD_Driver_mdelay(10);

	LCDC_IO_Disable(current_lcdc, LCDspec.lcd_bus_width);

	LCD_Driver_mdelay(10);

	LCD_PROT_INT();
	
#if !defined(CONFIG_FB_TCC92XX_HVGA_LMS350DF01) && !defined(CONFIG_FB_TCC92XX_WVGA_DX08D11VM0AAA)
	LCD_RESET_L();
 	LCD_POWER_OFF();
#endif//


}

EXPORT_SYMBOL(tca_bkl_powerdown);

/*****************************************************************************
* Function Name : tca_bkl_setpowerval()
******************************************************************************/


void tca_bkl_setpowerval(unsigned int inValue)
{
	printk("tca_bkl_setpowerval inValue (%d) (%d) (%d) \n", inValue, tcc_lcd_fb_on, bl_value); 

	if(tcc_lcd_fb_on)
	{
		backlight_ctrl(inValue);

		bl_value = inValue;
	}
}


/*****************************************************************************
* Function Name : tca_bkl_getpowerval()
******************************************************************************/
unsigned int tca_bkl_getpowerval(void)
{
	dprintk("tca_bkl_getpowerval inValue(%d)\n", bl_value); 

	return bl_value;
}


#ifdef __KERNEL__
EXPORT_SYMBOL(tca_bkl_getpowerval);
EXPORT_SYMBOL(tca_bkl_setpowerval);
#endif
