
/****************************************************************************
 *   FileName    : tca_backlight.c
 *   Description : 
 ****************************************************************************
 *
 *   TCC Version 1.0
 *   Copyright (c) Telechips, Inc.
 *   ALL RIGHTS RESERVED
 *
 ****************************************************************************/
/*****************************************************************************
*
* Header Files Include
*
******************************************************************************/
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/delay.h>
#include <linux/fb.h>
#include <linux/init.h>
#include <linux/dma-mapping.h>
#include <linux/interrupt.h>
#include <linux/workqueue.h>
#include <linux/wait.h>
#include <linux/platform_device.h>
#include <linux/clk.h>

#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/div64.h>
#include <asm/mach/map.h>
#ifdef CONFIG_PM
#include <linux/pm.h>
#endif
#include <linux/cpufreq.h>
#include <mach/bsp.h>
#include <mach/tca_ckc.h>


#ifdef __KERNEL__
#	if defined(_TCC79x_)
#		include <mach/tcc79x.h>
#	elif defined(_TCC83x_)
#		include <mach/tcc83x_virt.h>
#	endif
#else
#	if defined(_TCC79x_)
#		include "TCC79x_Virtual.h"
#	elif defined(_TCC83x_)
#		include "TCC83x_Virtual.h"
#	endif
#endif

#include <mach/tcc92xxfb_ioctrl.h>
#include <mach/tcc_grp_ioctrl.h>
#include <mach/tcc_scaler_ctrl.h>
#include <mach/tcc_used_mem.h>
#include <mach/tca_fb_hdmi.h>



#if 0
#define dprintk(msg...)	 { printk( "tca_hdmi: " msg); }
#else
#define dprintk(msg...)	 
#endif

#if defined(CONFIG_TCC_HDMI_UI_SIZE_1280_720)
#define FB_HDMI_TEST
#endif

//#define FB_SCALER_ON_THE_FLY

extern struct display_platform_data tcc_display_data;

char M2M_Scaler1_Ctrl_Detail(SCALER_TYPE *scale_img);
void grp_rotate_ctrl(G2D_BITBLIT_TYPE *g2d_p);

static char *fb_scaler_pbuf0;
static char *fb_scaler_pbuf1;
static char *fb_g2d_pbuf0;
static char *fb_g2d_pbuf1;

#if defined(CONFIG_TCC_OUTPUT_STARTER) && defined(USE_DUALOUTPUT) && defined(USE_DUALBUFFER)
static char *fb_scaler_pbuf2;
static char *fb_scaler_pbuf3;
static char *fb_g2d_pbuf2;
static char *fb_g2d_pbuf3;
#endif

#if defined(CONFIG_MACH_TCC9200) || defined(CONFIG_MACH_TCC9201) || defined(CONFIG_MACH_TCC9200S)
#define HDMI_DISPLAY_MAX_WIDTH 		1280
#define HDMI_DISPLAY_MAX_HEIGHT		720
#define HDMI_DEC_MAX_WIDTH 			1280
#define HDMI_DEC_MAX_HEIGHT			720
#elif defined(CONFIG_MACH_TCC8900)
#if defined(CONFIG_TCC_HDMI_UI_SIZE_1280_720)
#if 0
#define HDMI_DISPLAY_MAX_WIDTH 		1280
#define HDMI_DISPLAY_MAX_HEIGHT		720
#else
#define HDMI_DIRECT_OUTPUT_WIDTH		1280
#define HDMI_DIRECT_OUTPUT_HEIGHT	720

#define HDMI_DISPLAY_MAX_WIDTH 		1920
#define HDMI_DISPLAY_MAX_HEIGHT		1080
#endif

#else
#define HDMI_DISPLAY_MAX_WIDTH 		1920
#define HDMI_DISPLAY_MAX_HEIGHT		1080
#endif
#define HDMI_DEC_MAX_WIDTH 			1920
#define HDMI_DEC_MAX_HEIGHT			1080
#endif /* CONFIG_MACH_TCC9200 */


#define TVOUT_MAX_WIDTH		720
#define TVOUT_MAX_HEIGHT	480

/*****************************************************************************
*
* Defines
*
******************************************************************************/
//#define TCC_FB_UPSCALE

/*****************************************************************************
*
* structures
*
******************************************************************************/
typedef enum{
	M2M_YUV422SEQ0,
	M2M_YUV422SEQ1,
	M2M_YUV422SEP,
	M2M_YUV420SEP,
	M2M_RGB565,
	M2M_RGB555,
	M2M_RGB444,
	M2M_RGB454		
} M2M_FORMAT_TYPE;

typedef struct{
	unsigned char 	intlv_mode; // interleaved mode cb/cr order => cr/cb : 0, cb/cr : 1
	unsigned char 	intlv;	  // YUV type => interleaved : 1, seperate : 0
	M2M_FORMAT_TYPE ImgFormat;	  // Only YUV420/422 format in case of Interleaved configuration!!
}M2M_FORMAT_Info;

typedef struct {
	unsigned int  pSrcBase0;
	unsigned int  pSrcBase1;
	unsigned int  pSrcBase2;
	unsigned int  SrcHsize;
	unsigned int  SrcVsize;
	unsigned int  DstHsize;
	unsigned int  DstVsize;
	unsigned int  pDstBase0;
	unsigned int  pDstBase1;
	unsigned int  pDstBase2;
	M2M_FORMAT_Info SrcImgType;
	M2M_FORMAT_Info DstImgType;
	unsigned int  Hsize_Offset;
	unsigned int  Y_Offset;
	unsigned int  UV_Offset;
	unsigned char  PathType;  // direct_path  1  , Memmory type : 0  //  M2M direct_path
} M2M_SCALER_INFO;

/*****************************************************************************
*
* Variables
*
******************************************************************************/

/*****************************************************************************
*
* Functions
*
******************************************************************************/

static PLCDC pLCDC_HDMI = NULL;

static struct clk *scaler1_clk;	 	// video image scale
static struct clk *scaler0_clk;		// fb image scale

static struct clk *mem_hdmi_clk;

static struct clk *lcdc0_hdmi_clk;
static struct clk *lcdc1_hdmi_clk;

static struct clk *g2d_clk;

static char hdmi_lcdc;

static unsigned int FBimg_buf_addr;
static char buf_index = 0;

#if defined(CONFIG_TCC_HDMI_UI_SIZE_1280_720)
static char OnTheFly_started;

#if 0
void DEV_M2M_Scaler_Interrupt_Ctrl(unsigned int uiType)
{
	MEM_SCALER_CTRL(uiType);
}

//-----------------------------------------------------------------------------------
// m2m_scl_info->PathType = 0 : normal
// m2m_scl_info->PathType = 1 : direct path
//-----------------------------------------------------------------------------------
int DEV_M2M_Scaler_Ctrl(M2M_SCALER_INFO *m2m_scl_info)
{
	unsigned int ScaleRatV, ScaleRatH;
	unsigned char WAIT_CYCLE =1;
	unsigned int SrcOffsetH;
	
	ScaleRatH = (m2m_scl_info->SrcHsize*256)/m2m_scl_info->DstHsize;
	ScaleRatV = (m2m_scl_info->SrcVsize*256)/m2m_scl_info->DstVsize;

	//Gregory.2006.10.13.
	//DHsize = ((m2m_scl_info->DstHsize+7)>>3)<<3;
	//DVsize = ((m2m_scl_info->DstVsize+7)>>3)<<3;

	if (ScaleRatH > 128)      
		WAIT_CYCLE = 1;
	else if (ScaleRatH > 64)      
		WAIT_CYCLE = 3;
	else if (ScaleRatH > 32)      
		WAIT_CYCLE = 7;
	else 
		WAIT_CYCLE = 15;
	
	// Source Set!
	if(m2m_scl_info->Hsize_Offset == 0)
		SrcOffsetH = m2m_scl_info->SrcHsize; 
	else
		SrcOffsetH = m2m_scl_info->Hsize_Offset; 

	// Source Set!
	MEM_SCALER_SRC_ADDR(m2m_scl_info->pSrcBase0 + m2m_scl_info->Y_Offset, 
							m2m_scl_info->pSrcBase1 + m2m_scl_info->UV_Offset, 
							m2m_scl_info->pSrcBase2 + m2m_scl_info->UV_Offset);
	MEM_SCALER_SRC_SIZE(m2m_scl_info->SrcHsize, m2m_scl_info->SrcVsize);
	MEM_SCALER_SRC_CONFIG(MSC_CFG_ALL, m2m_scl_info->SrcImgType.intlv_mode, m2m_scl_info->SrcImgType.intlv, m2m_scl_info->SrcImgType.ImgFormat);
	
	if ((m2m_scl_info->SrcImgType.ImgFormat == M2M_YUV422SEP) || (m2m_scl_info->SrcImgType.ImgFormat == M2M_YUV420SEP))
	{
		if(m2m_scl_info->SrcImgType.intlv)
			MEM_SCALER_SRC_OFFSET_YC(SrcOffsetH, SrcOffsetH);
		else
			MEM_SCALER_SRC_OFFSET_YC(SrcOffsetH, (SrcOffsetH/2));
	}
	else        
	{
		MEM_SCALER_SRC_OFFSET_YC((SrcOffsetH*2), 0);
	}



	// Destination Set!!
	MEM_SCALER_DST_ADDR(m2m_scl_info->pDstBase0, m2m_scl_info->pDstBase1, m2m_scl_info->pDstBase2);
	MEM_SCALER_RATIO(ScaleRatH, ScaleRatV);
	MEM_SCALER_DST_SIZE(m2m_scl_info->DstHsize, m2m_scl_info->DstVsize);

	if ((m2m_scl_info->DstImgType.ImgFormat == M2M_YUV422SEP) || (m2m_scl_info->DstImgType.ImgFormat == M2M_YUV420SEP))
	{
		if(m2m_scl_info->DstImgType.intlv)
			MEM_SCALER_DST_OFFSET_YC(m2m_scl_info->DstHsize, m2m_scl_info->DstHsize);
		else
			MEM_SCALER_DST_OFFSET_YC(m2m_scl_info->DstHsize, (m2m_scl_info->DstHsize/2));
		MEM_SCALER_DST_CONFIG(MSC_CFG_ALL, 0, WAIT_CYCLE, 1, m2m_scl_info->DstImgType.intlv_mode, m2m_scl_info->PathType, m2m_scl_info->DstImgType.intlv, m2m_scl_info->DstImgType.ImgFormat);		
	}	
	else
	{
		MEM_SCALER_DST_OFFSET_YC(m2m_scl_info->DstHsize*2, 0);
		MEM_SCALER_DST_CONFIG(MSC_CFG_ALL, 0, WAIT_CYCLE, 1, 0, m2m_scl_info->PathType, 0, m2m_scl_info->DstImgType.ImgFormat);		
	}

	return 0;
}

void TCC_OnTheFly_init(void)
{
	dprintk(" %s : lcd:%d  scaler  \n", __func__, hdmi_lcdc);

	BITCLR(pLCDC_HDMI->LI2C, HwLIC_IEN);

	{
		volatile unsigned int data;
		data = (pLCDC_HDMI->LSTATUS & Hw31);
		while(1 && (pLCDC_HDMI->LCTRL & Hw0))
		{
			if (data != (pLCDC_HDMI->LSTATUS & Hw31))
				break;
		}
	}

	BITCLR(pLCDC_HDMI->LI2C, HwLIC_SRC);

	M2M_Scaler_SW_Reset(M2M_SCALER0);

	if(hdmi_lcdc)
	{
		BITCSET(HwDDI_CONFIG->ON_THE_FLY, Hw3+Hw2, Hw2);
	}
	else
	{
		BITCSET(HwDDI_CONFIG->ON_THE_FLY, Hw3+Hw2, 0);	
	}

	OnTheFly_started = 1;

}

void TCC_OnTheFly_displayupdate(M2M_SCALER_INFO *m2m_scl_info)
{
	BITSET(HwM2MSCALER0->MSCSTR, HwMSC_STATUS_IRDY);
	//Path is must be direct path, PathType must be set 1.
	DEV_M2M_Scaler_Ctrl(m2m_scl_info);    //direct_path
	DEV_M2M_Scaler_Interrupt_Ctrl(MSC_CTRL_CON_EN | MSC_CTRL_EN);
}

void TCC_OnTheFly_finish(void)
{
	dprintk(" %s : onTheFly started:%d \n", __func__,OnTheFly_started);

	if(OnTheFly_started)
	{
		OnTheFly_started = 0;
		DEV_M2M_Scaler_Interrupt_Ctrl(MSC_CTRL_CON_DIS | MSC_CTRL_DIS | MSC_CTRL_BUSY_INT_DIS);
		BITCSET(HwDDI_CONFIG->ON_THE_FLY, Hw3+Hw2, 0);
		M2M_Scaler_SW_Reset(M2M_SCALER0);
	}	
}

//#else
#endif

void DEV_M2M_Scaler1_Interrupt_Ctrl(unsigned int uiType)
{
	MEM_SCALER1_CTRL(uiType);
}

//-----------------------------------------------------------------------------------
// m2m_scl_info->PathType = 0 : normal
// m2m_scl_info->PathType = 1 : direct path
//-----------------------------------------------------------------------------------
int DEV_M2M_Scaler1_Ctrl(M2M_SCALER_INFO *m2m_scl_info)
{
	unsigned int ScaleRatV, ScaleRatH;
	unsigned char WAIT_CYCLE =1;
	unsigned int SrcOffsetH;
	
	ScaleRatH = (m2m_scl_info->SrcHsize*256)/m2m_scl_info->DstHsize;
	ScaleRatV = (m2m_scl_info->SrcVsize*256)/m2m_scl_info->DstVsize;

	//Gregory.2006.10.13.
	//DHsize = ((m2m_scl_info->DstHsize+7)>>3)<<3;
	//DVsize = ((m2m_scl_info->DstVsize+7)>>3)<<3;

	if (ScaleRatH > 128)      
		WAIT_CYCLE = 1;
	else if (ScaleRatH > 64)      
		WAIT_CYCLE = 3;
	else if (ScaleRatH > 32)      
		WAIT_CYCLE = 7;
	else 
		WAIT_CYCLE = 15;
	
	// Source Set!
	if(m2m_scl_info->Hsize_Offset == 0)
		SrcOffsetH = m2m_scl_info->SrcHsize; 
	else
		SrcOffsetH = m2m_scl_info->Hsize_Offset; 

	// Source Set!
	MEM_SCALER1_SRC_ADDR(m2m_scl_info->pSrcBase0 + m2m_scl_info->Y_Offset, 
							m2m_scl_info->pSrcBase1 + m2m_scl_info->UV_Offset, 
							m2m_scl_info->pSrcBase2 + m2m_scl_info->UV_Offset);
	MEM_SCALER1_SRC_SIZE(m2m_scl_info->SrcHsize, m2m_scl_info->SrcVsize);
	MEM_SCALER1_SRC_CONFIG(MSC_CFG_ALL, m2m_scl_info->SrcImgType.intlv_mode, m2m_scl_info->SrcImgType.intlv, m2m_scl_info->SrcImgType.ImgFormat);
	
	if ((m2m_scl_info->SrcImgType.ImgFormat == M2M_YUV422SEP) || (m2m_scl_info->SrcImgType.ImgFormat == M2M_YUV420SEP))
	{
		if(m2m_scl_info->SrcImgType.intlv)
			MEM_SCALER1_SRC_OFFSET_YC(SrcOffsetH, SrcOffsetH);
		else
			MEM_SCALER1_SRC_OFFSET_YC(SrcOffsetH, (SrcOffsetH/2));
	}
	else        
	{
		MEM_SCALER1_SRC_OFFSET_YC((SrcOffsetH*2), 0);
	}
	


	// Destination Set!!
	MEM_SCALER1_DST_ADDR(m2m_scl_info->pDstBase0, m2m_scl_info->pDstBase1, m2m_scl_info->pDstBase2);
	MEM_SCALER1_RATIO(ScaleRatH, ScaleRatV);
	MEM_SCALER1_DST_SIZE(m2m_scl_info->DstHsize, m2m_scl_info->DstVsize);

	if ((m2m_scl_info->DstImgType.ImgFormat == M2M_YUV422SEP) || (m2m_scl_info->DstImgType.ImgFormat == M2M_YUV420SEP))
	{
		if(m2m_scl_info->DstImgType.intlv)
			MEM_SCALER1_DST_OFFSET_YC(m2m_scl_info->DstHsize, m2m_scl_info->DstHsize);
		else
			MEM_SCALER1_DST_OFFSET_YC(m2m_scl_info->DstHsize, (m2m_scl_info->DstHsize/2));
		MEM_SCALER1_DST_CONFIG(MSC_CFG_ALL, 0, WAIT_CYCLE, 1, m2m_scl_info->DstImgType.intlv_mode, m2m_scl_info->PathType, m2m_scl_info->DstImgType.intlv, m2m_scl_info->DstImgType.ImgFormat);		
	}	
	else
	{
		MEM_SCALER1_DST_OFFSET_YC(m2m_scl_info->DstHsize*2, 0);
		MEM_SCALER1_DST_CONFIG(MSC_CFG_ALL, 0, WAIT_CYCLE, 1, 0, m2m_scl_info->PathType, 0, m2m_scl_info->DstImgType.ImgFormat);		
	}

	return 0;
}

void TCC_OnTheFly_init(void)
{
	dprintk(" %s : lcd:%d  scaler  \n", __func__, hdmi_lcdc);

	BITCLR(pLCDC_HDMI->LI0C, HwLIC_IEN);

	{
		volatile unsigned int data;
		data = (pLCDC_HDMI->LSTATUS & Hw31);
		while(1 && (pLCDC_HDMI->LCTRL & Hw0))
		{
			if (data != (pLCDC_HDMI->LSTATUS & Hw31))
				break;
		}
	}

	BITCLR(pLCDC_HDMI->LI0C, HwLIC_SRC);

	M2M_Scaler_SW_Reset(M2M_SCALER1);

	if(hdmi_lcdc)
	{
		BITCSET(HwDDI_CONFIG->ON_THE_FLY, Hw5+Hw4 + Hw2, Hw4 | Hw2);
		BITCSET(HwDDI_CONFIG->ON_THE_FLY, Hw5+Hw4, Hw5+Hw4);
	}
	else
	{
		BITCSET(HwDDI_CONFIG->ON_THE_FLY, Hw5+Hw4, 0);	
		BITCSET(HwDDI_CONFIG->ON_THE_FLY, Hw5+Hw4, Hw5+Hw4);
	}

	OnTheFly_started = 1;

}

void TCC_OnTheFly_displayupdate(M2M_SCALER_INFO *m2m_scl_info)
{

	BITSET(HwM2MSCALER1->MSCSTR, HwMSC_STATUS_IRDY);

	//Path is must be direct path, PathType must be set 1.
	DEV_M2M_Scaler1_Ctrl(m2m_scl_info);    //direct_path

	DEV_M2M_Scaler1_Interrupt_Ctrl(MSC_CTRL_CON_EN | MSC_CTRL_EN);
}

void TCC_OnTheFly_finish(void)
{
	dprintk(" %s : onTheFly started:%d \n", __func__,OnTheFly_started);

	if(OnTheFly_started)
	{
		OnTheFly_started = 0;
		DEV_M2M_Scaler1_Interrupt_Ctrl(MSC_CTRL_CON_DIS | MSC_CTRL_DIS | MSC_CTRL_BUSY_INT_DIS);
		BITCSET(HwDDI_CONFIG->ON_THE_FLY, Hw5+Hw4, Hw5+Hw4);
		M2M_Scaler_SW_Reset(M2M_SCALER1);
	}	
}

#endif


char M2M_Scaler_OnTheFly(char OnOff)
{
	volatile PDDICONFIG pDDICONFIG;

	pDDICONFIG = (volatile PDDICONFIG)tcc_p2v(HwDDI_CONFIG_BASE);


	M2M_Scaler_SW_Reset(M2M_SCALER1);

	if(OnOff)	{
		BITCSET(pDDICONFIG->ON_THE_FLY, Hw2 + Hw3 + Hw4 + Hw5, Hw4 + Hw3);
	}
	else	{
		BITCSET(pDDICONFIG->ON_THE_FLY, Hw4 + Hw5, Hw4 + Hw5);
	}
}


void TCC_HDMI_LCDC_init(void)
{
	lcdc0_hdmi_clk = clk_get(0, "lcdc0");
	BUG_ON(lcdc0_hdmi_clk == NULL);

	lcdc1_hdmi_clk = clk_get(0, "lcdc1");
	BUG_ON(lcdc1_hdmi_clk == NULL);

	scaler1_clk = clk_get(0, "m2m1");
	BUG_ON(scaler1_clk == NULL);	

	scaler0_clk = clk_get(0, "m2m0");
	BUG_ON(scaler0_clk == NULL);	

	mem_hdmi_clk = clk_get(NULL, "mem_hdmi");
	BUG_ON(mem_hdmi_clk == NULL);

	g2d_clk = clk_get(NULL, "overlay");
	BUG_ON(g2d_clk == NULL);

	fb_scaler_pbuf0 = (char *)FB_SCALE_ADDR0;
	fb_scaler_pbuf1 = (char *)FB_SCALE_ADDR1;

	fb_g2d_pbuf0 = (char *)FB_G2D_ADDR0;
	fb_g2d_pbuf1 = (char *)FB_G2D_ADDR1;

	#if defined(CONFIG_TCC_OUTPUT_STARTER) && defined(USE_DUALOUTPUT) && defined(USE_DUALBUFFER)
	fb_scaler_pbuf2 = (char *)FB_SCALE_ADDR2;
	fb_scaler_pbuf3 = (char *)FB_SCALE_ADDR3;
	fb_g2d_pbuf2 = (char *)FB_G2D_ADDR2;
	fb_g2d_pbuf3 = (char *)FB_G2D_ADDR3;
	#endif
}

extern struct tcc_freq_table_t gtHdmiClockLimitTable;
	
void TCC_HDMI_LCDC_OnOff(char hdmi_lcdc_num, char onoff)
{
	dprintk(" %s hdmi_lcdc_num:%d onoff:%d  \n", __func__, hdmi_lcdc_num, onoff);

	if(onoff)
	{
		clk_enable(scaler0_clk);
		//M2M_Scaler_SW_Reset(M2M_SCALER0);
		M2M_Scaler_SW_Reset(M2M_SCALER1);

		clk_enable(scaler1_clk);

		clk_enable(g2d_clk);

		tcc_cpufreq_set_limit_table(&gtHdmiClockLimitTable, TCC_FREQ_LIMIT_HDMI, 1);
		
		if(hdmi_lcdc_num)
		{
			clk_enable(lcdc1_hdmi_clk);
			pLCDC_HDMI = (volatile PLCDC)tcc_p2v(HwLCDC1_BASE);
			pLCDC_HDMI->LCTRL = 0;
			pLCDC_HDMI->LI0C = 0;
			pLCDC_HDMI->LI2C = 0;
		}
		else
		{
			clk_enable(lcdc0_hdmi_clk);
			pLCDC_HDMI = (volatile PLCDC)tcc_p2v(HwLCDC0_BASE);
			pLCDC_HDMI->LCTRL = 0;
			pLCDC_HDMI->LI0C = 0;
		}
		hdmi_lcdc = hdmi_lcdc_num;

		#ifdef FB_SCALER_ON_THE_FLY
		M2M_Scaler_OnTheFly(1);
		#endif//
	}
	else
	{
		#ifdef FB_SCALER_ON_THE_FLY
		M2M_Scaler_OnTheFly(0);
		#endif//
	
		tcc_cpufreq_set_limit_table(&gtHdmiClockLimitTable, TCC_FREQ_LIMIT_HDMI, 0);

		pLCDC_HDMI = NULL;
			
		if(hdmi_lcdc) {
			clk_disable(lcdc1_hdmi_clk);
		}
		else {
			clk_disable(lcdc0_hdmi_clk);
		}
		
		clk_disable(scaler1_clk);
		clk_disable(scaler0_clk);
		clk_disable(g2d_clk);
	}
}

void TCC_TVOUT_LCDC_OnOff(char hdmi_lcdc_num, char onoff)
{
	dprintk(" %s hdmi_lcdc_num:%d onoff:%d  \n", __func__, hdmi_lcdc_num, onoff);

	if(onoff)
	{
		clk_enable(scaler0_clk);
		//M2M_Scaler_SW_Reset(M2M_SCALER0);
		M2M_Scaler_SW_Reset(M2M_SCALER1);

		clk_enable(scaler1_clk);

		clk_enable(g2d_clk);

		
		if(hdmi_lcdc_num)
		{						
			pLCDC_HDMI = (volatile PLCDC)tcc_p2v(HwLCDC1_BASE);
			pLCDC_HDMI->LCTRL = 0;
			pLCDC_HDMI->LI0C = 0;
			pLCDC_HDMI->LI2C = 0;
		}
		else
		{
			pLCDC_HDMI = (volatile PLCDC)tcc_p2v(HwLCDC0_BASE);
			pLCDC_HDMI->LCTRL = 0;
			pLCDC_HDMI->LI0C = 0;
		}
		hdmi_lcdc = hdmi_lcdc_num;
	}
	else
	{
		pLCDC_HDMI = NULL;

		clk_disable(scaler1_clk);
		clk_disable(scaler0_clk);

		clk_disable(g2d_clk);

	}
}

unsigned char TCC_HDMI_MSC_Wait_signal_disable(char ch)
{
	unsigned int loop = 0;
	volatile PM2MSCALER	uiPM2M;

	if(ch == 0)
	{
		uiPM2M = (volatile PM2MSCALER)tcc_p2v(HwM2MSCALER0_BASE);
	}
	else if( ch == 1)
	{
		uiPM2M = (volatile PM2MSCALER)tcc_p2v(HwM2MSCALER1_BASE);
	}

	if( ISSET(uiPM2M->MSCCTR, HwMSC_CTRL_EN) )
	{
		if( ch == 0)
			M2M_Scaler_SW_Reset(M2M_SCALER0);
		else if( ch == 1)
			M2M_Scaler_SW_Reset(M2M_SCALER1);
		
		while(TRUE && (loop < 100000)) //check wheather is msc on
		{
			if(ISSET(uiPM2M->MSCSTR, HwMSC_STATUS_RDY))
				break;
			msleep(1);

			dprintk(" %s : wait ch(%d)\n", __func__, ch);
		}
		// MSC disable
		BITCLR(uiPM2M->MSCCTR, HwMSC_CTRL_EN);
	}

	dprintk(" %s : disable check is complete\n", __func__, ch);

	return TRUE;
}

unsigned char TCC_HDMI_MSC_enable(char ch)
{

	volatile PM2MSCALER	uiPM2M;

	if(ch == 0)
	{
		uiPM2M = (volatile PM2MSCALER)tcc_p2v(HwM2MSCALER0_BASE);
	}
	else if( ch == 1)
	{
		uiPM2M = (volatile PM2MSCALER)tcc_p2v(HwM2MSCALER1_BASE);
	}

	BITSET(uiPM2M->MSCCTR, HwMSC_CTRL_EN);

	return TRUE;
}

int TCC_HDMI_Cable_Detect(void)
{
	PGPIO pGPIO = (PGPIO)tcc_p2v(HwGPIO_BASE);

	//dprintk("%s\n", __func__);

	BITCLR(pGPIO->GPAEN, Hw14);
	BITCLR(pGPIO->GPAFN1, Hw28-Hw24);
	{volatile int ttt;for(ttt=0;ttt<500;ttt++);}

	if(ISSET(pGPIO->GPADAT, Hw14))
	{
 		//dprintk("HDMI cable is detected!!\n");
		return true;
	}
	else
	{
 		//dprintk("HDMI cable is not detected!!\n");
		return false;
	}
}

void TCC_HDMI_LCDC_Timing(struct lcdc_timimg_parms_t *mode)
{
    volatile unsigned int regl;
	unsigned int lcdc_id;

	PCKC	pCKC = (CKC *)&HwCLK_BASE;
	PLCDC	pLCDC;
	
	dprintk(" %s lcdc:[%d]  %d %d %d %d \n", __func__, hdmi_lcdc, mode->id, mode->iv, mode->ih, mode->lpw);


	if (hdmi_lcdc)	// lcdc1
	{
		lcdc_id = 1;
		pLCDC = (LCDC *)&HwLCDC1_BASE;
		pLCDC->LCTRL = 0;

		BITSET(HwDDI_CONFIG->HDMI_CTRL, HwDDIC_HDMI_CTRL_SEL_LCDC1);
		

		// disable lcdc
		regl = readl(&pLCDC->LCTRL);
		writel(regl & ~(0x00000001), &pLCDC->LCTRL);
		
		regl = readl(&pCKC->PCLK_LCD1);
		writel(regl & ~(0x10000000), &pCKC->PCLK_LCD1);	// EN: 0

		// set lcdc1 clock
		regl = readl(&pCKC->PCLK_LCD1);
		regl &= ~(0x1f000fff);
		writel(regl, &pCKC->PCLK_LCD1);	// EN : 0, init.
		regl |= 12<<24;					// SEL: HDMI PCLK(Variable)
		regl |= 0;						// DIV: 0
		writel(regl, &pCKC->PCLK_LCD1);
		regl |= 1 <<28;					// EN : 1
		writel(regl, &pCKC->PCLK_LCD1);
	}
	else		// lcdc0
	{
		lcdc_id = 0;
		pLCDC = (LCDC *)&HwLCDC0_BASE;

		pLCDC->LCTRL = 0;

		BITCLR(HwDDI_CONFIG->HDMI_CTRL, HwDDIC_HDMI_CTRL_SEL_LCDC1);

		// disable lcdc
		regl = readl(&pLCDC->LCTRL);
		writel(regl & ~(0x00000001), &pLCDC->LCTRL);

		// set lcdc0 clock
		regl = readl(&pCKC->PCLK_LCD0);
		regl &= ~(0x1f000fff);
		writel(regl, &pCKC->PCLK_LCD0);	// EN : 0, init.
		regl |= 12<<24;					// SEL: HDMI PCLK(Variable)
		regl |= 0;						// DIV: 0
		writel(regl, &pCKC->PCLK_LCD0);
		regl |= 1 <<28;					// EN : 1
		writel(regl, &pCKC->PCLK_LCD0);

	}

	//channel disable
	regl = readl(&pLCDC->LI0C);
	writel(regl & (~(0x10000000)), &pLCDC->LI0C);

	// set lctrl
	regl = readl(&pLCDC->LCTRL);
	regl &= ~(0x0000f3e0);
	regl |=((mode->id  & 0x1) <<15)
		 | ((mode->iv  & 0x1) <<14)
		 | ((mode->ih  & 0x1) <<13)
		 | ((mode->ip  & 0x1) <<12)
		 | ((mode->dp  & 0x1) << 9)
		 | ((mode->ni  & 0x1) << 8)
		 | ((mode->tv  & 0x1) << 7)
		 | ((mode->tft & 0x1) << 6)
		 | ((mode->stn & 0x1) << 5);
	regl = (regl & ~(0xf<<16)) | ((12)<<16);	//pxdw: 12 - rgb888
	writel(regl, &pLCDC->LCTRL);

	// set clkdiv
	writel(0x80000000, &pLCDC->LCLKDIV);
	
	// set timing
	writel((mode->lpw  <<16) | (mode->lpc  ), &pLCDC->LHTIME1);
	writel((mode->lswc <<16) | (mode->lewc ), &pLCDC->LHTIME2);
	writel((mode->fpw  <<16) | (mode->flc  ), &pLCDC->LVTIME1);
	writel((mode->fswc <<16) | (mode->fewc ), &pLCDC->LVTIME2);
	writel((mode->fpw2 <<16) | (mode->flc2 ), &pLCDC->LVTIME3);
	writel((mode->fswc2<<16) | (mode->fewc2), &pLCDC->LVTIME4);

	regl = readl(&pLCDC->LVTIME1);
	writel(regl | (mode->vdb  <<27) | (mode->vdf  <<22), &pLCDC->LVTIME1);

	// set lcdc.lds
#ifdef CONFIG_MACH_TCC8900
	if( mode->dp == 1)
	{
		regl = ((mode->flc+1 )<<16) | ((mode->lpc+1) >> 1);
	}
	else
#endif
	{
		regl = ((mode->flc+1)<<16) | (mode->lpc+1);
	}
	writel(regl, &pLCDC->LDS);

	// set lcdc.img dma interlace mode
	if (mode->ni)	// non-interlace mode (progressive mode)
	{
		regl = readl(&pLCDC->LI0C);
		writel(regl & ~(0x80000000), &pLCDC->LI0C);
		if(lcdc_id)
		{
			regl = readl(&pLCDC->LI1C);
			writel(regl & ~(0x80000000), &pLCDC->LI1C);
			regl = readl(&pLCDC->LI2C);
			writel(regl & ~(0x80000000), &pLCDC->LI2C);
		}
	}
	else			// interlace mode
	{
		regl = readl(&pLCDC->LI0C);
		writel(regl | (0x80000000), &pLCDC->LI0C);
		if(lcdc_id)
		{
			regl = readl(&pLCDC->LI1C);
			writel(regl | (0x80000000), &pLCDC->LI1C);
			regl = readl(&pLCDC->LI2C);
			writel(regl | (0x80000000), &pLCDC->LI2C);
		}
	}

	// enable lcdc
	regl = readl(&pLCDC->LCTRL);
	writel(regl | 0x0000000B, &pLCDC->LCTRL);

    return;
}



void TCC_HDMI_DISPLAY_UPDATE(struct tcc_lcdc_image_update *ImageInfo)
{

	unsigned int regl;
	unsigned int lcd_width = 0, lcd_height = 0, lcd_h_pos = 0, lcd_w_pos = 0, scale_x = 0, scale_y = 0;
	unsigned int img_width = 0, img_height = 0;
	unsigned int buffer_width = 0, y2r_option = 0;
	PLCDC_CHANNEL pLCDC_channel = NULL;
//	dprintk(" %s :fmt:%d Fw:%d Fh:%d Iw:%d Ih:%d \n", __func__,
//		ImageInfo->fmt,ImageInfo->Frame_width, ImageInfo->Frame_height, ImageInfo->Image_width, ImageInfo->Image_height);

	if((ImageInfo->Lcdc_layer >= 3) || (ImageInfo->fmt >TCC_LCDC_IMG_FMT_MAX))
		return;
		
	regl = pLCDC_HDMI->LDS; // get LCD size 

	lcd_width = (regl & 0xFFFF);
	lcd_height = ((regl>>16) & 0xFFFF);
	
	if((!lcd_width) || (!lcd_height))
		return;

	switch(ImageInfo->Lcdc_layer )
	{
		case 0:
			pLCDC_channel = (volatile PLCDC_CHANNEL)tcc_p2v(pLCDC_HDMI->LI0C);
			break;
		case 1:
			pLCDC_channel = (volatile PLCDC_CHANNEL)tcc_p2v(pLCDC_HDMI->LI1C);
			break;
		case 2:
			pLCDC_channel = (volatile PLCDC_CHANNEL)tcc_p2v(pLCDC_HDMI->LI2C);
			break;
	}
	
	img_width = ImageInfo->Frame_width;
	img_height= ImageInfo->Frame_height;

	if(!ImageInfo->enable)	{
		pLCDC_channel->LIC = 0;
		return;
	}

	{
		buffer_width = (((ImageInfo->Frame_width + 7)>>3)<<3);

		switch(ImageInfo->fmt)
		{
			case TCC_LCDC_IMG_FMT_RGB444:
			case TCC_LCDC_IMG_FMT_RGB565:
			case TCC_LCDC_IMG_FMT_RGB555:
				pLCDC_channel->LIO = buffer_width *2;
				y2r_option = HwLIC_Y2R;
				break;

			case TCC_LCDC_IMG_FMT_RGB888:
			case TCC_LCDC_IMG_FMT_RGB666:
				pLCDC_channel->LIO = buffer_width * 4;
				y2r_option = HwLIC_Y2R;
				break;
			
			case TCC_LCDC_IMG_FMT_YUV420SP:
			case TCC_LCDC_IMG_FMT_YUV422SP:
				pLCDC_channel->LIO = ((buffer_width / 2)<<16)|(buffer_width);
				y2r_option = HwLIC_Y2R;
				break;

			case TCC_LCDC_IMG_FMT_YUV422SQ:
				pLCDC_channel->LIO = buffer_width * 2;
				y2r_option = HwLIC_Y2R;
				break;

			case TCC_LCDC_IMG_FMT_YUV420ITL0:
			case TCC_LCDC_IMG_FMT_YUV420ITL1:
			case TCC_LCDC_IMG_FMT_YUV422ITL0:
			case TCC_LCDC_IMG_FMT_YUV422ITL1:
				pLCDC_channel->LIO = (buffer_width << 16)| (buffer_width);
				y2r_option = HwLIC_Y2R;
				break;

		}


		scale_x = 0;
		scale_y = 0;
		lcd_h_pos = 0;
		lcd_w_pos = 0;

		if(lcd_width > ImageInfo->Image_width)
			lcd_w_pos = (lcd_width - ImageInfo->Image_width)/2;
		
		if(lcd_height > ImageInfo->Image_height)
			lcd_h_pos = (lcd_height - ImageInfo->Image_height)/2;

	// position
		pLCDC_channel->LIP = ((lcd_h_pos << 16) | (lcd_w_pos));
	// scale
		pLCDC_channel->LISR =((scale_y<<4) | scale_x);
	// size
		pLCDC_channel->LIS =((ImageInfo->Image_height << 16) | (ImageInfo->Image_width));

	// position
		pLCDC_channel->LIP =((ImageInfo->offset_y << 16) | (ImageInfo->offset_x));

	// image address
		pLCDC_channel->LIBA0 = ImageInfo->addr0;
		pLCDC_channel->LIBA1 = ImageInfo->addr1;
		pLCDC_channel->LIBA2 = ImageInfo->addr2;


	//channel control
		pLCDC_channel->LIC = HwLIC_IEN | y2r_option | ImageInfo->fmt;
	}
	

}

char TCC_HDMI_FB_update(unsigned int width, unsigned int height, unsigned int bits_per_pixel, unsigned int addr)
{
	unsigned int regl = 0, g2d_rotate_need = 0, ret = 0;
	unsigned int lcd_width = 0, lcd_height = 0, lcd_h_pos = 0, lcd_w_pos = 0, scale_x = 0, scale_y = 0;
	unsigned int img_width = 0, img_height = 0;
	unsigned int chromaR = 0, chromaG = 0, chromaB = 0;
	unsigned int ifmt;
#if defined(CONFIG_TCC_HDMI_UI_SIZE_1280_720)
	unsigned int ui_width = HDMI_DIRECT_OUTPUT_WIDTH, ui_height = HDMI_DIRECT_OUTPUT_HEIGHT;
#endif

	PLCDC_CHANNEL pLCDC_HDMI_FB_CH;

	G2D_BITBLIT_TYPE g2d_p;
	SCALER_TYPE fbscaler;

	memset(&g2d_p, 0x00, sizeof(G2D_BITBLIT_TYPE));
	memset(&fbscaler, 0x00, sizeof(SCALER_TYPE));
	
	if(pLCDC_HDMI == NULL)
	{
		dprintk("%s - Err: HDMI LCDC is not valid\n", __func__);
		return 0;
	}

	FBimg_buf_addr = addr;

	regl = pLCDC_HDMI->LDS; // get LCD size 

	lcd_width = (regl & 0xFFFF);
	lcd_height = ((regl>>16) & 0xFFFF);

	if((!lcd_width) || (!lcd_height))
		return 0;

	chromaR = chromaG = chromaB = 0x00;


	if(hdmi_lcdc)
		pLCDC_HDMI_FB_CH = (volatile PLCDC_CHANNEL)tcc_p2v(pLCDC_HDMI->LI2C);
	else
		pLCDC_HDMI_FB_CH = (volatile PLCDC_CHANNEL)tcc_p2v(pLCDC_HDMI->LI0C);

	// lcd image offset
	if(bits_per_pixel == 32)		{
		ifmt = TCC_LCDC_IMG_FMT_RGB888;
	}
	else		{
		ifmt = TCC_LCDC_IMG_FMT_RGB565;
	}

#if defined(CONFIG_TCC_HDMI_UI_SIZE_1280_720)
	if(tcc_display_data.resolution == 0)
	{
		ui_width = 1280;
		ui_height = 720;
	}
	else if(tcc_display_data.resolution == 1)
	{
		ui_width = 720;
		ui_height = 576;
	}
	else if(tcc_display_data.resolution == 2)
	{	
		ui_width = 800;
		ui_height = 480;
	}

	if(lcd_width == ui_width && lcd_height == ui_height)
	{
		img_width = lcd_width;
		img_height = lcd_height;

		scale_x = 0;
		scale_y = 0;
		if(lcd_height > height)
			lcd_h_pos = (lcd_height - height)/2;
		#if 1
		else if(lcd_height < height)
			addr = addr + (width * (height - lcd_height) * 2);
		#endif//
		
		if(lcd_width > width)
			lcd_w_pos = (lcd_width - width)/2;


		// lcd image offset
		if(bits_per_pixel == 32)		{
			pLCDC_HDMI_FB_CH->LIO = img_width * 4;
		}
		else		{
			pLCDC_HDMI_FB_CH->LIO = img_width * 2;
		}

	// position
		pLCDC_HDMI_FB_CH->LIP = ((lcd_h_pos << 16) | (lcd_w_pos));
	// scale
		pLCDC_HDMI_FB_CH->LISR =((scale_y<<4) | scale_x);
	// size
		pLCDC_HDMI_FB_CH->LIS =((img_height << 16) | (img_width));

	// image address
		pLCDC_HDMI_FB_CH->LIBA0 = addr;

		// in case RGB888, Alpha blending enable...
		if(bits_per_pixel == 32)
		{
			TCC_HDMI_FB_Set_AlphaBlending();
		}
		else
		{
			BITCSET (pLCDC_HDMI_FB_CH->LIKR, 0xff <<  0, (chromaR)  <<  0); // key
			BITCSET (pLCDC_HDMI_FB_CH->LIKR, 0xff << 16, (0xF8)	<< 16); // keymask
			BITCSET (pLCDC_HDMI_FB_CH->LIKG, 0xff <<  0, (chromaG)  <<  0); // key
			BITCSET (pLCDC_HDMI_FB_CH->LIKG, 0xff << 16, (0xFC)	<< 16); // keymask
			BITCSET (pLCDC_HDMI_FB_CH->LIKB, 0xff <<  0, (chromaB)  <<  0); // key
			BITCSET (pLCDC_HDMI_FB_CH->LIKB, 0xff << 16, (0xF8)	<< 16); // keymask
			pLCDC_HDMI_FB_CH->LIC |= HwLIC_CEN;
		}

		//channel control
		pLCDC_HDMI_FB_CH->LIC |= HwLIC_IEN;
		BITCSET(pLCDC_HDMI_FB_CH->LIC, 0x0000001F, ifmt);

		pLCDC_HDMI->LCTRL |= (HwLCTRL_LEN | /* HwLCTRL_MSEL|*/ (Hw3+Hw1));

		pLCDC_HDMI->LSTATUS |= HwLSTATUS_RU;
	}
	else
#endif
	{
		if(width < height) {
			g2d_rotate_need = 1;
		}

		if(g2d_rotate_need ||(bits_per_pixel == 32))
		{
			g2d_p.responsetype = G2D_POLLING;
		
			g2d_p.src0 = (unsigned int)addr;
			if(bits_per_pixel == 32)
				g2d_p.srcfm.format = GE_RGB888;
			else
				g2d_p.srcfm.format = GE_RGB565;

			g2d_p.srcfm.data_swap = 0;
			
			g2d_p.src_imgx = width;
			g2d_p.src_imgy = height;

			if(g2d_rotate_need)
			g2d_p.ch_mode = ROTATE_270;
			else
				g2d_p.ch_mode = NOOP;

			g2d_p.crop_offx = 0;
			g2d_p.crop_offy = 0;
			g2d_p.crop_imgx = width;
			g2d_p.crop_imgy =  height;

			#if defined(CONFIG_TCC_OUTPUT_STARTER) && defined(USE_DUALOUTPUT) && defined(USE_DUALBUFFER)
				if(hdmi_lcdc)
				{
					if(buf_index)
						g2d_p.tgt0 = (unsigned int)fb_g2d_pbuf2;	// destination image address
					else
						g2d_p.tgt0 = (unsigned int)fb_g2d_pbuf3;	// destination image address
				}
				else
				{
					if(buf_index)
						g2d_p.tgt0 = (unsigned int)fb_g2d_pbuf0;	// destination image address
					else
						g2d_p.tgt0 = (unsigned int)fb_g2d_pbuf1;	// destination image address
				}
			#else
				if(buf_index)
					g2d_p.tgt0 = (unsigned int)fb_g2d_pbuf0;	// destination image address
				else
					g2d_p.tgt0 = (unsigned int)fb_g2d_pbuf1;	// destination image address
			#endif

			// destinaion format yuv_sq
			ifmt = TCC_LCDC_IMG_FMT_YUV422SQ;
			g2d_p.tgtfm.format = GE_YUV422_sq;
			g2d_p.tgtfm.data_swap = 0;


			if(g2d_rotate_need)		{
				g2d_p.dst_imgx = height;
				g2d_p.dst_imgy = width;
			}
			else		{
				g2d_p.dst_imgx = width;
				g2d_p.dst_imgy = height;
			}

			g2d_p.dst_off_x = 0;
			g2d_p.dst_off_y = 0;
			g2d_p.alpha_value = 255;

			g2d_p.alpha_type = G2d_ALPHA_NONE;
			
			grp_rotate_ctrl(&g2d_p);

			FBimg_buf_addr = g2d_p.tgt0;

			if(g2d_rotate_need)		{
				width = g2d_p.dst_imgx;
				height = g2d_p.dst_imgy;
			}
			

		}

#if 1//!defined(CONFIG_TCC_HDMI_UI_SIZE_1280_720)
		if(lcd_width > HDMI_DISPLAY_MAX_WIDTH) 
			img_width = lcd_width / 2;
		else
			img_width = lcd_width;

		if(lcd_height > HDMI_DISPLAY_MAX_HEIGHT) 
			img_height = lcd_height / 2;
		else
			img_height = lcd_height;

		fbscaler.responsetype = SCALER_NOWAIT;

		fbscaler.src_Yaddr = (char *)FBimg_buf_addr;

		if(ifmt == TCC_LCDC_IMG_FMT_YUV422SQ)
			fbscaler.src_fmt = SCALER_YUV422_sq0;
		else
			fbscaler.src_fmt = SCALER_RGB565;
			
		fbscaler.src_ImgWidth = width;
		fbscaler.src_ImgHeight = height;

		fbscaler.src_winLeft = 0;
		fbscaler.src_winTop = 0;
		fbscaler.src_winRight = width;
		fbscaler.src_winBottom = height;

		#if defined(CONFIG_TCC_OUTPUT_STARTER) && defined(USE_DUALOUTPUT) && defined(USE_DUALBUFFER)
			if(hdmi_lcdc)
			{
				if(buf_index)
					fbscaler.dest_Yaddr = fb_scaler_pbuf2;	// destination image address
				else
					fbscaler.dest_Yaddr = fb_scaler_pbuf3;	// destination image address
			}
			else
			{
				if(buf_index)
					fbscaler.dest_Yaddr = fb_scaler_pbuf0;	// destination image address
				else
					fbscaler.dest_Yaddr = fb_scaler_pbuf1;	// destination image address
			}
		#else
			if(buf_index)
				fbscaler.dest_Yaddr = fb_scaler_pbuf0;	// destination image address
			else
				fbscaler.dest_Yaddr = fb_scaler_pbuf1;	// destination image address
		#endif

		fbscaler.dest_Uaddr = fbscaler.dest_Yaddr + (img_width * img_height);	// destination image address

		buf_index = !buf_index;

#if defined(CONFIG_MACH_TCC8900)
		if( lcd_width == HDMI_DISPLAY_MAX_WIDTH && lcd_height == HDMI_DISPLAY_MAX_HEIGHT)
		{
		        img_width >>= 1;
		}
#endif /* CONFIG_MACH_TCC8900 */

		fbscaler.dest_fmt = SCALER_YUV422_sq0;		// destination image format
		fbscaler.dest_ImgWidth = img_width;	// destination image width
		fbscaler.dest_ImgHeight = img_height; // destination image height
		fbscaler.dest_winLeft = 0;;
		fbscaler.dest_winTop = 0;;
		fbscaler.dest_winRight = img_width;
		fbscaler.dest_winBottom = img_height;

		#ifdef FB_SCALER_ON_THE_FLY
		fbscaler.viqe_onthefly = 0x2;
		#else
		fbscaler.viqe_onthefly = 0;
		ret = DEV_M2M_Wait_signal_disable(1);

		if((ret == 0) || (ret >= M2M_WAIT_MAX))		{
			printk("ERROR : %s M2M not ready to run %d \n",__func__, ret);
		}
		#endif//

		M2M_Scaler1_Ctrl_Detail(&fbscaler);
	
		FBimg_buf_addr = (unsigned int)fbscaler.dest_Yaddr;

	//	dprintk(" %s FB[w:%d h:%d] Image[w:%d h:%d] LCD[w:%d h:%d] addr:0x%x index:%d \n",
	//		__func__,width, height, img_width, img_height, lcd_width, lcd_height, FBimg_buf_addr, buf_index);

	// lcd image offset
		pLCDC_HDMI_FB_CH->LIO = img_width * 2;

	// position
		pLCDC_HDMI_FB_CH->LIP = ((lcd_h_pos << 16) | (lcd_w_pos));

	// scale 2
		if(lcd_width > img_width)
			scale_x = 9;
		else
			scale_x = 0;

		if(lcd_height > img_height)
			scale_y = 9;
		else
			scale_y = 0;

		pLCDC_HDMI_FB_CH->LISR =((scale_y<<4) | scale_x);
	// size
		pLCDC_HDMI_FB_CH->LIS =((img_height << 16) | (img_width));

	// image address
	//	pLCDC_HDMI_FB_CH->LIBA0 = fbscaler.dest_Yaddr;

	//channel control
		BITCSET (pLCDC_HDMI_FB_CH->LIKR, 0xff <<  0, (chromaR)  <<  0); // key
		BITCSET (pLCDC_HDMI_FB_CH->LIKR, 0xff << 16, (0xF8)	<< 16); // keymask
		BITCSET (pLCDC_HDMI_FB_CH->LIKG, 0xff <<  0, (chromaG)  <<  0); // key
		BITCSET (pLCDC_HDMI_FB_CH->LIKG, 0xff << 16, (0xFC)	<< 16); // keymask
		BITCSET (pLCDC_HDMI_FB_CH->LIKB, 0xff <<  0, (chromaB)  <<  0); // key
		BITCSET (pLCDC_HDMI_FB_CH->LIKB, 0xff << 16, (0xF8)	<< 16); // keymask
	//	pLCDC_HDMI_FB_CH->LIC = HwLIC_IEN | TCC_LCDC_IMG_FMT_RGB565 | HwLIC_CEN;
	
		#ifdef FB_SCALER_ON_THE_FLY
		pLCDC_HDMI_FB_CH->LIC = HwLIC_IEN | HwLIC_SRC | HwLIC_Y2R |TCC_LCDC_IMG_FMT_YUV422SQ | HwLIC_CEN;
		pLCDC_HDMI->LCTRL |= (HwLCTRL_LEN | HwLCTRL_MSEL| (Hw3+Hw1));
		#else
		pLCDC_HDMI_FB_CH->LIC = HwLIC_IEN | HwLIC_Y2R |TCC_LCDC_IMG_FMT_YUV422SQ | HwLIC_CEN;
		pLCDC_HDMI->LCTRL |= (HwLCTRL_LEN | /* HwLCTRL_MSEL|*/ (Hw3+Hw1));
		#endif//


		pLCDC_HDMI->LSTATUS |= HwLSTATUS_RU;

#else
#if 1//def TCC_FB_UPSCALE
	M2M_SCALER_INFO OnTheFly;
	if(g2d_rotate_need ||(bits_per_pixel == 32))
	{
		OnTheFly.pSrcBase0 = FBimg_buf_addr;
	}
	else
	{
		OnTheFly.pSrcBase0 = addr;
	}

	OnTheFly.SrcHsize = width;
	OnTheFly.SrcVsize = height;

//8  
	OnTheFly.DstHsize = ((lcd_width>>3)<<3);
	OnTheFly.DstVsize = ((lcd_height>>3)<<3);
	
	OnTheFly.SrcImgType.intlv_mode = 0;
	OnTheFly.SrcImgType.intlv = 0;
	OnTheFly.SrcImgType.ImgFormat = M2M_RGB565;
	OnTheFly.DstImgType.intlv_mode = 0;
	OnTheFly.DstImgType.intlv = 0;
	OnTheFly.DstImgType.ImgFormat = M2M_RGB565;//M2M_YUV422SEQ0;
	
	OnTheFly.Y_Offset = 0;
	OnTheFly.UV_Offset = 0;
	OnTheFly.Hsize_Offset  =0;
	OnTheFly.PathType = 1;  // direct_path

	if(OnTheFly.DstHsize > lcd_width)
		img_width = lcd_width;
	else
		img_width = OnTheFly.DstHsize;		

	if(OnTheFly.DstVsize > lcd_height)
		img_height = lcd_height;
	else
		img_height = OnTheFly.DstVsize;

	if(lcd_width > img_width)
		lcd_w_pos = (lcd_width - img_width) / 2;

	if(lcd_height > img_height)
		lcd_h_pos = (lcd_height - img_height) / 2;
	

	if(OnTheFly_started == 0)
	{
		TCC_OnTheFly_init();
		BITCSET(pLCDC_HDMI->LCTRL, HwLCTRL_MSEL, HwLCTRL_MSEL);
		OnTheFly_started = 1;
		TCC_OnTheFly_displayupdate(&OnTheFly);

	}
	else
	{
		TCC_OnTheFly_displayupdate(&OnTheFly);

		scale_x = 0;
		scale_y = 0;
	
		// position
		pLCDC_HDMI_FB_CH->LIP = ((lcd_h_pos << 16) | (lcd_w_pos));
		// scale
		pLCDC_HDMI_FB_CH->LISR =((scale_y<<4) | scale_x);
		// size
		pLCDC_HDMI_FB_CH->LIS =((img_height << 16) | (img_width));

		BITCSET (pLCDC_HDMI_FB_CH->LIKR, 0xff <<  0, (chromaR)  <<  0); // key
		BITCSET (pLCDC_HDMI_FB_CH->LIKR, 0xff << 16, (0xF8)	<< 16); // keymask
		BITCSET (pLCDC_HDMI_FB_CH->LIKG, 0xff <<  0, (chromaG)  <<  0); // key
		BITCSET (pLCDC_HDMI_FB_CH->LIKG, 0xff << 16, (0xFC)	<< 16); // keymask
		BITCSET (pLCDC_HDMI_FB_CH->LIKB, 0xff <<  0, (chromaB)  <<  0); // key
		BITCSET (pLCDC_HDMI_FB_CH->LIKB, 0xff << 16, (0xF8)	<< 16); // keymask
		
		if(hdmi_lcdc)
			pLCDC_HDMI_FB_CH->LIC = HwLIC_IEN | HwLIC_CEN | HwLIC_SRC | TCC_LCDC_IMG_FMT_RGB565;
		else
			pLCDC_HDMI_FB_CH->LIC = HwLIC_IEN | HwLIC_SRC | TCC_LCDC_IMG_FMT_RGB565;

		pLCDC_HDMI->LCTRL |= (HwLCTRL_LEN | HwLCTRL_MSEL|(Hw3+Hw1));

		pLCDC_HDMI->LSTATUS |= HwLSTATUS_RU;

	}
#else
	img_width = lcd_width;
	img_height = lcd_height;

	scale_x = 0;
	scale_y = 0;
	if(lcd_height > height)
		lcd_h_pos = (lcd_height - height)/2;
	#if 1
	else if(lcd_height < height)
		addr = addr + (width * (height - lcd_height) * 2);
	#endif//
	
	if(lcd_width > width)
		lcd_w_pos = (lcd_width - width)/2;


	// lcd image offset
	if(bits_per_pixel == 32)
	{
		pLCDC_HDMI_FB_CH->LIO = img_width *4;
		ifmt = TCC_LCDC_IMG_FMT_RGB888;
	}
	else
	{
		pLCDC_HDMI_FB_CH->LIO = img_width * 2;
		ifmt = TCC_LCDC_IMG_FMT_RGB565;
	}

// position
	pLCDC_HDMI_FB_CH->LIP = ((lcd_h_pos << 16) | (lcd_w_pos));
// scale
	pLCDC_HDMI_FB_CH->LISR =((scale_y<<4) | scale_x);
// size
	pLCDC_HDMI_FB_CH->LIS =((img_height << 16) | (img_width));

// image address
	pLCDC_HDMI_FB_CH->LIBA0 = addr;

//channel control
	BITCSET (pLCDC_HDMI_FB_CH->LIKR, 0xff <<  0, (chromaR)  <<  0); // key
	BITCSET (pLCDC_HDMI_FB_CH->LIKR, 0xff << 16, (0xF8)	<< 16); // keymask
	BITCSET (pLCDC_HDMI_FB_CH->LIKG, 0xff <<  0, (chromaG)  <<  0); // key
	BITCSET (pLCDC_HDMI_FB_CH->LIKG, 0xff << 16, (0xFC)	<< 16); // keymask
	BITCSET (pLCDC_HDMI_FB_CH->LIKB, 0xff <<  0, (chromaB)  <<  0); // key
	BITCSET (pLCDC_HDMI_FB_CH->LIKB, 0xff << 16, (0xF8)	<< 16); // keymask
	pLCDC_HDMI_FB_CH->LIC |= HwLIC_IEN | ifmt | HwLIC_CEN;

	pLCDC_HDMI->LCTRL |= (HwLCTRL_LEN | /* HwLCTRL_MSEL|*/ (Hw3+Hw1));

	pLCDC_HDMI->LSTATUS |= HwLSTATUS_RU;

#endif//
#endif//
	}

	return 1;
}



void TCC_HDMI_FB_update_sync(void)
{
#if defined(CONFIG_TCC_HDMI_UI_SIZE_1280_720)
    unsigned int ui_width = HDMI_DIRECT_OUTPUT_WIDTH, ui_height = HDMI_DIRECT_OUTPUT_HEIGHT;
	unsigned int lcd_width = 0, lcd_height = 0, regl = 0;
#endif
	PLCDC_CHANNEL pLCDC_HDMI_FB_CH;

	if(pLCDC_HDMI == NULL)
	{
		printk("%s - Err: HDMI LCDC is not valid\n", __func__);
		return;
	}

	if(hdmi_lcdc)
		pLCDC_HDMI_FB_CH = (volatile PLCDC_CHANNEL)tcc_p2v(pLCDC_HDMI->LI2C);
	else
		pLCDC_HDMI_FB_CH = (volatile PLCDC_CHANNEL)tcc_p2v(pLCDC_HDMI->LI0C);

#if defined(CONFIG_TCC_HDMI_UI_SIZE_1280_720)
	lcd_width = (regl & 0xFFFF);
	lcd_height = ((regl>>16) & 0xFFFF);

        if(tcc_display_data.resolution == 0)
        {
                ui_width = 1280;
                ui_height = 720;
        }
        else if(tcc_display_data.resolution == 1)
        {
                ui_width = 720;
                ui_height = 576;
        }
        else if(tcc_display_data.resolution == 2)
        {
                ui_width = 800;
                ui_height = 480;
        }

	if(lcd_width != ui_width && lcd_height != ui_height)
	{
		pLCDC_HDMI_FB_CH->LIBA0 = FBimg_buf_addr;
	}
#else
	pLCDC_HDMI_FB_CH->LIBA0 = FBimg_buf_addr;
#endif
}

void TCC_HDMI_FB_Set_AlphaBlending(void)
{
	PLCDC_CHANNEL pLCDC_HDMI_FB_CH;

	if(hdmi_lcdc)
		pLCDC_HDMI_FB_CH = (volatile PLCDC_CHANNEL)tcc_p2v(pLCDC_HDMI->LI2C);
	else
		pLCDC_HDMI_FB_CH = (volatile PLCDC_CHANNEL)tcc_p2v(pLCDC_HDMI->LI0C);		

	// LCDC1 Alpha enable
	BITSET(pLCDC_HDMI_FB_CH->LIC, HwLIC_ASEL);
	BITSET(pLCDC_HDMI_FB_CH->LIC, HwLIC_AEN);
	
	pLCDC_HDMI->LSTATUS |= HwLSTATUS_RU;
}




