/* 
 * linux/drivers/char/tcc250_pmic.c
 *
 * Author:  <linux@telechips.com>
 * Created: 10th Jun, 2008 
 * Description: POWER MANAGEMENT IC DRIVER
 *
 * Copyright (c) Telechips, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <linux/init.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/moduleparam.h>

#include <linux/fs.h>
#include <linux/errno.h>
#include <linux/types.h>
#include <linux/fcntl.h>
#include <linux/delay.h>
#include <linux/i2c.h>

#include <asm/uaccess.h>
#include <asm/io.h>

#include <asm/arch/tcc83x_virt.h>

#define PMU_DEV_NAME	"pmu"
#define PMU_DEV_MAJOR	245

#define IOCTL_DRIVER_REVSTR	_IO(PMU_DEV_MAJOR, 1)
#define IOCTL_PMU_SETINIT	_IO(PMU_DEV_MAJOR, 2)
#define IOCTL_PMU_SETCORE	_IO(PMU_DEV_MAJOR, 3)
#define IOCTL_PMU_SETIO		_IO(PMU_DEV_MAJOR, 4)
#define IOCTL_PMU_SETMEM	_IO(PMU_DEV_MAJOR, 5)
#define IOCTL_PMU_SETSWLED	_IO(PMU_DEV_MAJOR, 6)
#define IOCTL_PMU_GETREG	_IO(PMU_DEV_MAJOR, 7)
#define IOCTL_PMU_SETREG	_IO(PMU_DEV_MAJOR, 8)
#define IOCTL_PMU_INITCHG	_IO(PMU_DEV_MAJOR, 9)

/* DEFINE */
#define _TCC250_PWM_INCLUDE 
#define _TCC250_DVS_INCLUDE

/**************************************	
Power Management Unit Register Mapping
***************************************/

//#define	TCC250_ANA_CTRL_REG			0x20		//LDOANA Regulator Management	Read / Write	0x0C
#define	TCC250_MEM_CTRL_REG			0x21		//BUCK0 Management (if VMEMSEL = 0)	Read / Write	0xD4
													//BUCK0 Management (if VMEMSEL = NC)	Read / Write	0xE8
																
#define	TCC250_CORE_CTRL_REG		0x22		//BUCK1 Management	Read / Write	0xC8
#define	TCC250_IO_CTRL_REG			0x23		//BUCK2 Management (if VBUSOK = 0)	Read / Write	0xE8
													//BUCK2 Management (if VBUSOK = 1)	Read / Write	0xF2
																
#define	TCC250_SWLED_CTRL_REG		0x24		//SWLED BOOST Management	Read / Write	0x00

/***********************	
Charger Register Mapping
*************************/

#define	TCC250_CHARGSET_REG			0x2A		//Charger Functionalities	Read / Write	0x8F
#define	TCC250_CHARGSTATUS_REG		0x2B		//Charger Status	Read Only	0x00
#define	TCC250_BATTERY_SET_REG		0x2C		//Battery Configuration	Read / Write	0x05
	

/***********************
System Register Mapping
************************/
#define	TCC250_DVS_CTRL_REG			0x00	//Digital Voltage Scaling Management
#define	TCC250_SYSCLK_CTRL_REG		0X01	//System Management
#define	TCC250_A1_RST_REG			0X02	//A1 Power Off Management


/* drivers/i2c/chips/tcc250.c */
extern struct i2c_client *tcc250_i2c_client;

typedef struct _PMU_Param{
	unsigned char nType;      // 0 : I2C Read, 1 : I2C Write , 2 : nSetValue is used
	unsigned char nSubAddr;   // Sub Addr;
	unsigned char pBuffer;    // data
	unsigned short nSetValue; // nType 3
} PMU_Param;

enum{
	PMU_TYPE_I2C_WRITE = 0,
	PMU_TYPE_I2C_READ,
	PMU_TYPE_SETTING
};

void _PDelay(unsigned int count)
{
    volatile unsigned int i;
    for (i=0;i<count;i++);
}

void TCC250PMIC_SEND_CMD(unsigned char hb, unsigned char lb)
{
   unsigned char cmd[2];

   cmd[0] = hb;
   cmd[1] = lb;

   i2c_master_send(tcc250_i2c_client, cmd, 2);

   _PDelay(50000);   
}

unsigned char TCC250PMIC_READ_CMD(unsigned char hb)
{
	unsigned char data;

	i2c_master_send(tcc250_i2c_client, &hb, 1);
	i2c_master_recv(tcc250_i2c_client, &data, 1);

	_PDelay(50000);

	return data;
}

/*****************************************************************************
* Function Name : TCC250_SetToBuck0()
******************************************************************************
* Desription  	: Set to Buck0 Voltage for Memory VDD
* Parameter   	: mV   - Seeting Voltage [mV], Max 3300mV, Min 1600mV
*                 PFM  - 0 is DCDC in PWM Mode, 1 is DCDC in PFM Mode
*                 DVS  - Digital Voltage Scaling Enable
* Return      	: None
* Note          : 
******************************************************************************/
void TCC250_SetToBuck0(unsigned short mV)
{
	if(mV < 1600) mV = 1600;
	else if(mV > 2000) mV = 2000;

#ifdef _TCC250_PWM_INCLUDE 
		TCC250PMIC_SEND_CMD(TCC250_MEM_CTRL_REG,  0x80 + (mV / 50 - (800/50)));	// PWM
#else
		TCC250PMIC_SEND_CMD(TCC250_MEM_CTRL_REG,  0xC0 + (mV / 50 - (800/50)));	// PFM

#endif

#ifdef _TCC250_DVS_INCLUDE
		TCC250PMIC_SEND_CMD(TCC250_DVS_CTRL_REG,  0x55);						// DVS Mode
#else
		TCC250PMIC_SEND_CMD(TCC250_DVS_CTRL_REG,  0xAA);						// Normal Mode
#endif

}

unsigned short TCC250_GetVoltageOfBuck0(void)
{
	unsigned char	Reg;
	unsigned short	mV;
	
	Reg = TCC250PMIC_READ_CMD(TCC250_MEM_CTRL_REG);
	mV = ((Reg & 0x3f) + (800/50)) * 50;

	return mV;
}

/*****************************************************************************
* Function Name : TCC250_SetToBuck1()
******************************************************************************
* Desription  	: Set to Buck1 Voltage for Core VDD
* Parameter   	: mV   - Seeting Voltage [mV], Max 1600mV, Min 800mV
*                 PFM  - 0 is DCDC in PWM Mode, 1 is DCDC in PFM Mode
*                 DVS  - Digital Voltage Scaling Enable
* Return      	: None
* Note          : 
******************************************************************************/
void TCC250_SetToBuck1(unsigned short mV)
{
	if(mV <  800) mV =  800;
	else if(mV > 1600) mV = 1600;

#ifdef _TCC250_PWM_INCLUDE 
		TCC250PMIC_SEND_CMD(TCC250_CORE_CTRL_REG, 0x80 + (mV / 50 - (800/50)));	// PWM
#else
		TCC250PMIC_SEND_CMD(TCC250_CORE_CTRL_REG, 0xC0 + (mV / 50 - (800/50)));	// PFM
#endif

#ifdef _TCC250_DVS_INCLUDE
		TCC250PMIC_SEND_CMD(TCC250_DVS_CTRL_REG,  0x55);						// DVS Mode
#else
		TCC250PMIC_SEND_CMD(TCC250_DVS_CTRL_REG,  0xCC);						// Normal Mode
#endif

}

unsigned short TCC250_GetVoltageOfBuck1(void)
{
	unsigned char	Reg;
	unsigned short	mV;
	
	Reg = TCC250PMIC_READ_CMD(TCC250_CORE_CTRL_REG);
	mV = ((Reg & 0x3f) + (800/50)) * 50;

	return mV;
}

/*****************************************************************************
* Function Name : TCC250_SetToBuck2()
******************************************************************************
* Desription  	: Set to Buck2 Voltage for I/O VDD
* Parameter   	: mV   - Seeting Voltage [mV], Max 3600mV, Min 3300mV
*                 PFM  - 0 is DCDC in PWM Mode, 1 is DCDC in PFM Mode
*                 DVS  - Digital Voltage Scaling Enable
* Return      	: None
* Note          : 
******************************************************************************/
void TCC250_SetToBuck2(unsigned short mV)
{
	if(mV < 2600) mV = 2600;
	else if(mV > 3600) mV = 3600;
	
#ifdef _TCC250_PWM_INCLUDE 
		TCC250PMIC_SEND_CMD(TCC250_IO_CTRL_REG,   0x80 + (mV / 50 - (800/50)));	// PWM
#else
		TCC250PMIC_SEND_CMD(TCC250_IO_CTRL_REG,   0xC0 + (mV / 50 - (800/50)));	// PFM
#endif

#ifdef _TCC250_DVS_INCLUDE
		TCC250PMIC_SEND_CMD(TCC250_DVS_CTRL_REG,  0x55);						// DVS Mode
#else
		TCC250PMIC_SEND_CMD(TCC250_DVS_CTRL_REG,  0xCC);						// Normal Mode
#endif

}

unsigned short TCC250_GetVoltageOfBuck2(void)
{
	unsigned char	Reg;
	unsigned short	mV;
	
	Reg = TCC250PMIC_READ_CMD(TCC250_IO_CTRL_REG);
	mV = ((Reg & 0x3f) + (800/50)) * 50;

	return mV;
}

#if 0
/*****************************************************************************
* Function Name : TCC250_SetToLDOANA()
******************************************************************************
* Desription  	: Set to LDO_ANA Voltage for Audio Codec
* Parameter   	: mV   - Seeting Voltage [mV], Max 3600mV, Min 3300mV
*                 DVS  - Digital Voltage Scaling Enable
* Return      	: None
* Note          : 
******************************************************************************/
void TCC250_SetToLDOANA(unsigned short mV)
{
	if(mV < 2700) mV = 2700;
	else if(mV > 3300) mV = 3300;

#ifdef _TCC250_DVS_INCLUDE
		TCC250PMIC_SEND_CMD(TCC250_ANA_CTRL_REG,   ((mV / 50) - 54));	// DVS Mode
		TCC250PMIC_SEND_CMD(TCC250_ANA_CTRL_REG,   (0x10 + ((mV / 50) - 54)));	// DVS Mode
#else
		TCC250PMIC_SEND_CMD(TCC250_ANA_CTRL_REG,   ((mV / 50) - 54));	// Normal Mode
		TCC250PMIC_SEND_CMD(TCC250_ANA_CTRL_REG,   (0x20 + ((mV / 50) - 54)));	// Normal Mode
#endif
}
#endif

/*****************************************************************************
* Function Name : TCC250_SetToSWLED()
******************************************************************************
* Desription  	: Set to SWLED Voltage for White LED Drive
* Parameter   	: mA   - Seeting Current [mA], Max 30mA, Min 0 mA
*                 DCS  - Digital Current Scaling Enable
* Return      	: None
* Note          : DSC Mode should be disable for Camera Flash
******************************************************************************/
void TCC250_SetSWLEDInit(void)
{
		TCC250PMIC_SEND_CMD(TCC250_SWLED_CTRL_REG,   0x00);	// SWLED  
		TCC250PMIC_SEND_CMD(TCC250_DVS_CTRL_REG,   0x40);	//Digital Voltage Scaling Management
		TCC250PMIC_SEND_CMD(TCC250_SWLED_CTRL_REG,   0x40);	// SWLED 
		TCC250PMIC_SEND_CMD(TCC250_SWLED_CTRL_REG,   0x47);	// SWLED  
		TCC250PMIC_SEND_CMD(TCC250_DVS_CTRL_REG,   0x40);	//Digital Voltage Scaling Management

		TCC250PMIC_SEND_CMD(TCC250_SWLED_CTRL_REG,   0xC7);//Digital Voltage Scaling Management
}

void TCC250_SetSWLEDoff(void)
{
		TCC250PMIC_SEND_CMD(TCC250_SWLED_CTRL_REG,   0xC0);	// SWLED  
		TCC250PMIC_SEND_CMD(TCC250_DVS_CTRL_REG,   0x40);	//Digital Voltage Scaling Management
		TCC250PMIC_SEND_CMD(TCC250_SWLED_CTRL_REG,   0x00);	// SWLED 
}


void TCC250_SetToSWLED(unsigned char mA, unsigned char DCS)
{
	if (mA > 30) mA = 30;
	
	if(mA == 0)
	{
		TCC250PMIC_SEND_CMD(TCC250_SWLED_CTRL_REG,   0x00);	// SWLED Off
		return;
	}
	else
		TCC250PMIC_SEND_CMD(TCC250_SWLED_CTRL_REG,   0x40 + mA);


	if(DCS)
		TCC250PMIC_SEND_CMD(TCC250_DVS_CTRL_REG,     0x40);	// DCS Mode
	else
		TCC250PMIC_SEND_CMD(TCC250_DVS_CTRL_REG,     0x80);	// Normal Mode

}

void TCC250_SetProtectSWLED(unsigned char mA)
{
	if (mA > 30) mA = 30;
    
	if(mA == 0)
	{
		TCC250PMIC_SEND_CMD(TCC250_SWLED_CTRL_REG,   0x00);	// SWLED Off
		return;
	}
	else
    	TCC250PMIC_SEND_CMD(TCC250_SWLED_CTRL_REG,   0xC0 + mA);
}

unsigned int TCC250_GetToSWLED(void)
{
	unsigned char	Reg;
	
	Reg = TCC250PMIC_READ_CMD(TCC250_SWLED_CTRL_REG);

    return (unsigned int)Reg;
}

void TCC250_PMIC_Init(void)
{
//	TCC250PMIC_SEND_CMD(TCC250_RTC_LDO, 0x00);		// RTC_LDO Enable
//	TCC250_SetToLDOANA(2800);	// Audio Codec VDD is 2800mV

	TCC250_SetToBuck0(1800);	// 1800mV for Memory VDD
	TCC250_SetToBuck1(1500);	// 1200mV for Core VDD
	TCC250_SetToBuck2(2800);	// 2800mV for I/O VDD
}

void TCC250_Charger_Init(void)
{
	TCC250PMIC_SEND_CMD(TCC250_CHARGSET_REG,   0x8F);	
	TCC250PMIC_SEND_CMD(TCC250_BATTERY_SET_REG,   0x05);	//default - 400mA
}

static int tcc83x_pmu_ioctl(struct inode *inode, struct file *filp, 
							unsigned int cmd, unsigned long arg)
{
	int addr = 0;
	int data = 0;
	PMU_Param ioControlParm;

	copy_from_user((void *)&ioControlParm, (const void *)arg, sizeof(ioControlParm));

	switch (cmd) {
		case IOCTL_DRIVER_REVSTR:
			printk("DRIVER REVSTR IOCTL...\n");
			break;
		case IOCTL_PMU_SETINIT:
			printk("IOCTL_PMU_SETINIT...\n");
			TCC250_PMIC_Init();
			break;

		case IOCTL_PMU_SETCORE:
			printk("PMU_SETCORE:%d\n", ioControlParm.nSetValue);
			if(ioControlParm.nType == PMU_TYPE_SETTING)
				TCC250_SetToBuck1(ioControlParm.nSetValue);	// 1500mV for Core VDD			
			break;

		case IOCTL_PMU_SETIO:
			printk("PMU_SETIO:%d\n", ioControlParm.nSetValue);
			if(ioControlParm.nType == PMU_TYPE_SETTING)
				TCC250_SetToBuck2(ioControlParm.nSetValue);	// 2800mV for I/O VDD
			break;

		case IOCTL_PMU_SETMEM:
			printk("PMU_SETMEM:%d\n", ioControlParm.nSetValue);
			if(ioControlParm.nType == PMU_TYPE_SETTING)
				TCC250_SetToBuck0(ioControlParm.nSetValue);	// 1800mV for Memory VDD
			break;

		case IOCTL_PMU_SETSWLED:
			printk("PMU_SETSWLED:%d\n", ioControlParm.nSetValue);
			if(ioControlParm.nType == PMU_TYPE_SETTING)
				TCC250_SetToSWLED((unsigned char)ioControlParm.nSetValue,1);
			break;
		case IOCTL_PMU_GETREG:
			printk("IOCTL_PMU_GETREG: 0x%x\n", ioControlParm.nSubAddr);		
			if(ioControlParm.nType == PMU_TYPE_I2C_READ)
			{
				ioControlParm.pBuffer  = TCC250PMIC_READ_CMD(ioControlParm.nSubAddr);
				addr = (ioControlParm.nSubAddr);
				printk("RAddr: 0x%x, Data:0x%x\n",addr,ioControlParm.pBuffer);		
			}
			break;
		case IOCTL_PMU_SETREG:
			printk("IOCTL_PMU_SETREG...\n");		
			if(ioControlParm.nType == PMU_TYPE_I2C_WRITE)
			{
				TCC250PMIC_SEND_CMD(ioControlParm.nSubAddr,ioControlParm.pBuffer);
				addr = ioControlParm.nSubAddr;
				data = ioControlParm.pBuffer;
				printk("WAddr: 0x%x, data: 0x%x\n", addr, data);		
			}

			break;

		case IOCTL_PMU_INITCHG:
			printk("IOCTL_PMU_INITCHG...\n");		
			if(ioControlParm.nType == PMU_TYPE_SETTING)
				TCC250_Charger_Init();
			break;

		default:
			//TCC250_SetSWLEDInit();
			printk("default...\n");		
			break;
	}

	return 0;
}

static int tcc83x_pmu_release(struct inode *inode, struct file *filp)
{
	return 0;
}

int tcc83x_pmu_open(struct inode *inode, struct file *filp)
{
	return 0;
}

struct file_operations tcc83x_pmu_fops =
{
	.owner		= THIS_MODULE,
	.open		= tcc83x_pmu_open,
	.ioctl		= tcc83x_pmu_ioctl,
	.release	= tcc83x_pmu_release,
};

static int __init pmu_init(void)
{
	int ret;

	ret = register_chrdev(PMU_DEV_MAJOR, PMU_DEV_NAME, &tcc83x_pmu_fops);
	if (ret < 0)
		return ret;

	TCC250_PMIC_Init();

    printk("pmu: init\n");
    return 0;
}

static void __exit pmu_exit(void)
{
	unregister_chrdev(PMU_DEV_MAJOR, PMU_DEV_NAME);
    printk("pmu: exit\n");
}


module_init(pmu_init);
module_exit(pmu_exit);

MODULE_AUTHOR("Telechips Inc. SYS4-3 linux@telechips.com");
MODULE_DESCRIPTION("TCC83x PMU driver");
MODULE_LICENSE("GPL");

