/*
 *  linux/drivers/char/tcc79xx_vcore.c
 *
 *  Copyright (C) 2008  Telechips, Inc.
 */

#include <linux/module.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/init.h>
#include <linux/fs.h>
#include <linux/device.h>

static dev_t vcore_devno;

static int vcore_open(struct inode *inode, struct file *filp)
{
	return 0;
}

static int vcore_mmap(struct file *file, struct vm_area_struct *vma)
{
	size_t size = vma->vm_end - vma->vm_start;

	if (remap_pfn_range(vma,
			    vma->vm_start,
			    vma->vm_pgoff,
			    size,
			    vma->vm_page_prot))
		return -EAGAIN;
	return 0;
}

static const struct file_operations vcore_fops = {
	.open	= vcore_open,
	.mmap	= vcore_mmap,
};

static struct class *vcore_class;

int __init tcc79xx_vcore_init(void)
{
	int result;

	result = alloc_chrdev_region(&vcore_devno, 0, 1, "vcore");
	if (result < 0) {
		printk(KERN_ERR "can't get major id");
		return result;
	}

	register_chrdev(MAJOR(vcore_devno), "vcore", &vcore_fops);

	vcore_class = class_create(THIS_MODULE, "vcore");
	device_create(vcore_class, NULL,
		      MKDEV(MAJOR(vcore_devno), 0), NULL,
		      "vcore");

	printk(KERN_INFO "vcore: registered, major=%d\n",
	       MAJOR(vcore_devno));

	return 0;
}

void __exit tcc79xx_vcore_cleanup(void)
{
}

MODULE_AUTHOR("Telechips");
MODULE_DESCRIPTION("TCC79xx Vcore driver");
MODULE_LICENSE("GPL");

module_init(tcc79xx_vcore_init);
module_exit(tcc79xx_vcore_cleanup);
