/*
 * drivers/media/video/tcc78xx/camera_core.c
 *
 * Copyright (C) 2004 Texas Instruments, Inc. 
 *
 * Video-for-Linux (Version 2) camera capture driver for
 * the OMAP H2 and H3 camera controller.
 *
 * Adapted from omap24xx driver written by Andy Lowe (source@mvista.com)
 * Copyright (C) 2003-2004 MontaVista Software, Inc.
 * 
 * This package is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation. 
 * 
 * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR 
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED 
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE. 
 *
 * History:
 *   27/03/05   Vladimir Barinov - Added support for power management
 */
 
#include <linux/autoconf.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/init.h>
#include <linux/fs.h>
#include <linux/vmalloc.h>
#include <linux/slab.h>
#include <linux/proc_fs.h>
#include <linux/ctype.h>
#include <linux/pagemap.h>
#include <linux/mm.h>
#include <linux/platform_device.h>
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <linux/videodev.h>
#include <linux/pci.h>
#include <linux/version.h>
//#include <asm/semaphore.h>
#include <asm/processor.h>
#include <linux/dma-mapping.h>
#include <linux/fb.h>
#include <linux/clk.h>
#include <linux/cpufreq.h>
#include <linux/syscalls.h>

#include <linux/poll.h>

#include <asm/io.h>
#include <asm/byteorder.h>
#include <asm/irq.h>
#include <mach/bsp.h>

#include "tcc_jpeg.h"
#include "tcc_jpegenc_app.h"
#include "tcc_jpegdec_app.h"
#include <mach/tcc_scaler_ioctrl.h>

#if 0
static int debug	   = 1;
#else
static int debug	   = 0;
#endif

#define dprintk(msg...)	if (debug) { printk( "TCC_JPEG: " msg); }

JPEG_ENC_INFO  gJPEG_ENC_Info;
JPEG_DEC_INFO  gJPEG_DEC_Info;
JPEG_BUFFER_CTRL gJPEG_Buffer_Info;
JPEG_DISPLAY_INFO JPEG_Info;
jpeg_parser_rsp gJpegInfo;
extern volatile char gJpegDecodeDone;
extern volatile char gJpegDecodeError;
extern volatile char gJpegEncodeDone;
unsigned char gJpegCodecMode = JPEG_UNKNOWN_MODE;

void *jpeg_remapped_rawdata_baseaddr, *jpeg_remapped_streamdata_baseaddr, *jpeg_header_remapped_address;

static struct clk *vcore_clk;
static struct clk *vcache_clk;
static struct clk *jpegd_clk;
static struct clk *jpege_clk;

typedef struct _int_data_t {
	wait_queue_head_t wq;
	spinlock_t lock;
	unsigned int count;
	unsigned char irq_reged;
	unsigned char opened_cnt;
} int_data_t;
static int_data_t jpg_data;

extern struct tcc_freq_table_t gtJpegClockLimitTable[];
extern struct tcc_freq_table_t gtJpegMaxClockLimitTable;

#ifdef CONFIG_VIDEO_TCCXX_CAMERA
#define NUM_FREQS 2
static unsigned int cam_alive_size;

static unsigned int gtJpegSizeTable[NUM_FREQS] =
{
	720 * 480 , //D1,	
	1280 * 720, //HD720P,
};

extern int cif_get_capturedone(void);

static int Get_Index_sizeTable(unsigned int Image_Size)
{
	int i;

	for(i=0; i<NUM_FREQS; i++)
	{
		if( gtJpegSizeTable[i] >= Image_Size)
		{
			return i;
		}
	}
	
	return (NUM_FREQS -1);
}

int
tccxxx_isalive_camera(void)
{
	unsigned short HSize, VSize;
	volatile PCKC pCKC = (PCKC)tcc_p2v(HwCLK_BASE);
	volatile PCIF pCIF = (PCIF)tcc_p2v(HwCIF_BASE);

	if(pCKC->PCLK_CIFMC & Hw28)
	{
		if(pCIF->ICPCR1 & Hw31)
		{
			HSize = (pCIF->IIS >> 16) - (pCIF->IIW1 >> 16) - (pCIF->IIW1 & 0xFFFF);
			VSize = (pCIF->IIS & 0xFFFF) - (pCIF->IIW2 >> 16) - (pCIF->IIW2 & 0xFFFF);

			dprintk("JPEG - Cam Alive = %d x %d \n", HSize, VSize);
			return HSize * VSize;
		}
	}

	return 0;

}
EXPORT_SYMBOL(tccxxx_isalive_camera);
#endif

extern  void drop_pagecache(void);
int tccxxx_sync_player(int sync)
{
	if(sync)
	{
		sys_sync();
		drop_pagecache();
	}

	return 0;
}
EXPORT_SYMBOL(tccxxx_sync_player);

static int clk_usecount = 0;
static void tccxxx_set_clock(unsigned char reset)
{
#ifdef CONFIG_VIDEO_TCCXX_CAMERA
	int index = 0;

	clk_usecount++;

	if(reset)
	{
		if(cam_alive_size && cif_get_capturedone()) //capture¿ preview ƴϹǷ max-clock set!!
		{
			dprintk("encode for captured-data \n");
			if(clk_usecount > 100000)
			{
				tcc_cpufreq_set_limit_table(&gtJpegClockLimitTable[0], TCC_FREQ_LIMIT_JPEG, 0);
				mdelay(10);
			}
			tcc_cpufreq_set_limit_table(&gtJpegMaxClockLimitTable, TCC_FREQ_LIMIT_JPEG, 1);
		}
	}
	else
	{
		if((cam_alive_size = tccxxx_isalive_camera()) >0)
		{
			index = Get_Index_sizeTable(cam_alive_size);		
			tcc_cpufreq_set_limit_table(&gtJpegClockLimitTable[index], TCC_FREQ_LIMIT_JPEG, 1);
		}
		else
		{
			tcc_cpufreq_set_limit_table(&gtJpegMaxClockLimitTable, TCC_FREQ_LIMIT_JPEG, 1);
		}
	}
#else
	tcc_cpufreq_set_limit_table(&gtJpegMaxClockLimitTable, TCC_FREQ_LIMIT_JPEG, 1);
#endif
}

static void tccxxx_reset_clock(void)
{
	tcc_cpufreq_set_limit_table(&gtJpegClockLimitTable[0], TCC_FREQ_LIMIT_JPEG, 0);
}


static int
tccxxx_jpeg_mmap(struct file *file, struct vm_area_struct *vma)
{
	vma->vm_page_prot = pgprot_noncached(vma->vm_page_prot);
	if(remap_pfn_range(vma,vma->vm_start, vma->vm_pgoff , vma->vm_end - vma->vm_start, vma->vm_page_prot))
	{
		return -EAGAIN;
	}

	vma->vm_ops		= NULL;
	vma->vm_flags 	|= VM_IO;
	vma->vm_flags 	|= VM_RESERVED;

	return 0;
}

int
tccxxx_jpeg_encode_init(void *phy_src, int ImageWidth, int ImageHeight, enum jpeg_quantisation_val val, EncodeInputType type)
{
	unsigned int uiOutputBufferSize;
	void *BufferY, *BufferU, *BufferV;

	JPEG_Exif_Header_Info_Init();

	BufferY = (void*)phy_src;
	BufferU = (void*)GET_ADDR_YUV42X_spU(BufferY, ImageWidth, ImageHeight);
	if(type == ENC_INPUT_422)
		BufferV = (void*)GET_ADDR_YUV422_spV(BufferU, ImageWidth, ImageHeight);		
	else	
		BufferV = (void*)GET_ADDR_YUV420_spV(BufferU, ImageWidth, ImageHeight);

	
	JPEG_BUS_Enable(JPEG_ENCODE_MODE);
	JPEG_SW_Reset(JPEG_ENCODE_MODE);

	uiOutputBufferSize = (gJPEG_Buffer_Info.pBaseBitstreamDataSize+1023)/1024;

	gJPEG_ENC_Info.is_rolling = FALSE;
	gJPEG_ENC_Info.y_addr = (unsigned int)BufferY;
	gJPEG_ENC_Info.u_addr = (unsigned int)BufferU;
	gJPEG_ENC_Info.v_addr = (unsigned int)BufferV;
	gJPEG_ENC_Info.ifrm_hsize = ImageWidth;
	gJPEG_ENC_Info.ibuf_vsize = ImageHeight;
	gJPEG_ENC_Info.q_value = val;
	gJPEG_ENC_Info.img_hsize = ImageWidth;
	gJPEG_ENC_Info.img_vsize = ImageHeight;
	gJPEG_ENC_Info.cbuf_addr = (unsigned int)gJPEG_Buffer_Info.pBaseBitstreamDataAddr;
	gJPEG_ENC_Info.cbuf_size = uiOutputBufferSize;
	gJPEG_ENC_Info.frame_cnt = FRAME_LINE_CNT;

	if(type == ENC_INPUT_422)
		gJPEG_ENC_Info.chroma = HwJP_CHROMA_422;
	else
		gJPEG_ENC_Info.chroma = Hw31|HwJP_CHROMA_422;  //TCC92XX Encoding Input = YUV420/422 :: Output => Only YUV422 Format

	JPEG_SET_Encode_Config(&gJPEG_ENC_Info);

	JPEG_INTERRUPT_Enable(JPEG_ENCODE_MODE);

	return 0;
}

int
tccxxx_jpeg_decode_init(uint32 file_length, Decode_Scale_Type ratio, 
								uint32* ImageWidth, uint32* ImageHeight, 
								uint32* ImagePadWidth, uint32* ImagePadHeight, 
								uint32* ImageFormat)
{
	JPEG_PARSER_STATUS ucJpegParserStatus;
	unsigned int uiIFrmWidth, uiIBufHeight;
	unsigned int uiImageWidth, uiImageHeight;
	unsigned int ucIsPadding, uiDecodeScale;
	unsigned int usPaddingWidth, usPaddingHeight;

	void *source_physical_address, *target_physical_address;
	void *source_remapped_address, *target_remapped_address;
	source_physical_address = gJPEG_Buffer_Info.pBaseBitstreamDataAddr;
	target_physical_address = gJPEG_Buffer_Info.pBaseRawDataAddr;	
	source_remapped_address = jpeg_remapped_streamdata_baseaddr;
	target_remapped_address = jpeg_remapped_rawdata_baseaddr;	

	JPEG_BUS_Enable(JPEG_DECODE_MODE);
	JPEG_SW_Reset(JPEG_DECODE_MODE);
	
	JPEG_Info.JpegFileLength = file_length;
	uiDecodeScale = ratio;

	ucJpegParserStatus = JPEG_IMAGE_Parser(JPEG_Info.JpegFileLength, source_remapped_address, TRUE, &gJpegInfo);

	if (ucJpegParserStatus == JPEG_PARSER_SUCCESS) {
		if (gJpegInfo.ImgChromaType == IMAGE_CHROMA_422S)  //YUV422S output format is YUV420 at TCC92XX.
			gJpegInfo.ImgChromaType = IMAGE_CHROMA_420;
		
		dprintk("source_virtual = 0x%x, HeaderSize = 0x%x \n", (unsigned int)source_remapped_address,  gJpegInfo.HeaderSize);
		memcpy(source_remapped_address, (void*)((unsigned int)source_remapped_address+gJpegInfo.HeaderSize), gJpegInfo.BitStreamSize);
	}
	else {
		return -1;
	}

	if (gJpegInfo.IsNeedsPadding == TRUE) {
		uiImageWidth = gJpegInfo.PaddingWidth;
		uiImageHeight = gJpegInfo.PaddingHeight;
	}
	else {
		uiImageWidth = gJpegInfo.ImageWidth;
		uiImageHeight = gJpegInfo.ImageHeight;
	}

	//uiDecodeScale = JPEG_Dec_Scale_Set(uiImageWidth, uiImageHeight, gJpegInfo.ImgChromaType);
	uiIFrmWidth = JPEG_Dec_IFrame_Width(uiDecodeScale, uiImageWidth);
	uiIBufHeight = JPEG_Dec_IBuf_Height(uiDecodeScale, uiImageHeight);
	ImagePaddingForDecode(gJpegInfo.ImgChromaType, uiIFrmWidth, uiIBufHeight, &ucIsPadding, &usPaddingWidth, &usPaddingHeight);

	if (ucIsPadding) {
		gJpegInfo.IsNeedsPadding = TRUE;
		uiIFrmWidth = usPaddingWidth;
		uiIBufHeight = usPaddingHeight;
	}

	if(uiDecodeScale == 1){
		*ImageWidth = gJpegInfo.ImageWidth/2;
		*ImageHeight = gJpegInfo.ImageHeight/2;
	}
	else if(uiDecodeScale == 2){
		*ImageWidth = gJpegInfo.ImageWidth/4;
		*ImageHeight = gJpegInfo.ImageHeight/4;
	}
	else{
		*ImageWidth = gJpegInfo.ImageWidth;
		*ImageHeight = gJpegInfo.ImageHeight;
	}

	*ImagePadWidth = uiIFrmWidth;
	*ImagePadHeight = uiIBufHeight;

	dprintk("Tar = %d x %d, Chroma = %d \n", gJpegInfo.ImageWidth, gJpegInfo.ImageHeight, gJpegInfo.ImgChromaType);
	dprintk("Pad = %d x %d \n", uiIFrmWidth, uiIBufHeight);
	
	if (gJpegInfo.ImgChromaType == HwJP_CHROMA_420)
		*ImageFormat  = HwJP_CHROMA_420;
	else
		*ImageFormat  = HwJP_CHROMA_422;

	gJPEG_DEC_Info.lcd_buffer = target_physical_address;
	gJPEG_DEC_Info.cbuf_addr = (unsigned int)source_physical_address;

	gJPEG_DEC_Info.cbuf_size = (gJPEG_Buffer_Info.pBaseBitstreamDataSize + 1023)/1024;
	gJPEG_DEC_Info.scale = uiDecodeScale;
	gJPEG_DEC_Info.ifrm_width = uiIFrmWidth;
	gJPEG_DEC_Info.ibuf_height = uiIBufHeight;
	gJPEG_DEC_Info.ImageFormat = gJpegInfo.ImgChromaType;

	JPEG_SET_Decode_Config(&gJPEG_DEC_Info);

	if(gJPEG_DEC_Info.ImageFormat == IMAGE_CHROMA_YONLY)
	{
		unsigned int Ydec, Udec, Vdec;
		
		Ydec = (unsigned int)target_remapped_address;
		Udec = GET_ADDR_YUV42X_spU(Ydec , uiIFrmWidth, uiIBufHeight );
		Vdec = GET_ADDR_YUV422_spV(Udec , uiIFrmWidth, uiIBufHeight );

		memset((char*)Udec, 0x80, uiIFrmWidth*uiIBufHeight/2);
		memset((char*)Vdec, 0x80, uiIFrmWidth*uiIBufHeight/2);
	}
	
	JPEG_INTERRUPT_Enable(JPEG_DECODE_MODE);

	return 0;
}

int tccxxx_jpeg_meminit(void *data, unsigned char isEnc)
{
	TCCXXX_JPEG_ENC_DATA *Enc_data;
	TCCXXX_JPEG_DEC_DATA *Dec_data;
	unsigned int raw_phy_addr, raw_size;
	unsigned int stream_phy_addr, stream_size;

	if(jpeg_remapped_rawdata_baseaddr != NULL)
		iounmap(jpeg_remapped_rawdata_baseaddr);

	if(jpeg_remapped_streamdata_baseaddr != NULL)
		iounmap(jpeg_remapped_streamdata_baseaddr);
	
	if(isEnc == TRUE)
	{
		Enc_data = (TCCXXX_JPEG_ENC_DATA *)data;
		
		raw_phy_addr = Enc_data->source_addr;
		if(Enc_data->src_format == ENC_INPUT_422)
			raw_size = Enc_data->width * Enc_data->height * 2;
		else
			raw_size = Enc_data->width * Enc_data->height * 3 / 2;

		stream_phy_addr = Enc_data->target_addr;
		stream_size = Enc_data->target_size;	
	}
	else
	{
		Dec_data = (TCCXXX_JPEG_DEC_DATA *)data;

		//Target is Raw-data in case of Decode.
		raw_phy_addr = Dec_data->target_addr;
		raw_size = Dec_data->target_size;

		//Source is Stream-data in case of Decode.
		stream_phy_addr = Dec_data->source_addr;
		stream_size = Dec_data->file_length;
	}

	jpeg_remapped_rawdata_baseaddr = (void *)ioremap_nocache(raw_phy_addr, PAGE_ALIGN(raw_size/*-PAGE_SIZE*/));
	if (jpeg_remapped_rawdata_baseaddr == NULL) {
		printk(KERN_ALERT "[raw] can not remap for jpeg\n");
		return -EFAULT;
	}	

	jpeg_remapped_streamdata_baseaddr = (void *)ioremap_nocache(stream_phy_addr, PAGE_ALIGN(stream_size/*-PAGE_SIZE*/));
	if (jpeg_remapped_streamdata_baseaddr == NULL) {
		printk(KERN_ALERT "[stream] can not remap for jpeg\n");
		return -EFAULT;
	}
	jpeg_header_remapped_address = jpeg_remapped_streamdata_baseaddr + stream_size-TCC_JPEG_HEADER_SIZE;
	
	dprintk("raw 	:: phy[0x%x ~ 0x%x], virt[0x%x ~ 0x%x], size = 0x%x \n",raw_phy_addr, raw_phy_addr+raw_size, (unsigned int)jpeg_remapped_rawdata_baseaddr, (unsigned int)jpeg_remapped_rawdata_baseaddr+raw_size, raw_size);
	dprintk("stream  :: phy[0x%x ~ 0x%x], virt[0x%x ~ 0x%x], size = 0x%x \n",stream_phy_addr, stream_phy_addr+stream_size, (unsigned int)jpeg_remapped_streamdata_baseaddr, (unsigned int)jpeg_remapped_streamdata_baseaddr+stream_size, stream_size);
	dprintk("header  :: phy[0x%x ~ 0x%x], virt[0x%x ~ 0x%x], size = 0x%x \n",stream_phy_addr + stream_size-TCC_JPEG_HEADER_SIZE, stream_phy_addr + stream_size, (unsigned int)jpeg_header_remapped_address, (unsigned int)jpeg_header_remapped_address + TCC_JPEG_HEADER_SIZE, TCC_JPEG_HEADER_SIZE);

	gJPEG_Buffer_Info.pBaseRawDataAddr 			= (void*)raw_phy_addr;
	gJPEG_Buffer_Info.pBaseRawDataSize 			= raw_size;
	gJPEG_Buffer_Info.pBaseBitstreamDataAddr 	= (void*)stream_phy_addr;
	gJPEG_Buffer_Info.pBaseBitstreamDataSize 	= stream_size;
	gJPEG_Buffer_Info.pBaseHeaderDataAddr 		= (void*)(stream_phy_addr + stream_size-TCC_JPEG_HEADER_SIZE);
	gJPEG_Buffer_Info.pBaseHeaderDataSize 		= TCC_JPEG_HEADER_SIZE;

	return 0;
}

int tccxxx_jpeg_exifheader_meminit(void *data)
{
	TCCXXX_JPEG_ENC_DATA *Enc_data;
	unsigned int base_phy_addr, total_size;

	if(jpeg_remapped_streamdata_baseaddr != NULL)
		iounmap(jpeg_remapped_streamdata_baseaddr);
	
	Enc_data = (TCCXXX_JPEG_ENC_DATA *)data;
	
	base_phy_addr = Enc_data->target_addr;
	total_size = Enc_data->header_offset + TCC_JPEG_HEADER_SIZE;	

	jpeg_remapped_streamdata_baseaddr = (void *)ioremap_nocache(base_phy_addr, PAGE_ALIGN(total_size));
	if (jpeg_remapped_streamdata_baseaddr == NULL) {
		printk(KERN_ALERT "[stream] can not remap for jpeg\n");
		return -EFAULT;
	}
	jpeg_header_remapped_address = jpeg_remapped_streamdata_baseaddr + total_size - TCC_JPEG_HEADER_SIZE;
	
	dprintk("exifHeader	:: phy[0x%x ~ 0x%x], virt[0x%x ~ 0x%x], size = 0x%x \n",base_phy_addr, base_phy_addr+total_size, (unsigned int)jpeg_remapped_streamdata_baseaddr, (unsigned int)jpeg_remapped_streamdata_baseaddr+total_size, total_size);

	gJPEG_Buffer_Info.pBaseRawDataAddr 			= (void*)base_phy_addr + Enc_data->thumb_offset;
	gJPEG_Buffer_Info.pBaseRawDataSize 			= Enc_data->thumb_size;
	gJPEG_Buffer_Info.pBaseBitstreamDataAddr 	= (void*)base_phy_addr + Enc_data->thumb_offset + Enc_data->thumb_size;
	gJPEG_Buffer_Info.pBaseBitstreamDataSize 	= Enc_data->header_offset - Enc_data->thumb_offset - Enc_data->thumb_size;
	gJPEG_Buffer_Info.pBaseHeaderDataAddr 		= (void*)base_phy_addr + Enc_data->header_offset;
	gJPEG_Buffer_Info.pBaseHeaderDataSize 		= TCC_JPEG_HEADER_SIZE;

	return 0;
}


int tccxxx_jpeg_encode(TCCXXX_JPEG_ENC_DATA *arg)
{
	TCCXXX_JPEG_ENC_DATA enc_data;
	void *HeaderBuffer;
	int ret = 0;

	if(copy_from_user(&enc_data, arg, sizeof(TCCXXX_JPEG_ENC_DATA)))
		return -EFAULT;

	if(enc_data.normal_op && jpg_data.opened_cnt > 1)
	{
		dprintk("JpegDevice(Enc) is already using \n");
		return -EFAULT;
	}

#ifdef CONFIG_VIDEO_TCCXX_CAMERA
	tccxxx_set_clock(1);
#endif

	ret = tccxxx_jpeg_meminit(&enc_data, TRUE);
	dprintk("src: 0x%x, size: %d x %d !!\n", enc_data.source_addr, enc_data.width, enc_data.height);

	if(ret >= 0)
	{
		tccxxx_jpeg_encode_init((void*)enc_data.source_addr, enc_data.width, enc_data.height, JPEG_SUPERFINE, enc_data.src_format);
		if((ret = JPEG_ENCODE_Start(enc_data.width, enc_data.height, &(enc_data.bitstream_size), &(enc_data.header_size))) >= 0)
		{
			if(EXIF_THUMBNAIL_MAKE_START_WIDTH < enc_data.width && EXIF_THUMBNAIL_MAKE_START_HEIGHT < enc_data.height){
				gEncodeOption.IsThumbnail = TRUE;
				JPEG_DECODE_For_Thumbnail(enc_data.header_size);
				JPEG_ENCODE_For_Thumbnail();
				ret = JPEG_ENCODE_Start(EXIF_THUMBNAIL_WIDTH, EXIF_THUMBNAIL_HEIGHT, &(enc_data.bitstream_size), &(enc_data.header_size));
			}

			//dprintk("Stream = 0x%x, Header = 0x%x \n", enc_data.bitstream_size, enc_data.header_size);

			if(ret >= 0)
			{
				HeaderBuffer = jpeg_header_remapped_address;
				memcpy(jpeg_remapped_streamdata_baseaddr+enc_data.bitstream_size, HeaderBuffer, enc_data.header_size);

				enc_data.thumb_size = 0; //thumb is in header!!
				enc_data.bitstream_offset = 0;
				enc_data.header_offset = enc_data.bitstream_size;
			}
		}
		JPEG_BUS_Disable(JPEG_ENCODE_MODE);		
	}
	dprintk("Stream = 0x%x, Header = 0x%x \n", enc_data.bitstream_size, enc_data.header_size);
	if (copy_to_user(arg, &enc_data, sizeof(TCCXXX_JPEG_ENC_DATA)))
		return -EFAULT;

#ifdef CONFIG_VIDEO_TCCXX_CAMERA
	tccxxx_set_clock(0);
#endif

	return ret;
}

int tccxxx_jpeg_decode(TCCXXX_JPEG_DEC_DATA *arg)
{
	TCCXXX_JPEG_DEC_DATA dec_data;
	unsigned char ratio;
	int ret = 0;

	if(copy_from_user(&dec_data, arg, sizeof(TCCXXX_JPEG_DEC_DATA)))
		return -EFAULT;

	if(dec_data.normal_op && jpg_data.opened_cnt > 1)
	{
		dprintk("JpegDevice(Dec) is already using \n");
		return -EFAULT;
	}

	ret = tccxxx_jpeg_meminit(&dec_data, FALSE);

	if(ret >= 0)
	{
		dec_data.image_format=0;
		ratio = dec_data.ratio;
		
		if(0 > tccxxx_jpeg_decode_init(dec_data.file_length, ratio, 
									&(dec_data.width), &(dec_data.height), 
									&(dec_data.pad_width), &(dec_data.pad_height),
									&(dec_data.image_format))
		)
		{
			dprintk(KERN_WARNING "JPEG DECODER ERROR\r\n");
			ret = -1;
		}
		else
		{
			dprintk(" Result: %dx%d [%dx%d], fmt: %d \n", dec_data.width, dec_data.height, dec_data.pad_width, dec_data.pad_height, dec_data.image_format);

			if(JPEG_DECODE_Start(JPEG_DECODE_MODE) < 0)
				ret = -1;
		}

#ifndef TCC_JPEGDEC_INT_POLL
		JPEG_BUS_Disable(JPEG_DECODE_MODE);		
#endif
	}
	if(copy_to_user(arg, &dec_data, sizeof(TCCXXX_JPEG_DEC_DATA)))
		return -EFAULT;
	
	return ret;
}

int tccxxx_jpeg_get_headerInfo(TCCXXX_JPEG_DEC_DATA *arg)
{
	TCCXXX_JPEG_DEC_DATA dec_data;
	JPEG_PARSER_STATUS ucJpegParserStatus;
	void *source_remapped_address;
	int ret = 0;

	if(copy_from_user(&dec_data, arg, sizeof(TCCXXX_JPEG_DEC_DATA)))
		return -EFAULT;

	ret = tccxxx_jpeg_meminit(&dec_data, FALSE);

	if(ret >= 0)
	{
		source_remapped_address = jpeg_remapped_streamdata_baseaddr;

		dprintk(" Check JPEG [addr: 0x%x, len: %d]===========> \n", (uint32)jpeg_remapped_streamdata_baseaddr, dec_data.file_length);

		JPEG_BUS_Enable(JPEG_DECODE_MODE);		
		JPEG_SW_Reset(JPEG_DECODE_MODE);

		ucJpegParserStatus = JPEG_IMAGE_Parser(dec_data.file_length, source_remapped_address, TRUE, &gJpegInfo);

		JPEG_BUS_Disable(JPEG_DECODE_MODE);		

		if (ucJpegParserStatus == JPEG_PARSER_SUCCESS)
		{
			dec_data.width 	 = gJpegInfo.ImageWidth;
			dec_data.height	 = gJpegInfo.ImageHeight;
			dec_data.pad_width  = gJpegInfo.PaddingWidth;
			dec_data.pad_height = gJpegInfo.PaddingHeight;

			if(dec_data.pad_width > 4095 || dec_data.pad_height > 4095)
			{
				printk("[size over]:: Impossible TCC H/W -Decode !!\n");
				ret = -1;
			}

			if(dec_data.width == 0 || dec_data.height == 0)
			{
				printk("[size 0x0]:: Impossible TCC H/W -Decode !!\n");
				ret = -1;
			}
			
			dprintk("Possible TCC-Decode - size: %dx%d !!\n", gJpegInfo.ImageWidth, gJpegInfo.ImageHeight);
		}	
		else
		{
			printk("[%d]:: Impossible TCC H/W -Decode !!\n", ucJpegParserStatus);
			ret = -1;
		}
	}
	if(copy_to_user(arg, &dec_data, sizeof(TCCXXX_JPEG_DEC_DATA)))
		return -EFAULT;
	
	return ret;
}

int tccxxx_jpeg_make_exifheader(TCCXXX_JPEG_ENC_DATA *arg)
{
	TCCXXX_JPEG_ENC_DATA enc_data;
	int ret = 0;

	if(copy_from_user(&enc_data, arg, sizeof(TCCXXX_JPEG_ENC_DATA)))
		return -EFAULT;

	ret = tccxxx_jpeg_exifheader_meminit(&enc_data);
	dprintk("src: 0x%x, size: %d x %d !!\n", enc_data.source_addr, enc_data.width, enc_data.height);

	if(ret >= 0)
	{	
		if(enc_data.thumb_size > 64*1024) //thumb is not jpeg but yuv. so we need to encode.
		{		
			gEncodeOption.ThumbnailInfo.thumbnail_width  = EXIF_THUMBNAIL_WIDTH;
			gEncodeOption.ThumbnailInfo.thumbnail_height = EXIF_THUMBNAIL_HEIGHT;
			JPEG_RESIZE_For_Thumbnail(EXIF_THUMBNAIL_WIDTH, EXIF_THUMBNAIL_HEIGHT, SCALER_YUV422_sq0);			
			gJPEG_Buffer_Info.pBaseRawDataAddr			+= enc_data.thumb_size;
			gJPEG_Buffer_Info.pBaseBitstreamDataAddr	+= enc_data.thumb_size;
			
			tccxxx_jpeg_encode_init((void*)gJPEG_Buffer_Info.pBaseRawDataAddr, EXIF_THUMBNAIL_WIDTH, EXIF_THUMBNAIL_HEIGHT, JPEG_SUPERFINE, ENC_INPUT_422);
			JPEG_THUMB_ENCODE_Start(EXIF_THUMBNAIL_WIDTH, EXIF_THUMBNAIL_HEIGHT, &(enc_data.thumb_size));

			enc_data.thumb_offset += (EXIF_THUMBNAIL_WIDTH * EXIF_THUMBNAIL_HEIGHT * 2 * 2);
		}

		JPEG_Make_ExifHeader(&enc_data, (unsigned int)jpeg_remapped_streamdata_baseaddr);
		enc_data.thumb_size = 0; //thumb is in header!!
	}
	dprintk("Stream = 0x%x, Header = 0x%x \n", enc_data.bitstream_size, enc_data.header_size);
	if(copy_to_user(arg, &enc_data, sizeof(TCCXXX_JPEG_ENC_DATA)))
		return -EFAULT;

	return ret;
}

static int
tccxxx_jpeg_ioctl(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg)
{
	switch (cmd)
	{
		case TCC_JPEG_IOCTL_ENC:
		case TCC_JPEG_IOCTL_V4L2_ENC:
			return tccxxx_jpeg_encode((TCCXXX_JPEG_ENC_DATA *)arg);

		case TCC_JPEG_IOCTL_DEC:
			return tccxxx_jpeg_decode((TCCXXX_JPEG_DEC_DATA *)arg);

		case TCC_JPEG_IOCTL_HEADER_INFO:
			return tccxxx_jpeg_get_headerInfo((TCCXXX_JPEG_DEC_DATA *)arg);
			
		case TCC_JPEG_IOCTL:
			break;

		case TCC_JPEG_IOCTL_MAKE_EXIFHEADER:
			return tccxxx_jpeg_make_exifheader((TCCXXX_JPEG_ENC_DATA *)arg);
			break;
			
		default:
			printk(KERN_ALERT "JPEG_IOCTL :: Error \n");
			break;
	}

	return 0;
}

// interrupt handler
/****************************************************************************
	IRQ_JPEG_ENC_Handler()

	Description : 
	Parameter : NONE
	Return : NONE
****************************************************************************/
static irqreturn_t tccxxx_jpeg_enc_handler(int irq, void *client_data)
{
	unsigned long IFlag, TempFlag;
	volatile PJPEGENCODER pJPEGENC = (PJPEGENCODER)tcc_p2v(HwJPEGENCODER_BASE);

	IFlag = pJPEGENC->JP_INT_FLAG;

	if(IFlag & HwJP_INT_FLAG_JOB_FINISHED)
	{
		TempFlag = pJPEGENC->JP_INT_ACK;
		JPEG_INTERRUPT_Disable(JPEG_ENCODE_MODE);
		JPEG_VIDEOCACHE_Drain();
		
		if(gJpegCodecMode == JPEG_ENCODE_MODE)
		{
			gJpegEncodeDone = 1;
			dprintk(KERN_ALERT "JPEG ENCODING DONE \n");
		}
	}
	else if (IFlag & HwJP_INT_FLAG_CODED_BUF_STAT)
	{
		// Todo : 
	}

	TempFlag = pJPEGENC->JP_INT_ACK;

	return IRQ_HANDLED;
}


/****************************************************************************
	IRQ_JPEG_DEC_Handler()

	Description : 
	Parameter : NONE
	Return : NONE
****************************************************************************/
static irqreturn_t tccxxx_jpeg_dec_handler(int irq, void *client_data)
{
	unsigned long IFlag, TempFlag;
	int_data_t *jpg_intr = client_data;

#ifdef TCC_JPEGDEC_INT_POLL
	if(gJpegCodecMode == JPEG_DECODE_MODE)
	{
		spin_lock_irq(&(jpg_intr->lock));
		jpg_intr->count++;
		spin_unlock_irq(&(jpg_intr->lock));
	}
#endif

	IFlag = HwJPEGDEC->JP_INT_FLAG;

	if (IFlag & HwJP_INT_FLAG_DECODING_ERR)
	{
		printk(KERN_ALERT "@@@ JPEG DECODING ERROR!! \n");
		gJpegDecodeError = 1;
		TempFlag = HwJPEGDEC->JP_INT_ACK;
		JPEG_INTERRUPT_Disable(JPEG_DECODE_MODE);
		JPEG_VIDEOCACHE_Drain();
	}
	else if (IFlag & HwJP_INT_FLAG_JOB_FINISHED)
	{
		TempFlag = HwJPEGDEC->JP_INT_ACK;
		JPEG_INTERRUPT_Disable(JPEG_DECODE_MODE);
		JPEG_VIDEOCACHE_Drain();
		
		 if (gJpegCodecMode == JPEG_DECODE_MODE ||
		 	  gJpegCodecMode == JPEG_DECODE_THUMB_MODE)
		{
			gJpegDecodeDone = 1;	
			dprintk(KERN_ALERT "JPEG DECODING DONE. \n");
		}
	}

	TempFlag = HwJPEGDEC->JP_INT_ACK;
#ifdef TCC_JPEGDEC_INT_POLL
	if(gJpegCodecMode == JPEG_DECODE_MODE)
	{
		wake_up_interruptible(&(jpg_intr->wq));
	}
#endif

	return IRQ_HANDLED;
}

static int
tccxxx_jpeg_release(struct inode *inode, struct file *filp)
{
	dprintk("tccxxx_jpeg_release \n");

	jpg_data.opened_cnt--;

	if(!jpg_data.opened_cnt)
	{
		if(jpg_data.irq_reged)
		{
			jpg_data.irq_reged = 0;
			
			free_irq(IRQ_JPGE, NULL);
#ifdef TCC_JPEGDEC_INT_POLL
			free_irq(IRQ_JPGD, &jpg_data);
#else
			free_irq(IRQ_JPGD, NULL);
#endif
		}

		iounmap(jpeg_remapped_rawdata_baseaddr);
		iounmap(jpeg_remapped_streamdata_baseaddr);

		jpeg_remapped_rawdata_baseaddr = NULL;
		jpeg_remapped_streamdata_baseaddr = NULL;
	}

	tccxxx_reset_clock();

	clk_disable(jpege_clk);
	clk_disable(jpegd_clk);
	clk_disable(vcache_clk);
	clk_disable(vcore_clk);

	return 0;
}

static int
tccxxx_jpeg_open(struct inode *inode, struct file *filp)
{
	int ret = 0;

	dprintk("tccxxx_jpeg_open!!\n");
	tccxxx_sync_player(0);

	clk_usecount = 0;

	clk_enable(vcore_clk);
	clk_enable(vcache_clk);
	clk_enable(jpegd_clk);
	clk_enable(jpege_clk);

	tccxxx_set_clock(0);

	if(!jpg_data.irq_reged)
	{
		if((ret = request_irq(IRQ_JPGE, tccxxx_jpeg_enc_handler, IRQF_DISABLED, "jpeg_enc", NULL)) < 0)
		{
			printk("FAILED to aquire jpeg_enc-irq\n");
			ret = -EFAULT; goto error;
		}
			
		if((ret = request_irq(IRQ_JPGD, tccxxx_jpeg_dec_handler, IRQF_DISABLED, "jpeg_dec", &jpg_data)) < 0)
		{
			printk("FAILED to aquire jpeg_dec-irq\n");
			ret = -EFAULT; goto error;
		}

		jpg_data.irq_reged = 1;
	}
	
#if defined(TCC_JPEGDEC_INT_POLL)
	filp->private_data = &jpg_data;
#endif
	jpg_data.opened_cnt++;

	return 0;	

error:
	clk_disable(jpege_clk);
	clk_disable(jpegd_clk);
	clk_disable(vcache_clk);
	clk_disable(vcore_clk);
	
	return ret;	
	
}

static unsigned int tccxxx_jpeg_poll(struct file *filp, poll_table *wait)
{
#if defined(TCC_JPEGDEC_INT_POLL)
	int_data_t *jpg_intr = (int_data_t *)filp->private_data;

	if (jpg_intr->count > 0) 
	{
		spin_lock_irq(&(jpg_intr->lock));
		jpg_intr->count--;
		spin_unlock_irq(&(jpg_intr->lock));

		JPEG_BUS_Disable(JPEG_DECODE_MODE); 

		if(gJpegDecodeDone)
			return (POLLIN | POLLRDNORM);
		else
			return POLLERR;
	}
	
	poll_wait(filp, &(jpg_intr->wq), wait);

	if (jpg_intr->count > 0) 
	{
		spin_lock_irq(&(jpg_intr->lock));
		jpg_intr->count--;
		spin_unlock_irq(&(jpg_intr->lock));

		JPEG_BUS_Disable(JPEG_DECODE_MODE);	
		
		if(gJpegDecodeDone)
			return (POLLIN | POLLRDNORM);
		else
			return POLLERR;
	} 
	else 
	{
		return 0;
	}
#endif

	return 0;

}


static struct file_operations tccxxx_jpeg_fops = 
{
	.owner			= THIS_MODULE,
	.ioctl				= tccxxx_jpeg_ioctl,
	.mmap			= tccxxx_jpeg_mmap,
	.open			= tccxxx_jpeg_open,
	.release			= tccxxx_jpeg_release,
	.poll				= tccxxx_jpeg_poll,
};


#define TCC_JPEG_MAJOR_ID 198
#define TCC_JPEG_MINOR_ID 1
#define TCC_JPEG_DEVICE_NAME "jpeg"

void __exit
tccxxx_jpeg_cleanup(void)
{
	//̽  ϱ  ڵ
	unregister_chrdev(TCC_JPEG_MAJOR_ID, TCC_JPEG_DEVICE_NAME);

	clk_put(vcore_clk);
	clk_put(vcache_clk);
	clk_put(jpegd_clk);
	clk_put(jpege_clk);
	
	dprintk(" ===========> tcc7901_jpeg_cleanup \n");

	return;
}


static char banner[] __initdata = KERN_INFO "TCC83XX JPEG driver initializing\n";
static struct class *jpeg_class;

int __init 
tccxxx_jpeg_init(void)
{
	//̽ ϱ  ڵ
  
	printk(banner);

#if defined(TCC_JPEGDEC_INT_POLL)
	memset(&jpg_data, 0, sizeof(int_data_t));
	spin_lock_init(&(jpg_data.lock));
	init_waitqueue_head(&(jpg_data.wq));
#endif
	
	vcore_clk = clk_get(NULL, "vcore");
	BUG_ON(vcore_clk == NULL);
	
	vcache_clk = clk_get(NULL, "vcache");
	BUG_ON(vcache_clk == NULL);
	
	jpegd_clk = clk_get(NULL, "jpegd");
	BUG_ON(jpegd_clk == NULL);
	
	jpege_clk = clk_get(NULL, "jpege");
	BUG_ON(jpege_clk == NULL);
	
	register_chrdev(TCC_JPEG_MAJOR_ID, TCC_JPEG_DEVICE_NAME, &tccxxx_jpeg_fops);
	
	jpeg_class = class_create(THIS_MODULE, TCC_JPEG_DEVICE_NAME);
	device_create(jpeg_class,NULL,MKDEV(TCC_JPEG_MAJOR_ID,TCC_JPEG_MINOR_ID),NULL,TCC_JPEG_DEVICE_NAME);

	jpeg_remapped_rawdata_baseaddr = NULL;
	jpeg_remapped_streamdata_baseaddr = NULL;

	return 0;
}


MODULE_AUTHOR("Telechips.");
MODULE_DESCRIPTION("TCC83XX Video for Linux camera driver");
MODULE_LICENSE("GPL");


module_init(tccxxx_jpeg_init);
module_exit(tccxxx_jpeg_cleanup);

