/****************************************************************************
 *   FileName    :  tcc_tsif_parallel_hwset.c
 *   Description : 
 ****************************************************************************
 *
 *   TCC Version 1.0
 *   Copyright (c) Telechips, Inc.
 *   ALL RIGHTS RESERVED
 *
 ****************************************************************************/

#include <linux/module.h>
#include <mach/io.h>

#include <bsp.h>
#include <linux/tcc_tsif_parallel_hwset.h>
#include <mach/irqs.h>

static void tcc_tsif_parallel_gpio_debug(volatile PGPIO regs)
{
#if TSIF_USE_DEBUG
	printk(KERN_DEBUG "## GPEFN debug ################\n");
	printk(KERN_DEBUG "GPEFN0(0x%08x): 0x%08x\n", &regs->GPEFN0, regs->GPEFN0);
	printk(KERN_DEBUG "GPEFN1(0x%08x): 0x%08x\n", &regs->GPEFN1, regs->GPEFN1);
	printk(KERN_DEBUG "GPEFN2(0x%08x): 0x%08x\n", &regs->GPEFN2, regs->GPEFN2);
	printk(KERN_DEBUG "GPEFN3(0x%08x): 0x%08x\n", &regs->GPEFN3, regs->GPEFN3);
	printk(KERN_DEBUG "###############################\n");
#endif
}

static void tcc_tsif_parallel_dma_debug(volatile PGDMACTRL regs, int ch)
{
#if TSIF_USE_DEBUG
	printk(KERN_DEBUG "## DMA debug ##################\n");	
	if(ch == 0) {
		printk(KERN_DEBUG "ST_SADR0-%d(0x%08x): 0x%08x\n", ch, &regs->ST_SADR0, regs->ST_SADR0);
		printk(KERN_DEBUG "SPARAM0-%d (0x%08x): 0x%08x\n", ch, &regs->SPARAM0,  regs->SPARAM0);
		printk(KERN_DEBUG "ST_DADR0-%d(0x%08x): 0x%08x\n", ch, &regs->ST_DADR0, regs->ST_DADR0);
		printk(KERN_DEBUG "DPARAM0-%d (0x%08x): 0x%08x\n", ch, &regs->DPARAM0, 	regs->DPARAM0);
		printk(KERN_DEBUG "C_DADR0-%d (0x%08x): 0x%08x\n", ch, &regs->C_DADR0, 	regs->C_DADR0);
		printk(KERN_DEBUG "HCOUNT0-%d (0x%08x): 0x%08x\n", ch, &regs->HCOUNT0, 	regs->HCOUNT0);
		printk(KERN_DEBUG "CHCTRL0-%d (0x%08x): 0x%08x\n", ch, &regs->CHCTRL0, 	regs->CHCTRL0);
		printk(KERN_DEBUG "RPTCTRL0-%d(0x%08x): 0x%08x\n", ch, &regs->RPTCTRL0, regs->RPTCTRL0);
		printk(KERN_DEBUG "EXTREQ0-%d (0x%08x): 0x%08x\n", ch, &regs->EXTREQ0, 	regs->EXTREQ0);
	} else if(ch == 1) {
		printk(KERN_DEBUG "ST_SADR1-%d(0x%08x): 0x%08x\n", ch, &regs->ST_SADR1, regs->ST_SADR1);
		printk(KERN_DEBUG "SPARAM1-%d (0x%08x): 0x%08x\n", ch, &regs->SPARAM1,  regs->SPARAM1);
		printk(KERN_DEBUG "ST_DADR1-%d(0x%08x): 0x%08x\n", ch, &regs->ST_DADR1, regs->ST_DADR1);
		printk(KERN_DEBUG "DPARAM1-%d (0x%08x): 0x%08x\n", ch, &regs->DPARAM1, 	regs->DPARAM1);
		printk(KERN_DEBUG "C_DADR1-%d (0x%08x): 0x%08x\n", ch, &regs->C_DADR1, 	regs->C_DADR1);
		printk(KERN_DEBUG "HCOUNT1-%d (0x%08x): 0x%08x\n", ch, &regs->HCOUNT1, 	regs->HCOUNT1);
		printk(KERN_DEBUG "CHCTRL1-%d (0x%08x): 0x%08x\n", ch, &regs->CHCTRL1, 	regs->CHCTRL1);
		printk(KERN_DEBUG "RPTCTRL1-%d(0x%08x): 0x%08x\n", ch, &regs->RPTCTRL1, regs->RPTCTRL1);
		printk(KERN_DEBUG "EXTREQ1-%d (0x%08x): 0x%08x\n", ch, &regs->EXTREQ1, 	regs->EXTREQ1);
	} else {
		printk(KERN_DEBUG "ST_SADR2-%d(0x%08x): 0x%08x\n", ch, &regs->ST_SADR2, regs->ST_SADR2);
		printk(KERN_DEBUG "SPARAM2-%d (0x%08x): 0x%08x\n", ch, &regs->SPARAM2,  regs->SPARAM2);
		printk(KERN_DEBUG "ST_DADR2-%d(0x%08x): 0x%08x\n", ch, &regs->ST_DADR2, regs->ST_DADR2);
		printk(KERN_DEBUG "DPARAM2-%d (0x%08x): 0x%08x\n", ch, &regs->DPARAM2, 	regs->DPARAM2);
		printk(KERN_DEBUG "C_DADR2-%d (0x%08x): 0x%08x\n", ch, &regs->C_DADR2, 	regs->C_DADR2);
		printk(KERN_DEBUG "HCOUNT2-%d (0x%08x): 0x%08x\n", ch, &regs->HCOUNT2, 	regs->HCOUNT2);
		printk(KERN_DEBUG "CHCTRL2-%d (0x%08x): 0x%08x\n", ch, &regs->CHCTRL2, 	regs->CHCTRL2);
		printk(KERN_DEBUG "RPTCTRL2-%d(0x%08x): 0x%08x\n", ch, &regs->RPTCTRL2, regs->RPTCTRL2);
		printk(KERN_DEBUG "EXTREQ2-%d (0x%08x): 0x%08x\n", ch, &regs->EXTREQ2, 	regs->EXTREQ2);
	}
	printk(KERN_DEBUG "CHCONFIG0 (0x%08x): 0x%08x\n", ch, &regs->CHCONFIG, regs->CHCONFIG);
	printk(KERN_DEBUG "###############################\n");
#endif
}

static void tcc_tsif_parallel_reg_debug(volatile struct tcc_tsif_regs* regs, int id)
{
#if TSIF_USE_DEBUG
	printk(KERN_DEBUG "## TSIF debug #################\n");
	printk(KERN_DEBUG "TSCR%d(0x%08x)  : 0x%08x\n", id, &regs->TSCR, regs->TSCR);
	printk(KERN_DEBUG "TSPID%d(0x%08x) : 0x%08x\n", id, &regs->TSPID, regs->TSPID);
	printk(KERN_DEBUG "TSCTRL%d(0x%08x): 0x%08x\n", id, &regs->TSCTRL, regs->TSCTRL);
	printk(KERN_DEBUG "TSCHS(0x%08x)  : 0x%08x\n", &HwTSIFPORTSEL_BASE, HwTSIFPORTSEL_BASE);
	printk(KERN_DEBUG "###############################\n");
#endif
}

static void tcc_tsif_parallel_clearfifopacket(struct tcc_tsif_parallel_handle *h)
{
	BITSET(h->regs->TSCR, Hw7);		//set to Hw7 : empties RX Fifo

	printk(KERN_DEBUG "%s\n", __func__);
}

static void tcc_tsif_parallel_set(struct tcc_tsif_parallel_handle *h)
{
	if(h->msb_first)		BITSET(h->regs->TSCR, Hw16);
	else					BITCLR(h->regs->TSCR, Hw16);

	if(h->big_endian)		BITSET(h->regs->TSCR, Hw15);
	else					BITCLR(h->regs->TSCR, Hw15);
		
	if(h->serial_mode)		BITSET(h->regs->TSCR, Hw14);
	else					BITCLR(h->regs->TSCR, Hw14);

	if(h->clk_polarity)		BITSET(h->regs->TSCR, Hw13);
	else					BITCLR(h->regs->TSCR, Hw13);

	if(h->valid_polarity)	BITSET(h->regs->TSCR, Hw12);
	else					BITCLR(h->regs->TSCR, Hw12);

	if(h->sync_polarity)	BITSET(h->regs->TSCR, Hw11);
	else					BITCLR(h->regs->TSCR, Hw11);

	if(h->sync_delay)		BITSET(h->regs->TSCR, Hw5);
	else					BITCLR(h->regs->TSCR, Hw5);
	BITSET(h->regs->TSCR, (h->sync_delay & 0x1f));

	printk(KERN_DEBUG "%s : mode set : %s, %s, %s, %s, %s, %s, sync delay %d\n", __func__, 
																		(h->msb_first		? "msb first" 		: "lsb first"),
																		(h->big_endian 		? "big endian" 		: "little endian"),
																		(h->serial_mode		? "serial mode" 	: "parallel mode"),
																		(h->clk_polarity	? "falling edge" 	: "rising edge"),
																		(h->valid_polarity 	? "valid high active": "valid low active"),
																		(h->sync_polarity	? "sync high active" : "sync low active"),
																		 h->sync_delay);
}

static void tcc_tsif_parallel_isr(struct tcc_tsif_parallel_handle *h)
{
	volatile PGDMACTRL dma_regs;
	
	if(h->dma_controller == 0)
		dma_regs = (volatile PGDMACTRL)tcc_p2v(HwGDMA0_BASE);
	else if(h->dma_controller == 1)
		dma_regs = (volatile PGDMACTRL)tcc_p2v(HwGDMA1_BASE);
	else if(h->dma_controller == 2)
		dma_regs = (volatile PGDMACTRL)tcc_p2v(HwGDMA2_BASE);
	else if(h->dma_controller == 3)
		dma_regs = (volatile PGDMACTRL)tcc_p2v(HwGDMA3_BASE);

	if(h->dma_ch == 0)
	{
		BITSET(dma_regs->CHCTRL0, Hw3);		//dma done flag set
		h->dma_phy_curpos = dma_regs->C_DADR0;
	}
	else if(h->dma_ch == 1)
	{
		BITSET(dma_regs->CHCTRL1, Hw3);		//dma done flag set
		h->dma_phy_curpos = dma_regs->C_DADR1;
	}
	else
	{
		BITSET(dma_regs->CHCTRL2, Hw3);		//dma done flag set
		h->dma_phy_curpos = dma_regs->C_DADR2;
	}

	h->dma_intr_cnt++;
}

static void tcc_tsif_parallel_dma_init(struct tcc_tsif_parallel_handle *h)
{
	volatile PGDMACTRL dma_regs;
	unsigned int mask_addr;

	if(h->dma_controller == 0)
		dma_regs = (volatile PGDMACTRL)tcc_p2v(HwGDMA0_BASE);
	else if(h->dma_controller == 1)
		dma_regs = (volatile PGDMACTRL)tcc_p2v(HwGDMA1_BASE);
	else if(h->dma_controller == 2)
		dma_regs = (volatile PGDMACTRL)tcc_p2v(HwGDMA2_BASE);
	else if(h->dma_controller == 3)
		dma_regs = (volatile PGDMACTRL)tcc_p2v(HwGDMA3_BASE);
	
	printk(KERN_DEBUG "[%s]: dma set dest addr 0x%08x:%dbyte\n", __func__, h->rx_dma.dma_addr, h->rx_dma.buf_size);

	if(h->dma_ch == 0) {
		BITCSET(dma_regs->ST_SADR0, (Hw32-Hw0), &(h->regs->TSDI));		//source addr set
		BITCLR(dma_regs->SPARAM0, (Hw32-Hw0));

		BITCSET(dma_regs->ST_DADR0, (Hw32-Hw0), h->rx_dma.dma_addr);	//destination addr set
		mask_addr = TSIF_MASK_SIZE << 8;								// 1mbyte mask set
		BITCSET(dma_regs->DPARAM0, (Hw32-Hw0), (mask_addr|TSIF_DMA_INCREASE_SIZE));		//inc & mask addr set

		BITCSET(dma_regs->HCOUNT0, (Hw32-Hw0), TSIF_DMA_HOPE_CNT(h->dma_intr_packet_cnt));		//hop count set
		BITCLR(dma_regs->CHCTRL0, (Hw32-Hw0));							//repeat ctrl reg set

		if(h->id == 0)	BITCSET(dma_regs->EXTREQ0, (Hw32-Hw0), Hw3);	//ext request set tsif0
		else			BITCSET(dma_regs->EXTREQ0, (Hw32-Hw0), Hw7);	//ext request set tsif1

		BITCSET(dma_regs->CHCTRL0, (Hw32-Hw0), (Hw15|Hw11|Hw9|Hw8|Hw5|Hw4|Hw2|Hw1));		//channel ctrl reg set
	} else if(h->dma_ch == 1) {
		BITCSET(dma_regs->ST_SADR1, (Hw32-Hw0), &(h->regs->TSDI));		//source addr set
		BITCLR(dma_regs->SPARAM1, (Hw32-Hw0));

		BITCSET(dma_regs->ST_DADR1, (Hw32-Hw0), h->rx_dma.dma_addr);	//destination addr set
		mask_addr = TSIF_MASK_SIZE << 8;
								// 1mbyte mask set
		BITCSET(dma_regs->DPARAM1, (Hw32-Hw0), (mask_addr|TSIF_DMA_INCREASE_SIZE));		//inc & mask addr set

		BITCSET(dma_regs->HCOUNT1, (Hw32-Hw0), TSIF_DMA_HOPE_CNT(h->dma_intr_packet_cnt));		//hop count set
		BITCLR(dma_regs->CHCTRL1, (Hw32-Hw0));							//repeat ctrl reg set

		if(h->id == 0)	BITCSET(dma_regs->EXTREQ1, (Hw32-Hw0), Hw3);	//ext request set tsif0
		else			BITCSET(dma_regs->EXTREQ1, (Hw32-Hw0), Hw7);	//ext request set tsif1

		BITCSET(dma_regs->CHCTRL1, (Hw32-Hw0), (Hw15|Hw11|Hw9|Hw8|Hw5|Hw4|Hw2|Hw1));			//channel ctrl reg set
	} else { 
		BITCSET(dma_regs->ST_SADR2, (Hw32-Hw0), &(h->regs->TSDI));		//source addr set
		BITCLR(dma_regs->SPARAM2, (Hw32-Hw0));

		BITCSET(dma_regs->ST_DADR2, (Hw32-Hw0), h->rx_dma.dma_addr);	//destination addr set
		mask_addr = TSIF_MASK_SIZE << 8;
								// 1mbyte mask set
		BITCSET(dma_regs->DPARAM2, (Hw32-Hw0), (mask_addr|TSIF_DMA_INCREASE_SIZE));		//inc & mask addr set

		BITCSET(dma_regs->HCOUNT2, (Hw32-Hw0), TSIF_DMA_HOPE_CNT(h->dma_intr_packet_cnt));		//hop count set
		BITCLR(dma_regs->CHCTRL2, (Hw32-Hw0));							//repeat ctrl reg set

		if(h->id == 0)	BITCSET(dma_regs->EXTREQ2, (Hw32-Hw0), Hw3);	//ext request set tsif0
		else			BITCSET(dma_regs->EXTREQ2, (Hw32-Hw0), Hw7);	//ext request set tsif1

		BITCSET(dma_regs->CHCTRL2, (Hw32-Hw0), (Hw15|Hw11|Hw9|Hw8|Hw5|Hw4|Hw2|Hw1));			//channel ctrl reg set
	}
}

static int tcc_tsif_parallel_dmastart(struct tcc_tsif_parallel_handle *h)
{
	volatile PGDMACTRL dma_regs;

	if(h->dma_controller == 0)
		dma_regs = (volatile PGDMACTRL)tcc_p2v(HwGDMA0_BASE);
	else if(h->dma_controller == 1)
		dma_regs = (volatile PGDMACTRL)tcc_p2v(HwGDMA1_BASE);
	else if(h->dma_controller == 2)
		dma_regs = (volatile PGDMACTRL)tcc_p2v(HwGDMA2_BASE);
	else if(h->dma_controller == 3)
		dma_regs = (volatile PGDMACTRL)tcc_p2v(HwGDMA3_BASE);

	h->dma_phy_rdpos 	= h->rx_dma.dma_addr;
	h->dma_phy_curpos 	= h->dma_phy_rdpos;
	h->dma_phy_maxaddr	= h->dma_phy_curpos + h->dma_total_size;
	h->dma_virt_rdpos 	= (unsigned int)h->rx_dma.v_addr;
	h->dma_virt_curpos 	= h->dma_virt_rdpos;
	h->dma_virt_maxaddr	= h->dma_virt_curpos + h->dma_total_size;
	h->dma_recv_size	= 0;
	h->dma_intr_cnt		= 0;

	tcc_tsif_parallel_dma_init(h);

	if(h->dma_ch == 0)	 	BITSET(dma_regs->CHCTRL0, 	Hw0);	//dma enable
	else if(h->dma_ch == 1)	BITSET(dma_regs->CHCTRL1, 	Hw0);	//dma enable
	else					BITSET(dma_regs->CHCTRL2, 	Hw0);	//dma enable
		
	tcc_tsif_parallel_dma_debug(dma_regs, h->dma_ch);

	BITSET(h->regs->TSCR, Hw31);								//tsif enable

	printk(KERN_DEBUG "%s\n", __func__);

	tcc_tsif_parallel_reg_debug(h->regs, h->id);

	return 0;
}

static int tcc_tsif_parallel_dmastop(struct tcc_tsif_parallel_handle *h)
{
	volatile PGDMACTRL dma_regs;

	if(h->dma_controller == 0)
		dma_regs = (volatile PGDMACTRL)tcc_p2v(HwGDMA0_BASE);
	else if(h->dma_controller == 1)
		dma_regs = (volatile PGDMACTRL)tcc_p2v(HwGDMA1_BASE);
	else if(h->dma_controller == 2)
		dma_regs = (volatile PGDMACTRL)tcc_p2v(HwGDMA2_BASE);
	else if(h->dma_controller == 3)
		dma_regs = (volatile PGDMACTRL)tcc_p2v(HwGDMA3_BASE);

	if(h->dma_ch == 0)		BITCLR(dma_regs->CHCTRL0, 	Hw0);	//dma disable
	else if(h->dma_ch == 1)	BITCLR(dma_regs->CHCTRL1, 	Hw0);	//dma disable
	else					BITCLR(dma_regs->CHCTRL2, 	Hw0);	//dma disable

	BITCLR(h->regs->TSCR, Hw31);								//tsif disable

	printk(KERN_DEBUG "%s\n", __func__);
	return 0;
}

static void tcc_tsif_parallel_dma_deinit(struct tcc_tsif_parallel_handle *h)
{
	volatile PGDMACTRL dma_regs;

	dma_regs = (volatile PGDMACTRL)tcc_p2v(HwGDMA0_BASE);
}

#if 0
static void tcc_tsif_parallel_irq_deinit(struct tcc_tsif_parallel_handle *h)
{
	volatile PPIC pic_regs = (volatile PPIC)tcc_p2v(HwPIC_BASE);

	//priority interrupt ctrl reg set
	BITCLR(pic_regs->INTMSK1,  	(HwINT1_TSIF0 << h->id));		//irq output mask
	BITCLR(pic_regs->IEN1,  	(HwINT1_TSIF0 << h->id));		//enable
}
#endif

static void tcc_tsif_parallel_hw_init(struct tcc_tsif_parallel_handle *h)
{
	volatile PGPIO gpio_regs = (volatile PGPIO)tcc_p2v(HwGPIO_BASE);
	unsigned int reg_val;

	//gpioe port configuration 
	BITCSET(gpio_regs->GPEFN1, (Hw32-Hw16), (Hw29|Hw28|Hw25|Hw24|Hw21|Hw20|Hw17|Hw16));
	BITCSET(gpio_regs->GPEFN2, (Hw28-Hw0),  (Hw25|Hw24|Hw21|Hw20|Hw17|Hw16|Hw13|Hw12|Hw9|Hw8|Hw5|Hw4|Hw1|Hw0));

	BITCSET(gpio_regs->GPEPD1, (Hw14-Hw12), Hw12);		// Clk port Internal Pull-up;
	BITCSET(gpio_regs->GPECD1, (Hw9|Hw8), (Hw9|Hw8));

	tcc_tsif_parallel_gpio_debug(gpio_regs);
 
	//tsif ch select
	if(h->id == 0) {
		BITCSET(HwTSIFPORTSEL_BASE, (Hw4-Hw2), Hw2);
    } else {
		BITCSET(HwTSIFPORTSEL_BASE, (Hw2-Hw0), Hw0);
	} 
	
	if(TSIF_BYTE_SIZE == 4)			reg_val = Hw17 | Hw9 | Hw8 | Hw7;
	else if(TSIF_BYTE_SIZE == 2)	reg_val = Hw17 | Hw8 | Hw7;
	else							reg_val = Hw17 | Hw7;
	BITCSET(h->regs->TSCR, 	(Hw32-Hw0), reg_val);
	
	//20100120 koo atsc   little endian  Ű tsif reg 0x47 detect  dma buf 4byte align ڹٲ . Ͽ 0x47 detect mode clear
	BITCLR(h->regs->TSCR, Hw17);
	
	reg_val = (TSIF_RXFIFO_THRESHOLD << 5) | Hw4;
	BITCSET(h->regs->TSCTRL,(Hw32-Hw0), reg_val);
	
	BITCLR(h->regs->TSPID, 	(Hw32-Hw0));
	
	tcc_tsif_parallel_dma_init(h);
	
	printk(KERN_DEBUG "%s\n", __func__);
}

static void tcc_tsif_parallel_hw_deinit(struct tcc_tsif_parallel_handle *h)
{
	//release GPIO
	if(h->gpio_set == 0x1234) {
		volatile PGPIO gpio_regs = (volatile PGPIO)tcc_p2v(HwGPIO_BASE);

		//gpioe port configuration
		BITCLR(gpio_regs->GPEFN1, (Hw32-Hw16));
		BITCLR(gpio_regs->GPEFN2, (Hw24-Hw0));
		
		if(h->id == 0) {
			BITCLR(HwTSIFPORTSEL_BASE, (Hw4-Hw2));
		} else {
			BITCLR(HwTSIFPORTSEL_BASE, (Hw2-Hw0));
		}

		BITCLR(h->regs->TSCR, 	(Hw32-Hw0));
		BITCLR(h->regs->TSCTRL,	(Hw32-Hw0));

		tcc_tsif_parallel_dma_deinit(h);
	}

	printk(KERN_DEBUG "%s\n", __func__);
}

/******************************
 * return value
 *
 * ret == 0: success
 * ret > 0 or ret < 0: fail
 ******************************/
int tcc_tsif_parallel_init(struct tcc_tsif_parallel_handle *h,
					volatile struct tcc_tsif_regs *regs,
					dma_alloc_f tea_dma_alloc,
					dma_free_f tea_dma_free,
					int dma_size,
					int tsif_ch,
					int dma_controller,
					int dma_ch)
{
	int ret = -1;

	if(regs)
	{
		h->regs 			= regs;
		h->dma_controller	= dma_controller;
		h->id 				= tsif_ch;
		h->dma_ch			= dma_ch;
		h->irq 				= (INT_DMA0_BASE + 3*dma_controller + dma_ch); //INT_DMA
		h->gpio_set 		= 0x1234;

		h->dma_stop 		= tcc_tsif_parallel_dmastop;
		h->dma_start 		= tcc_tsif_parallel_dmastart;
		h->clear_fifo_packet= tcc_tsif_parallel_clearfifopacket;
		h->tsif_set			= tcc_tsif_parallel_set;
		h->tsif_isr			= tcc_tsif_parallel_isr;

		h->tea_dma_alloc 	= tea_dma_alloc;
		h->tea_dma_free 	= tea_dma_free;

#ifdef VPU_DMA_USE        
		h->dma_total_size	= h->dma_phy_size;
#else
		h->dma_total_size 	= dma_size;
#endif

		if(h->tea_dma_alloc) {
			if(h->tea_dma_alloc(&(h->rx_dma), h->dma_total_size) == 0) {
				ret = 0;
			}
		} else {
			/* Already, tsif has rx_dma buf */
			ret = 0;
		}

		tcc_tsif_parallel_hw_init(h);

		if(ret)
			tcc_tsif_parallel_clean(h);
	}

	return ret;
}

void tcc_tsif_parallel_clean(struct tcc_tsif_parallel_handle *h)
{
	if(h) {
		if(h->tea_dma_free) {
			h->tea_dma_free(&(h->rx_dma));
		}

		tcc_tsif_parallel_hw_deinit(h);
	}
}


EXPORT_SYMBOL(tcc_tsif_parallel_init);
EXPORT_SYMBOL(tcc_tsif_parallel_clean);


