
/****************************************************************************
 *   FileName    : tca_remocon.h
 ****************************************************************************
 *
 *   TCC Version 1.0
 *   Copyright (c) Telechips, Inc.
 *   ALL RIGHTS RESERVED
 *
 ****************************************************************************/
#include <linux/types.h>


/*******************************************************************************
 Decide the remote type
 if you don't select any one, it will be the default remote controller
********************************************************************************/ 
//#define DARE_IR
//#define BUFFALO_IR
//#define LG_MKJ36998118
#define CS_2000_IR_X_CANVAS
//#define SUNNIWELL_IR
//#define NOVATEK_IR

#ifdef DARE_IR
#define CONFIG_DARE_IR
#endif //DARE_IR
#ifdef BUFFALO_IR
#define CONFIG_BUFFALO_IR
#endif //BUFFALO_IR
#ifdef LG_MKJ36998118
#define CONFIG_LG_MKJ36998118
#endif //LG_MKJ36998118
#ifdef CS_2000_IR_X_CANVAS
#define CONFIG_CS_2000_IR_X_CANVAS
#endif //CS_2000_IR_X_CANVAS
#ifdef SUNNIWELL_IR
#define CONFIG_SUNNIWELL_IR
#endif //SUNNIWELL_IR
#ifdef NOVATEK_IR
#define CONFIG_NOVATEK_IR
#endif //NOVATEK_IR


/*******************************************************************************
*	 Remote Control Interface Register Define   (Base Addr = 0xF05F3000)
********************************************************************************/
typedef	struct	_REMOCON {
	union	{
		uint	nPORT;
		struct  _REMOCON_PORT	{
			uint	PORT	:  32;	// PORT SELCT
			//uint	PORT	:  8;	// PORT SELCT
			//uint		: 24;
		}	bPORT;
	}	uPORT;
	union	{
		uint	nCOMMAND;
		struct	_REMOCON_COMMAND {
			uint	FF	:  1;	// interrupt acknowledge ( writing '1' for clear )
			uint	EN	:  1;
			uint	CLEAR	:  1;	// sent acknowledge ( 1 for disabled )
			uint	TH	:  9;	// FIFO THRESHOLD
			uint	WS	:  1;	// ir clk wait control
			uint	DEN	:  1;	// duration count enable
			uint	FWEN	:  1;	// FIFO write enable
			uint		: 17;
		}	bCOMMAND;
	}	uCOMMAND;
	union	{
		uint	nCONFIGURE;
		struct	_REMOCON_CONTROL {
#if defined(CONFIG_ARCH_TCC92X)		
			uint	CA	: 12;	// configure address
			uint		: 20;
#elif defined(CONFIG_ARCH_TCC93XX)
			uint INV : 1;
			uint       : 7;
			uint SCLK : 1;
			uint FIL : 1;
			uint FT : 2;
			uint      : 20;
#endif
		}	bCONFIGURE;
	}	uCONFIGURE;
	union	{
		uint	nSTATUS;
		struct	_REMOCON_STATUS {
			uint	ICF	: 12;	// interrupt configure ( 1 for pending )
			uint	OF	:  1;	// FIFO over flow
			uint		: 19;
		}	bSTATUS;
	}	uSTATUS;
//	union	{
//		uint	nEMPTY0;
//		struct	_REMOCON_EMPTY0	{
//			uint	EMPTY0	: 32;
//		}	bEMPTY0;
//	}	uEMPTY0;
	uint	EMPTY0;
	uint	EMPTY1;
	uint	EMPTY2;
	uint	EMPTY3;
	uint	EMPTY4;
	uint	EMPTY5;
	uint	EMPTY6;
	uint	EMPTY7;
	uint	EMPTY8;
	uint	EMPTY9;
	uint	EMPTY10;
	uint	EMPTY11;
	union	{
		uint	nCLKDIVIDE;
		struct	_REMOCON_CLKDIVED {
			uint	END_CNT	: 14;	// transfer end zero count 
			uint	CLK_DIV	: 18;	// clk clock divide for ir clk
		}	bCLKDIVIDE;
	}	uCLKDIVIDE;
}	REMOCON, *PREMOCON;

#if defined(CONFIG_ARCH_TCC92X)
#define HwREMOCON			((volatile REMOCON  *)(0xF05F3000))
#define Hw_REMOCON_BASE			*(volatile unsigned long *)0xF05F3000

#define Hw_REMOCON_RDATA	*(volatile unsigned long *)0xF05F3000
#define Hw_REMOCON_CMD		*(volatile unsigned long *)0xF05F3004
#define Hw_REMOCON_INPOL	*(volatile unsigned long *)0xF05F3008
#define Hw_REMOCON_STA		*(volatile unsigned long *)0xF05F3008
#define Hw_REMOCON_CLKDIV	*(volatile unsigned long *)0xF05F3040

#define	HwGPIOA_BASE  				*(volatile unsigned long *)0xF0102000  	// 
#define	HwGPIOA_FN0  				*(volatile unsigned long *)0xF0102024  	// 
#elif defined(CONFIG_ARCH_TCC93XX)
#define HwREMOCON					((volatile REMOCON  *)(0xB0101000))
#define Hw_REMOCON_BASE			*(volatile unsigned long *)0xB0101000

#define Hw_REMOCON_RDATA			*(volatile unsigned long *)0xB0101000
#define Hw_REMOCON_CMD			*(volatile unsigned long *)0xB0101004
#define Hw_REMOCON_INPOL			*(volatile unsigned long *)0xB0101008
#define Hw_REMOCON_STA			*(volatile unsigned long *)0xB010100c
#define Hw_REMOCON_CLKDIV			*(volatile unsigned long *)0xB0101040

//#define	HwGPIOA_BASE  				*(volatile unsigned long *)0xB010A000
//#define	HwGPIOA_FN0  				*(volatile unsigned long *)0xB010A024

#endif /* 0 */


//*******************************************
//	Remote KEY value
//*******************************************
#define REM_PWR			1	
#define REM_LOAD		2	
#define REM_SUBTITLE	3	
#define REM_AUDIO		4	
#define REM_ANGLE		5	
#define REM_INFO		6	

#define REM_KEY1		7	
#define REM_KEY2		8	
#define REM_KEY3		9	
#define REM_KEY4		10	
#define REM_KEY5		11	
#define REM_KEY6		12	
#define REM_KEY7		13	
#define REM_KEY8		14	
#define REM_KEY9		15	
#define REM_KEY0		16	
#define REM_KEY10		17	

#define REM_GOTO		18	
#define REM_TITLE		19	
#define REM_MENU		20	
#define REM_UP			21	
#define REM_DOWN		22	
#define REM_LEFT		23	
#define REM_RIGHT		24	
#define REM_ENTER		25	
#define REM_SETUP		26	
#define REM_RETURN		27	
                    	
#define REM_STOP		28	
#define REM_PLAY		29	
#define REM_PAUSE		30	
#define REM_FB			31	
#define REM_FF			32	
#define REM_LAST		33	
#define REM_NEXT		34	

#define REM_AB			35	
#define REM_REPEAT		36	
#define REM_STEP		37	
#define REM_PBC			38	
#define REM_ZOOM		39	
#define REM_MUTE		40	
#define REM_VOLDN		41	
#define REM_VOLUP		42	
#define REM_SLOW		43	
#define REM_RANDOM		44	
#define REM_KEYDEC		45	
#define REM_KEYADD		46	

#define REM_PROGRAM		47	
#define REM_CLEAR		48	
#define REM_RESUME		49	
#define REM_VIDEO		50	
#define REM_PN			51	
#define REM_DIGEST		52	
#define REM_HDMI		53	
#define REM_FUNCTION	54	
#define REM_EPG			55	
#define REM_TELETEXT	56	
#define REM_SERVICE		57	
#define REM_EXIT		58	

#define REM_GUIDE		59	
#define REM_LIST		60	
#define REM_BROWSE		61	
#define REM_RADIO		62	
#define REM_FADEC		63	
#define REM_FAADD		64	

#define REM_MODE		65	
#define REM_LANG		66	
#define REM_CHAR		67	


//*******************************************
//	Remote controller define
//*******************************************

#define STATUS0		0
#define STATUS1		1
#define STATUS2		2
#define STATUS3		3

/* 1 unit = 1 / [IOBUS/(HwREMOCON->uCLKDIVIDE.bCLKDIVIDE.CLK_DIV)]
              = 1 / [156/(500*20)]  us
              = 64.1us
*/
/* 1 unit = 1 / [IOBUS/(HwREMOCON->uCLKDIVIDE.bCLKDIVIDE.CLK_DIV)]
              = 1 / [165/(500*21)]  us
              = 63.63us
*/
/* 1 unit = 1 / [IOBUS/(HwREMOCON->uCLKDIVIDE.bCLKDIVIDE.CLK_DIV)]
			  = 1 / [165/(500*26)]	us
			  = 78.78us
*/

#if defined DARE_IR

//Low Bit:
#define LOW_MIN			0x0600
#define LOW_MAX			0x0BFF
//High Bit
#define HIGH_MIN		0x1600
#define HIGH_MAX		0x1DFF
//Start Bit
#define START_MIN		0x4000
#define START_MAX		0x50FF
//Repeat Bit
#define REPEAT_MIN		0x2000
#define REPEAT_MAX		0x27FF


#else
//Low Bit
#if (defined(CONFIG_ARCH_TCC92X) || defined(CONFIG_ARCH_TCC93XX))
#define LOW_MIN			0x0700
#define LOW_MAX			0x0B00
//High Bit
#define HIGH_MIN		0x1800
#define HIGH_MAX		0x1D00
//Start Bit
#define START_MIN		0x4000
#define START_MAX		0x5000
//Repeat Bit
#define REPEAT_MIN		0x2000
#define REPEAT_MAX		0x2700
#else
//Low Bit
#define LOW_MIN			0x0500
#define LOW_MAX			0x0B00
//High Bit
#define HIGH_MIN		0x1000
#define HIGH_MAX		0x1D00
//Start Bit
#define START_MIN		0x2500
#define START_MAX		0x5000
#endif
#endif

/*****************************************************************************
*
* IR remote controller scancode
*
******************************************************************************/
#define REM_ID_ERROR	0xFF

#if 0 
#define REMOCON_ID		0x0400

#define SCAN_PWR		0x0E00	//power
#define SCAN_LOAD		0x0E80	//eject
#define SCAN_SUBTITLE	0xFE00	//subitle, no mapping
#define SCAN_AUDIO		0xFE00	//audiono mapping
#define SCAN_ANGLE		0xFE00	//angle, no mapping
#define SCAN_INFO		0xFE00	//, no mapping

#define SCAN_KEY1		0x0E40	//"1"
#define SCAN_KEY2		0x0EC0	//"2"
#define SCAN_KEY3		0x0E20  //"3"
#define SCAN_KEY4		0x0EA0	//"4"
#define SCAN_KEY5		0x0E60  //"5"
#define SCAN_KEY6		0x0EE0	//"6"
#define SCAN_KEY7		0x0E10	//"7"
#define SCAN_KEY8		0x0E90	//"8"
#define SCAN_KEY9		0x0E50	//"9"
#define SCAN_KEY0		0x0ED0	//"0"
#define SCAN_KEY10		0xFE00	//"10+" , no mapping

#define SCAN_GOTO		0xFE00	//GOTO , no mapping
#define SCAN_TITLE		0xFE00	//tilte, no mapping
#define SCAN_MENU		0xFE00	//menu, no mapping
#define SCAN_UP			0x0E18	//up
#define SCAN_DOWN		0x0E98	//down
#define SCAN_LEFT		0x0ED8	//left
#define SCAN_RIGHT		0x0E58	//right
#define SCAN_ENTER		0x0E38	//enter
#define SCAN_SETUP		0x0E68	//setup
#define SCAN_RETURN	    0x0EE8	//return

#define SCAN_STOP		0x0EC8	//stop
#define SCAN_PLAY		0x0E28	//play
#define SCAN_PAUSE		0xFE00	//pause, no mapping
#define SCAN_FB			0x0E48	//fast backward
#define SCAN_FF			0x0EA8	//fast forward
#define SCAN_LAST		0x0EB0	//previous
#define SCAN_NEXT		0x0E88	//next

#define SCAN_AB			0x0E14	//A_B
#define SCAN_REPEAT		0x0E00	//repeat , no mapping
#define SCAN_STEP		0x0E08	//step
#define SCAN_PBC		0xFE00	//PBC, playback control (use in VCD2.0, DVD), no mapping
#define SCAN_ZOOM		0xFE00	//zoom, no mapping
#define SCAN_MUTE		0xFE00	//mute, no mapping
#define SCAN_VOLDN		0xFE00	//volume +, no mapping
#define SCAN_VOLUP		0xFE00	//volume -, no mapping
#define SCAN_SLOW		0xFE00	//slow, no mapping
#define SCAN_RANDOM	    0xFE00	//random, no mapping
#define SCAN_KEYDEC		0xFE00	//KEY-, no mapping
#define SCAN_KEYADD	    0xFE00	//KEY+, no mapping

#define SCAN_PROGRAM	0xFE00	//program, no mapping
#define SCAN_CLEAR		0xFE00	//clean, no mapping
#define SCAN_RESUME	    0xFE00	//resume, no mapping
#define SCAN_VIDEO		0xFE00	//video switching
#define SCAN_PN			0xFE00	//PAL/NTSC, , no mapping
#define SCAN_DIGEST		0xFE00	//Digest, no mapping
#define SCAN_HDMI		0xFE00	//HDMI, no mapping
#define SCAN_FUNCTION	0xFE00	//function, no mapping
#define SCAN_EPG		0xFE00	//Electronic Program Guide, no mapping
#define SCAN_TELETEXT	0xFE00	//Teletext, no mapping
#define SCAN_SERVICE	0xFE00	//service, no mapping
#define SCAN_EXIT		0xFE00	//exit, no mapping
#define SCAN_GUIDE		0xFE00	//guide, no mapping
#define SCAN_LIST		0xFE00	//list, no mapping
#define SCAN_BROWSE	    0xFE00	//browse, no mapping
#define SCAN_RADIO		0xFE00	//radio, no mapping
#define SCAN_FADEC		0xFE00	//fade -, no mapping
#define SCAN_FAADD		0xFE00	//fade +, no mapping

#elif defined (DARE_IR)

#define REMOCON_ID		0x40bf	//vendor ID, prodcut ID

#define SCAN_PWR		0x02fd	//
#define SCAN_LOAD		0xFFFF	//, no mapping
#define SCAN_SUBTITLE	0x906f 	//
#define SCAN_AUDIO		0xd827 	//
#define SCAN_ANGLE		0xFFFF	// no mapping
#define SCAN_INFO		0xe817	//

#define SCAN_KEY1		0x48b7	//"1"
#define SCAN_KEY2		0x6897	//"2"
#define SCAN_KEY3		0x40bf  //"3"
#define SCAN_KEY4		0x58a7 	//"4"
#define SCAN_KEY5		0x7887  //"5"
#define SCAN_KEY6		0xc03f 	//"6"
#define SCAN_KEY7		0xf00f	//"7"
#define SCAN_KEY8		0xd02f	//"8"
#define SCAN_KEY9		0xe01f  //"9"
#define SCAN_KEY0		0x22dd	//"0"
#define SCAN_KEY10		0xFFFF	//"10+" , no mapping

#define SCAN_GOTO		0xf807	//
#define SCAN_TITLE		0x807f	//<<<<<<<<<<<<< not implement
#define SCAN_MENU		0xFFFF	//menu, no mapping
#define SCAN_UP			0x827d	//up
#define SCAN_DOWN		0xa25d	//down
#define SCAN_LEFT		0xb24d	//left
#define SCAN_RIGHT		0xba45	//right
#define SCAN_ENTER		0x8877	//
#define SCAN_SETUP		0x50af	//
#define SCAN_RETURN	    0xFFFF	//, no mapping

#define SCAN_STOP		0x1ae5	//
#define SCAN_PLAY		0x0af5	//
#define SCAN_PAUSE		0xFFFF	//pause, no mapping
#define SCAN_FB			0xFFFF	//fast backward
#define SCAN_FF			0xFFFF	//fast forward
#define SCAN_LAST		0xb847	//
#define SCAN_NEXT		0x00ff	//

#define SCAN_AB			0xFFFF	//A_B
#define SCAN_REPEAT		0xc837  //
#define SCAN_STEP		0xFFFF	//, no mapping
#define SCAN_PBC		0xFFFF	//, no mapping
#define SCAN_ZOOM		0xFFFF	//, no mapping
#define SCAN_MUTE		0x708f	//
#define SCAN_VOLDN		0x9a65	//
#define SCAN_VOLUP		0x8a75	//
#define SCAN_SLOW		0xFFFF	//, no mapping
#define SCAN_RANDOM	    0xFFFF	//, no mapping
#define SCAN_KEYDEC		0xFFFF	//, no mapping
#define SCAN_KEYADD	    0xFFFF	//, no mapping

#define SCAN_PROGRAM	0xFFFF	//, no mapping
#define SCAN_CLEAR		0x12ed	//<<<<<<<<<<<<<< factory reset not implement
#define SCAN_RESUME	    0xFFFF	//, no mapping
#define SCAN_VIDEO		0x10ef	//<<<<<<<<<<<<<VIDEO mode, not implement
#define SCAN_PN			0x6a95	//<<<<<<<<<<< not implement
#define SCAN_DIGEST		0x609f	//<<<<<<<<<<<<PICTURE mode,  not implement
#define SCAN_HDMI		0x32cd	//<<<<<<<<<<< not implement
#define SCAN_FUNCTION	0xa05f	//<<<<<<<<<<< not implement
#define SCAN_EPG		0xFFFF	//, no mapping
#define SCAN_TELETEXT	0xFFFF	//, no mapping
#define SCAN_SERVICE	0xFFFF	//, no mapping
#define SCAN_EXIT		0xFFFF	//, no mapping
#define SCAN_GUIDE		0xFFFF	//, no mapping
#define SCAN_LIST		0xFFFF	//, no mapping
#define SCAN_BROWSE	    0xFFFF	//, no mapping
#define SCAN_RADIO		0x926d	//<<<<<<<<<<<<<<<AUDIO mode,  not implement
#define SCAN_FADEC		0xFFFF	//, no mapping
#define SCAN_FAADD		0xFFFF	//, no mapping

#elif 0
//*******************************************
//	ȿ011
//*******************************************
#define REMOCON_ID		0x00FF

#define SCAN_PWR		0x30CF
#define SCAN_LOAD		0xB04F
#define SCAN_MODE		0x906F
#define SCAN_LANG		0xF807
#define SCAN_CHAR		0x7887
#define SCAN_ZOOM		0x926D

#define SCAN_KEY1		0x00FF
#define SCAN_KEY2		0x807F
#define SCAN_KEY3		0x40BF
#define SCAN_KEY4		0xC03F
#define SCAN_KEY5		0x20DF
#define SCAN_KEY6		0xA05F
#define SCAN_KEY7		0x609F
#define SCAN_KEY8		0xE01F
#define SCAN_KEY9		0x10EF
#define SCAN_KEY0		0xD02F
#define SCAN_KEY10		0x50AF

#define SCAN_REPEAT		0x8A75
#define SCAN_AUDIO		0xE817
#define SCAN_CLEAR		0x52AD
#define SCAN_TITLE		0x38C7
#define SCAN_SETUP		0x9867

#define SCAN_MENU		0xB847
#define SCAN_UP			0x08F7
#define SCAN_DOWN		0x8877
#define SCAN_LEFT		0x48B7
#define SCAN_RIGHT		0xC837
#define SCAN_ENTER		0x708F
#define SCAN_RETURN		0xA857

#define SCAN_FB			0x02FD
#define SCAN_FF			0x827D
#define SCAN_LAST		0x42BD
#define SCAN_NEXT		0xC23D
#define SCAN_STOP		0xA25D
#define SCAN_PAUSE		0xB24D
#define SCAN_PLAY		0x22DD
#define SCAN_SLOW		0x32CD

#define SCAN_VOLUP		0xD22D
#define SCAN_VOLDN		0xF20D
#define SCAN_MUTE		0x0AF5

#elif defined (BUFFALO_IR)
	#define REMOCON_ID		0x1867

	#define SCAN_PWR		0x0001

	#define SCAN_HOME		0x41BE
	#define SCAN_PLAY		0x45BA
	#define SCAN_STOP		0x0001
	#define SCAN_PAUSE		0x0001
	#define SCAN_FB 		0x0001
	#define SCAN_FF 		0x0001

	#define SCAN_MUTE		0x0001
	#define SCAN_VOLUP		0x748B
	#define SCAN_VOLDN		0x0CF3

	#define SCAN_MENU		0x5CA3

	#define SCAN_UP			0x1CE3
	#define SCAN_DOWN		0x21DE
	#define SCAN_LEFT		0x3CC3
	#define SCAN_RIGHT		0x01FE

	#define SCAN_MTS		0x8C73
	#define SCAN_SPEED		0xCC33

	#define SCAN_ENTER		0x7C83
	#define SCAN_BACK		0x619E
#elif defined (LG_MKJ36998118)
	#define REMOCON_ID		0x106f

	#define SCAN_PWR		0x0001

	#define SCAN_HOME		0xAC53
	#define SCAN_PLAY		0x916E
	#define SCAN_STOP		0x0001
	#define SCAN_PAUSE		0x0001
	#define SCAN_FB 		0x0001
	#define SCAN_FF 		0x0001

	#define SCAN_MUTE		0x0001
	#define SCAN_VOLUP		0xA05F
	#define SCAN_VOLDN		0xE01F

	#define SCAN_MENU		0xE11E

	#define SCAN_UP			0x817E
	#define SCAN_DOWN		0xC13E
	#define SCAN_LEFT		0xF00F
	#define SCAN_RIGHT		0xB04F

	#define SCAN_MTS		0x0001
	#define SCAN_SPEED		0x0001

	#define SCAN_ENTER		0xD12E
	#define SCAN_BACK		0xED12
#elif defined (CS_2000_IR_VIDEO_VISION)
	#define REMOCON_ID		0x3B44
	#define REMOCON_REPEAT	0x3AAAAAAA

	#define SCAN_PWR		0x0001

	#define SCAN_HOME		0xFC03
	#define SCAN_PLAY		0x8877
	#define SCAN_STOP		0xC03F
	#define SCAN_PAUSE		0xE817
	#define SCAN_FB			0xA05F
	#define SCAN_FF			0xE01F

	#define SCAN_MUTE		0xDD22
	#define SCAN_VOLUP		0x8D72
	#define SCAN_VOLDN		0xCD32

	#define SCAN_MENU		0xB44B

	#define SCAN_UP			0xA0DF
	#define SCAN_DOWN		0xC8B7
	#define SCAN_LEFT		0xD12E
	#define SCAN_RIGHT		0x916E

	#define SCAN_MTS		0x8C73
	#define SCAN_SPEED		0xCC33

	#define SCAN_ENTER		0xBC43
	#define SCAN_BACK		0xB54A

	#define SCAN_NUM_1		0xC43B
	#define SCAN_NUM_2		0xA45B

#elif defined (CS_2000_IR_X_CANVAS)

	#define REMOCON_ID			0x106F
	#define REMOCON_REPEAT		0x10000000
	#define SCAN_PWR			0x0001

	#define SCAN_POWER			0x8877

	#define SCAN_NUM_1			0xC43B
	#define SCAN_NUM_2			0xA45B
	#define SCAN_NUM_3			0xE41B
	#define SCAN_NUM_4			0x946B
	#define SCAN_NUM_5			0xD42B
	#define SCAN_NUM_6			0xB44B
	#define SCAN_NUM_7			0xF40B
	#define SCAN_NUM_8			0x8C73
	#define SCAN_NUM_9			0xCC33
	#define SCAN_NUM_0			0x847B
	#define SCAN_NUM_MINUS		0x9966
	#define SCAN_NUM_PREV		0xAC53

	#define SCAN_MUTE			0xC837
	#define SCAN_INPUT			0xE817

	#define SCAN_VOLUP			0xA05F
	#define SCAN_VOLDN			0xE01F

	#define SCAN_CH_UP			0x807F
	#define SCAN_CH_DN			0xC03F

	#define SCAN_SLEEP			0xB847
	#define SCAN_CANCEL			0xED12
	#define SCAN_MENU			0xE11E
	#define SCAN_GUIDE			0xCAB5

	#define SCAN_UP				0x817E
	#define SCAN_DOWN			0xC13E
	#define SCAN_LEFT			0xF00F
	#define SCAN_RIGHT			0xB04F
	#define SCAN_CENTER			0x916E

	#define SCAN_AUTO_CHANNEL	0x956A
	#define SCAN_ADD_DELETE		0xD52A
	#define SCAN_SIZE			0xF708
	#define SCAN_MULTI_SOUND	0xA857

	#define SCAN_STOP			0xC6B9
	#define SCAN_PLAY			0x86F9
	#define SCAN_PAUSE			0xAED1
	#define SCAN_REC			0xDEA1
	#define SCAN_FB				0xB8C7
	#define SCAN_FF				0xF887
	#define SCAN_CURRENT			0xBCC3
	
#elif defined (SUNNIWELL_IR)

	#define REMOCON_ID		0x007F

	#define SCAN_PWR		0x0001

	#define SCAN_HOME		0xEC93
	#define SCAN_PLAY		0xE19E
	#define SCAN_STOP		0x0001
	#define SCAN_PAUSE		0x0001
	#define SCAN_FB 		0x0001
	#define SCAN_FF 		0x0001

	#define SCAN_MUTE		0x0001
	#define SCAN_VOLUP		0x80FF
	#define SCAN_VOLDN		0xC0BF

	#define SCAN_MENU		0xDCA3

	#define SCAN_UP 		0xA9D6
	#define SCAN_DOWN		0xA5DA
	#define SCAN_LEFT		0xCCB3
	#define SCAN_RIGHT		0xC1BE

	#define SCAN_MTS		0x0001
	#define SCAN_SPEED		0x0001

	#define SCAN_ENTER		0xB9C6
	#define SCAN_BACK		0x84FB

#elif defined (NOVATEK_IR)

#define REMOCON_ID		0x007F
#define REMOCON_REPEAT	0x0
#define SCAN_PWR		0xDD22
#define SCAN_LOAD		0xBC43
#define SCAN_SUBTITLE	0xD926
#define SCAN_AUDIO		0xFC03
#define SCAN_ANGLE		0xF807
#define SCAN_INFO		0xB847
#define SCAN_NUM_1		0xC936
#define SCAN_NUM_2		0xE817
#define SCAN_NUM_3		0xA857
#define SCAN_NUM_4		0xC13E
#define SCAN_NUM_5		0xE01F
#define SCAN_NUM_6		0xA05F
#define SCAN_NUM_7		0xD12e
#define SCAN_NUM_8		0xF00F
#define SCAN_NUM_9		0xB04F
#define SCAN_NUM_0		0xCD32
#define SCAN_NUM_10		0xEC13
#define SCAN_GOTO		0xAC53
#define SCAN_TITLE		0xD52A
#define SCAN_UP			0xF40B
#define SCAN_MENU		0xB44B
#define SCAN_LEFT		0xB54A
#define SCAN_ENTER		0xE41B
#define SCAN_RIGHT		0xB946
#define SCAN_SETUP		0xC53A
#define SCAN_DOWN		0xA55A
#define SCAN_RETURN		0xA45B
#define SCAN_STOP		0xAD52
#define SCAN_PLAY		0xA15E
#define SCAN_PAUSE		0xBD42
#define SCAN_FB			0xA956
#define SCAN_FF			0xB14E
#define SCAN_LAST		0xC43B
#define SCAN_NEXT		0x847B
#define SCAN_AB			0xF50A
#define SCAN_REPEAT		0x956A
#define SCAN_STEP		0xD42B
#define SCAN_PBC		0x946B
#define SCAN_ZOOM		0xED12
#define SCAN_MUTE		0x8D72
#define SCAN_VOLDN		0xCC33
#define SCAN_VOLUP		0x8C73
#define SCAN_SLOW		0xF10E
#define SCAN_RANDOM		0x916E
#define SCAN_KEYDEC		0xD02F
#define SCAN_KEYADD		0x906F
#define SCAN_PROGRAM	0xE11E
#define SCAN_CLEAR		0x817E
#define SCAN_RESUME		0xC03F
#define SCAN_VIDEO		0x807F
#define SCAN_PN			0xE916
#define SCAN_DIGEST		0x8976
#define SCAN_HDMI		0xC837
#define SCAN_FUNCTION	0x8877
#define SCAN_EPG		0xF906
#define SCAN_TELETEXT	0x9966
#define SCAN_SERVICE	0xD827
#define SCAN_EXIT		0x9867
#define SCAN_GUIDE		0xFD02
#define SCAN_LIST		0x9D62
#define SCAN_BROWSE		0xDC23
#define SCAN_RADIO		0x9C63
#define SCAN_FADEC		0xE51A
#define SCAN_FAADD		0x857A

#else
#define REMOCON_ID		0x00FF

#define SCAN_PWR		0xBA45
#define SCAN_LOAD		0x7887
#define SCAN_SUBTITLE	0xB24D
#define SCAN_AUDIO		0xF807
#define SCAN_ANGLE		0xF00F
#define SCAN_INFO		0x708F

#define SCAN_KEY1		0x926D
#define SCAN_KEY2		0xD02F
#define SCAN_KEY3		0x50AF
#define SCAN_KEY4		0x827D
#define SCAN_KEY5		0xC03F
#define SCAN_KEY6		0x40BF
#define SCAN_KEY7		0xA25D
#define SCAN_KEY8		0xE01F
#define SCAN_KEY9		0x609F
#define SCAN_KEY0		0x9A65
#define SCAN_KEY10		0xD827

#define SCAN_GOTO		0x58A7
#define SCAN_TITLE		0xAA55
#define SCAN_MENU		0x6897
#define SCAN_UP			0xE817
#define SCAN_DOWN		0x4AB5
#define SCAN_LEFT		0x6A95
#define SCAN_RIGHT		0x728D
#define SCAN_ENTER		0xC837
#define SCAN_SETUP		0x8A75
#define SCAN_RETURN		0x48B7

#define SCAN_STOP		0x5AA5
#define SCAN_PLAY		0x42BD
#define SCAN_PAUSE		0x7A85
#define SCAN_FB			0x52AD
#define SCAN_FF			0x629D
#define SCAN_LAST		0x8877
#define SCAN_NEXT		0x08F7

#define SCAN_AB			0xEA15
#define SCAN_REPEAT		0x2AD5
#define SCAN_STEP		0xA857
#define SCAN_PBC		0x28D7
#define SCAN_ZOOM		0xDA25
#define SCAN_MUTE		0x1AE5
#define SCAN_VOLDN		0x9867
#define SCAN_VOLUP		0x18E7
#define SCAN_SLOW		0xE21D
#define SCAN_RANDOM		0x22DD
#define SCAN_KEYDEC		0xA05F
#define SCAN_KEYADD		0x20DF

#define SCAN_PROGRAM	0xC23D
#define SCAN_CLEAR		0x02FD
#define SCAN_RESUME		0x807F
#define SCAN_VIDEO		0x00FF
#define SCAN_PN			0xD22D
#define SCAN_DIGEST		0x12ED
#define SCAN_HDMI		0x906F
#define SCAN_FUNCTION	0x10EF
#define SCAN_EPG		0xF20D
#define SCAN_TELETEXT	0x32CD
#define SCAN_SERVICE	0xB04F
#define SCAN_EXIT		0x30CF

#define SCAN_GUIDE		0xFA05
#define SCAN_LIST		0x3AC5
#define SCAN_BROWSE		0xB847
#define SCAN_RADIO		0x38C7
#define SCAN_FADEC		0xCA35
#define SCAN_FAADD		0x0AF5

#endif

// Scan-code mapping for keypad
typedef struct {
	unsigned short	rcode;
	unsigned char	vkcode;
}SCANCODE_MAPPING;

/*****************************************************************************
*
* External Variables
*
******************************************************************************/
#define NO_KEY		0
#if defined(BUFFALO_IR) || defined(LG_MKJ36998118) || defined(SUNNIWELL_IR)
static SCANCODE_MAPPING key_mapping[] = 
{
	{SCAN_MENU,		139},

	{SCAN_UP, 		103},
	{SCAN_DOWN,		108},
	{SCAN_LEFT,		105},
	{SCAN_RIGHT,	106},

	{SCAN_ENTER,	28},
	{SCAN_BACK,		158},

	{SCAN_MUTE,		200},
	{SCAN_VOLUP,	115},
	{SCAN_VOLDN,	114},

	{SCAN_MTS,		4},
	{SCAN_SPEED,	10},

	{SCAN_PLAY,		198},
	{SCAN_FB,		201},
	{SCAN_FF, 		202},
	{SCAN_STOP, 	199},

	{SCAN_HOME,		102},
};
#elif defined(CS_2000_IR_X_CANVAS)
static SCANCODE_MAPPING key_mapping[] = 
{
	{SCAN_POWER, 		107},

	{SCAN_NUM_1,		2},		// 1
	{SCAN_NUM_2,		3},		// 2
	{SCAN_NUM_3,		4},		// 3
	{SCAN_NUM_4,		5},		// 4
	{SCAN_NUM_5,		6},		// 5
	{SCAN_NUM_6,		7},		// 6
	{SCAN_NUM_7,		8},		// 7
	{SCAN_NUM_8,		9},		// 8
	{SCAN_NUM_9,		10},	// 9
	{SCAN_NUM_0,		11},	// 0
	{SCAN_NUM_MINUS,	12},	// MINUS
	{SCAN_NUM_PREV,		52},	// PERIOD

	{SCAN_MUTE,			200},	// MUTE
	{SCAN_INPUT,		233},	// F1

	{SCAN_VOLUP,		115},	// VOLUME_UP
	{SCAN_VOLDN,		114},	// VOLUME_DOWN

	{SCAN_CH_UP,		205},	// PAGE_UP
	{SCAN_CH_DN,		206},	// PAGE_DOWN

	{SCAN_SLEEP,		207},	// NOTIFICATION
	{SCAN_CANCEL,		158},	// BACK
	{SCAN_MENU,			139},	// MENU
	{SCAN_GUIDE,		102},	// HOME

	{SCAN_UP,			103},	// DPAD_UP
	{SCAN_DOWN,			108},	// DPAD_DOWN
	{SCAN_LEFT,			105},	// DPAD_LEFT
	{SCAN_RIGHT,		106},	// DPAD_RIGHT
#if 1
	{SCAN_CENTER,		232},	// DPAD_CENTER
#else
	{SCAN_CENTER,		28},	// ENTER
#endif

	{SCAN_AUTO_CHANNEL,	234},	// F2
	{SCAN_ADD_DELETE,	14},	// DEL
	{SCAN_SIZE,			236},	// F4
	{SCAN_MULTI_SOUND,	237},	// F5

	{SCAN_STOP, 		199},	// MEDIA_STOP
	{SCAN_PLAY,			198},	// MEDIA_PLAY_PAUSE
	{SCAN_PAUSE,		198},	// MEDIA_PLAY_PAUSE
	{SCAN_REC,			238},	// F6
#if 1
	{SCAN_FB,			201},	// MEDIA_REWIND
	{SCAN_FF,			202},	// MEDIA_FAST_FORWARD
#else
	{SCAN_FB,			204},	// MEDIA_PREVIOUS
	{SCAN_FF,			203},	// MEDIA_NEXT
#endif
	{SCAN_CURRENT,		15},	// TAB
};
#elif defined(NOVATEK_IR)
static SCANCODE_MAPPING key_mapping[] = 
{
	{SCAN_PWR,	 		116},	// POWER
//	{SCAN_LOAD,			0},		// OPEN/CLOSE
	{SCAN_SUBTITLE,		207},	// NOTIFICATION
	{SCAN_AUDIO,		237},	// F5
	{SCAN_ANGLE,		236},	// F4
	{SCAN_INFO,			233},	// F1
	{SCAN_NUM_1,		2},		// 1
	{SCAN_NUM_2,		3},		// 2
	{SCAN_NUM_3,		4},		// 3
	{SCAN_NUM_4,		5},		// 4
	{SCAN_NUM_5,		6},		// 5
	{SCAN_NUM_6,		7},		// 6
	{SCAN_NUM_7,		8},		// 7
	{SCAN_NUM_8,		9},		// 8
	{SCAN_NUM_9,		10},	// 9
	{SCAN_NUM_0,		11},	// 0
	{SCAN_NUM_10,		12},	// MINUS
	{SCAN_GOTO,			52},	// PERIOD
//	{SCAN_TITLE,		0},		// TITLE
	{SCAN_UP,			103},	// DPAD_UP
	{SCAN_MENU,			139},	// MENU
	{SCAN_LEFT,			105},	// DPAD_LEFT
#if 0
	{SCAN_ENTER,		232},	// DPAD_CENTER
#else
	{SCAN_ENTER,		28},	// ENTER
#endif
	{SCAN_RIGHT,		106},	// DPAD_RIGHT
	{SCAN_SETUP,		102},	// HOME
	{SCAN_DOWN,			108},	// DPAD_DOWN
	{SCAN_RETURN,		158},	// BACK
	{SCAN_STOP,			199},	// STOP
	{SCAN_PLAY,			198},	// PLAY_PAUSE
	{SCAN_PAUSE,		198},	// PLAY_PAUSE
#if 1
	{SCAN_FB,			201},	// MEDIA_REWIND
	{SCAN_FF,			202},	// MEDIA_FAST_FORWARD
#else
	{SCAN_FB,			204},	// MEDIA_PREVIOUS
	{SCAN_FF,			203},	// MEDIA_NEXT
#endif
	{SCAN_LAST,			206},	// PAGE_DOWN
	{SCAN_NEXT,			205},	// PAGE_UP
//	{SCAN_AB,			13},	// AB
//	{SCAN_REPEAT,		13},	// REPEAT
	{SCAN_STEP,			15},	// TAB
//	{SCAN_PBC,			13},	// PBC
//	{SCAN_ZOOM,			13},	// ZOOM
	{SCAN_MUTE,			200},	// MUTE
	{SCAN_VOLDN,		114},	// VOLDN
	{SCAN_VOLUP,		115},	// VOLUP
//	{SCAN_SLOW,			13},	// SLOW
//	{SCAN_RANDOM,		13},	// RANDOM
//	{SCAN_KEYDEC,		13},	// KEYDEC
//	{SCAN_KEYADD,		13},	// KEYADD
//	{SCAN_PROGRAM,		13},	// PROGRAM
	{SCAN_CLEAR,		14},	// DEL
//	{SCAN_RESUME,		13},	// RESUME
//	{SCAN_VIDEO,		13},	// VIDEO
//	{SCAN_PN,			13},	// P/N
//	{SCAN_DIGEST,		13},	// DIGEST
//	{SCAN_HDMI,			13},	// HDMI
//	{SCAN_FUNCTION,		13},	// FUNCTION
//	{SCAN_EPG,			13},	// EPG
//	{SCAN_TELETEXT,		13},	// TELETEXT
//	{SCAN_SERVICE,		13},	// SERVICE
//	{SCAN_EXIT,			13},	// EXIT
//	{SCAN_GUIDE,		102},	// GUIDE
//	{SCAN_LIST,			13},	// LIST
//	{SCAN_BROWSE,		13},	// BROWSE
//	{SCAN_RADIO,		13},	// TV/RADIO
//	{SCAN_FADEC,		13},	// FADEC
//	{SCAN_FAADD,		13},	// FAADD
};
#else
static SCANCODE_MAPPING key_mapping[] = 
{
	{SCAN_PWR, REM_PWR},	
	{SCAN_LOAD, REM_LOAD},
	{SCAN_SUBTITLE, REM_SUBTITLE},
	{SCAN_AUDIO, REM_AUDIO},	
	{SCAN_ANGLE, REM_ANGLE},
	{SCAN_INFO, REM_INFO},

	{SCAN_KEY1, REM_KEY1},	//"1"
	{SCAN_KEY2, REM_KEY2},	//"2"
	{SCAN_KEY3, REM_KEY3},	//"3"
	{SCAN_KEY4, REM_KEY4},	//"4"
	{SCAN_KEY5, REM_KEY5},	//"5"
	{SCAN_KEY6, REM_KEY6},	//"6"
	{SCAN_KEY7, REM_KEY7},	//"7"
	{SCAN_KEY8, REM_KEY8},	//"8"
	{SCAN_KEY9, REM_KEY9},	//"9"
	{SCAN_KEY0, REM_KEY0},	//"0"
	{SCAN_KEY10, REM_KEY10},//"10+"

	{SCAN_GOTO, REM_GOTO},	//GOTO
	{SCAN_TITLE, REM_TITLE},
	{SCAN_MENU, REM_MENU},
	{SCAN_UP, REM_UP},	
	{SCAN_DOWN, REM_DOWN},	
	{SCAN_LEFT, REM_LEFT},	
	{SCAN_RIGHT, REM_RIGHT},
	{SCAN_ENTER, REM_ENTER},
	{SCAN_SETUP, REM_SETUP},
	{SCAN_RETURN, REM_RETURN},
                    	
	{SCAN_STOP, REM_STOP},
	{SCAN_PLAY, REM_PLAY},
	{SCAN_PAUSE, REM_PAUSE},
	{SCAN_FB, REM_FB},
	{SCAN_FF, REM_FF},
	{SCAN_LAST, REM_LAST},
	{SCAN_NEXT, REM_NEXT},

	{SCAN_AB, REM_AB},
	{SCAN_REPEAT, REM_REPEAT},
	{SCAN_STEP, REM_STEP},	
	{SCAN_PBC, REM_PBC},
	{SCAN_ZOOM, REM_ZOOM},	
	{SCAN_MUTE, REM_MUTE},
	{SCAN_VOLDN, REM_VOLDN},	
	{SCAN_VOLUP, REM_VOLUP},
	{SCAN_SLOW, REM_SLOW},	
	{SCAN_RANDOM, REM_RANDOM},	
	{SCAN_KEYDEC, REM_KEYDEC},	
	{SCAN_KEYADD, REM_KEYADD},	

	{SCAN_PROGRAM, REM_PROGRAM},
	{SCAN_CLEAR, REM_CLEAR},	
	{SCAN_RESUME, REM_RESUME},	
	{SCAN_VIDEO, REM_VIDEO},	
	{SCAN_PN, REM_PN},
	{SCAN_DIGEST, REM_DIGEST},	
	{SCAN_HDMI, REM_HDMI},	
	{SCAN_FUNCTION, REM_FUNCTION},
	{SCAN_EPG, REM_EPG},	
	{SCAN_TELETEXT, REM_TELETEXT},
	{SCAN_SERVICE, REM_SERVICE},	
	{SCAN_EXIT, REM_EXIT},	

	{SCAN_GUIDE, REM_GUIDE},
	{SCAN_LIST, REM_LIST},	
	{SCAN_BROWSE, REM_BROWSE},
	{SCAN_RADIO, REM_RADIO},
	{SCAN_FADEC, REM_FADEC},
	{SCAN_FAADD, REM_FAADD},	
/*
	{SCAN_MODE, REM_MODE},	
	{SCAN_LANG, REM_LANG},
	{SCAN_CHAR, REM_CHAR},	
*/
};
#endif
 void    RemoconCommandOpen(void);
 void    RemoconCommandReset(void);
 void	RemoconConfigure(void);
 void    RemoconStatus(void);
 void	RemoconDivide(void);
 void    DisableRemocon(void);
 void	RemoconReset(void);
 void	RemoconInit(void);
 void	RemoconIrqClear(void);
 void	RemoconIntClear(void);
 void	RemoconIntWait(void);
 
