/*====*====*====*====*====*====*====*====*====*====*====*====*====*====*====*

               CAMERA    API    M O D U L E

                        EDIT HISTORY FOR MODULE

when        who       what, where, why
--------    ---       -------------------------------------------------------
10/xx/08  Telechips   Created file.
*====*====*====*====*====*====*====*====*====*====*====*====*====*====*====*/

/*===========================================================================

                           INCLUDE FILES FOR MODULE

===========================================================================*/
#include <linux/delay.h>
#include <asm/system.h>
#include <mach/hardware.h>
#include <asm/io.h>
#include "sensor_if.h"
#include "cam.h"
#include "tcc_cam_i2c.h"

#if defined(CONFIG_ARCH_TCC92X) || defined(CONFIG_ARCH_TCC93XX)
#include <mach/bsp.h>
#elif defined(CONFIG_ARCH_TCC79X)
#include <mach/tcc79x.h>
#endif

#ifdef CONFIG_VIDEO_CAMERA_SENSOR_S5K4BAFB

#if (0)
#if defined(CONFIG_MACH_TCC9200S)
#define CAM_PWR_ENABLE()		(HwGPIOD->GPEN |= Hw10);	(HwGPIOD->GPDAT |= Hw10);
#define CAM_PWR_DISABLE()		(HwGPIOD->GPEN |= Hw10);	(HwGPIOD->GPDAT &= ~Hw10);
#define CAM_PWRDN_ENABLE()		(HwGPIOF->GPEN |= Hw21);	(HwGPIOF->GPDAT |= Hw21);
#define CAM_PWRDN_DISABLE()		(HwGPIOF->GPEN |= Hw21);	(HwGPIOF->GPDAT &= ~Hw21);
#define CAM_RESET_HIGH()		(HwGPIOE->GPEN |= Hw2);		(HwGPIOE->GPDAT |= Hw2);
#define CAM_RESET_LOW()			(HwGPIOE->GPEN |= Hw2);		(HwGPIOE->GPDAT &= ~Hw2);
#elif defined(CONFIG_MACH_TCC9200) || defined(CONFIG_MACH_TCC9201) || defined(CONFIG_MACH_TCC8900) 
#define CAM_PWR_ENABLE()
#define CAM_PWR_DISABLE()
#define CAM_PWRDN_ENABLE()		//(HwGPIOF->GPEN |= Hw21);	(HwGPIOF->GPDAT |= Hw21);
#define CAM_PWRDN_DISABLE()		//(HwGPIOF->GPEN |= Hw21);	(HwGPIOF->GPDAT &= ~Hw21);
#define CAM_RESET_HIGH()		(HwGPIOE->GPEN |= Hw26);	(HwGPIOE->GPDAT |= Hw26);
#define CAM_RESET_LOW()			(HwGPIOE->GPEN |= Hw26);	(HwGPIOE->GPDAT &= ~Hw26);
#else
#define CAM_PWR_ENABLE()
#define CAM_PWR_DISABLE()
#define CAM_PWRDN_ENABLE()		(HwGPFEN |= Hw1); 	(HwGPFDAT |= Hw1);
#define CAM_PWRDN_DISABLE()		(HwGPFEN |= Hw1); 	(HwGPFDAT &= ~Hw1);
#define CAM_RESET_HIGH()		(HwGPEEN |= Hw9); 	(HwGPEDAT |= Hw9);
#define CAM_RESET_LOW()			(HwGPEEN |= Hw9); 	(HwGPEDAT &= ~Hw9);
#endif
#endif

/* Array of image sizes supported by MV9317.  These must be ordered from 
 * smallest image size to largest.
 */
struct capture_size sensor_sizes[] = {
	{ 1600, 1200 },	/* UXGA */
	{ 1280,  960 },	/* SXGA */
	{ 1024,  768 },	/* XGA */
	{  800,  600 },	/* SVGA */
	{  640,  480 },	/* VGA */
	{  320,  240 },	/* QVGA */
	{  176,  144 },	/* QCIF */
};


/* register initialization tables for sensor */
/* common sensor register initialization for all image sizes, pixel formats, 
 * and frame rates
 */
static struct sensor_reg sensor_initialize[] = {
#ifdef	TCC_VCORE_30FPS_CAMERASENSOR		// // LG, LX600/VX8560 Camera
	//==========================================================
	//  Name         :  for s5K4BAFB EVT1(UXGA 1-Chip)	 
	//  Version      :  v2.0 UXGA				    
	//  S/W version  :  s02				    
	//  PLL mode     :  MCLK 24MHz PCLK 32MHz)		   
	//  AGC gain     :  s66					    
	//  FPS          :  UXGA 7.5fps				   
	//  Date         :  2008.01.11				    
	//==========================================================

	//History
	//Gamma_Lee 
	//==========================================================
	//	PLL SET 
	//==========================================================
	{0xfc,0x02},
	{0x50,0x59}, // Mclk:25M, Pclk:36M  
	{0x51,0x0a}, // s=1, p=25           
	{0x52,0xD0}, // 31 fps,

	//==========================================================
	//	 CAMERA INITIAL 
	//==========================================================
	{0xfc,0x07},
	{0x66,0x01}, // Watch dog timer [0:off  1:on]   
	{0xfc,0x00},                                    
	{0x00,0xAA}, // For EDS Check                   
	{0x21,0x03},	                                   
	{0xfc,0x01},                                    
	{0x04,0x01}, // ARM Clock Divider                
	{0xfc,0x07},                                    
	{0x05,0x00}, // AE initial_HH                   
	{0x06,0x00}, // AE initial_HL                   
	{0x07,0x8b}, // AE initial_LH                   
	{0x08,0xf5}, // AE initial_LL                   
	{0x09,0x00},                                    
	{0x0a,0xb4}, // Rgain initial point             
	{0x0b,0x00},                                    
	{0x0c,0xea}, // Bgain initial point             
	{0x0d,0x00},                                    
	{0x0e,0x40}, // Ggain initial point             
	{0xfc,0x00},                                                   
	{0x70,0x02}, // Initial uploading               

	//==========================================================
	//	 Analog setting
	//==========================================================
	{0xfc,0x02},	      
	{0x55,0x1E}, // LineADLC on(s551a) off(s550a)        
	{0x56,0x10}, // BPR 16code                            
	{0x30,0x82}, // Analog offset (capture =?h)          
	{0x37,0x11}, //17	// Global Gain (default:31)             
	{0x57,0x80}, // LineADLC Roffset               
	{0x58,0x80}, // LineADLC Goffset           
	{0x59,0x80}, // LineADLC offset don't care  
	{0x44,0x64}, // clamp en[6]=1 on                      
	{0x4a,0x40}, // clamp level 0011h [7]~[4]              
	{0x2d,0x48}, // double shutter (default:00)          
	{0x4D,0x08}, // Voltage doubler (default:04)          
	{0x4e,0x1a}, // IO current 8mA set                   
	{0x4f,0xba}, // IO current 48mA set                  
	{0x66,0x41}, // 1st comp current 2uA                  
	{0x43,0xEF}, // ec_comp                               
	{0x62,0x60}, // LD control  CFPN_EN off              
	{0xfc,0x20},     
	{0x09,0x03}, // onejump delay for AE haunting    

	//==========================================================//
	//	 COMMAND SET                                         //
	//==========================================================//
	{0xfc,0x00},
	{0x20,0x02}, // Change AWB Mode
	{0x29,0x03}, // Y level   
	{0x2a,0xd0}, //ec}, //	
	{0x2b,0x04}, // C level   
	{0x2c,0x00}, //
	{0x62,0x02}, // Hue Control Enable
	{0x6c,0xc0}, // AE target, revised for Flicker 
	{0x6d,0x00}, 
//	{0x73,0x11}, // Frmae AE Enable	 , frame rate control, VFR
    {0x73,0x10}, // Frmae AE Enable	 , frame rate control, CFR
	{0x78,0x66}, // AGC Max 
	{0xfc,0x20}, 
	{0x16,0x5a}, // AGC frame AE start _for Prevating AE Hunting 
	{0x57,0x18}, // Stable_Frame_AE      
	{0xfc,0x00}, 
	{0x83,0x06}, // low condition shutter off  Double shutter off 
	             
	{0xfc,0x0b}, 
	{0x5c,0x65}, // AGC value to start shutter on/off suppress 
	{0x5d,0x65}, // AGC value to start double shutter on/off suppress                              
	{0xfc,0x20},  
	{0x25,0x00}, // CINTR Min         
	{0x2a,0x01}, // forbidden          
	{0x2b,0x02}, // For Forbidden Area 
	{0x2c,0x0a}, 
	{0x2d,0x00}, // For Forbidden Area  
	{0x2e,0x00}, 
	{0x2f,0x05}, // forbidden   
	{0x14,0x80}, // Brightness offset
	{0x01,0x00}, // Stepless_Off   
	{0x02,0x02}, // Flicker Dgain Mode     
	             
	{0xfc,0x07}, 
	{0x37,0x08}, // Flicker setting 	             
	{0xfc,0x00}, 
	{0x72,0xac}, // Flicker for 36MHz Revised for Flicker
	{0x74,0x08}, // flicker 60Hz    
	             
	{0xfc,0x01}, 
	{0x0c,0x02}, // Full YC Enable   
	             
	//==========================================================   
	//	 Table Set for Sub-Sampling                          
	//==========================================================  
	{0xfc,0x03},
	{0x00,0x06},
	{0x01,0xe2},
	{0x05,0x46},
	{0x07,0xb6},
	{0x0e,0x04},
	{0x12,0x03},
	{0x2c,0x00},

	{0xfc,0x03},
	{0x3d,0x00}, // POST_HSTART_H
	{0x3e,0x02}, // POST_HSTART_L
	{0x3f,0x06}, // POST_HWIDTH_H
	{0x40,0x40}, // POST_HWIDTH_L
	{0x41,0x00}, // POST_VSTART_H
	{0x42,0x02}, // POST_VSTART_L
	{0x43,0x04}, // POST_VHEIGHT_H
	{0x44,0xb0}, // POST_VHEIGHT_L
	{0x45,0x06}, // IMG_HSIZEH
	{0x46,0x40}, // IMG_HSIZEL
	{0x47,0x04}, // IMG_VSIZEH
	{0x48,0xb0}, // IMG_VSIZEL
	{0x49,0x00}, // PSF
	{0x4a,0x40}, // MSFXH
	{0x4b,0x00}, // MSFXL
	{0x4c,0x40}, // MSFYH
	{0x4d,0x00}, // MSFYL
	{0x4e,0x05}, // DW_HSIZEH
	{0x4f,0x00}, // DW_HSIZEL
	{0x50,0x03}, // DW_VSIZEH
	{0x51,0xc0}, // DW_VSIZEL
	{0x52,0x00}, // STARTXH
	{0x53,0x00}, // STARTXL
	{0x54,0x00}, // STARTYH
	{0x55,0x00}, // STARTYL
	{0x56,0x05}, // CLIP_HSIZEH
	{0x57,0x00}, // CLIP_HSIZEL
	{0x58,0x03}, // CLIP_VSIZEH
	{0x59,0xc0}, // CLIP_VSIZEL
	{0x5a,0x00}, // SEL_MAIN
	{0x5b,0x00}, // HTERMH
	{0x5c,0x08}, // HTERMM
	{0x5d,0x9c}, // HTERML

	{0xfc,0x04},
	{0xc0,0x05}, //04}, //05 
	{0xc1,0x40}, 
	{0xc2,0x02}, 
	{0xc3,0x87}, 
	{0xc4,0x03}, 
	{0xc5,0x26}, 
	{0xc6,0x02}, 
	{0xc7,0x5c}, 
	{0xc8,0x01}, 
	{0xc9,0x03}, 
	{0xca,0x01}, 
	{0xcb,0x03}, 
	{0xcc,0x08},
	{0xce,0x01},
	{0xd2,0x01},

	{0xfc,0x03},  //snap shot: YCbCr order
	{0x2e,0x01},	
	{0x5e,0x00},
	{0x8e,0x00},
	{0xbe,0x00},
	{0xee,0x00},

	{0xfc,0x04},  //preview : YCbCr order
	{0x2e,0x01},
	{0x5e,0x01},
	{0x8e,0x01},
	{0xbe,0x01},
	{0xee,0x00},  

	//==========================================================
	//	 COLOR MATRIX                                        
	//==========================================================
	{0xfc,0x01},		
	{0x51,0x07},			
	{0x52,0x40},			
	{0x53,0xfc},			
	{0x54,0x80},			
	{0x55,0x00},			
	{0x56,0x3f},			
	{0x57,0xfe},			
	{0x58,0x7f},			
	{0x59,0x06},			
	{0x5A,0x6a},			
	{0x5B,0xff},			
	{0x5C,0x18},			
	{0x5D,0xff},			
	{0x5E,0xd7},			
	{0x5F,0xfd},			
	{0x60,0x1c},			
	{0x61,0x07},			
	{0x62,0x0d},			               

	//==========================================================
	//	 EDGE ENHANCEMENT                                       
	//==========================================================
	{0xfc,0x0b}, //KF310
	{0x42,0x50}, //50 //50},   // Edge AGC MIN              
	{0x43,0x60}, //60 //60},   // Edge AGC MAX           
	{0x45,0x0a}, //04 //0a},   //24  // positive gain AGC MIN 
	{0x49,0x06}, //0a //06},   // positive gain AGC MAX  
	{0x4d,0x09}, //10 //0a},   //24  // negative gain AGC MIN  
	{0x51,0x06}, //0a //06},   // negative gain AGC MAX  
	{0xfc,0x05}, //05 
	{0x33,0x08}, //08  // APTCLP  
	{0x34,0x38}, //40 //38},  //24  // APTCLP  
	{0x35,0x0a}, //08 //08},  //04  // APTSC   
	{0x36,0x1a}, //1f //1f},  //0b  // ENHANCE 
	{0x3f,0x00}, //00  // NON-LIN 
	{0x42,0x10}, //10  // EGFALL  
	{0x43,0x00}, //00  // HLFALL  
	{0x45,0xa0}, //a0  // EGREF   
	{0x46,0x7a}, //7a  // HLREF   
	{0x47,0x40}, //40  // LLREF   
	{0x48,0x0c}, //0c    
	{0x49,0x31}, //31  // CSSEL  EGSEL  CS_DLY 
	{0x40,0x41}, //41  // Y delay  

	//========================================================== 
	//	 NEW EDGE ENHANCEMENT                                       
	//========================================================== 	
	{0xfc,0x1d},		
	{0x86,0x02},		
	{0x87,0xc0},		
	{0x88,0x03},		
	{0x89,0xc0},		
	{0x8a,0x02},		
	{0x8b,0x00},		
	{0x8c,0x00},		
	{0x8d,0x00},		
	{0x8e,0x00},		
	{0x8f,0xd0},		
	{0x90,0x00},		
	{0x91,0x00},		
	{0x92,0x02},		
	{0x93,0x00},		
	{0x94,0x00},		
	{0x95,0x00},		
	{0x96,0x00},		
	{0x97,0xd0},		
	{0x98,0x00},		
	{0x99,0x00},		
	{0x9a,0x02},		
	{0x9b,0x60},		
	{0x9c,0x00},		
	{0x9d,0x00},		
	{0x9e,0x02},		
	{0x9f,0x60},		
	{0xa0,0x00},		
	{0xa1,0x00},		
	{0xa2,0x09},		
	{0x85,0x01},  // EE_Wide_Luma ON/OFF 1:ON, 0:OFF
	{0xfc,0x00},
	{0x89,0x03},  // Edge negative/positive suppress ON

	//========================================================== 
	//	 GAMMA                                                
	//==========================================================                  
	{0xfc,0x1d},
	{0x00,0x05},  
	{0x01,0x0a},  
	{0x02,0x33},  
	{0x03,0xbd},  
	{0x04,0x00},  
	{0x05,0x33},  
	{0x06,0x8f},  
	{0x07,0xdc},  
	{0x08,0x15},  
	{0x09,0x56},  
	{0x0a,0x38},  
	{0x0b,0x57},  
	{0x0c,0x74},  
	{0x0d,0x8f},  
	{0x0e,0xaa},  
	{0x0f,0xa9},  
	{0x10,0xc2},  
	{0x11,0xdb},  
	{0x12,0xf3},  
	{0x13,0xaa},  
	{0x14,0x0a},  
	{0x15,0x22},  
	{0x16,0x3d},  
	{0x17,0x57},  
	{0x18,0xff},  
	{0x19,0x6b},  
	{0x1a,0x7a},  
	{0x1b,0x8f},  
	{0x1c,0x9e},  
	{0x1d,0xff},  
	{0x1e,0xa8},  
	{0x1f,0xb8},  
	{0x20,0xc3},  
	{0x21,0xd3},  
	{0x22,0xff},  
	{0x23,0xe0},  
	{0x24,0xeb},  
	{0x25,0xf2},  
	{0x26,0xf8},  
	{0x27,0xff},  
	{0x28,0xfc},  
	{0x29,0xff},  
	{0x2a,0xf0},  
	{0x2b,0x00},  
	{0x2c,0x0a},  
	{0x2d,0x33},  
	{0x2e,0xbd},  
	{0x2f,0x00},  
	{0x30,0x33},  
	{0x31,0x8f},  
	{0x32,0xdc},  
	{0x33,0x15},  
	{0x34,0x56},  
	{0x35,0x38},  
	{0x36,0x57},  
	{0x37,0x74},  
	{0x38,0x8f},  
	{0x39,0xaa},  
	{0x3a,0xa9},  
	{0x3b,0xc2},  
	{0x3c,0xdb},  
	{0x3d,0xf3},  
	{0x3e,0xaa},  
	{0x3f,0x0a},  
	{0x40,0x22},  
	{0x41,0x3d},  
	{0x42,0x57},  
	{0x43,0xff},  
	{0x44,0x6b},  
	{0x45,0x7a},  
	{0x46,0x8f},  
	{0x47,0x9e},  
	{0x48,0xff},  
	{0x49,0xa8},  
	{0x4a,0xb8},  
	{0x4b,0xc3},  
	{0x4c,0xd3},  
	{0x4d,0xff},  
	{0x4e,0xe0},  
	{0x4f,0xeb},  
	{0x50,0xf2},  
	{0x51,0xf8},  
	{0x52,0xff},  
	{0x53,0xfc},  
	{0x54,0xff},  
	{0x55,0xf0},  
	{0x56,0x00},  
	{0x57,0x0a},  
	{0x58,0x33},  
	{0x59,0xbd},  
	{0x5a,0x00},  
	{0x5b,0x33},  
	{0x5c,0x8f},  
	{0x5d,0xdc},  
	{0x5e,0x15},  
	{0x5f,0x56},  
	{0x60,0x38},  
	{0x61,0x57},  
	{0x62,0x74},  
	{0x63,0x8f},  
	{0x64,0xaa},  
	{0x65,0xa9},  
	{0x66,0xc2},  
	{0x67,0xdb},  
	{0x68,0xf3},  
	{0x69,0xaa},  
	{0x6a,0x0a},  
	{0x6b,0x22},  
	{0x6c,0x3d},  
	{0x6d,0x57},  
	{0x6e,0xff},  
	{0x6f,0x6b},  
	{0x70,0x7a},  
	{0x71,0x8f},  
	{0x72,0x9e},  
	{0x73,0xff},  
	{0x74,0xa8},  
	{0x75,0xb8},  
	{0x76,0xc3},  
	{0x77,0xd3},  
	{0x78,0xff},  
	{0x79,0xe0},  
	{0x7a,0xeb},  
	{0x7b,0xf2},  
	{0x7c,0xf8},  
	{0x7d,0xff},  
	{0x7e,0xfc},  
	{0x7f,0xff},  
	{0x80,0xf0},                

	//==========================================================    
	//   HUE CONTROL                                          
	//========================================================== 
	{0xfc,0x00},                               
	{0x48,0x40},	 // 2000K 
	{0x49,0x40}, 
	{0x4a,0xF0}, 
	{0x4b,0x00}, 
	{0x4c,0x40}, 
	{0x4d,0x40}, 
	{0x4e,0x00}, 
	{0x4f,0x00}, 
	             
	{0x50,0x48}, 	 // 3000K          
	{0x51,0x3e}, 	
	{0x52,0xf6}, 	
	{0x53,0x00}, 	
	{0x54,0x40}, 	
	{0x55,0x40},                    
	{0x56,0x00}, 	                   
	{0x57,0xf0},    

	{0x58,0x35}, //36}, //3a}, //3d}, //40},   // 5000K          
	{0x59,0x4a}, //40},//3a},//37}, //35},          
	{0x5a,0xfc},	
	{0x5b,0x00}, 
	{0x5c,0x40}, 
	{0x5d,0x3a}, 
	{0x5e,0x00},          
	{0x5f,0x08},//00}, //f8},//f4}, //f0},                                

	//========================================================== //   
	//	SUPPRESS FUNCTION                                    //
	//========================================================== //
	{0xfc,0x00},	    
	{0x7e,0xf4},                                                            

	//========================================================== //   
	//	 BPR                                                  // 
	//========================================================== // 
	{0xfc,0x01},         
	{0x3d,0x10},             
	{0xfc,0x0b},   
	{0x0b,0x00},	// ISP BPR On Start                  
	{0x0c,0x40},	// Th13 AGC Min                     
	{0x0d,0x5a},	// Th13 AGC Max                     
	{0x0e,0x00},	// Th1 Max H for AGCMIN             
	{0x0f,0x15},	// 0822 semi Th1 Max L for AGCMIN    
	{0x10,0x00},	// Th1 Min H for AGCMAX             
	{0x11,0x20},	// 0822 semi Th1 Min L for AGCMAX   
	{0x12,0x00},	// Th3 Max H for AGCMIN             
	{0x13,0x7f},	// Th3 Max L for AGCMIN             
	{0x14,0x03},	// Th3 Min H for AGCMAX             
	{0x15,0xff},	// Th3 Min L for AGCMAX             
	{0x16,0x48},	// Th57 AGC Min                     
	{0x17,0x60},	// Th57 AGC Max                     
	{0x18,0x00},	// Th5 Max H for AGCMIN             
	{0x19,0x00},	// Th5 Max L for AGCMIN             
	{0x1a,0x00},	// Th5 Min H for AGCMAX             
	{0x1b,0x20},	// Th5 Min L for AGCMAX             
	{0x1c,0x00},	// Th7 Max H for AGCMIN             
	{0x1d,0x00},	// Th7 Max L for AGCMIN             
	{0x1e,0x00},	// Th7 Min H for AGCMAX             
	{0x1f,0x20},	// Th7 Min L for AGCMAX 	                           

	//========================================================== // 	  
	//	 GR/GB CORRECTION                                     // 
	//========================================================== //
	{0xfc,0x01}, 
	{0x45,0x0c},   	                       
	{0xfc,0x0b},                                  
	{0x21,0x00},	// Start AGC                   
	{0x22,0x05},	// AGCMIN                      
	{0x23,0x33},	// AGCMAX                      
	{0x24,0x0a},	// G Th AGCMIN                 
	{0x25,0x25},	// G Th AGCMAX                 
	{0x26,0x0a},	// RB Th AGCMIN                
	{0x27,0x25},	// RB Th AGCMAX                             

	//==========================================================//   
	//	 NR                                                  //   
	//==========================================================// 
	{0xfc,0x01},   
	{0x4C,0x01},	// NR Enable   
	{0x49,0x15},	// Sig_Th Mult 
	{0x4B,0x0A},	// Pre_Th Mult 

	{0xfc,0x0b},                              
	{0x28,0x00},	// NR start AGC	                           
	{0x29,0x00},	// SIG Th AGCMIN H       
	{0x2a,0x14},	// SIG Th AGCMIN L       
	{0x2b,0x00},	// SIG Th AGCMAX H       
	{0x2c,0x14},	// SIG Th AGCMAX L       
	{0x2d,0x00},	// PRE Th AGCMIN H       
	{0x2e,0xc0},	// PRE Th AGCMIN L     
	{0x2f,0x01},	// PRE Th AGCMAX H   
	{0x30,0x00},	// PRE Th AGCMAX L   
	{0x31,0x00},	// POST Th AGCMIN H      
	{0x32,0xd0},	// POST Th AGCMIN L   
	{0x33,0x01},	// POST Th AGCMAX H      
	{0x34,0x10},	// POST Th AGCMAX L                 

	//========================================================== //
	//	 1D-Y/C-SIGMA-LPF                                     //
	//========================================================== //
	{0xfc,0x01},                             
	{0x05,0xC0}, 

	{0xfc,0x0b},                             
	{0x35,0x00},	// YLPF start AGC         
	{0x36,0x20},	// YLPF01 AGCMIN          
	{0x37,0x50},	// YLPF01 AGCMAX          
	{0x38,0x00},	// YLPF SIG01 Th AGCMINH  
	{0x39,0x18},	// YLPF SIG01 Th AGCMINL   
	{0x3a,0x00},	// YLPF SIG01 Th AGCMAXH   
	{0x3b,0x40},	// YLPF SIG01 Th AGCMAXH               
	{0x3c,0x20},	// YLPF02 AGCMIN                   
	{0x3d,0x50},	// YLPF02 AGCMAX                   
	{0x3e,0x00},	// YLPF SIG02 Th AGCMINH   
	{0x3f,0x30},	// YLPF SIG02 Th AGCMINL   
	{0x40,0x00},	// YLPF SIG02 Th AGCMAXH   
	{0x41,0x40},	// YLPF SIG02 Th AGCMAXH   
	{0xd4,0x20},	// CLPF AGCMIN            
	{0xd5,0x50},	// CLPF AGCMAX            
	{0xd6,0xb0},	// CLPF SIG01 Th AGCMIN   
	{0xd7,0xf0},	// CLPF SIG01 Th AGCMAX   
	{0xd8,0xb0},	// CLPF SIG02 Th AGCMIN   
	{0xd9,0xf0},	// CLPF SIG02 Th AGCMAX                            

	//==========================================================  //
	//	COLOR SUPPRESS                                        //
	//==========================================================  //
	{0xfc,0x0b},	// COLOR SUPPRESS         
	{0x08,0x58},	// Color suppress AGC MIN 
	{0x09,0x04},	// Color suppress MIN H   
	{0x0a,0x00},	// Color suppress MIN L                            

	//==========================================================  //
	//	 SHADING                                               //
	//==========================================================  //
	{0xfc,0x09},
	//Shading file for 3BAFX
	//90000  // shading off
	// DSP9_SH_WIDTH_H 
	{0x01,0x06},
	{0x02,0x40},
	// DSP9_SH_HEIGHT_H 
	{0x03,0x04},
	{0x04,0xB0},
	// DSP9_SH_XCH_R 
	{0x05,0x03},
	{0x06,0x20},
	{0x07,0x02},
	{0x08,0xa5},
	// DSP9_SH_XCH_G 
	{0x09,0x03},
	{0x0A,0x2a},
	{0x0B,0x02},
	{0x0C,0x78},
	// DSP9_SH_XCH_B 
	{0x0D,0x02},
	{0x0E,0xF9},
	{0x0F,0x02},
	{0x10,0x58},
	// DSP9_SH_Del_eH_R 
	{0x1D,0x80},
	{0x1E,0x00},
	{0x1F,0x80},
	{0x20,0x00},
	{0x23,0x79},
	{0x24,0xE6},
	{0x21,0x82},
	{0x22,0x9A},
	// DSP9_SH_Del_eH_G 
	{0x25,0x80},
	{0x26,0x00},
	{0x27,0x80},
	{0x28,0x00},
	{0x2B,0x79},
	{0x2C,0xE6},
	{0x29,0x86},
	{0x2A,0xBC},
	// DSP9_SH_Del_eH_B 
	{0x2D,0x79},
	{0x2E,0xE6},
	{0x2F,0x78},
	{0x30,0xBE},
	{0x33,0x80},
	{0x34,0x00},
	{0x31,0x80},
	{0x32,0x00},
	//, DSP9_SH_VAL_R0H 
	{0x35,0x01},
	{0x36,0x00},
	{0x37,0x01},
	{0x38,0x10},
	{0x39,0x01},
	{0x3A,0x35},
	{0x3B,0x01},
	{0x3C,0x68},
	{0x3D,0x01},
	{0x3E,0x84},
	{0x3F,0x01},
	{0x40,0xA3},
	{0x41,0x01},
	{0x42,0xC7},
	{0x43,0x01},
	{0x44,0xEC},
	// DSP9_SH_VAL_G0H 
	{0x45,0x01},
	{0x46,0x00},
	{0x47,0x01},
	{0x48,0x0C},
	{0x49,0x01},
	{0x4A,0x29},
	{0x4B,0x01},
	{0x4C,0x4D},
	{0x4D,0x01},
	{0x4E,0x5F},
	{0x4F,0x01},
	{0x50,0x78},
	{0x51,0x01},
	{0x52,0x8C},
	{0x53,0x01},
	{0x54,0xA7},
	// DSP9_SH_VAL_B0H 
	{0x55,0x01},
	{0x56,0x00},
	{0x57,0x01},
	{0x58,0x0A},
	{0x59,0x01},
	{0x5A,0x23},
	{0x5B,0x01},
	{0x5C,0x43},
	{0x5D,0x01},
	{0x5E,0x57},
	{0x5F,0x01},
	{0x60,0x6A},
	{0x61,0x01},
	{0x62,0x7E},
	{0x63,0x01},
	{0x64,0x9C},
	// DSP9_SH_M_R2_R1H 
	{0x65,0x00},
	{0x66,0xA5},
	{0x67,0xDF},
	{0x68,0x02},
	{0x69,0x97},
	{0x6A,0x7E},
	{0x6B,0x05},
	{0x6C,0xD4},
	{0x6D,0xDB},
	{0x6E,0x07},
	{0x6F,0xEF},
	{0x70,0xF2},
	{0x71,0x0A},
	{0x72,0x5D},
	{0x73,0xF8},
	{0x74,0x0D},
	{0x75,0x1E},
	{0x76,0xED},
	{0x77,0x10},
	{0x78,0x32},
	{0x79,0xD3},
	// DSP9_SH_M_R2_G1H 
	{0x7A,0x00},
	{0x7B,0x87},
	{0x7C,0x7C},
	{0x7D,0x02},
	{0x7E,0x1D},
	{0x7F,0xEE},
	{0x80,0x04},
	{0x81,0xC3},
	{0x82,0x58},
	{0x83,0x06},
	{0x84,0x7B},
	{0x85,0xAA},
	{0x86,0x08},
	{0x87,0x77},
	{0x88,0xB9},
	{0x89,0x0A},
	{0x8A,0xB7},
	{0x8B,0x87},
	{0x8C,0x0D},
	{0x8D,0x3B},
	{0x8E,0x12},
	// DSP9_SH_M_R2_B1H 
	{0x8F,0x00},
	{0x90,0x93},
	{0x91,0x91},
	{0x92,0x02},
	{0x93,0x4E},
	{0x94,0x42},
	{0x95,0x05},
	{0x96,0x30},
	{0x97,0x15},
	{0x98,0x07},
	{0x99,0x0F},
	{0x9A,0xAB},
	{0x9B,0x09},
	{0x9C,0x39},
	{0x9D,0x09},
	{0x9E,0x0B},
	{0x9F,0xAC},
	{0xA0,0x2F},
	{0xA1,0x0E},
	{0xA2,0x69},
	{0xA3,0x1E},
	// DSP9_SH_SUB_RR0H 
	{0xA4,0x62},
	{0xA5,0xC6},
	{0xA6,0x20},
	{0xA7,0xEC},
	{0xA8,0x13},
	{0xA9,0xC1},
	{0xAA,0x1E},
	{0xAB,0x64},
	{0xAC,0x1A},
	{0xAD,0x56},
	{0xAE,0x17},
	{0xAF,0x3D},
	{0xB0,0x14},
	{0xB1,0xCB},
	// DSP9_SH_SUB_RG0H 
	{0xB2,0x78},
	{0xB3,0xEE},
	{0xB4,0x28},
	{0xB5,0x4F},
	{0xB6,0x18},
	{0xB7,0x2F},
	{0xB8,0x25},
	{0xB9,0x35},
	{0xBA,0x20},
	{0xBB,0x3F},
	{0xBC,0x1C},
	{0xBD,0x74},
	{0xBE,0x19},
	{0xBF,0x75},
	// DSP9_SH_SUB_RB0H 
	{0xC0,0x6F},
	{0xC1,0x07},
	{0xC2,0x25},
	{0xC3,0x02},
	{0xC4,0x16},
	{0xC5,0x34},
	{0xC6,0x22},
	{0xC7,0x29},
	{0xC8,0x1D},
	{0xC9,0x9B},
	{0xCA,0x1A},
	{0xCB,0x1F},
	{0xCC,0x17},
	{0xCD,0x5F},

	{0x00,0x02},	// shading on     

	//==========================================================// 
	//	 X-SHADING                                           // 
	//==========================================================//
	{0xfc,0x1B},	// X-SHADING 
	{0x80,0x01},            
	{0x81,0x00},
	{0x82,0x4c},
	{0x83,0x00},
	{0x84,0x86},
	{0x85,0x03},
	{0x86,0x5e},
	{0x87,0x00},
	{0x88,0x07},
	{0x89,0xa4},
	{0x90,0x00},
	{0x91,0xd3},
	{0x92,0x01},
	{0x93,0x09},
	{0x94,0x01},
	{0x95,0x3a},
	{0x96,0x01},
	{0x97,0x5c},
	{0x98,0x01},
	{0x99,0x70},
	{0x9a,0x01},
	{0x9b,0x5f},
	{0x9c,0x01},
	{0x9d,0x2d},
	{0x9e,0x00},
	{0x9f,0x9b},
	{0xa0,0x00},
	{0xa1,0xbd},
	{0xa2,0x00},
	{0xa3,0xd9},
	{0xa4,0x00},
	{0xa5,0xf8},
	{0xa6,0x01},
	{0xa7,0x05},
	{0xa8,0x00},
	{0xa9,0xfd},
	{0xaa,0x00},
	{0xab,0xdf},
	{0xac,0x00},
	{0xad,0x35},
	{0xae,0x00},
	{0xaf,0x33},
	{0xb0,0x00},
	{0xb1,0x29},
	{0xb2,0x00},
	{0xb3,0x20},
	{0xb4,0x00},
	{0xb5,0x2a},
	{0xb6,0x00},
	{0xb7,0x38},
	{0xb8,0x00},
	{0xb9,0x3d},
	{0xba,0x07},
	{0xbb,0xb2},
	{0xbc,0x07},
	{0xbd,0x86},
	{0xbe,0x07},
	{0xbf,0x54},
	{0xc0,0x07},
	{0xc1,0x2b},
	{0xc2,0x07},
	{0xc3,0x27},
	{0xc4,0x07},
	{0xc5,0x41},
	{0xc6,0x07},
	{0xc7,0x6e},
	{0xc8,0x07},
	{0xc9,0x4a},
	{0xca,0x06},
	{0xcb,0xfe},
	{0xcc,0x06},
	{0xcd,0xb5},
	{0xce,0x06},
	{0xcf,0x86},
	{0xd0,0x06},
	{0xd1,0x73},
	{0xd2,0x06},
	{0xd3,0x92},
	{0xd4,0x06},
	{0xd5,0xe0},

	{0xfc,0x0b},	// x-shading temp. correlation factor
	{0xda,0x00},	// t0(3100K)
	{0xdb,0x9c},
	{0xdc,0x00},	// tc(5100K)
	{0xdd,0xd1},	// default eeh

	{0xfc,0x1b},          
	{0x80,0x01},	// X-Shading On                      

	//=========================================================//
	//	 AE WINDOW WEIGHT                                   //
	//=========================================================//
	{0xfc,0x00},	// AE WINDOW WEIGHT  
	{0x03,0x4b},	// AE Suppress On   
	{0xfc,0x06},
	{0x01,0x35},	// UXGA AE Window   
	{0x03,0xc2},
	{0x05,0x48},
	{0x07,0xb8},
	{0x09,0x50},
	{0x0b,0xbc},
	{0x0d,0x2c},
	{0x0f,0x8f},
	{0x31,0x2a},	// Subsampling AE Window 
	{0x33,0x61},
	{0x35,0x28},
	{0x37,0x5c},
	{0x39,0x4d},
	{0x3b,0x5a},
	{0x3d,0x36},
	{0x3f,0x43},   

	{0xfc,0x20},	   	
	{0x60,0x11},	 		
	{0x61,0x11},	 		
	{0x62,0x11},	 		
	{0x63,0x11},	 		
	{0x64,0x11},	 		
	{0x65,0x11},	 	 
	{0x66,0x11},	 	 
	{0x67,0x11},	 		
	{0x68,0x11},	 	 
	{0x69,0x11},	 	 
	{0x6a,0x11},	 	 
	{0x6b,0x11},	 	 
	{0x6c,0x11},	 	 
	{0x6d,0x11},	 	 
	{0x6e,0x11},	 	 
	{0x6f,0x11},	 	 
	{0x70,0x11},	 	 
	{0x71,0x11},	 	 
	{0x72,0x11},	 	 
	{0x73,0x11},	 	 
	{0x74,0x11},	 	 
	{0x75,0x11},	 	 
	{0x76,0x11},	 	 
	{0x77,0x11},	 	                         

	//========================================================== // 
	//	 SAIT AWB                                             // 
	//========================================================== //
	{0xfc,0x07}, 
	{0x3c,0x10},	// AWB cut R min  
	{0x3d,0x10},	// AWB cut B min  
	{0x3e,0x10},	// AWB cut R max  
	{0x3f,0x10},	// AWB cut B max  

	{0xfc,0x01},
	{0xc8,0xe0},	// AWB Y Max 
	{0xfc,0x00},
	{0x3e,0x10},	// AWB Y_min   

	{0xfc,0x00},	
	{0x3e,0x10},	// AWB Y_min Normal     
	{0x3d,0x04},	// AWB Y_min Low
	{0x32,0x02},	// AWB moving average 8 frame 
	{0x81,0x10},	// AWB G gain suppress Disable 
	{0xbc,0xf0},	
	{0xfc,0x22},	  
	{0x8a,0x0a},	// AWB difference threshold check 
	{0x8b,0x03},                       
	{0x8c,0x04},	// AWB Min Y Weight check 
	{0x8d,0x06},	// AWB Max Y Weight check 

	{0xfc,0x07},
	{0x97,0x00},	// S/W Y_lowpass Filter OFF                                  

	//=================================                            
	// White Point                                                 
	//================================= 
	{0xfc,0x22},	          
	{0x01,0xd8},   // D65                                      
	{0x03,0xa1},                                              
	{0x05,0xcd},   // 5000K               
	{0x07,0xba},                           
	{0x09,0xb0},   // CWF                                   
	{0x0b,0xe1},                                             
	{0x0d,0xa0},   // 3000K                                        
	{0x0f,0xf0},                                                     
	{0x11,0x8f},   // A                                
	{0x12,0x00},                                                     
	{0x13,0xfc},                                        
	{0x15,0x80},   // 2000K                               
	{0x16,0x01},                                                      
	                                     
	//=================================                                   
	// Basic Setting                             
	//================================= 
	{0xfc,0x22},
	{0xA8,0xff},                 
	{0xA0,0x01},
	{0xA1,0x3D},
	{0xA2,0x0D},
	{0xA3,0xAD},
	{0xA4,0x07},
	{0xA5,0xE7},
	{0xA6,0x12},
	{0xA7,0xB7},
	{0xA9,0x02},
	{0xAA,0x8A},
	{0xAB,0x23},
	{0xAC,0xA2},
	{0xAD,0x02},
	{0xAE,0x04},
	{0xAF,0x09},
	{0xB0,0x84},
	{0x94,0x36},
	{0x95,0xB3},
	{0x96,0x5E},
	{0x97,0xE8},
	{0xD0,0xb0},
	{0xD1,0x30},
	{0xD2,0x00},
	{0xD3,0x21},
	{0xD4,0xFF}, //Low Temp ν 
	{0xDB,0x30},
	{0xDC,0x7B},
	{0xDD,0x21},
	{0xE7,0x00},
	{0xE8,0xB8},
	{0xE9,0x00},
	{0xEA,0xE0},
	{0xEB,0x00},
	{0xEC,0x00},
	{0xEE,0xa0}, //AWB swing

	//================================= 
	// Pixel Filter Setting             
	//================================= 
	{0xFC,0x07},
	{0x95,0xCF},

	{0xfc,0x01},
	{0xd3,0x4f},
	{0xd4,0x00},
	{0xd5,0x3a},
	{0xd6,0x80},
	{0xd7,0x60},
	{0xd8,0x00},
	{0xd9,0x49},
	{0xda,0x00},
	{0xdb,0x24},
	{0xdc,0x4b},
	{0xdd,0x26},
	{0xde,0x54},
	{0xdf,0x20},
	{0xe0,0xc9},
	{0xe1,0x19},
	{0xe2,0x4c},
	{0xe3,0x31},
	{0xe4,0x40},
	{0xe5,0x39},
	{0xe6,0x40},
	{0xe7,0x40},
	{0xe8,0x37},
	{0xe9,0x40},
	{0xea,0x22},
	{0xeb,0x00},                                         

	//=================================
	// AWB3 - Polygon Region  
	//=================================
	{0xfc,0x22},
	{0x18,0x00},
	{0x19,0x70},
	{0x1a,0xca},
	{0x1b,0x00},
	{0x1c,0x65},
	{0x1d,0xc9},
	{0x1e,0x00},
	{0x1f,0x67},
	{0x20,0xb4},
	{0x21,0x00},
	{0x22,0x80},
	{0x23,0x92},
	{0x24,0x00},
	{0x25,0x9e},
	{0x26,0x7c},
	{0x27,0x00},
	{0x28,0xb3},
	{0x29,0x6c},
	{0x2a,0x00},
	{0x2b,0xcd},
	{0x2c,0x66},
	{0x2d,0x00},
	{0x2e,0xd0},
	{0x2f,0x6f},
	{0x30,0x00},
	{0x31,0xbb},
	{0x32,0x7a},
	{0x33,0x00},
	{0x34,0xac},
	{0x35,0x86},
	{0x36,0x00},
	{0x37,0x95},
	{0x38,0xa1},
	{0x39,0x00},
	{0x3a,0x83},
	{0x3b,0xc2},
	{0x3c,0x00},
	{0x3d,0x00},                                                                 

	//=================================  
	// Moving Equation Weight            
	//=================================  
	{0xfc,0x22},
	{0x98,0x07},	

	//================================= 
	// EIT Threshold                    
	//================================= 
	{0xfc,0x22},
	{0xb1,0x00},  // Sunny 
	{0xb2,0x03},	
	{0xb3,0x00},
	{0xb4,0xC1},

	{0xb5,0x00},	// Cloudy 
	{0xb6,0x03},	
	{0xb7,0x00},
	{0xb9,0xc2},

	{0xd7,0x00},	// Shade 
	{0xd8,0x35},
	{0xd9,0x20},
	{0xda,0x81},
	                  
	//================================= 
	// Gain Offset                      
	//================================= 
	{0xfc,0x00},
	{0x79,0xf2},  // Global R gain
	{0x7a,0x0b},  // Global B gain 
	              
	{0xfc,0x07},  
	{0x11,0xfe},  // Global G gain 
	              
	{0xfc,0x22},         
	{0x58,0x00},  //ff}, //f8}, //f3},  // D65 R Offset   
	{0x59,0xf6},  //f8},  // D65 B Offset   
	{0x5A,0x00},  //ff}, //f9}, //f8},	// 5000K R Offset 
	{0x5B,0xfe},  //00},	// 5000K B Offset 
	{0x5C,0x02},  //00},	// CWF R Offset   
	{0x5D,0x00},  // CWF B Offset   
	{0x5E,0x02},	// 3000K R Offset 
	{0x5F,0x00},  //08},	// 3000K B Offset 
	{0x60,0x04},	// A R Offset     
	{0x61,0x06},  //08},	// A B Offset     
	{0x62,0x0a},	// 2000K R Offset 
	{0x63,0x06},  //08},	// 2000K B Offset 

	{0xde,0x00},	// LARGE OBJECT BUG FIX 
	{0xf0,0x6a},	// RB Ratio 

	//================================= 
	// Green Stablity Enhance           
	//================================= 
	{0xfc,0x22},
	{0xb9,0x00},
	{0xba,0x00},
	{0xbb,0x00},
	{0xbc,0x00},
	{0xe5,0x01},
	{0xe6,0xff},
	{0xbd,0x2c},   
	{0x67,0x00},                                                   

	//========================================================== 
	//	 Special Effect                                        
	//========================================================== 
	//{0xfc,0x07},  // Special Effect 
	//{0x30,0xc0},
	//{0x31,0x20},
	//{0x32,0x40},
	//{0x33,0xc0},
	//{0x34,0x00},
	//{0x35,0xb0},                     

	//==========================================================//
	//	ETC                                                 //
	//==========================================================//
	{0xfc,0x01},                
	{0x01,0x00}, //01},	// VCK Inversion //for high pclk setting
	{0x00,0x90},	// sck inv.      
	{0xfc,0x02},
	{0x03,0x20},	// SCK inv.      
	{0xfc,0x00},                                                    
	{0x02,0x09},	// 800X600
#else	// LG, Matterhorn Camera
	//==========================================================
	//	PLL SET 
	//==========================================================
	{0xfc,0x02},
	{0x50,0x58},//5a},
	{0x51,0x0a},
	{0x52,0x80},
	//==========================================================
	//	CAMERA INITIAL 
	//==========================================================
	{0xfc,0x07},
	{0x66,0x01},
	{0xfc,0x00},
	{0x00,0xaa},	//for ESD_CHECK
	{0x21,0x03},
	{0xfc,0x01},
	{0x04,0x01},
	{0xfc,0x02},
	{0x55,0x1e},
	{0x56,0x10},
	{0x57,0x80},
	{0x58,0x80},
	{0x59,0x80},
	{0x2d,0x48},
	{0x4d,0x08},
	{0x37,0x17},
	{0x30,0x82},
	{0x66,0x41},
	{0x43,0xef},
	{0x62,0x60},
	{0xfc,0x02},
	{0x4a,0x40},
	//==========================================================   
	//	Table Set for Sub-Sampling
	//==========================================================  
	{0xfc,0x03},
	{0x2c,0x00},
	{0x05,0x46},
	{0x07,0xb6},
	{0x0e,0x04},
	{0x12,0x03},
	{0xfc,0x04},
	{0xc2,0x03},
	{0xc3,0x17},
	{0xc5,0x26},
	{0xc7,0x5e},
	{0xce,0x04},
	{0xd2,0x04},
	{0xec,0x00},
	{0xc0,0x05},
	{0xc1,0x40},
	{0xfc,0x07},
	{0x05,0x00},
	{0x06,0x00},
	{0x07,0x8b},
	{0x08,0xf5},
	{0x09,0x00},
	{0x0a,0x89},
	{0x0b,0x00},
	{0x0c,0xb0},
	{0x0d,0x00},
	{0x0e,0x40},
	{0xfc,0x00},
	{0x70,0x02},
	{0xfc,0x03},
	{0x2e,0x00},
	{0x5e,0x00},
	{0x8e,0x00},
	{0xbe,0x00},
	{0xee,0x00},
	{0xfc,0x04},
	{0x2e,0x00},
	{0x5e,0x00},
	{0x8e,0x00},
	{0xbe,0x00},
	{0xee,0x00},
	{0xfc,0x00},
	{0x73,0x11},
	{0x02,0x09},
	{0x20,0x02},
	{0xfc,0x00},                                                                        
	{0x02,0x09},		 // 800 x 600

	//==========================================================
	//	ISP Setting
	//==========================================================
	{0xfc,0x00},
	{0x6c,0xb0},
	{0x6d,0x00},
	{0xfc,0x20},
	{0x16,0x5a},
	{0xfc,0x00},
	{0x78,0x63},	
	{0xfc,0x20},    
	{0x16,0x60},    
	{0xfc,0x20},    
	{0x57,0x18},
	{0x2c,0x30},
	{0x2e,0x00},
	{0x14,0x70},
	{0x01,0x02},
	{0xfc,0x07},
	{0x11,0x01},
	{0xfc,0x07},
	{0x3e,0x0a},
	{0xfc,0x01},
	{0xc8,0xa0},
	{0xfc,0x00},
	{0x3e,0x10},
	{0x3d,0x04},
	{0xfc,0x22},
	{0x8c,0x04},
	{0x8d,0x06},
	{0xfc,0x00},
	{0x32,0x02},
	{0x81,0x10},
	{0xbc,0xf0},
	{0x29,0x03},	//Brightness Offset High
	{0x2a,0xc0},	//Brightness Offset Low
	{0x2b,0x03},	//Color Saturation High
	{0x2c,0x80},	//Color Saturation Low
	{0xfc,0x07},
	{0x37,0x00},
	{0x64,0x6f},
	{0xfc,0x00},
	{0x72,0xa0},//a3},
	{0x74,0x18}, 	// 0x18 : 60hz auto 0x14 : 50hz auto 0x08 : 60hz fixed 0x04 : 50hz fixed flicker
	{0xfc,0x20},
	{0x02,0x02},
	{0xfc,0x00},
	{0x62,0x02},
	{0xfc,0x02},
	{0x4e,0x00},
	{0x4e,0x00},
	{0x4e,0x00},
	{0x4e,0x00},
	{0x4f,0x0a},
	{0x4f,0x0a},
	{0x4f,0x0a},
	{0x4f,0x0a},
	{0xfc,0x01},
	{0x0c,0x03},
	{0x0c,0x03},
	{0x01,0x00},
	{0xfc,0x00},
	{0x7e,0xf4},
	{0xfc,0x01},
	{0x3d,0x10},
	{0xfc,0x02},
	{0x03,0x20},
	{0xfc,0x01},
	{0x00,0x90},
	{0xfc,0x07},
	{0x3a,0xaa},
	{0xfc,0x02},
	{0x4e,0x1b},
	{0x4f,0xda},
	{0xfc,0x07},
	{0x97,0x00},

	//==========================================================
	//	COLOR MATRIX
	//==========================================================
	{0xfc,0x01},
	{0x51,0x08},
	{0x52,0x9b},
	{0x53,0xfc},
	{0x54,0x07},
	{0x55,0xff},
	{0x56,0x5e},
	{0x57,0xfd},
	{0x58,0x0e},
	{0x59,0x07},
	{0x5a,0xee},
	{0x5b,0xff},
	{0x5c,0x05},
	{0x5d,0xff},
	{0x5e,0x7a},
	{0x5f,0xfc},
	{0x60,0x23},
	{0x61,0x08},
	{0x62,0x64},
	//==========================================================
	//	EDGE SUPPRESSION
	//==========================================================
	{0xfc,0x00},
	{0x89,0x03},
	{0xfc,0x0b},
	{0x42,0x50},
	{0x43,0x60},
	{0x45,0x30},
	{0x49,0x10},
	{0x4d,0x30},
	{0x51,0x10},
	//==========================================================
	//	EDGE ENHANCEMENT
	//==========================================================
	{0xfc,0x05},
	{0x34,0x34},
	{0x35,0x08},
	{0x36,0x0b},
	{0x3f,0x00},
	{0x42,0x10},
	{0x43,0x00},
	{0x45,0xa0},
	{0x46,0x7a},
	{0x47,0x40},
	{0x48,0x0c},
	{0x49,0x31},
	{0x40,0x41},
	//==========================================================
	//	GAMMA
	//==========================================================
	{0xfc,0x1d},
	{0x00,0x0a},
	{0x01,0x1e},
	{0x02,0x42},
	{0x03,0x07},
	{0x04,0x01},
	{0x05,0x9e},
	{0x06,0xfa},
	{0x07,0x42},
	{0x08,0x75},
	{0x09,0x5a},
	{0x0a,0xa5},
	{0x0b,0xc9},
	{0x0c,0xec},
	{0x0d,0x0d},
	{0x0e,0xab},
	{0x0f,0x2b},
	{0x10,0x48},
	{0x11,0x64},
	{0x12,0x7c},
	{0x13,0xff},
	{0x14,0x91},
	{0x15,0xa2},
	{0x16,0xb0},
	{0x17,0xbd},
	{0x18,0xff},
	{0x19,0xc8},
	{0x1a,0xd2},
	{0x1b,0xda},
	{0x1c,0xe0},
	{0x1d,0xff},
	{0x1e,0xe5},
	{0x1f,0xea},
	{0x20,0xed},
	{0x21,0xef},
	{0x22,0xff},
	{0x23,0xf0},
	{0x24,0xf3},
	{0x25,0xf5},
	{0x26,0xf8},
	{0x27,0xff},
	{0x28,0xfb},
	{0x29,0xff},
	{0x2a,0xf0},
	{0x2b,0x0a},
	{0x2c,0x1e},
	{0x2d,0x42},
	{0x2e,0x07},
	{0x2f,0x01},
	{0x30,0x9e},
	{0x31,0xfa},
	{0x32,0x42},
	{0x33,0x75},
	{0x34,0x5a},
	{0x35,0xa5},
	{0x36,0xc9},
	{0x37,0xec},
	{0x38,0x0d},
	{0x39,0xab},
	{0x3a,0x2b},
	{0x3b,0x48},
	{0x3c,0x64},
	{0x3d,0x7c},
	{0x3e,0xff},
	{0x3f,0x91},
	{0x40,0xa2},
	{0x41,0xb0},
	{0x42,0xbd},
	{0x43,0xff},
	{0x44,0xc8},
	{0x45,0xd2},
	{0x46,0xda},
	{0x47,0xe0},
	{0x48,0xff},
	{0x49,0xe5},
	{0x4a,0xea},
	{0x4b,0xed},
	{0x4c,0xef},
	{0x4d,0xff},
	{0x4e,0xf0},
	{0x4f,0xf3},
	{0x50,0xf5},
	{0x51,0xf8},
	{0x52,0xff},
	{0x53,0xfb},
	{0x54,0xff},
	{0x55,0xf0},
	{0x56,0x0a},
	{0x57,0x1e},
	{0x58,0x42},
	{0x59,0x07},
	{0x5a,0x01},
	{0x5b,0x9e},
	{0x5c,0xfa},
	{0x5d,0x42},
	{0x5e,0x75},
	{0x5f,0x5a},
	{0x60,0xa5},
	{0x61,0xc9},
	{0x62,0xec},
	{0x63,0x0d},
	{0x64,0xab},
	{0x65,0x2b},
	{0x66,0x48},
	{0x67,0x64},
	{0x68,0x7c},
	{0x69,0xff},
	{0x6a,0x91},
	{0x6b,0xa2},
	{0x6c,0xb0},
	{0x6d,0xbd},
	{0x6e,0xff},
	{0x6f,0xc8},
	{0x70,0xd2},
	{0x71,0xda},
	{0x72,0xe0},
	{0x73,0xff},
	{0x74,0xe5},
	{0x75,0xea},
	{0x76,0xed},
	{0x77,0xef},
	{0x78,0xff},
	{0x79,0xf0},
	{0x7a,0xf3},
	{0x7b,0xf5},
	{0x7c,0xf8},
	{0x7d,0xff},
	{0x7e,0xfb},
	{0x7f,0xff},
	{0x80,0xf0},
	//==========================================================
	//	HUE CONTROL
	//==========================================================
	{0xfc,0x00},
	{0x62,0x03},
	{0x48,0x42},	//2000K
	{0x49,0x40},
	{0x4a,0xf0},
	{0x4b,0xf4},
	{0x4c,0x40},
	{0x4d,0x6a},
	{0x4e,0x08},
	{0x4f,0x03},
	{0x50,0x46},	//3000K
	{0x51,0x48},
	{0x52,0xf8},
	{0x53,0xf4},
	{0x54,0x40},
	{0x55,0x6a},
	{0x56,0x08},
	{0x57,0xfd},
	{0x58,0x30},	//5000K
	{0x59,0x38},
	{0x5a,0x02},
	{0x5b,0xf0},
	{0x5c,0x30},
	{0x5d,0x4a},
	{0x5e,0x0c},
	{0x5f,0xfd},

	//==========================================================   
	//	Bad Pixel Replacement
	//==========================================================  		
	{0xfc,0x0b},
	{0x0b,0x00},
	{0x0c,0x40},
	{0x0d,0x5a},
	{0x0e,0x00},
	{0x0f,0x20},
	{0x10,0x00},
	{0x11,0x10},
	{0x12,0x00},
	{0x13,0x00},
	{0x14,0xff},
	{0x15,0xff},
	{0x16,0x48},
	{0x17,0x60},
	{0x18,0x00},
	{0x19,0x00},
	{0x1a,0x00},
	{0x1b,0x20},
	{0x1c,0x00},
	{0x1d,0x00},
	{0x1e,0x00},
	{0x1f,0x20},
	//==========================================================
	//	GR/GB CORRECTION
	//==========================================================
	{0xfc,0x01},
	{0x45,0x0c},
	{0xfc,0x0b},
	{0x21,0x00},
	{0x22,0x58},
	{0x23,0x68},
	{0x24,0x1a},
	{0x25,0x28},//0x16
	{0x26,0x1a},
	{0x27,0x28},//0x20
	//==========================================================
	//	Noise Reduction
	//==========================================================
	{0xfc,0x01},
	{0x4c,0x01},
	{0x49,0x15},
	{0x4b,0x0a},
	{0xfc,0x0b},
	{0x28,0x00},
	{0x29,0x00},
	{0x2a,0x14},
	{0x2b,0x00},
	{0x2c,0x14},
	{0x2d,0x00},
	{0x2e,0x90},
	{0x2f,0x00},
	{0x30,0xc0},
	{0x31,0x00},
	{0x32,0xa0},
	{0x33,0x00},
	{0x34,0xd0},
	//==========================================================
	//	1D-Y/C-SIGMA-LPF
	//==========================================================
	{0xfc,0x01},
	{0x05,0xc0},
	{0xfc,0x0b},
	{0x35,0x00},
	{0x36,0x60},
	{0x37,0x68},
	{0x38,0x00},
	{0x39,0x08},
	{0x3a,0x00},
	{0x3b,0x30},
	{0x3c,0x60},
	{0x3d,0x68},
	{0x3e,0x00},
	{0x3f,0x30},
	{0x40,0x00},
	{0x41,0x38},
	{0xd4,0x60},
	{0xd5,0x68},
	{0xd6,0xb0},
	{0xd7,0xb8},
	{0xd8,0xb0},
	{0xd9,0xb8},
	//==========================================================
	//	COLOR SUPPRESS
	//==========================================================
	{0xfc,0x0b},
	{0x08,0x73},
	{0x09,0x03},
	{0x0a,0x40},
	//==========================================================
	//	SHADING
	//==========================================================
	{0xfc,0x09},
	{0x01,0x06},
	{0x02,0x40},
	{0x03,0x04},
	{0x04,0xb0},
	{0x05,0x03},	//0x02
	{0x06,0x1a},	//0x10 0x08 0xf8
	{0x07,0x02},
	{0x08,0x2b},	//0x4b 0x5b 0x70 0x8a
	{0x09,0x03},
	{0x0a,0x02},
	{0x0b,0x02},
	{0x0c,0x58},
	{0x0d,0x03},	//0x02
	{0x0e,0x22},	//0x18  0x10 0xf0 0xd0
	{0x0f,0x02},
	{0x10,0x5d},	//0x51  0x39
	{0x1d,0x80},
	{0x1e,0x00},
	{0x1f,0x80},
	{0x20,0x00},
	{0x21,0x80},
	{0x22,0x00},
	{0x23,0x80},
	{0x24,0x00},
	{0x25,0x80},
	{0x26,0x00},
	{0x27,0x80},
	{0x28,0x00},
	{0x29,0x80},
	{0x2a,0x00},
	{0x2b,0x80},
	{0x2c,0x00},
	{0x2d,0x80},
	{0x2e,0x00},
	{0x2f,0x80},
	{0x30,0x00},
	{0x31,0x80},
	{0x32,0x00},
	{0x33,0x7d},
	{0x34,0x7c},
	{0x35,0x01},
	{0x36,0x0a},
	{0x37,0x01},
	{0x38,0x1e},
	{0x39,0x01},
	{0x3a,0x41},
	{0x3b,0x01},
	{0x3c,0x72},
	{0x3d,0x01},
	{0x3e,0x8c},
	{0x3f,0x01},
	{0x40,0xa7},
	{0x41,0x01},
	{0x42,0xc5},
	{0x43,0x01},
	{0x44,0xec},
	{0x45,0x01},
	{0x46,0x00},
	{0x47,0x01},
	{0x48,0x0c},
	{0x49,0x01},
	{0x4a,0x29},
	{0x4b,0x01},
	{0x4c,0x4d},
	{0x4d,0x01},
	{0x4e,0x5f},
	{0x4f,0x01},
	{0x50,0x78},
	{0x51,0x01},
	{0x52,0x92},
	{0x53,0x01},
	{0x54,0xb1},
	{0x55,0x01},
	{0x56,0x00},
	{0x57,0x01},
	{0x58,0x0a},
	{0x59,0x01},
	{0x5a,0x23},
	{0x5b,0x01},
	{0x5c,0x3d},
	{0x5d,0x01},
	{0x5e,0x4f},
	{0x5f,0x01},
	{0x60,0x62},
	{0x61,0x01},
	{0x62,0x76},
	{0x63,0x01},
	{0x64,0x8e},
	{0x65,0x00},
	{0x66,0xa4},
	{0x67,0xdf},
	{0x68,0x02},
	{0x69,0x93},
	{0x6a,0x7e},
	{0x6b,0x05},
	{0x6c,0xcb},
	{0x6d,0xdb},
	{0x6e,0x07},
	{0x6f,0xe3},
	{0x70,0xb2},
	{0x71,0x0a},
	{0x72,0x4d},
	{0x73,0xf8},
	{0x74,0x0d},
	{0x75,0x0a},
	{0x76,0xad},
	{0x77,0x10},
	{0x78,0x19},
	{0x79,0xd3},
	{0x7a,0x00},
	{0x7b,0x93},
	{0x7c,0xdd},
	{0x7d,0x02},
	{0x7e,0x4f},
	{0x7f,0x75},
	{0x80,0x05},
	{0x81,0x32},
	{0x82,0xc7},
	{0x83,0x07},
	{0x84,0x13},
	{0x85,0x56},
	{0x86,0x09},
	{0x87,0x3d},
	{0x88,0xd3},
	{0x89,0x0b},
	{0x8a,0xb2},
	{0x8b,0x3f},
	{0x8c,0x0e},
	{0x8d,0x70},
	{0x8e,0x9a},
	{0x8f,0x00},
	{0x90,0x9c},
	{0x91,0xf4},
	{0x92,0x02},
	{0x93,0x73},
	{0x94,0xcf},
	{0x95,0x05},
	{0x96,0x84},
	{0x97,0x92},
	{0x98,0x07},
	{0x99,0x82},
	{0x9a,0xaa},
	{0x9b,0x09},
	{0x9c,0xcf},
	{0x9d,0x3c},
	{0x9e,0x0c},
	{0x9f,0x6a},
	{0xa0,0x48},
	{0xa1,0x0f},
	{0xa2,0x53},
	{0xa3,0xcd},
	{0xa4,0x63},
	{0xa5,0x5f},
	{0xa6,0x21},
	{0xa7,0x1f},
	{0xa8,0x13},
	{0xa9,0xdf},
	{0xaa,0x1e},
	{0xab,0x93},
	{0xac,0x1a},
	{0xad,0x7f},
	{0xae,0x17},
	{0xaf,0x61},
	{0xb0,0x14},
	{0xb1,0xeb},
	{0xb2,0x6e},
	{0xb3,0xcd},
	{0xb4,0x24},
	{0xb5,0xef},
	{0xb6,0x16},
	{0xb7,0x29},
	{0xb8,0x22},
	{0xb9,0x17},
	{0xba,0x1d},
	{0xbb,0x8c},
	{0xbc,0x1a},
	{0xbd,0x12},
	{0xbe,0x17},
	{0xbf,0x53},
	{0xc0,0x68},
	{0xc1,0x63},
	{0xc2,0x22},
	{0xc3,0xcb},
	{0xc4,0x14},
	{0xc5,0xe0},
	{0xc6,0x20},
	{0xc7,0x1e},
	{0xc8,0x1b},
	{0xc9,0xd6},
	{0xca,0x18},
	{0xcb,0x8f},
	{0xcc,0x15},
	{0xcd,0xf9},
	{0x00,0x02},
	//==========================================================
	//	X-Shading
	//==========================================================
	{0xfc,0x1b},
	{0x80,0x00},
	{0x81,0x00},
	{0x82,0x4c},
	{0x83,0x00},
	{0x84,0x86},
	{0x85,0x03},
	{0x86,0x5e},
	{0x87,0x00},
	{0x88,0x07},
	{0x89,0xa4},
	{0x90,0x00},
	{0x91,0xd3},
	{0x92,0x01},
	{0x93,0x09},
	{0x94,0x01},
	{0x95,0x3a},
	{0x96,0x01},
	{0x97,0x5c},
	{0x98,0x01},
	{0x99,0x70},
	{0x9a,0x01},
	{0x9b,0x5f},
	{0x9c,0x01},
	{0x9d,0x2d},
	{0x9e,0x00},
	{0x9f,0x9b},
	{0xa0,0x00},
	{0xa1,0xbd},
	{0xa2,0x00},
	{0xa3,0xd9},
	{0xa4,0x00},
	{0xa5,0xf8},
	{0xa6,0x01},
	{0xa7,0x05},
	{0xa8,0x00},
	{0xa9,0xfd},
	{0xaa,0x00},
	{0xab,0xdf},
	{0xac,0x00},
	{0xad,0x35},
	{0xae,0x00},
	{0xaf,0x33},
	{0xb0,0x00},
	{0xb1,0x29},
	{0xb2,0x00},
	{0xb3,0x20},
	{0xb4,0x00},
	{0xb5,0x2a},
	{0xb6,0x00},
	{0xb7,0x38},
	{0xb8,0x00},
	{0xb9,0x3d},
	{0xba,0x07},
	{0xbb,0xb2},
	{0xbc,0x07},
	{0xbd,0x86},
	{0xbe,0x07},
	{0xbf,0x54},
	{0xc0,0x07},
	{0xc1,0x2b},
	{0xc2,0x07},
	{0xc3,0x27},
	{0xc4,0x07},
	{0xc5,0x41},
	{0xc6,0x07},
	{0xc7,0x6e},
	{0xc8,0x07},
	{0xc9,0x4a},
	{0xca,0x06},
	{0xcb,0xfe},
	{0xcc,0x06},
	{0xcd,0xb5},
	{0xce,0x06},
	{0xcf,0x86},
	{0xd0,0x06},
	{0xd1,0x73},
	{0xd2,0x06},
	{0xd3,0x92},
	{0xd4,0x06},
	{0xd5,0xe0},

	{0xfc,0x0b},
	{0xda,0x00},
	{0xdb,0x9c},
	{0xdc,0x00},
	{0xdd,0xd1},
	{0xfc,0x00},
	{0x81,0x10},
	{0xfc,0x1b},
	{0x80,0x01},

	//
	{0xfc,0x00},
	{0x03,0x4b},
	{0xfc,0x06},
	{0x01,0x35},
	{0x03,0xc2},
	{0x05,0x48},
	{0x07,0xb8},
	{0x31,0x2a},
	{0x33,0x61},
	{0x35,0x28},
	{0x37,0x5c},
	//==========================================================
	// AE Window
	//==========================================================
	//ave
	{0xfc,0x20},
	{0x60,0x11},
	{0x61,0x11},
	{0x62,0x11},
	{0x63,0x11},
	{0x64,0x11},
	{0x65,0x11},
	{0x66,0x11},
	{0x67,0x11},
	{0x68,0x11},
	{0x69,0x11},
	{0x6a,0x11},
	{0x6b,0x11},
	{0x6c,0x11},
	{0x6d,0x11},
	{0x6e,0x11},
	{0x6f,0x11},
	{0x70,0x11},
	{0x71,0x11},
	{0x72,0x11},
	{0x73,0x11},
	{0x74,0x11},
	{0x75,0x11},
	{0x76,0x11},
	{0x77,0x11},

	//==========================================================
	//	SAIT AWB
	//==========================================================
	// White Point For Hue Control MWB
	{0xfc,0x22},
	{0x01,0xcf},
	{0x03,0x86},
	{0x05,0xbc},
	{0x07,0xb2},
	{0x09,0xae},
	{0x0b,0xd5},
	{0x0d,0x95},
	{0x0f,0xf4},
	{0x10,0x00},
	{0x11,0x79},
	{0x12,0x00},
	{0x13,0xfd},
	{0x15,0x7f},
	{0x16,0x00},
	{0x17,0xff},


	// Basic Setting
	{0xfc,0x22}, 	//AWB Basic
	{0xA0,0x01}, 
	{0xA1,0x3F}, 
	{0xA2,0x0E}, 
	{0xA3,0x8A}, 
	{0xA4,0x07}, 
	{0xA5,0xF6}, 
	{0xA6,0x11}, 
	{0xA7,0x9A}, 
	{0xA9,0x02},
	{0xAA,0xA8},
	{0xAB,0x7F},
	{0xAC,0xE8},
	{0xAD,0x02},
	{0xAE,0xA1},
	{0xAF,0x4F},
	{0xB0,0xCF},
	{0xBD,0x80},
	{0x94,0x37},
	{0x95,0xCC},
	{0x96,0x62},
	{0x97,0x3A},
	{0xD0,0xA0},
	{0xD1,0x25},
	{0xD2,0x42},
	{0xD3,0x20},
	{0xD4,0xa0},	//Low Temp ν 
	{0xDB,0x25},
	{0xDC,0x78},
	{0xDD,0x20},
	{0xe7,0x00},	//Low Temp AWB
	{0xe8,0xbb},
	{0xe9,0x00},
	{0xea,0x72},
	{0xeb,0x03},
	{0xec,0x84},
	{0xEE,0x98},

	{0xfc,0x00},
	{0x8a,0x08},
	{0x8b,0x03},


	// Pixel Filter Setting 
	{0xfc,0x07},
	{0x95,0xcf},
	{0xfc,0x01},
	{0xd3,0x4e},
	{0xd4,0x80},
	{0xd5,0x3e},
	{0xd6,0x00},
	{0xd7,0x5d},
	{0xd8,0x80},
	{0xd9,0x4d},
	{0xda,0x00},
	{0xdb,0x1d},
	{0xdc,0xa8},
	{0xdd,0x26},
	{0xde,0x95},
	{0xdf,0x21},
	{0xe0,0xb1},
	{0xe1,0x1c},
	{0xe2,0x1a},
	{0xe3,0x40},
	{0xe4,0x40},
	{0xe5,0x40},
	{0xe6,0x40},
	{0xe7,0x1f},
	{0xe8,0x39},
	{0xe9,0x36},
	{0xea,0x23},
	{0xeb,0x00},

	// Polygon AWB Region Tune
	{0xfc,0x22},
	{0x18,0x00},
	{0x19,0x59},
	{0x1a,0xe9},
	{0x1b,0x00},
	{0x1c,0x5b},
	{0x1d,0xcb},
	{0x1e,0x00},
	{0x1f,0x7e},
	{0x20,0xa0},
	{0x21,0x00},
	{0x22,0x91},
	{0x23,0x88},
	{0x24,0x00},
	{0x25,0xa3},
	{0x26,0x74},
	{0x27,0x00},
	{0x28,0xb9},
	{0x29,0x60},
	{0x2a,0x00},
	{0x2b,0xd3},
	{0x2c,0x5a},
	{0x2d,0x00},
	{0x2e,0xd5},
	{0x2f,0x74},
	{0x30,0x00},
	{0x31,0xbc},
	{0x32,0x7d},
	{0x33,0x00},
	{0x34,0xad},
	{0x35,0x89},
	{0x36,0x00},
	{0x37,0x9a},
	{0x38,0xa6},
	{0x39,0x00},
	{0x3a,0x75},
	{0x3b,0xde},
	{0x3c,0x00},
	{0x3d,0x00},
	{0x3e,0x00},
	{0x3f,0x00},
	{0x40,0x00},
	{0x41,0x00},
	{0x42,0x00},
	{0x43,0x00},
	{0x44,0x00},
	{0x45,0x00},
	{0x46,0x00},
	{0x47,0x00},
	{0x48,0x00},
	{0x49,0x00},
	{0x4a,0x00},
	{0x4b,0x00},
	{0x4c,0x00},
	{0x4d,0x00},
	{0x4e,0x00},
	{0x4f,0x00},
	{0x50,0x00},
	{0x51,0x00},
	{0x52,0x00},
	{0x53,0x00},
	{0x54,0x00},
	{0x55,0x00},
	{0x56,0x00},

	// Moving Equation Weight
	{0xfc,0x22},
	{0x98,0x07},

	//EIT Threshold
	{0xfc,0x22},
	{0xb1,0x00},
	{0xb2,0x02},
	{0xb3,0x00},
	{0xb4,0xc1},
	{0xb5,0x00},
	{0xb6,0x02},
	{0xb7,0x00},
	{0xb9,0xc2},
	{0xd7,0x00},
	{0xd8,0x35},
	{0xd9,0x20},
	{0xda,0x81},

	//==========================================================
	//	R, B Offset
	//==========================================================
	{0xfc,0x00},
	{0x79,0xf2},	//0xf1//0xf5//0xf6 Global R Offset
	{0x7a,0x1c},	//0x09//0x05//0x08 Global B Offset
	{0xde,0x00},
	{0xf0,0x6a},
	{0xfc,0x22},	// Offset
	{0x58,0xf9},
	{0x59,0xf7},
	{0x5a,0x05},
	{0x5b,0xf5},
	{0x5c,0x0a},//0c
	{0x5d,0xfb},
	{0x5e,0x0a},//0c
	{0x5f,0xfe},
	{0x60,0x0a},
	{0x61,0x06},
	{0x62,0x0a},
	{0x63,0x06},

	// Green Stablity Enhance
	{0xfc,0x22},
	{0xb9,0x00},
	{0xba,0x00},
	{0xbb,0x00},
	{0xbc,0x00},
	{0xe5,0x01},
	{0xe6,0xff},
	// AWB2_OPTION : BD_OUTDOORCLASY
	{0xbd,0x80},	//0x8c
	{0x70,0xf0},

	//==========================================================
	//	Special function sephia B/W etc
	//==========================================================
	{0xfc,0x07},
	{0x30,0xc0},
	{0x31,0x20},
	{0x32,0x40},
	{0x33,0xc0},
	{0x34,0x00},
	{0x35,0xb0},


	//==========================================================
	//	EE Wide LUMA
	//==========================================================
	{0xfc,0x1d},
	{0x86,0x02},
	{0x87,0xc0},
	{0x88,0x03},
	{0x89,0xc0},
	{0x8a,0x02},
	{0x8b,0x00},
	{0x8c,0x00},
	{0x8d,0x00},
	{0x8e,0x00},
	{0x8f,0xd0},
	{0x90,0x00},
	{0x91,0x00},
	{0x92,0x02},
	{0x93,0x00},
	{0x94,0x00},
	{0x95,0x00},
	{0x96,0x00},
	{0x97,0xd0},
	{0x98,0x00},
	{0x99,0x00},
	{0x9a,0x02},
	{0x9b,0x60},
	{0x9c,0x00},
	{0x9d,0x00},
	{0x9e,0x02},
	{0x9f,0x60},
	{0xa0,0x00},
	{0xa1,0x00},
	{0xa2,0x09},
	{0x85,0x01},

	//==========================================================
	//	Color Correction Coefficient
	//==========================================================
	{0xfc,0x0b},
	{0x60,0x08},
	{0x61,0x9b},
	{0x62,0xfc},
	{0x63,0x07},
	{0x64,0xff},
	{0x65,0x5e},
	{0x66,0xfd},
	{0x67,0x0e},
	{0x68,0x07},
	{0x69,0xee},
	{0x6a,0xff},
	{0x6b,0x05},
	{0x6c,0xff},
	{0x6d,0x7a},
	{0x6e,0xfc},
	{0x6f,0x23},
	{0x70,0x08},
	{0x71,0x64},
	{0x72,0x04},    
	{0x73,0xfd},    
	{0x74,0xfd},
	{0x75,0xea},
	{0x76,0x01},
	{0x77,0x19},
	{0x78,0xfd},
	{0x79,0xbf},
	{0x7a,0x05},
	{0x7b,0x16},
	{0x7c,0x01},
	{0x7d,0x2b},
	{0x7e,0xff},
	{0x7f,0xbf},
	{0x80,0xf8},
	{0x81,0x8e},
	{0x82,0x0b},
	{0x83,0xb3},
	{0x84,0x04},
	{0x85,0xfd},
	{0x86,0xfd},
	{0x87,0xea},
	{0x88,0x01},
	{0x89,0x19},
	{0x8a,0xfd},
	{0x8b,0xbf},
	{0x8c,0x05},
	{0x8d,0x16},
	{0x8e,0x01},
	{0x8f,0x2b},
	{0x90,0xff},
	{0x91,0xbf},
	{0x92,0xf8},
	{0x93,0x8e},
	{0x94,0x0b},
	{0x95,0xb3},


	///////////////////////////////////////////////////
	//   Stable Range For Cintr
	///////////////////////////////////////////////////
	{0xfc,0x20},
	{0x52,0x10},
	{0x53,0x10},

	///////////////////////////////////////////////////
	//   Mirror Option
	///////////////////////////////////////////////////
	{0xfc,0x20},
	{0x0f,0x01},       // Weight Mirrored  63 to 0
	{0xfc,0x00},
	{0x75,0x05},
#endif
    {REG_TERM, VAL_TERM}
};


static struct sensor_reg sensor_set_preview[] = {
	//===================================
	//	CAMERA PREVIEW(800*600)
	//===================================
	{0xfc,0x02},
	{0x2d,0x48},  //double shutter width control.double shutter enable
	{0x44,0x63},  //Reserved
	{0xfc,0x03},  
	{0x02,0x04}, //CIS_FRAME_V_DEPTH_H
	{0xfc,0x20},  
	{0x14,0x70},  //80 //BRIGHT_OFFSET,                   
	{0xfc,0x00},  
	{0x03,0x4b},  // AE/AWB On
	{0x7e,0x74},  // Suppress On
	{0x89,0x03},  // Edge Suppress On 
	{0xfc,0x20},                     
	{0x16,0x5a},  //50 // Frame AE Start
	{0xfc,0x02},         
	{0x30,0x82},  // Analog offset       
	{0x37,0x11},  //0d // Global Gain
	{0x60,0x00},  // Blank_Adrs
	{0x45,0x0e},  // CDS Timing for Average Sub_Sampling
	{0x47,0x2f},  // Reserved
	{0xfc,0x00},                
	{0x02,0x09},  // 800 x 600 Recovery
	{REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_set_capture[] = {
	//===================================
	//	1600*1200 SNAPSHOT
	//===================================
	{0xfc, 0x00},
	{0x03, 0x48},
	{0xfc, 0x02},
	{0x30, 0x90},
	{0x37, 0x20},                                                                        
	{0x60, 0x01},		                                                        
	{0x45, 0xff},                                                                        
	{0x47, 0x0f},		                                                     
	{0xfc, 0x00},		 
	{0x02, 0x00},
//	{0xfc, 0x00},     
//	{0x75, 0x05},  //H.V mirror	
    {REG_TERM, VAL_TERM}
};

 struct sensor_reg* sensor_reg_common[3] =
{
	sensor_initialize,
	sensor_set_preview,
	sensor_set_capture
};

static struct sensor_reg sensor_brightness_0[] = {
	{0xfc,0x00},
	{0x29,0x02},
	{0x2a,0xa0},
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_brightness_1[] = {
	{0xfc,0x00},
	{0x29,0x03},
	{0x2a,0xa0},
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_brightness_2[] = {
	{0xfc,0x00},
	{0x29,0x04},
	{0x2a,0xa0},
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_brightness_3[] = {
	{0xfc,0x00},
	{0x29,0x05},
	{0x2a,0xa0},
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_brightness_4[] = {
	{0xfc,0x00},
	{0x29,0x03},
	{0x2a,0xa0},
    {REG_TERM, VAL_TERM}
};

struct sensor_reg* sensor_reg_brightness[5] =
{
	sensor_brightness_0,
	sensor_brightness_1,
	sensor_brightness_2,
	sensor_brightness_3,
	sensor_brightness_4
};


static struct sensor_reg sensor_awb_auto[] = {
	//===================================
	//	CameraWB_Auto
	//===================================
	{0xfc,0x00},             
	{0x04,0x00},
	{0x05,0x00},

	{0xfc,0x22},             
	{0x05,0xca},	// CRWB
	{0x07,0xc8},	// CBWB

	{0xfc,0x00},
	{0x30,0x00},
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_awb_daylight[] = {
	//===================================
	//	CameraWB_Daylight
	//===================================

	{0xfc,0x00},             
	{0x04,0x00},
	{0x05,0x00},

	{0xfc,0x22},             
	{0x05,0xd0},	// CRWB
	{0x07,0x98},	// CBWB

	{0xfc,0x00},
	{0x30,0x02},
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_awb_incandescent[] = {
//==========================================================
//	CAMERA_WB_Tungsten  
//==========================================================

	{0xfc,0x00},             
	{0x04,0x00},
	{0x05,0x00},
	{0xfc,0x22},            
	{0x05,0xa0},  //9d	// CRWB
	{0x07,0xdc},  //e0//f5// CBWB
	{0xfc,0x00},
	{0x30,0x02},
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_awb_fluorescent[] = {
	//===================================
	//	CameraWB_fluorescent
	//===================================

	{0xfc,0x00},             
	{0x04,0x00},
	{0x05,0x00},

	{0xfc,0x22},             
	{0x05,0xb0},	// CRWB
	{0x07,0xbf},	// CBWB

	{0xfc,0x00},
	{0x30,0x02},
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_awb_cloudy[] = {
	//===================================
	//	CameraWB_Cloudy
	//===================================
	{0xfc,0x00},             
	{0x04,0x00},
	{0x05,0x00},

	{0xfc,0x22},             
	{0x05,0xe8},	// CRWB
	{0x07,0x90},	// CBWB

	{0xfc,0x00},
	{0x30,0x05},
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_awb_sunset[] = {
    {REG_TERM, VAL_TERM}
};

struct sensor_reg* sensor_reg_awb[6] =
{
	sensor_awb_auto,
	sensor_awb_daylight,
	sensor_awb_incandescent,
	sensor_awb_fluorescent,
	sensor_awb_cloudy,
	sensor_awb_sunset
	
};


static struct sensor_reg sensor_iso_auto[] = {
	//===================================
	//	CameraIso_Auto 
	//===================================

	{0xfc,0x00},
	{0x6c,0xa0},
	{0x6d,0x00},
	{0x78,0x6a},

	{0xfc,0x01},
	{0x2b,0x02},  // Gr_H
	{0x2c,0x90},  // Gr_L
	{0x2d,0x02},  // Gb_H
	{0x2e,0x90},  // Gb_L
	{0x2f,0x02},  // R_H 
	{0x30,0x90},  // R_L 
	{0x31,0x02},  // B_H 
	{0x32,0x90},  // B_L 
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_iso_100[] = {
	//===================================
	//	CameraIso_100 
	//===================================

	{0xfc,0x00},
	{0x6c,0x98},
	{0x6d,0x00},
	{0x78,0x6b},

	{0xfc,0x01},
	{0x2b,0x02},  // Gr_H
	{0x2c,0xc0},  // Gr_L
	{0x2d,0x02},  // Gb_H
	{0x2e,0xc0},  // Gb_L
	{0x2f,0x02},  // R_H 
	{0x30,0xc0},  // R_L 
	{0x31,0x02},  // B_H 
	{0x32,0xc0},  // B_L 
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_iso_200[] = {
	//===================================
	//	CameraIso_200 
	//===================================
	{0xfc,0x00},
	{0x6c,0xd0},
	{0x6d,0x00},
	{0x78,0x70},

	{0xfc,0x01},
	{0x2b,0x03},  // Gr_H
	{0x2c,0x80},  // Gr_L
	{0x2d,0x03},  // Gb_H
	{0x2e,0x80},  // Gb_L
	{0x2f,0x03},  // R_H 
	{0x30,0x80},  // R_L 
	{0x31,0x03},  // B_H 
	{0x32,0x80},  // B_L 
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_iso_400[] = {
	//===================================
	//	CameraIso_400 
	//===================================

	{0xfc,0x00},
	{0x6c,0x40},
	{0x6d,0x01},
	{0x78,0x6d},

	{0xfc,0x01},
	{0x2b,0x04},  // Gr_H
	{0x2c,0x00},  // Gr_L
	{0x2d,0x04},  // Gb_H
	{0x2e,0x00},  // Gb_L
	{0x2f,0x04},  // R_H 
	{0x30,0x00},  // R_L 
	{0x31,0x04},  // B_H 
	{0x32,0x00},  // B_L 
	{REG_TERM, VAL_TERM}
};

struct sensor_reg* sensor_reg_iso[4] =
{
	sensor_iso_auto,
	sensor_iso_100,
	sensor_iso_200,
	sensor_iso_400
};


static struct sensor_reg sensor_effect_normal[] = {
	
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_effect_gray[] = {
	
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_effect_negative[] = {
	
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_effect_sepia[] = {
	
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_effect_sharpness[] = {
	
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_effect_sketch[] = {
	
    {REG_TERM, VAL_TERM}
};

struct sensor_reg* sensor_reg_effect[6] =
{
	sensor_effect_normal,
	sensor_effect_gray,
	sensor_effect_negative,
	sensor_effect_sepia,
	sensor_effect_sharpness,
	sensor_effect_sketch,
};

static struct sensor_reg sensor_reg_flipnone[] = {
	//===================================
	//	CAMERA_FLIP_NONE  (1/4)
	//===================================
	{0xfc,0x20},
	{0x0f,0x00},        // Weight Mirrored  (0 - 63)
	{0xfc,0x00},
	{0x75,0x04},
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_reg_hflip[] = {
	//===================================
	//	CAMERA_FLIP_MIRROR_ONLY  (2/4)
	//===================================
	{0xfc,0x20},
	{0x0f,0x00},        // Weight Mirrored  (0 - 63)
	{0xfc,0x00},
	{0x75,0x06},
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_reg_vflip[] = {
	//===================================
	//	CAMERA_FLIP_WATER_ONLY   (3/4)
	//===================================
	{0xfc,0x20},
	{0x0f,0x01},        // Weight Mirrored  (63 - 0)
	{0xfc,0x00},
	{0x75,0x07},
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_reg_hvflip[] = {
	//===================================
	//	CAMERA_FLIP_WATER_MIRROR (4/4)
	//===================================
	{0xfc,0x20},
	{0x0f,0x01},       // Weight Mirrored  (63 - 0)
	{0xfc,0x00},
	{0x75,0x05},
    {REG_TERM, VAL_TERM}
};

struct sensor_reg* sensor_reg_flip[4] =
{
	sensor_reg_flipnone,
	sensor_reg_hflip,
	sensor_reg_vflip,
	sensor_reg_hvflip,
};


static struct sensor_reg sensor_secne_auto[] = {
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_secne_night[] = {
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_secne_landscape[] = {
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_secne_portrait[] = {
	{REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_secne_sport[] = {
	{REG_TERM, VAL_TERM}
};

struct sensor_reg* sensor_reg_scene[5] =
{
	sensor_secne_auto,
	sensor_secne_night,
	sensor_secne_landscape,
	sensor_secne_portrait,
	sensor_secne_sport
};

static struct sensor_reg sensor_me_mtrix[] = {
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_me_center_weighted[] = {
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_me_spot[] = {
    {REG_TERM, VAL_TERM}
};

struct sensor_reg* sensor_reg_metering_exposure[3] =
{
	sensor_me_mtrix,
	sensor_me_center_weighted,
	sensor_me_spot,
};

static struct sensor_reg sensor_af_single[] = {
    {REG_TERM, VAL_TERM}
};

static struct sensor_reg sensor_af_manual[] = {
    {REG_TERM, VAL_TERM}
};

struct sensor_reg* sensor_reg_af[2] =
{
	sensor_af_single,
	sensor_af_manual,
};

static int write_regs(const struct sensor_reg reglist[])
{
	int err;
	int err_cnt = 0;	
	unsigned char data[132];
	unsigned char bytes;
	const struct sensor_reg *next = reglist;
	
	while (!((next->reg == REG_TERM) && (next->val == VAL_TERM)))
	{
		bytes = 0;
		data[bytes]= next->reg&0xff;	bytes++;
		data[bytes]= next->val&0xff;	bytes++;
		
		err = DDI_I2C_Write(data, 1, 1);
        
        if (err)
        {
			err_cnt++;
			if(err_cnt >= 3)
			{
				printk("ERROR: Sensor I2C !!!! \n"); 
				return err;
			}
        }
        else
        {
        	err_cnt = 0;
            next++;
        }
	}

	return 0;
}

void sensor_delay(int ms)
{
#if 1 //ZzaU
	unsigned int msec;

	msec = ms / 10; //10msec unit

	if(!msec)
		msleep(1);
	else
		msleep(msec);
#else
	int totCnt, iCnt, temp = 0;

	totCnt = ms*20000;
	for(iCnt = 0; iCnt < totCnt; iCnt++)
	{
		temp++;
	}
#endif
}

static int sensor_open(void)
{
	sensor_power_disable();
	sensor_delay(10);

	sensor_power_enable();
	sensor_delay(10);

	sensor_powerdown_disable();
	sensor_delay(10);

	sensor_reset_low();
	sensor_delay(10);

	sensor_powerdown_enable();
	sensor_delay(5);

	CIF_Open();
	sensor_delay(40);
	
	sensor_reset_high();
	sensor_delay(15);

	return write_regs(sensor_reg_common[0]);
}

static int sensor_close(void)
{
	CIF_ONOFF(OFF);

	sensor_reset_low();
	sensor_power_disable();
	sensor_powerdown_disable();

	CIF_Close();
	msleep(5);

	return 0;
}

static int sensor_preview(void)
{
	return write_regs(sensor_reg_common[1]);
}

static int sensor_capture(void)
{
	return write_regs(sensor_reg_common[2]);
}

static int sensor_capturecfg(int width, int height)
{
	return 0;
}

static int sensor_zoom(int val)
{
	return 0;
}

static int sensor_autofocus(int val)
{
	return write_regs(sensor_reg_af[val]);
}

static int sensor_effect(int val)
{
	return write_regs(sensor_reg_effect[val]);
}

static int sensor_flip(int val)
{
	return write_regs(sensor_reg_flip[val]);
}

static int sensor_iso(int val)
{
	return write_regs(sensor_reg_iso[val]);
}

static int sensor_me(int val)
{
	return write_regs(sensor_reg_metering_exposure[val]);
}

static int sensor_wb(int val)
{
	return write_regs(sensor_reg_awb[val]);
}

static int sensor_bright(int val)
{
	return write_regs(sensor_reg_brightness[val]);
}

static int sensor_scene(int val)
{
	return write_regs(sensor_reg_scene[val]);
}

static int sensor_check_esd(int val)
{
	return 0;
}

static int sensor_check_luma(int val)
{
	return 0;
}

/**********************************************************
*    Function  
**********************************************************/
void sensor_init_fnc(SENSOR_FUNC_TYPE *sensor_func)
{
	sensor_func->Open 			= sensor_open;
	sensor_func->Close 			= sensor_close;

	sensor_func->Set_Preview 	= sensor_preview;
	sensor_func->Set_Capture 	= sensor_capture;
	sensor_func->Set_CaptureCfg = sensor_capturecfg;

	sensor_func->Set_Zoom		= sensor_zoom;
	sensor_func->Set_AF 		= sensor_autofocus;
	sensor_func->Set_Effect 	= sensor_effect;
	sensor_func->Set_Flip 		= sensor_flip;
	sensor_func->Set_ISO 		= sensor_iso;
	sensor_func->Set_ME 		= sensor_me;
	sensor_func->Set_WB 		= sensor_wb;
	sensor_func->Set_Bright 	= sensor_bright;
	sensor_func->Set_Scene 		= sensor_scene;

	sensor_func->Check_ESD 		= sensor_check_esd;
	sensor_func->Check_Luma 	= sensor_check_luma;
}

#endif

