/*
 * linux/drivers/mmc/host/tcc92xx_sdhc.c  
 *
 * Author:  <linux@telechips.com>
 * Created: June 10, 2008
 * Description: SD/MMC Host Driver for Telechips Boards.
 *
 * Copyright (C) 2008-2009 Telechips 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *
 * ChangeLog:
 *
 *	Nov, 2008	:	Added Suspend and Resume functions
 *
 */
#include <linux/fcntl.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/init.h>
#include <linux/ioport.h>
#include <linux/platform_device.h>
#include <linux/interrupt.h>
#include <linux/highmem.h>
#include <linux/dma-mapping.h>
#include <linux/delay.h>
#include <linux/spinlock.h>
#include <linux/timer.h>
#include <linux/mmc/host.h>
#include <linux/mmc/mmc.h>
#include <linux/mmc/sdio.h>
#include <linux/mmc/card.h>
#include <linux/clk.h>

#include <linux/pci.h>

#include <asm/io.h>
#include <asm/irq.h>
#include <asm/scatterlist.h>
#include <asm/mach-types.h>

#include <mach/hardware.h>
#include <mach/irqs.h>
#include <mach/bsp.h>
#include <mach/tca_ckc.h>

#include "tcc92xx_sdhc.h"

#if defined(CONFIG_MACH_TCC9200) || defined(CONFIG_MACH_TCC9201) || defined(CONFIG_MACH_TCC8900)
#include <mach/tcc_pca953x.h>
#endif

#if defined(CONFIG_BCM4325_WIFI) 
#include <linux/earlysuspend.h>
#endif

#undef SD_EXTENTION_SUB  //MCC
#define SD_CLOCK_CONTROL

/*
 * Controller registers
 */
#define TCCSDHC_DMA_ADDRESS 		0x00
#define TCCSDHC_BLOCK_SIZE			0x04

#define TCCSDHC_MAKE_BLKSZ(dma, blksz) (((dma & 0x7) << 12) | (blksz & 0xFFF))

#define TCCSDHC_BLOCK_COUNT 		0x06
#define TCCSDHC_ARGUMENT			0x08
//#define TCCSDHC_TRANSFER_MODE		0x0C
//#define TCCSDHC_COMMAND 			0x0E
#define TCCSDHC_TMODE_COM			0x0C

#define TCCSDHC_MAKE_CMD(c, f) (((c & 0xff) << 8) | (f & 0xff))

#define TCCSDHC_RESPONSE10			0x10
#define TCCSDHC_RESPONSE32			0x14
#define TCCSDHC_RESPONSE54			0x18
#define TCCSDHC_RESPONSE76			0x1c
#define TCCSDHC_BUFFER				0x20
#define TCCSDHC_PRESENT_STATE		0x24
#define TCCSDHC_HOST_CONTROL		0x28
//#define TCCSDHC_POWER_CONTROL		0x29
#define TCCSDHC_BLOCK_GAP_CONTROL	0x2A
//#define TCCSDHC_WAKE_UP_CONTROL 	0x2B
#define TCCSDHC_CLOCK_CONTROL		0x2C
#define TCCSDHC_TIMEOUT_CONTROL 	0x2E
//#define TCCSDHC_SOFTWARE_RESET		0x2F
#define TCCSDHC_INT_STATUS			0x30
#define TCCSDHC_INT_ENABLE			0x34
#define TCCSDHC_SIGNAL_ENABLE		0x38
#define TCCSDHC_ACMD12_ERR			0x3C

#define TCCSDHC_ADMA_ERROR			0x54
#define TCCSDHC_ADMA_ADDRESS		0x58

#undef	 DEBUG

#ifdef	DEBUG
#define SD_DEBUG(x...)	printk(x)
#define SD_ERROR(x...)	printk(x)
#else
#define SD_DEBUG(x...)
#define SD_ERROR(x...)
#endif

#define DRIVER_NAME			"tcc-mmc"
#define DRIVER_NAME_CORE0	"tcc-mmc0"
#define DRIVER_NAME_CORE1	"tcc-mmc1"
#define DETECT_TIMEOUT		(HZ)
#define SDMMC_FIFO_CNT		1024
#define SDMMC_TIMEOUT_TICKS	(1000*HZ/1000)	/* 1000ms */

PSDCHCTRL	pSDCHCTRL = HwSDCHCTRL;
PSDHOST pSDMMC_SLOT0 = HwSDCORE0SLOT0;
PSDHOST pSDMMC_SLOT1 = HwSDCORE1SLOT0;

#if defined(CONFIG_MMC_TCC_DEMO_BOARD_SD_WAKEUP)
int sd_detect_flag;
#endif

static bool bChanged = 0;

static void tcc_mmc_start_command(struct tcc_mmc_host *host, struct mmc_command *cmd);
static void tcc_mmc_tasklet_finish(unsigned long param);
static void init_mmc_host(struct tcc_mmc_host *host);

static int tcc_sw_reset(struct tcc_mmc_host *host, uint8_t rst_bits)
{
	int timeout = 100;

	tcc_writew(rst_bits<<8 | tcc_readw(host->iobase+TCCSDHC_TIMEOUT_CONTROL), host->iobase+TCCSDHC_TIMEOUT_CONTROL);
	

	while (--timeout) {
		if (!(tcc_readw(host->iobase+TCCSDHC_TIMEOUT_CONTROL)& rst_bits<<8)){

			break;
		}
		/* Must be mdelay not msleep, as called from interrupt context */
		mdelay(1);
	}
	if (!timeout) {
		SD_DEBUG("timed out waiting for reset\n");
		return -ETIMEDOUT;
	}

	return 0;
}

int tcc_mmc_card_inserted(struct tcc_mmc_host *host)
{
	if(host->id ==1)
	{
#if defined(CONFIG_MACH_TCC9200S)
		return (HwGPIOA->GPDAT& Hw13) ? 0 : 1;	
#else
#if !defined(SD_EXTENTION_SUB)
		return (HwGPIOA->GPDAT &Hw6) ? 0 : 1;
#else
		return (HwGPIOF->GPDAT &Hw24) ? 0 : 1;		
#endif
#endif
	} 

	else if(host->id ==0){	
#if defined(CONFIG_MACH_TCC9200S)		
#if defined(CONFIG_BCM4325_WIFI_9200S_DEMO_BOARD)
		return 1;	// bcm4325 		
#else
		return (HwGPIOA->GPDAT& Hw10) ? 0 : 1;		
#endif
#else
#if !defined(SD_EXTENTION_SUB)
		return (HwGPIOA->GPDAT& Hw10) ? 0 : 1;		
#else
		return (HwGPIOF->GPDAT& Hw10) ? 0 : 1;		
#endif
#endif
	}	
	
}

/*
 * Card insert/remove timer handler
 */
static void tcc_mmc_poll_event(unsigned long data)
{
	struct tcc_mmc_host *host = (struct tcc_mmc_host *) data;	

	if (!tcc_mmc_card_inserted(host)) {
		/* card removed */
		if (host->card_inserted == 1) {
			host->card_inserted = 0;
			bChanged = 1;
			printk("SD card%d removed\n",host->id);
			if (host && bChanged) {
#if defined(CONFIG_MMC_TCC_DEMO_BOARD_SD_WAKEUP)
					if(host->id==0){
						sd_detect_flag=0;
					}
				#endif
#ifdef SD_CLOCK_CONTROL
				if(host->id==0)
					clk_disable(host->fclk);
#endif					
				mmc_detect_change(host->mmc, msecs_to_jiffies(0));
				bChanged = 0;
			}
		}
	} else if (host->card_inserted == 0) {
		host->card_inserted = 1;
		bChanged = 1;
		printk("SD card%d inserted\n",host->id);
		if (host && bChanged) {
#ifdef SD_CLOCK_CONTROL		

			if(host->id==0)
				clk_enable(host->fclk);	// wifi always enable 
#endif
			//mmc_detect_change(host->mmc, (5 * HZ) / 2);
			//mmc_detect_change(host->mmc, msecs_to_jiffies(500));			
			mmc_detect_change(host->mmc, msecs_to_jiffies(0));			
			bChanged = 0;
		}
	}

	mod_timer(&host->detect_timer, jiffies + DETECT_TIMEOUT);
}

static void tcc_mmc_clock_onoff(struct tcc_mmc_host *host, unsigned char onoff)
{
	u16 temp_val;

	if (onoff == 1) {
//		BITSET(HwIOBUSCFG->HCLKEN0, HwIOBUSCFG_SD);

		temp_val = tcc_readw(host->iobase+TCCSDHC_CLOCK_CONTROL);
		tcc_writew(temp_val|HwSDCLKSEL_SCK_EN, host->iobase+TCCSDHC_CLOCK_CONTROL);


	} else if (onoff == 0) {

		temp_val = tcc_readw(host->iobase+TCCSDHC_CLOCK_CONTROL);
		tcc_writew(temp_val&~HwSDCLKSEL_SCK_EN, host->iobase+TCCSDHC_CLOCK_CONTROL);


		//FIXME: need to be revised for power saving. currently bus clock is always running.
		if ((pSDMMC_SLOT0->CLK & HwSDCLKSEL_SCK_EN) == 0  &&(pSDMMC_SLOT1->CLK & HwSDCLKSEL_SCK_EN) == 0) {
			//HwBCLKCTR &= ~HwBCLKCTR_SD_ON;
		}
	}
}

static char *tcc_mmc_kmap_atomic(struct scatterlist *sg, unsigned long *flags)
{
	local_irq_save(*flags);
	return kmap_atomic(sg_page(sg), KM_BIO_SRC_IRQ) + sg->offset;
}

static void tcc_mmc_kunmap_atomic(void *buffer, unsigned long *flags)
{
	kunmap_atomic(buffer, KM_BIO_SRC_IRQ);
	local_irq_restore(*flags);
}

static int tcc_mmc_adma_table_pre(struct tcc_mmc_host *host, struct mmc_data *data)
{
	int direction;

	u8 *desc;
	u8 *align;
	dma_addr_t addr;
	dma_addr_t align_addr;
	int len, offset;

	struct scatterlist *sg;
	int i;
	char *buffer;
	unsigned long flags;

	/*
	 * The spec does not specify endianness of descriptor table.
	 * We currently guess that it is LE.
	 */

	if (data->flags & MMC_DATA_READ)
		direction = DMA_FROM_DEVICE;
	else
		direction = DMA_TO_DEVICE;

	/*
	 * The ADMA descriptor table is mapped further down as we
	 * need to fill it with data first.
	 */

	host->align_addr = dma_map_single(mmc_dev(host->mmc),
		host->align_buffer, 128 * 4, direction);
	if (dma_mapping_error(mmc_dev(host->mmc), host->align_addr))
		goto fail;
	BUG_ON(host->align_addr & 0x3);

	host->sg_count = dma_map_sg(mmc_dev(host->mmc),
		data->sg, data->sg_len, direction);
	if (host->sg_count == 0)
		goto unmap_align;

	desc = host->adma_desc;
	align = host->align_buffer;

	align_addr = host->align_addr;

	for_each_sg(data->sg, sg, host->sg_count, i) {
		addr = sg_dma_address(sg);
		len = sg_dma_len(sg);

//		if(len==65536)
//		{
//		printk(" addr:0x%x len:%d ",addr, len);
//		}
		/*
		 * The SDHCI specification states that ADMA
		 * addresses must be 32-bit aligned. If they
		 * aren't, then we use a bounce buffer for
		 * the (up to three) bytes that screw up the
		 * alignment.
		 */
		offset = (4 - (addr & 0x3)) & 0x3;
		if (offset) {
			if (data->flags & MMC_DATA_WRITE) {
				buffer = tcc_mmc_kmap_atomic(sg, &flags);
				WARN_ON(((long)buffer & PAGE_MASK) > (PAGE_SIZE - 3));
				memcpy(align, buffer, offset);
				tcc_mmc_kunmap_atomic(buffer, &flags);
			}

			desc[7] = (align_addr >> 24) & 0xff;
			desc[6] = (align_addr >> 16) & 0xff;
			desc[5] = (align_addr >> 8) & 0xff;
			desc[4] = (align_addr >> 0) & 0xff;

			BUG_ON(offset > 65536);

			desc[3] = (offset >> 8) & 0xff;
			desc[2] = (offset >> 0) & 0xff;

			desc[1] = 0x00;
			desc[0] = 0x21; /* tran, valid */

			align += 4;
			align_addr += 4;

			desc += 8;

			addr += offset;
			len -= offset;
		}

		desc[7] = (addr >> 24) & 0xff;
		desc[6] = (addr >> 16) & 0xff;
		desc[5] = (addr >> 8) & 0xff;
		desc[4] = (addr >> 0) & 0xff;

		BUG_ON(len > 65536);

		desc[3] = (len >> 8) & 0xff;
		desc[2] = (len >> 0) & 0xff;

		desc[1] = 0x00;
		desc[0] = 0x21; /* tran, valid */

		desc += 8;

		/*
		 * If this triggers then we have a calculation bug
		 * somewhere. :/
		 */
		WARN_ON((desc - host->adma_desc) > (128 * 2 + 1) * 4);
	}

	desc -= 8;

	/*
	 * Add a terminating entry.
	 */
//	desc[7] = 0;
//	desc[6] = 0;
//	desc[5] = 0;
//	desc[4] = 0;
//	desc[3] = 0;
//	desc[2] = 0;
//	desc[1] = 0x00;
	desc[0] = 0x27; /* nop, end, valid */

	/*
	 * Resync align buffer as we might have changed it.
	 */
	if (data->flags & MMC_DATA_WRITE) {
		dma_sync_single_for_device(mmc_dev(host->mmc),
			host->align_addr, 128 * 4, direction);
	}

	host->adma_addr = dma_map_single(mmc_dev(host->mmc),	host->adma_desc, (128 * 2 + 1) * 4, DMA_TO_DEVICE);
	
//	printk("ADMA address: 0x%x\n",host->adma_addr);
	
	if (dma_mapping_error(mmc_dev(host->mmc), host->adma_addr))
		goto unmap_entries;
	BUG_ON(host->adma_addr & 0x3);

	return 0;

unmap_entries:
	dma_unmap_sg(mmc_dev(host->mmc), data->sg,
		data->sg_len, direction);
unmap_align:
	dma_unmap_single(mmc_dev(host->mmc), host->align_addr,
		128 * 4, direction);
fail:
	return -EINVAL;
}

static void tcc_mmc_adma_table_post(struct tcc_mmc_host *host, struct mmc_data *data)
{
	int direction;

	struct scatterlist *sg;
	int i, size;
	u8 *align;
	char *buffer;
	unsigned long flags;

	if (data->flags & MMC_DATA_READ)
		direction = DMA_FROM_DEVICE;
	else
		direction = DMA_TO_DEVICE;

	dma_unmap_single(mmc_dev(host->mmc), host->adma_addr,
		(128 * 2 + 1) * 4, DMA_TO_DEVICE);

	dma_unmap_single(mmc_dev(host->mmc), host->align_addr,
		128 * 4, direction);

	if (data->flags & MMC_DATA_READ) {
		dma_sync_sg_for_cpu(mmc_dev(host->mmc), data->sg,
			data->sg_len, direction);

		align = host->align_buffer;

		for_each_sg(data->sg, sg, host->sg_count, i) {
			if (sg_dma_address(sg) & 0x3) {
				size = 4 - (sg_dma_address(sg) & 0x3);

				buffer = tcc_mmc_kmap_atomic(sg, &flags);
				WARN_ON(((long)buffer & PAGE_MASK) > (PAGE_SIZE - 3));
				memcpy(buffer, align, size);
				tcc_mmc_kunmap_atomic(buffer, &flags);

				align += 4;
			}
		}
	}

	dma_unmap_sg(mmc_dev(host->mmc), data->sg,
		data->sg_len, direction);
}

static void tcc_mmc_finish_data(struct tcc_mmc_host *host)
{
	struct mmc_data *data;
	u16 blocks;

	BUG_ON(!host->data);
#ifdef SD_CLOCK_CONTROL
	clk_enable(host->fclk);
#endif

	data = host->data;
	host->data = NULL;

	if (host->flags & TCC_MMC_REQ_USE_DMA) {
		if (!host->is_direct) {
			if (host->flags & TCC_MMC_USE_ADMA) {
				tcc_mmc_adma_table_post(host, data);
			} else {
				dma_unmap_sg(NULL, data->sg, data->sg_len,
					(data->flags & MMC_DATA_READ)?DMA_FROM_DEVICE:DMA_TO_DEVICE);
			}
		} else {
			dma_unmap_sg(NULL, data->sg, data->sg_len,
				(data->flags & MMC_DATA_READ)?DMA_FROM_DEVICE:DMA_TO_DEVICE);
		}
	}

	/*
	 * Controller doesn't count down when in single block mode.
	 */
	if (data->blocks == 1) {
		blocks = (data->error == 0) ? 0 : 1;
	} else {
		blocks = tcc_readw(host->iobase+TCCSDHC_BLOCK_COUNT);
	}
	data->bytes_xfered = data->blksz * (data->blocks - blocks);

	if (!data->error && blocks) {
		printk(KERN_ERR "%s: Controller signalled completion even "
				"though there were blocks left.\n",
				mmc_hostname(host->mmc));
		data->error = -EIO;
	}
#if 0  //It must be enabled. When It doesn't sends STOP command
	if (data->stop) {
		/*
		 * The controller needs a reset of internal state machines
		 * upon error conditions.
		 */
		if (data->error) {
			tcc_sw_reset(host, HwSD_SRESET_RSTCMD);
			tcc_sw_reset(host, HwSD_SRESET_RSTDAT);           
		}
		tcc_mmc_start_command(host, data->stop);
	} else {
		tasklet_schedule(&host->finish_tasklet);
	}
#endif	
	tasklet_schedule(&host->finish_tasklet);
	
#ifdef SD_CLOCK_CONTROL
	clk_disable(host->fclk);
#endif
}

//FIXME: Not yet implemented...
static void tcc_transfer_pio(struct tcc_mmc_host *host)
{
	BUG_ON(!host->data);
	printk("\nenter %s  ", __func__);
}

static void tcc_mmc_start_command(struct tcc_mmc_host *host, struct mmc_command *cmd)
{
	u32 resptype;
	u32 cmdtype;
	u32 mask;
	unsigned long timeout;    
	int cmd_reg = 0x00000000;
	unsigned int uiIntStatusEn;

	/* Wait max 10 ms */
#if defined(CONFIG_ATHEROS_WIFI)
	timeout = jiffies + HZ/100;
#else 
	timeout = 10;    
#endif 

	cmdtype = 0;

	mask = HwSD_STATE_NOCMD;
	if ((cmd->data != NULL) || (cmd->flags & MMC_RSP_BUSY)) {
		mask |= HwSD_STATE_NODAT;
	}

	/* We shouldn't wait for data inihibit for stop commands, even
	   though they might use busy signaling */
	if (host->mrq->data && (cmd == host->mrq->data->stop)) {
		mask &= ~HwSD_STATE_NODAT;
	}

//	while (pSDMMC_SLOT0->STATE & mask) {
	while (tcc_readl(host->iobase+TCCSDHC_PRESENT_STATE) & mask) {
#if defined(CONFIG_ATHEROS_WIFI)
        if (time_is_after_jiffies(timeout)) {
#else
		if (timeout == 0) {
#endif
			printk(KERN_ERR "%s: Controller never released "
					"inhibit bit(s).\n", mmc_hostname(host->mmc));
			cmd->error = -EIO;
			tasklet_schedule(&host->finish_tasklet);
			return;
		}
#ifndef CONFIG_ATHEROS_WIFI /* atheros*/
		timeout--;
		mdelay(1);
#endif 
	}

	mod_timer(&host->timer, jiffies + 10 * HZ);

	host->cmd = cmd;    

	switch (mmc_resp_type(cmd)) {
		case MMC_RSP_NONE:
			resptype = 0;
			break;
		case MMC_RSP_R1:
			resptype = 2;
			break;
		case MMC_RSP_R1B:
			resptype = 3;
			break;
		case MMC_RSP_R2:
			resptype = 1;
			break;
		default:
			resptype = 2;
			break;
	}

	uiIntStatusEn = tcc_readl(host->iobase+TCCSDHC_INT_ENABLE);

	uiIntStatusEn |= HwSDINT_EN_TDONE | HwSDINT_EN_CDONE;

	if (cmd->data) {
		host->data = cmd->data;
		host->data_early = 0;

		cmd_reg |= HwSD_COM_TRANS_DATSEL | HwSD_COM_TRANS_DIR;

		if (cmd->data->blocks > 1) {
			cmd_reg |= HwSD_COM_TRANS_MS | HwSD_COM_TRANS_BCNTEN;
			if(cmd->opcode !=SD_IO_RW_EXTENDED) {
				cmd_reg |= HwSD_COM_TRANS_ACMD12;  // It is related STOP command
			}
		}

		if (cmd->data->flags & MMC_DATA_WRITE)
			cmd_reg &= ~HwSD_COM_TRANS_DIR;

		if (host->flags & TCC_MMC_USE_DMA)
			host->flags |= TCC_MMC_REQ_USE_DMA;

		if (host->flags & TCC_MMC_REQ_USE_DMA)
			cmd_reg |= HwSD_COM_TRANS_DMAEN;        

		/*
		 * Always adjust the DMA selection as some controllers
		 * (e.g. JMicron) can't do PIO properly when the selection
		 * is ADMA.
		 */
		if (host->flags & TCC_MMC_USE_DMA) {
			unsigned short ctrl = tcc_readw(host->iobase + TCCSDHC_HOST_CONTROL);
			ctrl &= ~HwSD_CTRL_DMA_MASK;
			if (!host->is_direct) {
				if ((host->flags & TCC_MMC_REQ_USE_DMA) && (host->flags & TCC_MMC_USE_ADMA))
					ctrl |= HwSD_CTRL_ADMA32;
				else
					ctrl |= HwSD_CTRL_SDMA;
			} else {
				ctrl |= HwSD_CTRL_SDMA;
			}
			tcc_writew(ctrl, host->iobase + TCCSDHC_HOST_CONTROL);
		}

		if (host->flags & TCC_MMC_REQ_USE_DMA) {
			if (!host->is_direct) {
				if (host->flags  & TCC_MMC_USE_ADMA) {
					tcc_mmc_adma_table_pre(host,cmd->data);
					writel(host->adma_addr, host->iobase+TCCSDHC_ADMA_ADDRESS);
				} else {
					int count;
					count = dma_map_sg(NULL, cmd->data->sg, cmd->data->sg_len,
						(cmd->data->flags & MMC_DATA_READ)?DMA_FROM_DEVICE:DMA_TO_DEVICE);
					BUG_ON(count != 1);

					tcc_writel(sg_dma_address(cmd->data->sg), host->iobase+TCCSDHC_DMA_ADDRESS);
				}
			} else {
				tcc_writel(host->req->dma, host->iobase+TCCSDHC_DMA_ADDRESS);
			}
		} else {
			if (!host->is_direct) {
				host->cur_sg = cmd->data->sg;
				host->num_sg = cmd->data->sg_len;
				host->offset = 0;
				host->remain = host->cur_sg->length;
			} else {
				printk(KERN_ERR "%s: only support DMA!!!", mmc_hostname(host->mmc));
			}
		}
		/* reset is_direct */
		//host->is_direct = 0;
		
	}    

	if (((cmd->opcode==SD_IO_RW_DIRECT)&& ((cmd->arg & (0x03<<28)) ==0)  
				&& ((cmd->arg&(0x1ff<<9)) ==(SDIO_CCCR_ABORT<<9)) &&(cmd->arg&0x07)) 
			|| (cmd->opcode==MMC_STOP_TRANSMISSION)) {
		/* Use R5b For CMD52, Function 0, I/O Abort 
		 * Need to be revised for handling CMD12       
		 */
		cmdtype = HwSD_COM_TRANS_ABORT;
	}

	if (cmd->flags & MMC_RSP_CRC)
		cmd_reg |= HwSD_COM_TRANS_CRCHK;

	if (cmd->flags & MMC_RSP_OPCODE)
		cmd_reg |= HwSD_COM_TRANS_CICHK;

	cmd_reg |= (cmd->opcode << 24) |cmdtype| (resptype << 16);

	if (tcc_readl(host->iobase+TCCSDHC_INT_STATUS) & HwSDINT_STATUS_ERR) {
		tcc_writew(tcc_readw(host->iobase+TCCSDHC_TIMEOUT_CONTROL)|(HwSD_SRESET_RSTCMD<<8), host->iobase+TCCSDHC_TIMEOUT_CONTROL);
		while (tcc_readw(host->iobase+TCCSDHC_TIMEOUT_CONTROL) & HwSD_SRESET_RSTCMD<<8);
			msleep(1);
	}
	
	tcc_writel(tcc_readl(host->iobase+TCCSDHC_INT_STATUS),host->iobase+TCCSDHC_INT_STATUS);

	if (cmd->data) {
		tcc_writew((0x07<<12) | cmd->data->blksz, host->iobase+TCCSDHC_BLOCK_SIZE);
		tcc_writew(cmd->data->blocks , host->iobase+TCCSDHC_BLOCK_COUNT);
	} else {
		tcc_writel(0, host->iobase+TCCSDHC_DMA_ADDRESS);	
		tcc_writew(0, host->iobase+TCCSDHC_BLOCK_SIZE);
		tcc_writew(0, host->iobase+TCCSDHC_BLOCK_COUNT);
	}

	tcc_writel(cmd->arg, host->iobase+TCCSDHC_ARGUMENT);
	
	/* Enable transfer interrupt sources */
	tcc_writel(uiIntStatusEn, host->iobase+TCCSDHC_INT_ENABLE);
	tcc_writel(cmd_reg, host->iobase +TCCSDHC_TMODE_COM);
	SD_DEBUG("%s: opcode=%d, arg=%x\n", __func__, cmd->opcode, cmd->arg);
	//SD_DEBUG("data:%x,blksz:%d,blk:%d,flags:%x\n\n", data, data->blksz, data->blocks, data->flags);		
}

static void tcc_mmc_finish_command(struct tcc_mmc_host *host)
{
	BUG_ON(host->cmd == NULL);

	if (host->cmd->flags & MMC_RSP_PRESENT) {
		if (host->cmd->flags & MMC_RSP_136) {
			host->cmd->resp[0] = tcc_readl(host->iobase+TCCSDHC_RESPONSE76);
			host->cmd->resp[1] = tcc_readl(host->iobase+TCCSDHC_RESPONSE54);
			host->cmd->resp[2] = tcc_readl(host->iobase+TCCSDHC_RESPONSE32);
			host->cmd->resp[3] = tcc_readl(host->iobase+TCCSDHC_RESPONSE10);

			SD_DEBUG("%s: R2: resp[0]=0x%08x, resp[1]=0x%08x, resp[2]=0x%08x, resp[3]=0x%08x\n",
					__func__, host->cmd->resp[0], host->cmd->resp[1], host->cmd->resp[2], host->cmd->resp[3]);

			host->cmd->resp[0] = (host->cmd->resp[0] << 8) | ((host->cmd->resp[1] & 0xFF000000) >> 24);
			host->cmd->resp[1] = (host->cmd->resp[1] << 8) | ((host->cmd->resp[2] & 0xFF000000) >> 24);
			host->cmd->resp[2] = (host->cmd->resp[2] << 8) | ((host->cmd->resp[3] & 0xFF000000) >> 24);
			host->cmd->resp[3] <<= 8;                

		} else {
			host->cmd->resp[0] = tcc_readl(host->iobase+TCCSDHC_RESPONSE10);
			
			SD_DEBUG("%s: R1: resp[0]=0x%08x\n",  __func__, host->cmd->resp[0]);
		}
	}

	host->cmd->error = 0;

	if (host->data && host->data_early)
		tcc_mmc_finish_data(host);

	if (!host->cmd->data)
		tasklet_schedule(&host->finish_tasklet);

	host->cmd = NULL;
}

static void tcc_mmc_request(struct mmc_host *mmc, struct mmc_request *mrq)
{
	struct tcc_mmc_host *host = mmc_priv(mmc);
	unsigned long flags;

	spin_lock_irqsave(&host->lock, flags);
#ifdef SD_CLOCK_CONTROL
	clk_enable(host->fclk);
#endif

	BUG_ON(host->mrq != NULL);

	host->mrq = mrq;

	if(!tcc_mmc_card_inserted(host)) {
		if (mrq->data && !(mrq->data->flags & MMC_DATA_READ)) {
			mrq->cmd->error = 0;
			mrq->data->bytes_xfered = mrq->data->blksz *
						  mrq->data->blocks;
		} else
			mrq->cmd->error = -ENOMEDIUM;	

		tasklet_schedule(&host->finish_tasklet);
		spin_unlock_irqrestore(&host->lock, flags);
	//	mmc_request_done(mmc, mrq);
		return;		
		
	} else {
		tcc_mmc_start_command(host, mrq->cmd);        
	}
	
	mmiowb();
	spin_unlock_irqrestore(&host->lock, flags);
}

/* High speed mode threshold (Hz).
 * 
 * Although high speed mode should be suitable for all speeds not all
 * controller/card combinations are capable of meeting the higher
 * tolerances for (e.g.) clock rise/fall times.  Therefore, default
 * mode is used where possible for improved compatibility. */
 
#define SDIO_CLOCK_FREQ_HIGH_SPD 25000000

static void tcc_hw_set_high_speed(struct mmc_host *mmc, int hs)
{
	struct tcc_mmc_host *host = mmc_priv(mmc);
	unsigned long flags;
	unsigned short host_ctrl = 0;

	spin_lock_irqsave(&host->lock, flags);

	host_ctrl= tcc_readw(host->iobase+TCCSDHC_HOST_CONTROL);
	host_ctrl &= ~HwSD_POWER_HS;
	
	if (hs) {
		//host_ctrl |= HwSD_POWER_HS;
	}
	
	tcc_writew(host_ctrl, host->iobase+TCCSDHC_HOST_CONTROL);

	spin_unlock_irqrestore(&host->lock, flags);
}

static void tcc_mmc_set_ios(struct mmc_host *mmc, struct mmc_ios *ios)
{
	unsigned long flags;    
	unsigned uBCLKCTR;
	struct tcc_mmc_host *host = mmc_priv(mmc);
	uint32_t temp_reg;    
	unsigned short host_ctrl = 0;	


	long dwMaxClockRate = host->peri_clk;
	int i = 0; /* 2^i is the divisor value */
	u32 clk_div=0;

	spin_lock_irqsave(&host->lock, flags);

	uBCLKCTR = HwIOBUSCFG->HCLKEN0;
	HwIOBUSCFG->HCLKEN0 |= HwIOBUSCFG_SD;

	if (ios->clock != 0) {
		tcc_hw_set_high_speed(mmc, ios->clock >SDIO_CLOCK_FREQ_HIGH_SPD);
		/* shift MaxClockRate until we find the closest frequency <= target */
		while ((ios->clock < dwMaxClockRate) && (i < 8)) {
			dwMaxClockRate = dwMaxClockRate >> 1;
			i++;
		}

		//printk("\nclock value = %d\n", i);                 
		if(i==0)
			clk_div = 0;								
		else
			clk_div = 1<<(i-1);
		
		temp_reg = tcc_readl(host->iobase+TCCSDHC_CLOCK_CONTROL);
		tcc_writew(temp_reg & ~HwSDCLKSEL_SCK_EN, host->iobase+TCCSDHC_CLOCK_CONTROL);

		udelay(10);

		tcc_writew((clk_div << 8)|HwSDCLKSEL_INCLK_EN, host->iobase+TCCSDHC_CLOCK_CONTROL);

		while (!(tcc_readl(host->iobase+TCCSDHC_CLOCK_CONTROL) & HwSDCLKSEL_INCLK_STABLE)) 	{
			udelay(100);
		}
		temp_reg = tcc_readl(host->iobase+TCCSDHC_CLOCK_CONTROL);
//		tcc_writew(temp_reg|(HwSDCLKSEL_SCK_EN|HwSDCLKSEL_INCLK_EN), host->iobase+TCCSDHC_CLOCK_CONTROL);			 
		tcc_writew(temp_reg|(HwSDCLKSEL_SCK_EN), host->iobase+TCCSDHC_CLOCK_CONTROL);			 				
		udelay(200);
	}

	switch (ios->power_mode) {
		case MMC_POWER_OFF:
			SD_DEBUG("-----------------------MMC_POWER_OFF %d\n",host->id);
		
#if defined(CONFIG_MACH_TCC9200) || defined(CONFIG_MACH_TCC9201) || defined(CONFIG_MACH_TCC8900)
			/* power */

			if (host->id == 1) {
				tcc_pca953x_setup(PCA9539_U3_SLAVE_ADDR, SD0, OUTPUT, OFF, SET_DIRECTION|SET_VALUE); 		 
				/* slot0 port2 - GPIO mode */
				BITCLR(HwGPIOE->GPFN1, Hw32-Hw16);	/* GPIOE[15:12] */
				BITCLR(HwGPIOE->GPFN2, Hw24-Hw16);	/* GPIOE[21:20] */
				//BITCLR(HwGPIOA->GPFN0, Hw28-Hw24);	/* GPIOA[6] 	*/
				/* output mode */
				BITSET(HwGPIOE->GPEN, Hw21|Hw20|Hw15|Hw14|Hw13|Hw12);
				//BITSET(HwGPIOA->GPEN, Hw6);
			}
			//mdelay(10);						 
#endif
			break;
		case MMC_POWER_UP:
			SD_DEBUG("-----------------------MMC_POWER_UP %d\n", host->id);
#if defined(CONFIG_MACH_TCC9200) || defined(CONFIG_MACH_TCC9201) || defined(CONFIG_MACH_TCC8900)
			/* power */
			if (host->id == 1) {
				tcc_pca953x_setup(PCA9539_U3_SLAVE_ADDR, SD0, OUTPUT, OFF, SET_DIRECTION|SET_VALUE);			
				msleep(100);			
				tcc_pca953x_setup(PCA9539_U3_SLAVE_ADDR, SD0, OUTPUT, ON, SET_DIRECTION|SET_VALUE);
			}	
			//mdelay(10);
#endif			
			init_mmc_host(host);

			break;
		case MMC_POWER_ON:
			SD_DEBUG("-----------------------MMC_POWER_ON %d\n",host->id);
	//		init_mmc_host(host);
			break;
	}

	switch (ios->bus_width) {
		case MMC_BUS_WIDTH_1:
			SD_DEBUG("\n/***********1 bit mode***************/ \n"); 
			if(host->id ==1){
#ifdef CONFIG_MACH_TCC9200S
				BITCSET(HwGPIOE->GPFN0, HwPORTCFG_GPFN6_MASK|HwPORTCFG_GPFN7_MASK,
							HwPORTCFG_GPFN6(3)|HwPORTCFG_GPFN7(3));
				BITCSET(HwGPIOE->GPFN1, HwPORTCFG_GPFN0_MASK|HwPORTCFG_GPFN1_MASK|HwPORTCFG_GPFN2_MASK|HwPORTCFG_GPFN3_MASK,
							HwPORTCFG_GPFN0(3)|HwPORTCFG_GPFN1(3)|HwPORTCFG_GPFN2(3)|HwPORTCFG_GPFN3(3));
#endif
			}else if(host->id == 0){
#ifdef CONFIG_MACH_TCC9200S
				//gpio	//
//				BITCSET(HwGPIOB->GPFN3, HwPORTCFG_GPFN1_MASK|HwPORTCFG_GPFN2_MASK|HwPORTCFG_GPFN3_MASK|HwPORTCFG_GPFN4_MASK|HwPORTCFG_GPFN5_MASK|HwPORTCFG_GPFN6_MASK,		
//							HwPORTCFG_GPFN1(2)|HwPORTCFG_GPFN2(2)|HwPORTCFG_GPFN3(2)|HwPORTCFG_GPFN4(2)|		HwPORTCFG_GPFN5(2)|HwPORTCFG_GPFN6(2)	);
#endif

			}

			host_ctrl =	tcc_readw(host->iobase+TCCSDHC_HOST_CONTROL);
			tcc_writew(host_ctrl&~(HwSD_POWER_SD4 | HwSD_POWER_SD8),host->iobase+TCCSDHC_HOST_CONTROL);
			host_ctrl =	tcc_readw(host->iobase+TCCSDHC_HOST_CONTROL);
			tcc_writew(host_ctrl|(HwSD_POWER_POW|HwSD_POWER_VOL33),host->iobase+TCCSDHC_HOST_CONTROL);
			host_ctrl =	tcc_readw(host->iobase+TCCSDHC_BLOCK_GAP_CONTROL);	
			tcc_writew(host_ctrl& ~Hw3, host->iobase+TCCSDHC_BLOCK_GAP_CONTROL);			

			break;
		case MMC_BUS_WIDTH_4:
			SD_DEBUG("\n/***********4 bit mode***************/\n "); 
			if(host->id ==1)
			{
#ifdef CONFIG_MACH_TCC9200S
				BITCSET(HwGPIOE->GPFN0, HwPORTCFG_GPFN6_MASK|HwPORTCFG_GPFN7_MASK,
							HwPORTCFG_GPFN6(3)|HwPORTCFG_GPFN7(3));
				BITCSET(HwGPIOE->GPFN1, HwPORTCFG_GPFN0_MASK|HwPORTCFG_GPFN1_MASK|HwPORTCFG_GPFN2_MASK|HwPORTCFG_GPFN3_MASK,
							HwPORTCFG_GPFN0(3)|HwPORTCFG_GPFN1(3)|HwPORTCFG_GPFN2(3)|HwPORTCFG_GPFN3(3));
#endif
			}
			else if(host->id ==0){		
			}

			host_ctrl = tcc_readw(host->iobase+TCCSDHC_HOST_CONTROL);
			tcc_writew(host_ctrl&~(HwSD_POWER_SD8),host->iobase+TCCSDHC_HOST_CONTROL);
			host_ctrl = tcc_readw(host->iobase+TCCSDHC_HOST_CONTROL);
			tcc_writew(host_ctrl|(HwSD_POWER_POW|HwSD_POWER_VOL33|HwSD_POWER_SD4),host->iobase+TCCSDHC_HOST_CONTROL);
			host_ctrl = tcc_readw(host->iobase+TCCSDHC_BLOCK_GAP_CONTROL);	
			tcc_writew(host_ctrl|Hw3, host->iobase+TCCSDHC_BLOCK_GAP_CONTROL);
			break;
		case MMC_BUS_WIDTH_8:
			SD_DEBUG("\n/***********8 bit mode***************/\n ");
			if (host->id == 0) {
			} else {
			}

			host_ctrl = tcc_readw(host->iobase+TCCSDHC_HOST_CONTROL);
			tcc_writew(host_ctrl|(HwSD_POWER_POW|HwSD_POWER_VOL33|HwSD_POWER_SD8),host->iobase+TCCSDHC_HOST_CONTROL);
			host_ctrl = tcc_readw(host->iobase+TCCSDHC_BLOCK_GAP_CONTROL);	
			tcc_writew(host_ctrl&~Hw3, host->iobase+TCCSDHC_BLOCK_GAP_CONTROL);
			
			break;
	}

	host->bus_mode = ios->bus_mode;

	//need to add resetting a controller?

	HwIOBUSCFG->HCLKEN0 = uBCLKCTR;	
	mmiowb();	 
	spin_unlock_irqrestore(&host->lock, flags);
}

static int tcc_mmc_get_ro(struct mmc_host *mmc)
{
	struct tcc_mmc_host *host = mmc_priv(mmc);
	unsigned long flags;    
	uint32_t reg=0;    
#ifdef SD_CLOCK_CONTROL
	clk_enable(host->fclk);
#endif

	spin_lock_irqsave(&host->lock, flags);
	reg = tcc_readl(host->iobase + TCCSDHC_PRESENT_STATE);
#ifdef SD_CLOCK_CONTROL
	clk_disable(host->fclk);
#endif
	spin_unlock_irqrestore(&host->lock, flags);

	return !(reg & HwSD_STATE_SDWP);
}

//fix me, MCC
static int tcc_mmc_get_cd(struct mmc_host *mmc)
{
	struct tcc_mmc_host *host = mmc_priv(mmc);
	unsigned int status;

	status=	tcc_mmc_card_inserted;
	udelay(500);	// jitter?
	
	return (tcc_mmc_card_inserted(host)); // return 1 card inserted, return 0 card not inserted
}


static void tcc_sdio_hw_enable_int(struct mmc_host *mmc, uint32_t sigs)
{
	struct tcc_mmc_host *host = mmc_priv(mmc);
	unsigned long flags;
	uint32_t stat_en;

	spin_lock_irqsave(&host->lock, flags);
	stat_en=tcc_readl(host->iobase+TCCSDHC_INT_ENABLE);	
	tcc_writel( stat_en | sigs, host->iobase+TCCSDHC_INT_ENABLE);

	spin_unlock_irqrestore(&host->lock, flags);
}

static void tcc_sdio_hw_disable_int(struct mmc_host *mmc, uint32_t sigs)
{
	struct tcc_mmc_host *host = mmc_priv(mmc);
	unsigned long flags;
	uint32_t stat_en;    

	spin_lock_irqsave(&host->lock, flags);

	stat_en=tcc_readl(host->iobase+TCCSDHC_INT_ENABLE);
	tcc_writel(stat_en & ~sigs, host->iobase+TCCSDHC_INT_ENABLE);

	spin_unlock_irqrestore(&host->lock, flags);
}

static void tcc_sdio_enable_card_int(struct mmc_host *mmc)
{
	tcc_sdio_hw_enable_int(mmc, HwSDINT_EN_CDINT);
}

static void tcc_sdio_disable_card_int(struct mmc_host *mmc)
{
	tcc_sdio_hw_disable_int(mmc, HwSDINT_EN_CDINT);
}


static void tcc_mmc_check_status(struct tcc_mmc_host *host)
{
	unsigned int status;
	
	SD_DEBUG("%s: cd detect\n",mmc_hostname(host->mmc));
	// 1: insert  0:remove 
	status=tcc_mmc_get_cd(host->mmc);

	host->card_inserted=status;

	if(status^host->oldstat)
	{
		SD_DEBUG("%s: Slot status change detected (%d -> %d)\n", mmc_hostname(host->mmc), host->oldstat, status);
		if (status)
			mmc_detect_change(host->mmc, msecs_to_jiffies(1000));		
		else
			mmc_detect_change(host->mmc, 0);	
	}

#ifdef SD_CLOCK_CONTROL
	if(host->id==0)
		clk_enable(host->fclk); // wifi always enable
#endif

	host->oldstat=status;

}
/*
 * ISR for handling card insertion and removal
 */
static irqreturn_t tcc_mmc_cd_irq(int irq, void *dev_id)
{
	struct tcc_mmc_host *host = (struct tcc_mmc_host *)dev_id;
	
	spin_lock(&host->lock);

	tcc_mmc_check_status(host);

	spin_unlock(&host->lock);

	return IRQ_HANDLED;
}
/*
 * Interrupt handling
 */
static void tcc_mmc_cmd_irq(struct tcc_mmc_host *host, u32 intmask)
{
	BUG_ON(intmask == 0);

	if (!host->cmd) {
		printk(KERN_ERR "%s: Got command interrupt 0x%08x even "
				"though no command operation was in progress.\n",
				mmc_hostname(host->mmc), (unsigned)intmask);
		return;
	}

	if (intmask & HwSDINT_STATUS_CMDTIME) {
		host->cmd->error = -ETIMEDOUT;
	} else if (intmask & (HwSDINT_STATUS_CMDCRC | HwSDINT_STATUS_CMDEND | HwSDINT_STATUS_CINDEX)) {
		host->cmd->error = -EILSEQ;
	}

	if (host->cmd->error) {
		tasklet_schedule(&host->finish_tasklet);
	} else if (intmask & HwSDINT_STATUS_CDONE) {
		tcc_mmc_finish_command(host);
	}
}

static void tcc_mmc_data_irq(struct tcc_mmc_host *host, u32 intmask)
{
	BUG_ON(intmask == 0);

	if (!host->data) {
		/*
		 * A data end interrupt is sent together with the response
		 * for the stop command.
		 */
		if (intmask & HwSDINT_STATUS_TDONE)
			return;

		printk(KERN_ERR "%s: Got data interrupt 0x%08x even "
				"though no data operation was in progress.\n",
				mmc_hostname(host->mmc), (unsigned)intmask);
		return;
	}

	if (intmask & HwSDINT_STATUS_DATTIME) {
		host->data->error = -ETIMEDOUT;
	} else if (intmask & (HwSDINT_STATUS_DATCRC | HwSDINT_STATUS_DATEND)) {
		host->data->error = -EILSEQ;
	} else if (intmask & HwSDINT_STATUS_ADMA) {
		host->data->error = -EIO;
	}

	if (host->data->error) {
		tcc_mmc_finish_data(host);
	} else {
		if (intmask & (HwSDINT_STATUS_RDRDY | HwSDINT_STATUS_WRRDY)) {
			printk(" \n %x \n ", intmask);
			tcc_transfer_pio(host);
		}

		/*
		 * We currently don't do anything fancy with DMA
		 * boundaries, but as we can't disable the feature
		 * we need to at least restart the transfer.
		 */
		if (intmask & HwSDINT_STATUS_DMA){
			tcc_writel(tcc_readl(host->iobase+TCCSDHC_DMA_ADDRESS), host->iobase+TCCSDHC_DMA_ADDRESS);			
		}			

		if (intmask & HwSDINT_STATUS_TDONE) {
			if (host->cmd) {
				/*
				 * Data managed to finish before the
				 * command completed. Make sure we do
				 * things in the proper order.
				 */
				host->data_early = 1;
			} else {
				tcc_mmc_finish_data(host);
			}
		}
	}
}

static irqreturn_t tcc_mmc_interrupt_handler(int irq, void *dev_id)
{
	unsigned int IntStatus;
	irqreturn_t result;    
	int cardint = 0;    
	struct tcc_mmc_host *host = (struct tcc_mmc_host *) dev_id;

	spin_lock(&host->lock);
#ifdef SD_CLOCK_CONTROL
	clk_enable(host->fclk);
#endif

	IntStatus = tcc_readl(host->iobase+TCCSDHC_INT_STATUS);


	if (IntStatus == 0 || IntStatus == 0xffffffff ) {
		result=IRQ_NONE;
		goto out;
	}

	if (IntStatus & HwSDINT_CMD_MASK) {
		tcc_writel(IntStatus & HwSDINT_CMD_MASK,host->iobase+TCCSDHC_INT_STATUS);		
		tcc_mmc_cmd_irq(host, IntStatus & HwSDINT_CMD_MASK);
	}

	if (IntStatus & HwSDINT_DATA_MASK) {
		tcc_writel(IntStatus & HwSDINT_DATA_MASK,host->iobase+TCCSDHC_INT_STATUS);		
		tcc_mmc_data_irq(host, IntStatus & HwSDINT_DATA_MASK);
	}

	IntStatus &= ~(HwSDINT_CMD_MASK |HwSDINT_DATA_MASK);
	IntStatus &= ~HwSDINT_STATUS_ERR;
#if defined(CONFIG_ATHEROS_WIFI)
	if(IntStatus & HwSDINT_STATUS_CDINT) {
		if (readl(host->iobase + TCCSDHC_INT_ENABLE) & HwSDINT_STATUS_CDINT) {
		cardint =1;
	}
	}
#else
	if(IntStatus & HwSDINT_STATUS_CDINT) {
		cardint =1;
	}
#endif 

	IntStatus &= ~ HwSDINT_STATUS_CDINT;

	if (IntStatus) {
		SD_DEBUG(KERN_ERR "%s: Unexpected interrupt 0x%08x.\n",   mmc_hostname(host->mmc), IntStatus);    
		tcc_writel(IntStatus, host->iobase+TCCSDHC_INT_STATUS);		
	}

	result=IRQ_HANDLED;    

	mmiowb();

out:
	spin_unlock(&host->lock);    

	if (cardint)
		mmc_signal_sdio_irq(host->mmc);

#ifdef SD_CLOCK_CONTROL
	clk_disable(host->fclk);
#endif

	return result;
}

static void tcc_enable_sdio_irq(struct mmc_host *mmc, int enable)
{
	if (enable) {
		tcc_sdio_enable_card_int(mmc);
	} else {
		tcc_sdio_disable_card_int(mmc);
	}
}

static struct mmc_host_ops tcc_mmc_ops = {
	.request		 = tcc_mmc_request,
	.set_ios		 = tcc_mmc_set_ios,
	.get_ro			 = tcc_mmc_get_ro,
	.get_cd			 = tcc_mmc_get_cd,
	.enable_sdio_irq = tcc_enable_sdio_irq,	
};

/*
 * Tasklets
 */
static void tcc_mmc_tasklet_finish(unsigned long param)
{
	struct tcc_mmc_host *host;
	unsigned long flags;
	struct mmc_request *mrq;

	host = (struct tcc_mmc_host *)param;

	spin_lock_irqsave(&host->lock, flags);

	del_timer(&host->timer);

	mrq = host->mrq;

	/*
	 * The controller needs a reset of internal state machines
	 * upon error conditions.
	 */
	if (mrq->cmd->error ||
			(mrq->data && (mrq->data->error ||
						   (mrq->data->stop && mrq->data->stop->error)))) {

		/* Spec says we should do both at the same time, but Ricoh
		   controllers do not like that. */
		tcc_sw_reset(host, HwSD_SRESET_RSTCMD);
		tcc_sw_reset(host, HwSD_SRESET_RSTDAT);                        
	}

	host->mrq = NULL;
	host->cmd = NULL;
	host->data = NULL;

	mmiowb();
#ifdef SD_CLOCK_CONTROL
	clk_disable(host->fclk);
#endif
	/* reset is_direct */
	host->is_direct = 0;
	
	spin_unlock_irqrestore(&host->lock, flags);
	mmc_request_done(host->mmc, mrq);
}

static void tcc_mmc_timeout_timer(unsigned long data)
{
	struct tcc_mmc_host *host;
	unsigned long flags;

	host = (struct tcc_mmc_host*)data;

	spin_lock_irqsave(&host->lock, flags);

	if (host->mrq) {
		printk(KERN_ERR "%s: Timeout waiting for hardware "
				"interrupt.\n", mmc_hostname(host->mmc));

		if (host->data) {
			host->data->error = -ETIMEDOUT;
			tcc_mmc_finish_data(host);
		} else {
			if (host->cmd)
				host->cmd->error = -ETIMEDOUT;
			else
				host->mrq->cmd->error = -ETIMEDOUT;

			tasklet_schedule(&host->finish_tasklet);
		}
	}    
	mmiowb();
	spin_unlock_irqrestore(&host->lock, flags);
}

static void init_mmc_host(struct tcc_mmc_host *host)
{
#if defined(SD_EXTENTION_SUB)
	static int i=0;

	if(i==0)
	{
		tcc_pca953x_setup(PCA9539_U3_SLAVE_ADDR, ATAPI, OUTPUT, LOW, SET_DIRECTION | SET_VALUE);
		msleep(100);	
		tcc_pca953x_setup(PCA9539_U3_SLAVE_ADDR, ATAPI, OUTPUT, HIGH, SET_DIRECTION | SET_VALUE);
		msleep(100);		
		BITCSET(HwGPIOF->GPFN2,HwPORTCFG_GPFN1_MASK|HwPORTCFG_GPFN0_MASK ,HwPORTCFG_GPFN1(0)|HwPORTCFG_GPFN0(0));	
		BITSET(HwGPIOF->GPEN, Hw16|Hw17);
		BITSET( HwGPIOF->GPDAT , Hw16|Hw17); 
		i=1;		
	}
#endif

	if(host->id == 0)
	{
	//clk 
		tca_ckc_setperi(PERI_SDMMC0,ENABLE,480000, PCDIRECTPLL1);
	//signal
#if defined(CONFIG_MACH_TCC9200S)	
		BITCSET(pSDCHCTRL->SDPORTCTRL, HwSD_PORTCTRL_SLOT0_MASK, HwSD_PORTCTRL_SLOT0(6));
#if defined(CONFIG_BCM4325_WIFI_9200S_DEMO_BOARD)
				BITSET(HwGPIOE->GPEN, Hw4);
				BITSET(HwGPIOE->GPDAT, Hw4);
				msleep(10);
				BITSET(HwGPIOD->GPEN, Hw11);
				BITSET(HwGPIOD->GPDAT, Hw11);
#endif

#else
#if !defined(SD_EXTENTION_SUB)	
		BITCSET(pSDCHCTRL->SDPORTCTRL, HwSD_PORTCTRL_SLOT0_MASK, HwSD_PORTCTRL_SLOT0(5));
#else
		BITCSET(pSDCHCTRL->SDPORTCTRL, HwSD_PORTCTRL_SLOT0_MASK, HwSD_PORTCTRL_SLOT0(3));
#endif
#endif

		BITSET(HwIOBUSCFG->HCLKEN0, HwIOBUSCFG_SD);
		BITSET(HwIOBUSCFG->HRSTEN0, HwIOBUSCFG_SD);

		host->peri_clk = tca_ckc_getperi(PERI_SDMMC0) * 100;
		SD_DEBUG("tcc-sd%d: clock %lu\n", host->id,host->peri_clk);	

		pSDMMC_SLOT0->STSEN = 0x03ff00ff;
		pSDMMC_SLOT0->INTEN = 0x03ff01ff;

		BITCSET(pSDMMC_SLOT0->TIME, 0x00FF, 0x0E);



	//gpio
#if defined(CONFIG_MACH_TCC9200S)		
		BITSET(HwGPIOB->GPEN, Hw25|Hw26|Hw27|Hw28|Hw29|Hw30);	// SD_CLK(6), SD_CMD(6), SD_D0(6), SD_D1(6), SD_D2(6) SD_D3(6)	
		BITCSET(HwGPIOB->GPFN3, HwPORTCFG_GPFN1_MASK|HwPORTCFG_GPFN2_MASK|HwPORTCFG_GPFN3_MASK|HwPORTCFG_GPFN4_MASK|HwPORTCFG_GPFN5_MASK|HwPORTCFG_GPFN6_MASK,		
					HwPORTCFG_GPFN1(2)|HwPORTCFG_GPFN2(2)|HwPORTCFG_GPFN3(2)|HwPORTCFG_GPFN4(2)|		HwPORTCFG_GPFN5(2)|HwPORTCFG_GPFN6(2)	);

		BITCSET(HwGPIOB->GPCD1, 0x3FFC0000, 0x3FFC0000);	// driver strength

		BITSET(HwGPIOB->GPEN, Hw25|Hw26|Hw27|Hw28|Hw29|Hw30);	// SD_CLK(6), SD_CMD(6), SD_D0(6), SD_D1(6), SD_D2(6) SD_D3(6)
		BITCSET(HwGPIOB->GPFN3, 0x0FFFFFF0, 0x02222220);		// SD_CLK(6), SD_CMD(6), SD_D0(6), SD_D1(6), SD_D2(6) SD_D3(6)
		//BITSET(HwGPIOB->GPCD1, Hw29|Hw28 | Hw26|Hw25 | Hw24|Hw23 | Hw22|Hw21 | Hw20|Hw19 | Hw18|Hw17);	// drive strength //FIXME

		HwGPIOA->GPDAT &= ~Hw10;		

#else
#if !defined(SD_EXTENTION_SUB)	
		// SD port 5
		// GPIOB[3:0] - SD1_D[0:3]
		BITCSET(HwGPIOB->GPFN0, Hw16-Hw0, HwPORTCFG_GPFN3(2)|HwPORTCFG_GPFN2(2)|HwPORTCFG_GPFN1(2)|HwPORTCFG_GPFN0(2));
		// GPIOB[14:12] - SD1_CMD, SD1_CLK, SD1_WP
		BITCSET(HwGPIOB->GPFN1, Hw24-Hw16,HwPORTCFG_GPFN5(2)|HwPORTCFG_GPFN4(2));

		BITCSET(HwGPIOB->GPCD0, 0x0F0000FF, 0x0C000000);	// driver strength


		// GPIOA[10] - SD_CD (Card Detect)
		BITCLR(HwGPIOA->GPFN1, Hw12-Hw8);
		BITCLR(HwGPIOA->GPEN, Hw10);	// input mode
		HwGPIOA->GPDAT &= ~Hw10;
#else
		// SD port 3
		// GPIOF[3:0] - SD3_D[0:3]
		BITCSET(HwGPIOF->GPFN0, Hw16-Hw0, HwPORTCFG_GPFN3(2)|HwPORTCFG_GPFN2(2)|HwPORTCFG_GPFN1(2)|HwPORTCFG_GPFN0(2));
		// GPIOB[9:8] - SD3_CMD, SD3_CLK
		BITCSET(HwGPIOF->GPFN1, Hw8-Hw0,HwPORTCFG_GPFN0(2)|HwPORTCFG_GPFN1(2));

		BITCSET(HwGPIOF->GPCD0, 0x000F00FF, 0x00040000);	// driver strength


		// GPIOF[10] - SD_CD (Card Detect)
		BITCLR(HwGPIOF->GPFN1, Hw12-Hw8);
		BITCLR(HwGPIOF->GPEN, Hw10);	// input mode
		HwGPIOF->GPDAT &= ~Hw10;
#endif

#endif
		pSDMMC_SLOT0->CONTL &= ~(HwSD_POWER_SD4 | HwSD_POWER_SD8);
		pSDMMC_SLOT0->CONTL |= HwSD_POWER_POW;

		//pSDCHCTRL->SDPORTDLY0 = 0x07FFFFFF;
	}		
	else if(host->id==1)		
	{
		//clk 
		tca_ckc_setperi(PERI_SDMMC1,ENABLE,480000, PCDIRECTPLL1);
		//signal
#if defined(CONFIG_MACH_TCC9200S)
		BITCSET(pSDCHCTRL->SDPORTCTRL, HwSD_PORTCTRL_SLOT2_MASK, HwSD_PORTCTRL_SLOT2(4));
#else
#if !defined(SD_EXTENTION_SUB)	
		BITCSET(pSDCHCTRL->SDPORTCTRL, HwSD_PORTCTRL_SLOT2_MASK, HwSD_PORTCTRL_SLOT2(2));
#else
		BITCSET(pSDCHCTRL->SDPORTCTRL, HwSD_PORTCTRL_SLOT2_MASK, HwSD_PORTCTRL_SLOT2(1));
#endif	//SD_EXTENTION_SUB
#endif
		BITSET(HwIOBUSCFG->HCLKEN0, HwIOBUSCFG_SD);
		BITSET(HwIOBUSCFG->HRSTEN0, HwIOBUSCFG_SD);

		host->peri_clk = tca_ckc_getperi(PERI_SDMMC1) * 100;
		SD_DEBUG("tcc-sd%d: clock %lu\n",host->id, host->peri_clk); 

		pSDMMC_SLOT1->STSEN = 0x03ff00ff;
		pSDMMC_SLOT1->INTEN = 0x03ff01ff;

		BITCSET(pSDMMC_SLOT1->TIME, 0x00FF, 0x0E);

	//gpio	
#if defined(CONFIG_MACH_TCC9200S)	
		BITCSET(HwGPIOE->GPFN0, HwPORTCFG_GPFN6_MASK|HwPORTCFG_GPFN7_MASK,
					HwPORTCFG_GPFN6(3)|HwPORTCFG_GPFN7(3));
		BITCSET(HwGPIOE->GPFN1, HwPORTCFG_GPFN0_MASK|HwPORTCFG_GPFN1_MASK|HwPORTCFG_GPFN2_MASK|HwPORTCFG_GPFN3_MASK,
					HwPORTCFG_GPFN0(3)|HwPORTCFG_GPFN1(3)|HwPORTCFG_GPFN2(3)|HwPORTCFG_GPFN3(3));

		BITCSET(HwGPIOE->GPCD0, 0x00FFF000, 0x00003000);	// driver strength

		HwGPIOA->GPDAT &= ~Hw13;
#else
#if !defined(SD_EXTENTION_SUB)	
		// SD port 2
		// GPIOE[15:12] - SD0_D[0:3]
		BITCSET(HwGPIOE->GPFN1, Hw32-Hw16, Hw29|Hw25|Hw21|Hw17);
		// GPIOE[21:20] - SD0_CMD, SD0_CLK
		BITCSET(HwGPIOE->GPFN2, Hw24-Hw0, Hw21|Hw17|Hw13|Hw9|Hw5|Hw1);				/* GPIOE[21:20] - SD0_CMD, SD0_CLK */		

		BITCSET(HwGPIOE->GPCD0, 0xFF000000, 0x00000000);			/* driver strength */
		BITCSET(HwGPIOE->GPCD1, 0x00000FFF, 0x00000300);		

		// GPIOA[6] - SD_CD (Card Detect)
		BITCLR(HwGPIOA->GPFN0, Hw28-Hw24);
		BITCLR(HwGPIOA->GPEN, Hw6); 	// input mode
		HwGPIOA->GPDAT &= ~Hw6;
#else
		// SD port 1
		// GPIOF[23:18] - SD0_D[0:3],SD0_CMD, SD0_CLK
		BITCSET(HwGPIOF->GPFN2,HwPORTCFG_GPFN2_MASK|HwPORTCFG_GPFN3_MASK|HwPORTCFG_GPFN4_MASK|HwPORTCFG_GPFN5_MASK|HwPORTCFG_GPFN6_MASK|HwPORTCFG_GPFN7_MASK, 
					HwPORTCFG_GPFN2(2)|HwPORTCFG_GPFN3(2)|HwPORTCFG_GPFN4(2)|HwPORTCFG_GPFN5(2)|HwPORTCFG_GPFN6(2)|HwPORTCFG_GPFN7(2)); 	

		BITCSET(HwGPIOF->GPCD1, 0x0000FFF0, 0x0000C000);	// driver strength
		
		// GPIOF[24] - SD_CD (Card Detect)
		BITCLR(HwGPIOF->GPFN3, Hw4-Hw0);
		BITCLR(HwGPIOF->GPEN, Hw24);	// input mode
		HwGPIOF->GPDAT &= ~Hw24;
		
#endif

#endif
		pSDMMC_SLOT1->CONTL &= ~(HwSD_POWER_SD4 | HwSD_POWER_SD8);
		pSDMMC_SLOT1->CONTL |= HwSD_POWER_POW;

		//pSDCHCTRL->SDPORTDLY2 = 0x07FFFFFF;
	}
	
}

static int __init tcc_mmc_probe(struct platform_device *pdev)
{
	struct mmc_host *mmc;
	struct tcc_mmc_host *host = NULL;
	volatile PPIC pPIC = (volatile PPIC)tcc_p2v(HwPIC_BASE);
	int ret = 0, cd_irq=0;

#if defined(CONFIG_MMC_TCC_DEMO_BOARD_SD_WAKEUP)
	sd_detect_flag = 0;
#endif
	mmc = mmc_alloc_host(sizeof(struct tcc_mmc_host), &pdev->dev);
	if (!mmc)
		return -ENOMEM;

	host = mmc_priv(mmc);
	host->mmc = mmc;

	host->id = pdev->id;

	#ifdef SD_CLOCK_CONTROL
	if(host->id == 0){
		host->fclk = clk_get(NULL,"sdhc0_slot0");		
	}else if(host->id == 1){
		host->fclk = clk_get(NULL,"sdhc1_slot0");		
	}
	#endif

#if defined(CONFIG_MACH_TCC9200) || defined(CONFIG_MACH_TCC9201) || defined(CONFIG_MACH_TCC8900)
	//tcc_pca953x_setup(PCA9539_U3_SLAVE_ADDR, SD0, OUTPUT, LOW, SET_DIRECTION | SET_VALUE);
	//msleep(100);	
	//tcc_pca953x_setup(PCA9539_U3_SLAVE_ADDR, SD0, OUTPUT, HIGH, SET_DIRECTION | SET_VALUE);
	//msleep(100);	
	if(host->id == 0)
	{
		tcc_pca953x_setup(PCA9539_U3_SLAVE_ADDR, SD1, OUTPUT, OFF, SET_DIRECTION | SET_VALUE);
		msleep(100);	
		tcc_pca953x_setup(PCA9539_U3_SLAVE_ADDR, SD1, OUTPUT, ON, SET_DIRECTION | SET_VALUE);
		msleep(100);	
	}
#endif

	init_mmc_host(host);

	host->res = platform_get_resource(pdev, IORESOURCE_MEM, 0);
	host->irq = platform_get_irq(pdev,  0);
	host->cd_irq = platform_get_irq(pdev,  1);

	host->iobase = host->res->start;

	host->flags =0;

	// Use for EMMC.
#if defined(CONFIG_MMC_TCC_SUPPORT_EMMC	)
	if(host->id == 0)   //slot 1 is used for EMMC in TCC EVB.
		host->flags |= TCC_MMC_USE_EMMC;
#endif

	//ADMA
	host->flags |= TCC_MMC_USE_DMA;
#if defined(CONFIG_MMC_TCC_ADMA)
	host->flags |= TCC_MMC_USE_DMA|	TCC_MMC_USE_ADMA;	
#endif

	if (host->id == 1) {
		mmc->caps |= MMC_CAP_SDIO_IRQ;
		mmc->caps |= MMC_CAP_4_BIT_DATA;
//		mmc->caps |= MMC_CAP_8_BIT_DATA;	// if support MMC 8bit by H/W
		mmc->caps |= MMC_CAP_SD_HIGHSPEED;
		mmc->caps |= MMC_CAP_MMC_HIGHSPEED;
		mmc->ops = &tcc_mmc_ops;
		mmc->f_min = 100000;
		mmc->f_max = 48000000;	// support High-speed
		mmc->ocr_avail = MMC_VDD_32_33 | MMC_VDD_33_34;
	} else if (host->id == 0) {
		mmc->caps |= MMC_CAP_SDIO_IRQ;
		mmc->caps |= MMC_CAP_4_BIT_DATA;
		//mmc->caps |= MMC_CAP_8_BIT_DATA;	// Telechips EVB does not support Core1 MMC 8bit
		mmc->caps |= MMC_CAP_SD_HIGHSPEED;  // Uncomment, If support High speed SD.
		mmc->caps |= MMC_CAP_MMC_HIGHSPEED; // Uncomment, If support High speed MMC.
		mmc->ops = &tcc_mmc_ops;
		mmc->f_min = 100000;
#if defined(CONFIG_BCM4325_WIFI)
		mmc->f_max = 24000000;
#else
		mmc->f_max = 48000000;	// support High-speed
#endif
		mmc->ocr_avail = MMC_VDD_32_33 | MMC_VDD_33_34;
	}

	if (host->flags & TCC_MMC_USE_ADMA) {
		/*
		 * We need to allocate descriptors for all sg entries
		 * (128) and potentially one alignment transfer for
		 * each of those entries.
		 */
		host->adma_desc = kmalloc((128 * 2 + 1) * 4, GFP_KERNEL);
		host->align_buffer = kmalloc(128 * 4, GFP_KERNEL);
		if (!host->adma_desc || !host->align_buffer) {
			kfree(host->adma_desc);
			kfree(host->align_buffer);
			printk(KERN_WARNING "%s: Unable to allocate ADMA "
				"buffers. Falling back to standard DMA.\n",
				mmc_hostname(mmc));
			host->flags &= ~TCC_MMC_USE_ADMA;
		}
	}

	/*
	 * Maximum number of segments. Hardware cannot do scatter lists.
	 */
	if (host->flags & TCC_MMC_USE_DMA) {
		if(host->flags & TCC_MMC_USE_ADMA) {
			mmc->max_hw_segs = 128;
		}else{
			mmc->max_hw_segs = 1;			
		}
	} else {
		mmc->max_hw_segs = 128;
	}
	mmc->max_phys_segs = 128;

	/*
	 * Maximum number of sectors in one transfer. Limited by DMA boundary
	 * size (512KiB).
	 */
	mmc->max_req_size = 524288;

	/*
	 * Maximum segment size. Could be one segment with the maximum number
	 * of bytes.
	 */
 	if (host->flags & TCC_MMC_USE_ADMA)
//		mmc->max_seg_size = 65535;
		mmc->max_seg_size = 32768;
	else
	mmc->max_seg_size = mmc->max_req_size;

	/*
	 * Maximum block size. This varies from controller to controller and
	 * is specified in the capabilities register.
	 */
	//	mmc->max_blk_size = (caps & SDHCI_MAX_BLOCK_MASK) >> SDHCI_MAX_BLOCK_SHIFT;
	//	if (mmc->max_blk_size >= 3) {
	//		printk(KERN_WARNING "%s: Invalid maximum block size, assuming 512\n",
	//			host->slot_descr);
	mmc->max_blk_size = 512;
	//	} else
	//		mmc->max_blk_size = 512 << mmc->max_blk_size;

	/*
	 * Maximum block count.
	 */
	mmc->max_blk_count = 65535;

	host->dev = &pdev->dev;
	platform_set_drvdata(pdev, host);

	mmc_add_host(mmc);

	spin_lock_init(&host->lock);


	/*
	 * Init tasklets.
	 */
	tasklet_init(&host->finish_tasklet,tcc_mmc_tasklet_finish, (unsigned long)host);

	setup_timer(&host->timer, tcc_mmc_timeout_timer, (unsigned long)host);

#if 1
	pPIC->SEL1		|= HwINT1_SD0 <<host->id;
	pPIC->INTMSK1	|= HwINT1_SD0 <<host->id;
	pPIC->MODE1 	|= HwINT1_SD0 <<host->id;	// Level trigger
	pPIC->CLR1		|= HwINT1_SD0 <<host->id;		
#endif	

	snprintf(host->slot_desc, 16, "TCC_SD_SLOT%d", host->id);

	ret = request_irq(host->irq, tcc_mmc_interrupt_handler, IRQF_DISABLED, host->slot_desc, host);	
	if (ret)
		goto error;

	//Card detect
	if(host->cd_irq<0)
	{
		/* configuration the SD card insertion detection */
		init_timer(&host->detect_timer);
		host->detect_timer.function = tcc_mmc_poll_event;
		host->detect_timer.data = (unsigned long) host;
		host->detect_timer.expires = jiffies + DETECT_TIMEOUT;
		
		add_timer(&host->detect_timer);
	}
	else
	{
		host->oldstat= 0;

#if defined(CONFIG_MACH_TCC9200S) 		
		if(	host->id==1)
		{
			BITCSET(HwEINTSEL->EINTSEL2, HwEINTSEL2_EINT9_MASK, HwEINTSEL2_EINT9(SEL_GPIOA13)); 
		}			
		else if(host->id==0)
		{
			BITCSET(HwEINTSEL->EINTSEL2, HwEINTSEL2_EINT8_MASK, HwEINTSEL2_EINT8(SEL_GPIOA10)); 		
		}
		else
		{
		}
#elif defined(CONFIG_MACH_TCC9200) || defined(CONFIG_MACH_TCC9201) ||defined(CONFIG_MACH_TCC8900)
		if( host->id==1)
		{
			BITCSET(HwEINTSEL->EINTSEL2, HwEINTSEL2_EINT9_MASK, HwEINTSEL2_EINT9(SEL_GPIOA6)); 
		}			
		else if(host->id==0)
		{
			BITCSET(HwEINTSEL->EINTSEL2, HwEINTSEL2_EINT8_MASK, HwEINTSEL2_EINT8(SEL_GPIOA10)); 		
		}
		else
		{
		}
#endif	

		pPIC->SEL0		|= 1<<host->cd_irq;
		pPIC->INTMSK0	|= 1<<host->cd_irq;
		pPIC->MODE0 	&= ~(1<<host->cd_irq); // edge trigger
		pPIC->MODEA0	|= 1<<host->cd_irq;	//both edge
		pPIC->IEN0		|= 1<<host->cd_irq;		
		pPIC->CLR0		|= 1<<host->cd_irq;	

		ret = request_irq(host->cd_irq, tcc_mmc_cd_irq, IRQF_DISABLED, host->slot_desc, host);	
		if (ret)
			goto error;			

		tcc_mmc_check_status(host);
	}
	
	/* initialize the card-insert status */
	if (tcc_mmc_card_inserted(host)) {
		host->card_inserted = 1;

	} else {
		host->card_inserted = 0;
	}

	#ifdef SD_CLOCK_CONTROL
	if(host->id == 0 && host->card_inserted == 1)	
		clk_enable(host->fclk);	// wifi always enable 
	#endif
	
	printk("%s%d: init.\n", DRIVER_NAME,host->id);
	return ret;

error:
	printk("%s%d: failed.\n", DRIVER_NAME,host->id);
	return ret;   
}

static int tcc_mmc_remove(struct platform_device *pdev)
{
	struct mmc_host *mmc = platform_get_drvdata(pdev);

	platform_set_drvdata(pdev, NULL);

	if (mmc) {
		struct tcc_mmc_host *host = mmc_priv(mmc);

		mmc_remove_host(mmc);
		del_timer(&host->timer);
		del_timer_sync(&host->timer);        
		mmc_free_host(mmc);

		tasklet_kill(&host->finish_tasklet);

		kfree(host->adma_desc);
		kfree(host->align_buffer);

		host->adma_desc = NULL;
		host->align_buffer = NULL;
	}

	return 0;
}

#if defined(CONFIG_BCM4325_WIFI) 
int gSdio_suspend;
struct early_suspend dhdpm;
EXPORT_SYMBOL(dhdpm);

void register_mmc_card_pm(struct early_suspend *cardpm)
{
	if((cardpm != NULL) && (cardpm->suspend !=NULL) && (cardpm->resume != NULL))
	{
		dhdpm.suspend = cardpm->suspend;
		dhdpm.resume = cardpm->resume;
		printk("%s: [WiFi] Callbacks registered successfully. \n",__FUNCTION__);
	}
	else
		printk("%s: Error: Null pointer!! \n", __FUNCTION__);
}

EXPORT_SYMBOL(register_mmc_card_pm);

void unregister_mmc_card_pm(void)
{
	printk("%s: [WIFI] Unregistering suspend/resume callbacks.  \n", __FUNCTION__);
	dhdpm.suspend = NULL;
	dhdpm.resume  = NULL;	
}
EXPORT_SYMBOL(unregister_mmc_card_pm);
#endif /* CONFIG_BCM4325_WIFI */

#ifdef CONFIG_PM
static int tcc_mmc_suspend(struct platform_device *pdev, pm_message_t mesg)
{
	int ret = 0;
	volatile PPIC pPIC = (volatile PPIC)tcc_p2v(HwPIC_BASE);

	struct tcc_mmc_host *host = platform_get_drvdata(pdev);

	if (host && host->suspended)
		return 0;

#if defined(CONFIG_BCM4325_WIFI)
	if(host->id == 0)
	{
		if(dhdpm.suspend != NULL) {
			dhdpm.suspend(NULL);
			gSdio_suspend = 1;
		}
		else
			printk("[WiFi] %s: WIFI is not running.\n",__FUNCTION__);
		host->suspended = 1;
	}
#endif	/* CONFIG_BRCM_WL_HOSTWAKEUP */

	if (host) {
		ret = mmc_suspend_host(host->mmc, mesg);
		if (ret == 0)
			host->suspended = 1;
	}

	//disable cd interrupt
	if(!(host->cd_irq<0))
		pPIC->INTMSK0	&= ~(1<<host->cd_irq);

#if defined(CONFIG_MACH_TCC9200S)
        BITSET(HwGPIOE->GPEN, Hw27);
        BITCLR(HwGPIOE->GPDAT, Hw27);
#elif defined(CONFIG_MACH_TCC9200) || defined(CONFIG_MACH_TCC9201) ||defined(CONFIG_MACH_TCC8900)
        if(host->id == 0)
        	tcc_pca953x_setup(PCA9539_U3_SLAVE_ADDR, SD1, OUTPUT, OFF, SET_DIRECTION | SET_VALUE);
#endif	

	return ret;
}

static int tcc_mmc_resume(struct platform_device *pdev)
{
	int ret = 0;
	volatile PPIC pPIC = (volatile PPIC)tcc_p2v(HwPIC_BASE);	
	struct tcc_mmc_host *host = platform_get_drvdata(pdev);

#if defined(CONFIG_MACH_TCC9200S)
        BITSET(HwGPIOE->GPEN, Hw27);
        BITSET(HwGPIOE->GPDAT, Hw27);
#elif defined(CONFIG_MACH_TCC9200) || defined(CONFIG_MACH_TCC9201) ||defined(CONFIG_MACH_TCC8900)
		if(host->id == 0){   		
                tcc_pca953x_setup(PCA9539_U3_SLAVE_ADDR, SD1, OUTPUT, ON, SET_DIRECTION | SET_VALUE);
		msleep(100);
	}

#endif

	if (host && !host->suspended)
		return 0;

	if (host) {
		ret = mmc_resume_host(host->mmc);
		if (ret == 0)
			host->suspended = 0;
	}

	if(host){
		host->oldstat=tcc_mmc_get_cd(host->mmc);		
	}

#if defined(CONFIG_BCM4325_WIFI) 
	if(host->id == 0)
	{
		host->mmc->ios.bus_width = MMC_BUS_WIDTH_4;
		host->mmc->ios.clock = host->mmc->f_max;
		tcc_mmc_set_ios(host->mmc, &host->mmc->ios);

		if(dhdpm.resume != NULL) {
			dhdpm.resume(NULL);
			gSdio_suspend = 0;
		}
		else
			printk("[WiFi] %s: WIFI is not running.\n",__FUNCTION__);
		host->suspended = 0;

	}
#endif	/* CONFIG_BRCM_WL_HOSTWAKEUP */

	//enable cd interrupt
	if(!(host->cd_irq<0))
		pPIC->INTMSK0	|= 1<<host->cd_irq;

	return ret;
}
#else
#define tcc_mmc_suspend	NULL
#define tcc_mmc_resume	NULL
#endif

#if defined(CONFIG_MMC_TCC_SDHC_CORE0)
static struct platform_driver tcc_mmc_core0_driver = {
	.probe		= tcc_mmc_probe,
	.remove		= tcc_mmc_remove,
	.suspend	= tcc_mmc_suspend,
	.resume		= tcc_mmc_resume,
	.driver		= {
		.name	= DRIVER_NAME_CORE0,
	},
};
#endif

#if defined(CONFIG_MMC_TCC_SDHC_CORE1)
static struct platform_driver tcc_mmc_core1_driver = {
	.probe		= tcc_mmc_probe,
	.remove 	= tcc_mmc_remove,
	.suspend	= tcc_mmc_suspend,
	.resume 	= tcc_mmc_resume,
	.driver 	= {
		.name	= DRIVER_NAME_CORE1,
	},
};
#endif

static int __init tcc_mmc_init(void)
{
#if defined(CONFIG_MMC_TCC_SDHC_CORE0)
	platform_driver_register(&tcc_mmc_core0_driver);
#endif
#if defined(CONFIG_MMC_TCC_SDHC_CORE1)
	platform_driver_register(&tcc_mmc_core1_driver);	
#endif
	return 0;

}

static void __exit tcc_mmc_exit(void)
{
#if defined(CONFIG_MMC_TCC_SDHC_CORE0)
	platform_driver_unregister(&tcc_mmc_core0_driver);
#endif
#if defined(CONFIG_MMC_TCC_SDHC_CORE1)
	platform_driver_unregister(&tcc_mmc_core1_driver);
#endif


}

module_init(tcc_mmc_init);
module_exit(tcc_mmc_exit);

MODULE_DESCRIPTION("Telechips SD/MMC Card driver");
MODULE_LICENSE("GPL");
MODULE_ALIAS(DRIVER_NAME);
MODULE_AUTHOR("Telechips Inc. linux@telechips.com");
