
/****************************************************************************
 *   FileName    : tca_alarm.C
 *   Description : 
 ****************************************************************************
 *
 *   TCC Version 1.0
 *   Copyright (c) Telechips, Inc.
 *   ALL RIGHTS RESERVED
 *
 ****************************************************************************/
#ifdef __KERNEL__
#	if defined(_TCC79x_)
#		include <mach/tcc79x.h>
#	elif defined(_TCC83x_)
#		include <mach/tcc83x_virt.h>
#	endif
#else
#	if defined(_TCC79x_)
#		include "TCC79x_Virtual.h"
#	elif defined(_TCC83x_)
#		include "TCC83x_Virtual.h"
#	endif
#endif

/*****************************************************************************
*
* Header Files Include
*
******************************************************************************/
#include "tca_alarm.h"

/*****************************************************************************
*
* Defines
*
******************************************************************************/

/*****************************************************************************
*
* structures
*
******************************************************************************/


/*****************************************************************************
*
* Variables
*
******************************************************************************/



/*****************************************************************************
*
* Functions
*
******************************************************************************/

/*****************************************************************************
* Function Name : tca_rtcgettime()
******************************************************************************/
void tca_alarm_gettime(rtctime *pTime)
{
	unsigned uCON;
	
	BITSET(HwRTCCON, Hw1); // RTC Register write enabled
	BITSET(HwINTCON, Hw0);	// Interrupt Block Write Enable

	pTime->wSecond			= HwALMSEC;
	pTime->wMinute			= HwALMMIN;
	pTime->wHour			= HwALMHOUR;
	pTime->wDayOfWeek 		= HwALMDAY;
	pTime->wDay				= HwALMDATE;
	pTime->wMonth			= HwALMMON;
	pTime->wYear			= HwALMYEAR;
	uCON = HwRTCALM;
	
	BITCLR(HwINTCON, Hw0);	// Interrupt Block Write Disable
	BITCLR(HwRTCCON, Hw1);	// RTC Register write Disable

	/* Second */
	if (ISZERO(uCON, Hw0)) //HwRTCALM_SECEN_EN
		pTime->wSecond	= (unsigned char) 0;
	else
		pTime->wSecond	= FROM_BCD( pTime->wSecond );

	/* Minute */
	if (ISZERO(uCON, Hw1))//HwRTCALM_MINEN_EN
		pTime->wMinute	= (unsigned char) 0;
	else
		pTime->wMinute	= FROM_BCD( pTime->wMinute );

	/* Hour */
	if (ISZERO(uCON, Hw2))//HwRTCALM_HOUREN_EN
		pTime->wHour	= (unsigned char) 0;
	else
		pTime->wHour	= FROM_BCD( pTime->wHour );

	/* date */
	if (ISZERO(uCON, Hw3))//HwRTCALM_DATEEN_EN
		pTime->wDay	= (unsigned char) 0;
	else
		pTime->wDay	= FROM_BCD( pTime->wDay );

	/* month */
	if (ISZERO(uCON, Hw5))//HwRTCALM_MONEN_EN
		pTime->wMonth	= (unsigned char) 0;
	else
		pTime->wMonth	= FROM_BCD( pTime->wMonth );

	/* year */
	if (ISZERO(uCON, Hw6))//HwRTCALM_YEAREN_EN
		pTime->wYear	= (unsigned short) 0;
	else
		pTime->wYear	= FROM_BCD( pTime->wYear );

	/* weekdays */
	if (ISZERO(uCON, Hw4))//HwRTCALM_DAYEN_EN
		pTime->wDayOfWeek		= 1;
	else
		pTime->wDayOfWeek = FROM_BCD( pTime->wDayOfWeek );

	HwRTCALM  = uCON;

}

/*****************************************************************************
* Function Name : tca_rtcsettime()
******************************************************************************/
void tca_alarm_settime(rtctime *pTime)
{
	unsigned	uCON;

	BITSET(HwRTCCON, Hw1);	// RTC Register write enabled
	BITSET(HwINTCON, Hw0);	// Interrupt Block Write Enable
	uCON	= 0xEF; // Not wDayOfWeek
	/* Second */
	if ( pTime->wSecond > 59 )
		BITCLR(uCON, Hw0);//HwRTCALM_SECEN_EN
	else
		HwALMSEC	= TO_BCD( pTime->wSecond );

	/* Minute */
	if ( pTime->wMinute > 59 )
		BITCLR(uCON, Hw1);//HwRTCALM_MINEN_EN
	else
		HwALMMIN	= TO_BCD( pTime->wMinute );

	/* Hour */
	if ( pTime->wHour > 23 )
		BITCLR(uCON, Hw2);//HwRTCALM_HOUREN_EN
	else
		HwALMHOUR	= TO_BCD( pTime->wHour );

	/* Date */
	if ( pTime->wDay > 31 || pTime->wDay < 1 )
		BITCLR(uCON, Hw3);//HwRTCALM_DATEEN_EN
	else
		HwALMDATE	= TO_BCD( pTime->wDay );

	/* month */
	if (pTime->wMonth > 12 || pTime->wMonth < 1)
		BITCLR(uCON, Hw5);//HwRTCALM_MONEN_EN
	else
		HwALMMON	= TO_BCD( pTime->wMonth );

	/* year */
	if (pTime->wYear > 2099 || pTime->wYear < 1900)
		BITCLR(uCON, Hw6);//HwRTCALM_YEAREN_EN
	else
		HwALMYEAR	= TO_BCD( pTime->wYear );

	/* Day */
	if ( pTime->wDayOfWeek > 6)
		BITCLR(uCON, Hw4);//HwRTCALM_DAYEN_EN
	else
		HwALMDAY	= pTime->wDayOfWeek+1;

	// Enable ALARM
	HwRTCALM	= uCON;

	BITCLR(HwRTCIM,Hw3|Hw1|Hw0);	// Normal Mode,PMWKUP Active low, Disable alarm interrupt mode
#if defined(_TCC79x_)
	BITSET(HwRTCIM,Hw0);	////Suports on the edge alarm interrupt
#elif  defined(_TCC83x_)
	BITSET(HwRTCIM,Hw1|Hw0);	//Support on the level alarm interrupt
#endif
	HwRTCPEND	= 0;
	#if defined(_TCC79x_)
	HwRTCSTR	= HwRTCSTR;
	#endif
	BITCLR(HwINTCON,Hw0);	// Normal Mode,PMWKUP Active low, Disable alarm interrupt mode
	BITCSET(HwRTCCON, Hw1, Hw6);	// Enable Alarm Interrupt

}

/************************************************************************************************
* FUNCTION		:  tca_alarm_setint
*
* DESCRIPTION	: 
*
************************************************************************************************/
#if 1
void tca_alarm_setint(void)
{
	rtctime lpTime;

	//Set Alarm
    tca_rtc_gettime(&lpTime);
	lpTime.wMinute += 1;
	tca_alarm_settime(&lpTime);
	
}
#else
void tca_alarm_setint(rtctime *lpTime)
{
    tca_alarm_settime(lpTime);
}
#endif
/************************************************************************************************
* FUNCTION		:  tca_alarm_setpmwkup
*
* DESCRIPTION	: 
*
************************************************************************************************/
#if 1
void tca_alarm_setpmwkup(void)
{
	rtctime lpTime;

	//Set Alarm
    tca_rtc_gettime(&lpTime);
	lpTime.wMinute += 1;
	tca_alarm_settime(&lpTime);
	
#if defined(_TCC79x_)
	HwCLR = HwINT_RTC;
	HwINTMSK 	|= HwINT_RTC;
	HwSEL	|= HwINT_RTC;
#elif  defined(_TCC83x_)
	HwCLR = HwIEN_RTC;
	HwIEN 	&= ~HwIEN_RTC;
	HwSEL	&= ~HwIEN_RTC;
	
	HwIEN 	|= HwIEN_RTC_P;
	HwSEL	|= HwIEN_RTC_P;
#elif  defined(_TCC80x_)
	HwCREQ_A = Hw19;
	HwIEN_A &= ~Hw19;
	HwIRQSEL_A	&= ~Hw19;
	
	HwIEN_A	|= Hw18;
	HwIRQSEL_A	|= Hw18;
#endif

	BITSET(HwRTCCON, Hw1);//HwRTCCON_RTCEN
	BITSET(HwINTCON, Hw0);//HwINTCON_INTWREN
	BITCLR(HwRTCIM, 0xf);
//	BITSET(HwRTCIM, HwRTCIM_ALMINT_EDGE_EN|HwRTCIM_PMWKUP_ACTIVE_HIGH|HwRTCIM_PWDN_POWERDOWN_MODE);	
	BITSET(HwRTCIM, Hw0|Hw2|Hw3);	

//	BITSET(HwRTCCON, HwRTCCON_RTCEN | HwRTCCON_WUOUT_EN);
	BITSET(HwRTCCON, Hw1 | Hw7);
	//PMWKUP Disable Start
	//HwRTCCON = 0;
	//BITSET(HwRTCCON, HwRTCCON_RTCEN);
	//BITSET(HwINTCON, HwINTCON_INTWREN);
	//HwRTCIM = 0;
	//BITSET(HwRTCIM, HwRTCIM_ALMINT_EDGE_EN);
	//PMWKUP Disable End


}
#else
void tca_alarm_setpmwkup(rtctime *lpTime)
{

    tca_alarm_settime(lpTime);
    
#if defined(_TCC79x_)
    HwCLR = HwINT_RTC;
    HwINTMSK    |= HwINT_RTC;
    HwSEL   |= HwINT_RTC;
#elif  defined(_TCC83x_)
    HwCLR = HwIEN_RTC;
    HwIEN   &= ~HwIEN_RTC;
    HwSEL   &= ~HwIEN_RTC;
    
    HwIEN   |= HwIEN_RTC_P;
    HwSEL   |= HwIEN_RTC_P;
#endif

	BITSET(HwRTCCON, HwRTCCON_RTCEN);
	BITSET(HwINTCON, HwINTCON_INTWREN);
	BITCLR(HwRTCIM, 0xf);
	BITSET(HwRTCIM, HwRTCIM_ALMINT_EDGE_EN|HwRTCIM_PMWKUP_ACTIVE_HIGH|HwRTCIM_PWDN_POWERDOWN_MODE);	

	BITSET(HwRTCCON, HwRTCCON_RTCEN | HwRTCCON_WUOUT_EN);
	//PMWKUP Disable Start
	//HwRTCCON = 0;
	//BITSET(HwRTCCON, HwRTCCON_RTCEN);
	//BITSET(HwINTCON, HwINTCON_INTWREN);
	//HwRTCIM = 0;
	//BITSET(HwRTCIM, HwRTCIM_ALMINT_EDGE_EN);
	//PMWKUP Disable End


}
#endif

