
/****************************************************************************
 *   FileName    : tca_alarm.C
 *   Description : 
 ****************************************************************************
 *
 *   TCC Version 1.0
 *   Copyright (c) Telechips, Inc.
 *   ALL RIGHTS RESERVED
 *
 ****************************************************************************/

/*****************************************************************************
*
* Header Files Include
*
******************************************************************************/
#include <linux/kernel.h>
#include <asm/io.h>
#include "tca_alarm.h"

/*****************************************************************************
*
* Defines
*
******************************************************************************/

/*****************************************************************************
*
* structures
*
******************************************************************************/


/*****************************************************************************
*
* Variables
*
******************************************************************************/



/*****************************************************************************
*
* Functions
*
******************************************************************************/

/*****************************************************************************
* Function Name : tca_rtcgettime()
******************************************************************************/
volatile void tca_alarm_gettime(unsigned int devbaseaddresss, rtctime *pTime)
{
	unsigned uCON;
	
	PRTC	pRTC = (PRTC)devbaseaddresss;
#if 0
	BITSET(pRTC->RTCCON, Hw1);	// RTC Register write enabled
	BITSET(pRTC->INTCON, Hw0);	// Interrupt Block Write Enable
#endif
	pTime->wSecond			= pRTC->ALMSEC;
	pTime->wMinute			= pRTC->ALMMIN;
	pTime->wHour			= pRTC->ALMHOUR;
	pTime->wDayOfWeek 		= pRTC->ALMDAY;
	pTime->wDay				= pRTC->ALMDATE;
	pTime->wMonth			= pRTC->ALMMON;
	pTime->wYear			= pRTC->ALMYEAR;
	uCON = pRTC->RTCALM;
	
	BITCLR(pRTC->INTCON, Hw0);	// Interrupt Block Write Disable
	BITCLR(pRTC->RTCCON, Hw1);	// RTC Register write Disable

	/* Second */
	if (ISZERO(uCON, Hw0))
		pTime->wSecond	= (unsigned char) 0;
	else
		pTime->wSecond	= FROM_BCD( pTime->wSecond );

	/* Minute */
	if (ISZERO(uCON, Hw1))
		pTime->wMinute	= (unsigned char) 0;
	else
		pTime->wMinute	= FROM_BCD( pTime->wMinute );

	/* Hour */
	if (ISZERO(uCON, Hw2))
		pTime->wHour	= (unsigned char) 0;
	else
		pTime->wHour	= FROM_BCD( pTime->wHour );

	/* date */
	if (ISZERO(uCON, Hw3))
		pTime->wDay	= (unsigned char) 0;
	else
		pTime->wDay	= FROM_BCD( pTime->wDay );

	/* month */
	if (ISZERO(uCON, Hw5))
		pTime->wMonth	= (unsigned char) 0;
	else
		pTime->wMonth	= FROM_BCD( pTime->wMonth );

	/* year */
	if (ISZERO(uCON, Hw6))
		pTime->wYear	= (unsigned short) 0;
	else
//		pTime->wYear	= FROM_BCD( pTime->wYear );
		pTime->wYear	= (FROM_BCD( pTime->wYear>>8 )*100) + FROM_BCD( pTime->wYear&0x00FF );	// YearFix : hjbae

	/* weekdays */
	if (ISZERO(uCON, Hw4))
		pTime->wDayOfWeek		= 1;
	else
		pTime->wDayOfWeek = FROM_BCD( pTime->wDayOfWeek );

	pRTC->RTCALM  = uCON;

}

/*****************************************************************************
* Function Name : tca_rtcsettime()
******************************************************************************/
volatile void tca_alarm_settime(unsigned int devbaseaddresss, rtctime *pTime)
{
	unsigned	uCON;
	PRTC	pRTC = (PRTC)devbaseaddresss;
#if defined(CONFIG_ARCH_TCC93XX)
	PPIC	pPIC = (PPIC)tcc_p2v(HwPIC_BASE);
#endif

	BITSET(pRTC->RTCCON, Hw1);	// RTC Register write enabled
	BITSET(pRTC->INTCON, Hw0);	// Interrupt Block Write Enable

	uCON	= 0xEF; // Not wDayOfWeek
	/* Second */
	if ( pTime->wSecond > 59 )
		BITCLR(uCON, Hw0);//HwRTCALM_SECEN_EN
	else
		pRTC->ALMSEC	= TO_BCD( pTime->wSecond );

	/* Minute */
	if ( pTime->wMinute > 59 )
		BITCLR(uCON, Hw1);//HwRTCALM_MINEN_EN
	else
		pRTC->ALMMIN	= TO_BCD( pTime->wMinute );

	/* Hour */
	if ( pTime->wHour > 23 )
		BITCLR(uCON, Hw2);//HwRTCALM_HOUREN_EN
	else
		pRTC->ALMHOUR	= TO_BCD( pTime->wHour );

	/* Date */
	if ( pTime->wDay > 31 || pTime->wDay < 1 )
		BITCLR(uCON, Hw3);//HwRTCALM_DATEEN_EN
	else
		pRTC->ALMDATE	= TO_BCD( pTime->wDay );

	/* month */
	if (pTime->wMonth > 12 || pTime->wMonth < 1)
		BITCLR(uCON, Hw5);//HwRTCALM_MONEN_EN
	else
		pRTC->ALMMON	= TO_BCD( pTime->wMonth );

	/* year */
	if (pTime->wYear > 2099 || pTime->wYear < 1900)
		BITCLR(uCON, Hw6);//HwRTCALM_YEAREN_EN
	else
//		pRTC->ALMYEAR	= TO_BCD( pTime->wYear );
		pRTC->ALMYEAR	= (TO_BCD( pTime->wYear/100 ) << 8) | TO_BCD( pTime->wYear%100 );	// YearFix : hjbae

	/* Day */
	if ( pTime->wDayOfWeek > 6)
		BITCLR(uCON, Hw4);//HwRTCALM_DAYEN_EN
	else
		pRTC->ALMDAY	= pTime->wDayOfWeek+1;


	// Enable ALARM
	pRTC->RTCALM	= uCON;

	//BITCSET(pRTC->RTCIM, Hw1, Hw0);	// INTMODE : Supports on the edge alarm interrupt.
	BITSET(pRTC->RTCIM, Hw1|Hw0);	//INTMODE : Supports on the level alarm interrupt.
	BITSET(pRTC->RTCIM, Hw3);		// Operation mode - Power Down Mode

	BITSET(pRTC->RTCIM, Hw2);		// Wakeup mode selection bit - PMWKUP active high

	pRTC->RTCPEND	= 0;			// alarm clear
	BITSET(pRTC->RTCSTR, Hw0);		// alarm clear

	BITSET(pRTC->RTCCON, Hw6|Hw7);  // Alarm Interrupt Output Enable (Hw6) Wake-up Output Enable (Hw7)

	if( !(pRTC->INTCON & Hw15) )
	{
		BITSET(pRTC->RTCCON,Hw0);		// RTC Start bit - Halt
		BITSET(pRTC->INTCON,Hw15);		// protect bit - enable
		BITCLR(pRTC->RTCCON,Hw0);		// RTC Start bit - Run
	}

	printk("RTC[%02X] INT[%04X]", pRTC->RTCCON, pRTC->INTCON);
	printk(" ALM[%02X] IM[%01X] PEND[%01X] STR[%02X]", pRTC->RTCALM, pRTC->RTCIM, pRTC->RTCPEND, pRTC->RTCSTR);
	//printk("\n%04x.%02x.%02x-%02x:%02x:%02x", pRTC->ALMYEAR, pRTC->ALMMON, pRTC->ALMDATE, pRTC->ALMHOUR, pRTC->ALMMIN, pRTC->ALMSEC);

	BITCLR(pRTC->INTCON,Hw0);		// Interrupt Block Write Enable bit - disable
	BITCLR(pRTC->RTCCON, Hw1);		// RTC write enable bit - disable


	//BITCLR(HwPMU->WKUPEN, HwPMU_WKUP_RTCWKUP);	// RTC WKUP Disable
	//BITCLR(HwPMU->WKUPPOL, HwPMU_WKUP_RTCWKUP);	// ACTIVE HIGH
#if defined(CONFIG_ARCH_TCC93XX)
	#if 1
	pPIC->CLR0 |= HwINT0_RTC;
	pPIC->SEL0	|= HwINT0_RTC;
	pPIC->IEN0 |= HwINT0_RTC;
	//pPIC->MODE0 |= HwINT0_RTC; // Level-triggered
	//pPIC->POL0
	//pPIC->WKEN0 |= HwINT0_RTC;
	#endif
#else
	#if 1
	HwPIC->CLR1 |= HwINT1_RTC;
	HwPIC->SEL1	|= HwINT1_RTC;
	HwPIC->IEN1 |= HwINT1_RTC;
	//HwPIC->MODE1 |= HwINT1_RTC; // Level-triggered
	//HwPIC->POL1 
	//HwPIC->WKEN1 |= HwINT1_RTC;
	#endif
#endif
	
}

/************************************************************************************************
* FUNCTION		:  tca_alarm_setint
*
* DESCRIPTION	: 
*
************************************************************************************************/
volatile void tca_alarm_setint(unsigned int devbaseaddresss)
{
	volatile rtctime lpTime;

	//Set Alarm
    tca_rtc_gettime(devbaseaddresss, (rtctime *)&lpTime);
#if defined(WINCE_ONLY)
	if(lpTime.wSecond < 57){
		lpTime.wSecond += 3;
	}
	else{
		if(lpTime.wMinute < 59){
			lpTime.wMinute += 1;
			lpTime.wSecond = 0;
		}
		else{
			if(lpTime.wHour < 23){
				lpTime.wHour += 1;
				lpTime.wMinute = 0;
				lpTime.wSecond = 0;
			}
			else{
				lpTime.wHour = 0;
				lpTime.wMinute = 0;
				lpTime.wSecond = 0;
			}
		}
	}
	
#else
	if(lpTime.wSecond < 55)
		lpTime.wSecond += 5;
#endif

	tca_alarm_settime(devbaseaddresss, (rtctime *)&lpTime);
}
/************************************************************************************************
* FUNCTION		:  tca_alarm_setpmwkup
*
* DESCRIPTION	: 
*
************************************************************************************************/
volatile void tca_alarm_setpmwkup(unsigned int rtcbaseaddresss,unsigned int vicbaseaddresss)
{
	rtctime lpTime;
	PRTC	pRTC = (PRTC)rtcbaseaddresss;
	PPIC    pPIC = (PPIC)vicbaseaddresss;

	//Set Alarm
    tca_rtc_gettime(rtcbaseaddresss, (rtctime *)&lpTime);
	

#if defined(WINCE_ONLY)
	
	if(lpTime.wSecond < 57){
		lpTime.wSecond += 3;
	}
	else{
		if(lpTime.wMinute < 59){
			lpTime.wMinute += 1;
			lpTime.wSecond = 0;
		}
		else{
			if(lpTime.wHour < 23){
				lpTime.wHour += 1;
				lpTime.wMinute = 0;
				lpTime.wSecond = 0;
			}
	else{
				lpTime.wHour = 0;
				lpTime.wMinute = 0;
		lpTime.wSecond = 0;
	}
		}
	}
	
#else
	if(lpTime.wSecond < 55)
		lpTime.wSecond += 5;
#endif

	tca_alarm_settime(rtcbaseaddresss, (rtctime *)&lpTime);

#if defined(CONFIG_ARCH_TCC93XX)
	pPIC->CLR0		= HwINT0_RTC;
	pPIC->MSTS0		|= HwINT0_RTC;
	pPIC->SEL0		|= HwINT0_RTC;
#else
	pPIC->CLR1 		= HwINT1_RTC;
	pPIC->MSTS1 		|= HwINT1_RTC;
	pPIC->SEL1		|= HwINT1_RTC;
#endif

	BITSET(pRTC->RTCCON, Hw1);// Enable RTCEN
	BITSET(pRTC->INTCON, Hw0);// Enable INTWREN
	BITCLR(pRTC->RTCIM, 0xf);
	BITSET(pRTC->RTCIM, Hw0|Hw2|Hw3);	 // Enable ALMINT_EDGE_EN, Enable PMWKUP_ACTIVE_HIGH, Enable PWDN_POWERDOWN_MODE
	BITSET(pRTC->RTCCON, Hw1 | Hw7);
	
	//PMWKUP Disable Start
		//pRTC->RTCCON = 0;
		//BITSET(pRTC->RTCCON, Hw1);
		//BITSET(pRTC->INTCON, Hw0);
		//pRTC->RTCIM = 0;
		//BITSET(pRTC->RTCIM, Hw0);
	//PMWKUP Disable End
}

