
/****************************************************************************
 *   FileName    :  tca_spi_hwset.c
 *   Description : 
 ****************************************************************************
 *
 *   TCC Version 1.0
 *   Copyright (c) Telechips, Inc.
 *   ALL RIGHTS RESERVED
 *
 ****************************************************************************/

#if defined(__KERNEL__)
#   include <linux/module.h>
#   if defined(_TCC79x_)
#       include <mach/tcc79x.h>
#   elif defined(_TCC83x_)
#       include <mach/tcc83x_virt.h>
#   endif
#else
#   if defined(_TCC79x_)
#       include "TCC79x_Virtual.h"
#   elif defined(_TCC83x_)
#       include "TCC83x_Virtual.h"
#   endif
#   include <stdlib.h>
#endif

#include "tca_spi_hwset.h"

#if defined(_TCC79x_)
static int tca_spi_isenabledma(struct tca_spi_handle *h)
{
    return (h->regs->DMACTRL & Hw0) ? 1 : 0;
}

static int tca_spi_dmastop(struct tca_spi_handle *h)
{
    BITCLR(h->regs->DMACTRL, Hw31 | Hw30); /* disable DMA Transmit & Receive */
    BITSET(h->regs->DMAICR, Hw29| Hw28);
    BITCLR(h->regs->DMACTRL, Hw0); /* DMA disable */
	return 0;
}

static int tca_spi_dmastart(struct tca_spi_handle *h)
{
    BITSET(h->regs->DMACTRL, Hw31 | Hw30); /* enable DMA Transmit & Receive */
    BITCLR(h->regs->DMACTRL, Hw17 | Hw16 | Hw15 | Hw14); /* set Multiple address mode */

    BITCLR(h->regs->DMAICR, Hw16); /* disable DMA Packet Interrupt */
    BITSET(h->regs->DMAICR, Hw17); /* enable DMA Done Interrupt */
    BITCLR(h->regs->DMAICR, Hw20); /* set rx interrupt */

    BITSET(h->regs->DMACTRL, Hw0); /* DMA enable */
	return 0;
}

static int tca_spi_dmastart_slave(struct tca_spi_handle *h)
{
    //BITSET(h->regs->DMACTRL, Hw31 | Hw30); /* enable DMA Transmit & Receive */
    BITSET(h->regs->DMACTRL, Hw30); /* enable DMA Transmit */
    BITCLR(h->regs->DMACTRL, Hw17 | Hw16 | Hw15 | Hw14); /* set Multiple address mode */

    BITSET(h->regs->DMAICR, Hw16); /* enable DMA Packet Interrupt */
    //BITSET(h->regs->DMAICR, Hw17); /* enable DMA Done Interrupt */
    BITCLR(h->regs->DMAICR, Hw20); /* set rx interrupt */

    //h->regs->DMAICR = h->regs->DMAICR | (256 & 0x1FFF);
    BITSET(h->regs->DMAICR, Hw16); /* enable DMA Packet Interrupt */

    BITSET(h->regs->DMACTRL, Hw4); /* MPEG2-TS mode */
    //BITCLR(h->regs->DMACTRL, Hw5 | Hw4); /* MPEG2-TS mode */

    BITSET(h->regs->DMACTRL, Hw0); /* DMA enable */
	return 0;
}

static void tca_spi_clearfifopacket(struct tca_spi_handle *h)
{
    /* clear tx/rx FIFO & Packet counter  */
    BITSET(h->regs->MODE, Hw15 | Hw14);
    BITSET(h->regs->DMACTRL, Hw2);
    BITCLR(h->regs->DMACTRL, Hw2);
    BITCLR(h->regs->MODE, Hw15 | Hw14);
}

static void tca_spi_setpacketcnt(struct tca_spi_handle *h, int size)
{
    /* set packet count & size */
    h->regs->PACKET = (size & 0x1FFF);
}

static void tca_spi_setpacketcnt_slave(struct tca_spi_handle *h, int size)
{
    unsigned int packet_cnt = (h->dma_total_packet_cnt & 0x1FFF) - 1;
    unsigned int packet_size = (MPEG_PACKET_SIZE & 0x1FFF);
    unsigned int intr_packet_cnt = (h->dma_intr_packet_cnt & 0x1FFF) - 1;

    if (packet_size != size) {
        size = packet_size;
    }

    h->regs->PACKET = (packet_cnt << 16) | packet_size;
    BITCSET(h->regs->DMAICR, 0x1FFF, intr_packet_cnt);
}

static void tca_spi_setbitwidth(struct tca_spi_handle *h, int width)
{
    int width_value = (width - 1) & 0x1F;

    /* set bit width */
    BITCLR(h->regs->MODE, Hw12 | Hw11 | Hw10 | Hw9 | Hw8);
    h->regs->MODE |= (width_value << 8);
    if (width_value & Hw4) {
        BITCLR(h->regs->DMACTRL, Hw28);
    } else {
        BITSET(h->regs->DMACTRL, Hw28);
    }
}

static void tca_spi_setdmaaddr(struct tca_spi_handle *h)
{
    /* set dma txbase/rxbase & request DMA tx/rx */
    h->regs->TXBASE = h->tx_dma.dma_addr;
    h->regs->RXBASE = h->rx_dma.dma_addr;

    h->regs->INTEN = h->tx_dma.dma_addr ? (h->regs->INTEN | Hw31) : (h->regs->INTEN & ~Hw31);
    h->regs->INTEN = h->rx_dma.dma_addr ? (h->regs->INTEN | Hw30) : (h->regs->INTEN & ~Hw30);
}

static void tca_spi_setdmaaddr_slave(struct tca_spi_handle *h)
{
    /* set dma txbase/rxbase & request DMA tx/rx */
    h->regs->RXBASE = h->rx_dma.dma_addr;
    h->regs->INTEN = h->rx_dma.dma_addr ? (h->regs->INTEN | Hw30) : (h->regs->INTEN & ~Hw30);
}

static void tca_spi_hwinit(struct tca_spi_handle *h)
{
    /* init => set SPI mode, set Master mode ... */
    memset((void *)(h->regs), 0, sizeof(struct tca_spi_regs));

    h->set_bit_width(h, 32);

    /* [SCK] Tx: risiing edge, Rx: falling edge & enable Operation */
    BITSET((h->regs)->MODE, Hw17 | Hw18 | Hw3);

    h->set_dma_addr(h);
    h->clear_fifo_packet(h);
}

static void tca_spi_hwinit_slave(struct tca_spi_handle *h)
{
    /* init => set SPI mode, set Master mode ... */
    memset((void *)(h->regs), 0, sizeof(struct tca_spi_regs));

    h->set_bit_width(h, 32);

    /* [SCK] Tx: risiing edge, Rx: falling edge & enable Operation */
    BITSET((h->regs)->MODE, Hw17 | Hw18 | Hw3);
    BITSET((h->regs)->MODE, Hw2 | Hw5);

    h->set_dma_addr(h);
    h->clear_fifo_packet(h);
}

static void tca_spi_set_mpegtspidmode(struct tca_spi_handle *h, int is_set)
{
    if (is_set) {
        BITSET(h->regs->DMACTRL, Hw19 | Hw18);
    } else {
        BITCLR(h->regs->DMACTRL, Hw19 | Hw18);
    }
}

/******************************
 * return value
 *
 * ret == 0: success
 * ret > 0 or ret < 0: fail
 ******************************/
int tca_spi_init(tca_spi_handle_t *h,
                 volatile struct tca_spi_regs *regs,
                 int irq,
                 dma_alloc_f tea_dma_alloc,
                 dma_free_f tea_dma_free,
                 int id,
                 int is_slave)
{
    int ret = -1;

    if (h) { memset(h, 0, sizeof(tca_spi_handle_t)); }
    if (regs) {
        h->regs = regs;
        h->irq = irq;
        h->id = id;

        h->dma_stop = tca_spi_dmastop;
        h->dma_start = is_slave ? tca_spi_dmastart_slave : tca_spi_dmastart;
        h->clear_fifo_packet = tca_spi_clearfifopacket;
        h->set_packet_cnt = is_slave ? tca_spi_setpacketcnt_slave : tca_spi_setpacketcnt;
        h->set_bit_width = tca_spi_setbitwidth;
        h->set_dma_addr = is_slave ? tca_spi_setdmaaddr_slave : tca_spi_setdmaaddr;
        h->hw_init = is_slave ? tca_spi_hwinit_slave : tca_spi_hwinit;
        h->is_enable_dma = tca_spi_isenabledma;
        h->set_mpegts_pidmode = tca_spi_set_mpegtspidmode;

        h->tea_dma_alloc = tea_dma_alloc;
        h->tea_dma_free = tea_dma_free;

        /* interrupt init */
        BITSET(HwSEL, Hw5);
        BITSET(HwMODE, Hw5);
        BITSET(HwIRQMSK, Hw5);
        
        if (h->id == 0) {
            /* port config  (SPI0) */
            BITCLR(HwPORTCFG3, Hw0 | Hw1 | Hw2 | Hw3);
            BITCLR(HwPORTCFG4, 0xFFF00000);
            
            /* GPSB port config (channel 0 port mapping == 6) */
            HwGPSB_PCFG = (HwGPSB_PCFG & ~0x000000FF) | (0x6);
        } else {
            /* port config  (SPI1) */
            BITCLR(HwPORTCFG7, Hw28 - Hw12);
            
            /* GPSB port config (channel 0 port mapping == 5) */
            HwGPSB_PCFG = (HwGPSB_PCFG & ~0x0000FF00) | (0x500);
        }

        if (is_slave) {
            h->rx_dma.buf_size = TSIF_DTCM_DMA_MEM_SIZE;
            h->rx_dma.v_addr = (void *)TSIF_RX_BUFFER_BASE_VA;
            h->rx_dma.dma_addr = TSIF_RX_BUFFER_BASE_PA;
            ret = 0;
        } else {
            if (h->tea_dma_alloc) {
        if (h->tea_dma_alloc(&(h->tx_dma), SPI_DMA_SIZE) == 0) {
            if (h->tea_dma_alloc(&(h->rx_dma), SPI_DMA_SIZE) == 0) {
                ret = 0;
            }
        }
            }
        }

        if (ret) { tca_spi_clean(h); }
    }
    return ret;
}

void tca_spi_clean(tca_spi_handle_t *h)
{
    if (h) {
        if (h->tea_dma_free) {
            h->tea_dma_free(&(h->tx_dma));
            h->tea_dma_free(&(h->rx_dma));
        }
        memset(h, 0, sizeof(tca_spi_handle_t));
    }
}

#if defined(__KERNEL__)
EXPORT_SYMBOL(tca_spi_init);
EXPORT_SYMBOL(tca_spi_clean);
#endif

#endif

