/*
 * linux/drivers/char/tsif/tcc_tsif_module.c
 *
 * Author:  <linux@telechips.com>
 * Created: 1st April, 2009
 * Description: Driver for Telechips TS Parallel/Serial Controllers
 *
 * Copyright (c) Telechips, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/spinlock.h>
#include <linux/workqueue.h>
#include <linux/interrupt.h>
#include <linux/delay.h>
#include <linux/errno.h>
#include <linux/err.h>
#include <linux/clk.h>
#include <linux/dma-mapping.h>
#include <linux/poll.h>
#include <linux/spi/spi.h>

#include <linux/kernel.h>
#include <linux/time.h>
#include <linux/timer.h>
#include <linux/sched.h>

#include <asm/uaccess.h>
#include <asm/io.h>
#include <asm/dma.h>
#include <asm/atomic.h>

#include <bsp.h>
#include <mach/gpio.h>
#include <linux/spi/tcc_tsif_module.h>
#include "tcc/tca_tsif_module_hwset.h"
#include "tsdemux/TSDEMUX_ref_timer.h"
#include "tsdemux/TSDEMUX_sys.h"

static unsigned int guiPCRPID;
static struct clk *gpsb_clk;

struct tcc_tsif_module_pri_handle {
    wait_queue_head_t wait_q;
    struct mutex mutex;
    int open_cnt;
    u32 gpio_port;
};


static struct tcc_tsif_module_handle tsif_module_handle;
static struct tcc_tsif_module_pri_handle tsif_module_pri;

static int __init tsif_module_drv_probe(struct platform_device *pdev)
{
    struct resource *port = NULL;

	port = platform_get_resource(pdev, IORESOURCE_IO, 0);
    if (!port) {
        return -ENXIO;
    }
    tsif_module_pri.gpio_port = port->start;

    platform_set_drvdata(pdev, &tsif_module_handle);

	/* TODO: device_remove_file(&pdev->dev, &dev_attr_tcc_port); */
    //device_create_file(&pdev->dev, &dev_attr_tcc_port);

    printk("%s%d: init\n", pdev->name, TSIF_CH);
    return 0;
}

#ifdef CONFIG_PM
/*
 * NOTE: before suspend, you must close spi.
 */
static int tsif_module_drv_suspend(struct platform_device *dev, pm_message_t state)
{
	return 0;
}
static int tsif_module_drv_resume(struct platform_device *dev)
{
	return 0;
}
#else
#define tsif_drv_suspend NULL
#define tsif_drv_resume  NULL
#endif

static struct platform_driver tsif_module_platform_driver = {
	.driver = {
		.name	= "tcc-tsif-module",
		.owner	= THIS_MODULE,
	},
	.suspend = tsif_module_drv_suspend,
	.resume  = tsif_module_drv_resume,
};

static int tsif_module_dxb_power(unsigned long flag)
{
	#if defined(CONFIG_MACH_TCC9300ST)
        if (flag == 1) {
                /* GPIO_EXPAND DXB_ON Power-on */
                //gpio_direction_output(TCC_GPEXT1(8), 1);
                //gpio_set_value(TCC_GPEXT1(8), 1);
        } else if (flag == 2) {
                /* GPIO_EXPAND DEEPPWDN Power-on */
                //gpio_direction_output(TCC_GPEXT2(8), 1);
                //gpio_set_value(TCC_GPEXT2(8), 1);
        } else if (flag == 0) {
                /* GPIO_EXPAND DXB_ON Power-off */
                //gpio_direction_output(TCC_GPEXT1(8), 0);
                //gpio_set_value(TCC_GPEXT1(8), 0);

                /* GPIO_EXPAND DEEPPWDN Power-off */
                //gpio_direction_output(TCC_GPEXT2(8), 0);
                //gpio_set_value(TCC_GPEXT2(8), 0);
        } else {
                return -EINVAL;
        }
	#elif defined(CONFIG_MACH_TCC9300CM)
	if (flag == 1) {
		/* GPIO_EXPAND DXB_ON Power-on */
        	gpio_direction_output(TCC_GPEXT2(12), 1);
        	gpio_set_value(TCC_GPEXT2(12), 1);
        	gpio_direction_output(TCC_GPEXT1(15), 1);
        	gpio_set_value(TCC_GPEXT1(15), 1);
	} else if (flag == 0) {
		/* GPIO_EXPAND DXB_ON Power-off */
        	gpio_direction_output(TCC_GPEXT2(12), 0);
        	gpio_set_value(TCC_GPEXT2(12), 0);
        	gpio_direction_output(TCC_GPEXT1(15), 0);
        	gpio_set_value(TCC_GPEXT1(15), 0);
	} else {
		return -EINVAL;
	}
	#else
	if (flag == 1) {
		/* GPIO_EXPAND DXB_ON Power-on */
        	gpio_direction_output(TCC_GPEXT1(8), 1);
        	gpio_set_value(TCC_GPEXT1(8), 1);
	} else if (flag == 2) {
        	/* GPIO_EXPAND DEEPPWDN Power-on */
        	gpio_direction_output(TCC_GPEXT2(8), 1);
        	gpio_set_value(TCC_GPEXT2(8), 1);
	} else if (flag == 3) {
			/* GPIO_EXPAND DXB_PWRDWN_N_IN Power-on */
        	gpio_direction_output(TCC_GPEXT2(9), 1);
        	gpio_set_value(TCC_GPEXT2(9), 1);
	} else if (flag == 0) {
		/* GPIO_EXPAND DXB_ON Power-off */
        	gpio_direction_output(TCC_GPEXT1(8), 0);
        	gpio_set_value(TCC_GPEXT1(8), 0);

        	/* GPIO_EXPAND DEEPPWDN Power-off */
        	gpio_direction_output(TCC_GPEXT2(8), 0);
        	gpio_set_value(TCC_GPEXT2(8), 0);

        	/* GPIO_EXPAND PWRDWN_N_IN Power-off */
        	gpio_direction_output(TCC_GPEXT2(9), 0);
        	gpio_set_value(TCC_GPEXT2(9), 0);
	} else {
		return -EINVAL;
	}
	#endif

	return 0;
}


static void tea_free_dma_linux(struct tea_dma_buf *tdma)
{
    if(tdma) {
        if(tdma->v_addr != 0) {
            dma_free_writecombine(0, tdma->buf_size, tdma->v_addr, tdma->dma_addr);

			printk("tcc-tsif-module : dma buffer free @0x%X(Phy=0x%X), size:%d\n", (unsigned int)tdma->v_addr, (unsigned int)tdma->dma_addr, tdma->buf_size);
        }
        memset(tdma, 0, sizeof(struct tea_dma_buf));
    }
}

static int tea_alloc_dma_linux(struct tea_dma_buf *tdma, unsigned int size)
{
    int ret = -1;

    if(tdma) {
        tea_free_dma_linux(tdma);

        tdma->buf_size = size;
        tdma->v_addr = dma_alloc_writecombine(0, tdma->buf_size, &tdma->dma_addr, GFP_KERNEL);

        ret = tdma->v_addr ? 0 : 1;

		printk("tcc-tsif-module : dma buffer alloc @0x%X(Phy=0x%X), size:%d\n", (unsigned int)tdma->v_addr, (unsigned int)tdma->dma_addr, tdma->buf_size);
    }

    return ret;
} 

static irqreturn_t tsif_module_dma_handler(int irq, void *dev_id)
{
	struct tcc_tsif_module_handle *tspi 	= (struct tcc_tsif_module_handle *)dev_id;
    struct tcc_tsif_module_pri_handle *tpri = (struct tcc_tsif_module_pri_handle *)tspi->private_data;
    
	tsif_module_handle.tsif_isr(tspi); 
	
	if (tpri->open_cnt > 0) {
        //Check PCR & Make STC
        if(guiPCRPID < 0x1FFF) {
            int start_pos, search_pos, search_size;
            start_pos = tsif_module_handle.q_pos - tsif_module_handle.dma_intr_packet_cnt;
            if(start_pos > 0) {
                search_pos = start_pos;
                search_size = tsif_module_handle.dma_intr_packet_cnt;
            }
            else {
               search_pos = 0;
               search_size = tsif_module_handle.q_pos;
            }

            TSDEMUX_MakeSTC((unsigned char *)tsif_module_handle.rx_dma.v_addr + search_pos*TSIF_PACKET_SIZE, search_size*TSIF_PACKET_SIZE, guiPCRPID);      
        }
		wake_up(&(tpri->wait_q));
	}

    return IRQ_HANDLED;
}

#ifdef GDMA
static int tsif_get_readable_cnt(struct tcc_tsif_module_handle *h)
{
	int dma_phy_rdpos = 0;
	int dma_phy_curpos = 0;
	int dma_recv_size = 0;
	int ret = 0;
	
	if(h){
		dma_phy_curpos = h->dma_phy_curpos;
		dma_phy_rdpos = h->dma_phy_rdpos;

		if( dma_phy_curpos ) {
    	    if(dma_phy_curpos > dma_phy_rdpos) {
			    dma_recv_size = dma_phy_curpos - dma_phy_rdpos;
		    } else { 
			    dma_recv_size = (h->dma_phy_maxaddr - dma_phy_rdpos) + (dma_phy_curpos - h->rx_dma.dma_addr);
		    }
		}
	
		if(dma_recv_size > 0){
			ret = dma_recv_size / TSIF_PACKET_SIZE;
		}
	
		return ret;
	}

    return 0;
}

static ssize_t tcc_tsif_module_read(struct file *filp, char *buf, size_t len, loff_t *ppos)
{
	int readable_cnt;

	readable_cnt = tsif_get_readable_cnt(&tsif_module_handle); 
	if ( len < (readable_cnt * TSIF_PACKET_SIZE) ) {
		int copy_byte = 0;
		int copy_cnt = 0;

		copy_cnt = len / TSIF_PACKET_SIZE;
		copy_byte = copy_cnt * TSIF_PACKET_SIZE;
		
		if (copy_cnt <= readable_cnt) {		
			int dma_phy_rdpos;
			int dma_phy_curpos;
			int offset;

			dma_phy_rdpos = tsif_module_handle.dma_phy_rdpos;
    	    dma_phy_curpos = tsif_module_handle.dma_phy_curpos;

			if((dma_phy_rdpos + copy_byte) < tsif_module_handle.dma_phy_maxaddr) {
				offset = dma_phy_rdpos - tsif_module_handle.rx_dma.dma_addr;
				
				if(copy_to_user(buf, (tsif_module_handle.rx_dma.v_addr + offset), copy_byte)) {
					return -EFAULT;
				}

        	    tsif_module_handle.dma_phy_rdpos = dma_phy_rdpos + copy_byte;
			} else {
				int first_size;
				int second_size;
			
				offset = dma_phy_rdpos - tsif_module_handle.rx_dma.dma_addr;
			
				first_size 	= tsif_module_handle.dma_phy_maxaddr - dma_phy_rdpos;
				if(copy_to_user(buf, (tsif_module_handle.rx_dma.v_addr + offset), first_size)) {
					return -EFAULT;
				}
			
				second_size	= copy_byte - first_size;
				if(copy_to_user((buf + first_size), tsif_module_handle.rx_dma.v_addr, second_size)) {
					return -EFAULT;
				}
	
    	        tsif_module_handle.dma_phy_rdpos = tsif_module_handle.rx_dma.dma_addr + second_size;
			}
		
			if(tsif_module_handle.dma_intr_cnt > 1){
				printk("%d\n", tsif_module_handle.dma_intr_cnt);
        	}
		
			tsif_module_handle.dma_intr_cnt = 0;
		
			return copy_byte;
		}
	}

	return 0;
}

static unsigned int tcc_tsif_module_poll(struct file *filp, struct poll_table_struct *wait)
{
    if(tsif_get_readable_cnt(&tsif_module_handle) > tsif_module_handle.dma_intr_packet_cnt) {
    //if(tsif_get_readable_cnt(&tsif_module_handle) > 0) {
		return  (POLLIN | POLLRDNORM);
    } 

	poll_wait(filp, &(tsif_module_pri.wait_q), wait);

    if(tsif_get_readable_cnt(&tsif_module_handle) > tsif_module_handle.dma_intr_packet_cnt) {
    //if(tsif_get_readable_cnt(&tsif_module_handle) > 0) {
		return  (POLLIN | POLLRDNORM);
    }

    return 0;
}
#else
static int tsif_get_readable_cnt(struct tcc_tsif_module_handle *h)
{
	if( h ){
		int dma_pos = h->cur_q_pos;
		int q_pos = h->q_pos;
		int readable_cnt = 0;

		if( dma_pos > q_pos ){
			readable_cnt = dma_pos - q_pos;
		} else if(dma_pos < q_pos) {
			readable_cnt = h->dma_total_packet_cnt - q_pos;
			readable_cnt += dma_pos;
		}

		return readable_cnt;
	}

	return 0;
}

static ssize_t tcc_tsif_module_read(struct file *filp, char *buf, size_t len, loff_t *ppos)
{
    int readable_cnt = 0, copy_cnt = 0;
    int copy_byte = 0;

    readable_cnt = tsif_get_readable_cnt(&tsif_module_handle);
    if (readable_cnt > 0) {
        copy_byte = readable_cnt * TSIF_PACKET_SIZE;
        if (copy_byte > len) {
            copy_byte = len;
        }

        copy_byte -= copy_byte % TSIF_PACKET_SIZE;
        copy_cnt = copy_byte / TSIF_PACKET_SIZE;
        copy_cnt -= copy_cnt % tsif_module_handle.dma_intr_packet_cnt;
        copy_byte = copy_cnt * TSIF_PACKET_SIZE;

        if (copy_cnt >= tsif_module_handle.dma_intr_packet_cnt) {
            int offset = tsif_module_handle.q_pos * TSIF_PACKET_SIZE;
            if (copy_cnt > tsif_module_handle.dma_total_packet_cnt - tsif_module_handle.q_pos) {
                int first_copy_byte = (tsif_module_handle.dma_total_packet_cnt - tsif_module_handle.q_pos) * TSIF_PACKET_SIZE;
                int first_copy_cnt = first_copy_byte / TSIF_PACKET_SIZE;
                int second_copy_byte = (copy_cnt - first_copy_cnt) * TSIF_PACKET_SIZE;

                if (copy_to_user(buf, tsif_module_handle.rx_dma.v_addr + offset, first_copy_byte)) {
                    return -EFAULT;
                }
                if (copy_to_user(buf + first_copy_byte, tsif_module_handle.rx_dma.v_addr, second_copy_byte)) {
                    return -EFAULT;
                }

                tsif_module_handle.q_pos = copy_cnt - first_copy_cnt;
            } else {
                if (copy_to_user(buf, tsif_module_handle.rx_dma.v_addr + offset, copy_byte)) {
                    return -EFAULT;
                }

                tsif_module_handle.q_pos += copy_cnt;
                if (tsif_module_handle.q_pos >= tsif_module_handle.dma_total_packet_cnt) {
                    tsif_module_handle.q_pos = 0;
                }
            }
            return copy_byte;
        }
    }

    return 0;
}

static unsigned int tcc_tsif_module_poll(struct file *filp, struct poll_table_struct *wait)
{
    if(tsif_get_readable_cnt(&tsif_module_handle) > tsif_module_handle.dma_intr_packet_cnt) {
		return  (POLLIN | POLLRDNORM);
    } 

	poll_wait(filp, &(tsif_module_pri.wait_q), wait);

    if(tsif_get_readable_cnt(&tsif_module_handle) > tsif_module_handle.dma_intr_packet_cnt) {
		return  (POLLIN | POLLRDNORM);
    }

    return 0;
}
#endif //GDMA

static ssize_t tcc_tsif_module_write(struct file *filp, const char *buf, size_t len, loff_t *ppos)
{
    return 0;
}

static int tcc_tsif_module_ioctl(struct inode *inode, struct file *filp, unsigned int cmd, unsigned long arg)
{
	int ret = 0;

	switch (cmd) {
        case IOCTL_TSIF_DMA_START:
			{
				struct tcc_tsif_module_param param;

				if(copy_from_user(&param, (void *)arg, sizeof(struct tcc_tsif_module_param) )) {
					printk("cannot copy from user tsif_module_param in IOCTL_TSIF_DMA_START !!! \n");
					return -EFAULT;
				}
				
				if(((TSIF_PACKET_SIZE * param.ts_total_packet_cnt) > tsif_module_handle.dma_total_size)
					|| (param.ts_total_packet_cnt <= 0)){
					printk("so big ts_total_packet_cnt !!! \n");
					return -EFAULT;
				}

				tsif_module_handle.dma_stop(&tsif_module_handle);

				tsif_module_handle.dma_total_packet_cnt = param.ts_total_packet_cnt;
				tsif_module_handle.dma_intr_packet_cnt = param.ts_intr_packet_cnt;

				tsif_module_handle.q_pos = 0;
				tsif_module_handle.cur_q_pos = 0;

				tsif_module_handle.tsif_set(&tsif_module_handle);
				tsif_module_handle.clear_fifo_packet(&tsif_module_handle);

				tsif_module_handle.dma_start(&tsif_module_handle);
			}
			break;
			
		case IOCTL_TSIF_DMA_STOP:
	            tsif_module_handle.dma_stop(&tsif_module_handle);
    	    break;
			
		case IOCTL_TSIF_GET_MAX_DMA_SIZE:
			{
	            struct tcc_tsif_module_param param;

	            param.ts_total_packet_cnt = tsif_module_handle.dma_total_size / TSIF_PACKET_SIZE;
	            param.ts_intr_packet_cnt = 1;

	            if (copy_to_user((void *)arg, (void *)&param, sizeof(struct tcc_tsif_module_param))) {
	                printk("cannot copy to user tsif_module_param in IOCTL_TSIF_GET_MAX_DMA_SIZE !!! \n");
	                return -EFAULT;
	            }
	        }
	        break;
			
		case IOCTL_TSIF_SET_PID:
			{
				struct tcc_tsif_module_pid_param param;

				if (copy_from_user(&param, (void *)arg, sizeof(struct tcc_tsif_module_pid_param))) {
					printk("cannot copy form user tcc_tsif_pid_param in IOCTL_TSIF_SET_PID !!!\n");
					return -EFAULT;
				}
				
				if (param.valid_data_cnt <= PID_MATCH_TABLE_MAX_CNT) {
					volatile unsigned long* PIDT;
					int i;
				
					for(i=0; i< (PID_MATCH_TABLE_MAX_CNT); i++)
					{
						if(tsif_module_handle.id == 0){
							//PIDT = (volatile unsigned long *)tcc_p2v(HwTSIF0_PIDT(i));
							PIDT = (volatile unsigned long *)tcc_p2v(HwGPSB_PIDT(i));
						}
						else{
							//PIDT = (volatile unsigned long *)tcc_p2v(HwTSIF1_PIDT(i));
							PIDT = (volatile unsigned long *)tcc_p2v(HwGPSB_PIDT(i));
						}

						*PIDT = 0;
					}
					
					for(i=0; i < (param.valid_data_cnt); i++) 
					{
						if(tsif_module_handle.id == 0){
							//PIDT = (volatile unsigned long *)tcc_p2v(HwTSIF0_PIDT(i));
							PIDT = (volatile unsigned long *)tcc_p2v(HwGPSB_PIDT(i));
						}
						else{
							//PIDT = (volatile unsigned long *)tcc_p2v(HwTSIF1_PIDT(i));
							PIDT = (volatile unsigned long *)tcc_p2v(HwGPSB_PIDT(i));
						}
						*PIDT = ((param.pid_data)[i] & 0x1FFF);
						//BITSET(*PIDT, Hw13);

						BITSET(*PIDT, Hw30);
					}
				}
			}
            break;

		case IOCTL_TSIF_DXB_POWER:
			{
				unsigned long flag;
				if (copy_from_user((void *)&flag, (const void *)arg, sizeof(flag))) {
					return -EFAULT;
				}
				if (tsif_module_dxb_power(flag)) {
					return -EINVAL;
				}
			}
			break;

        case IOCTL_TSIF_SET_PCRPID:
            {
                if (copy_from_user((void *)&guiPCRPID, (const void *)arg, sizeof(int))) {
                    return -EFAULT;
                }
                printk("Set PCR PID[0x%X]\n", guiPCRPID);
                if( guiPCRPID < 0x1FFF){
                    TSDEMUX_Open();
                }
            }
            break;

        case IOCTL_TSIF_GET_STC:
             {
                 unsigned int uiSTC;
                 uiSTC = TSDEMUX_GetSTC();
                 //printk("STC %d\n", uiSTC);
                 if (copy_to_user((void *)arg, (void *)&uiSTC, sizeof(int))) {
                     printk("cannot copy to user tcc_tsif_param in IOCTL_TSIF_GET_PCR !!! \n");
                     return -EFAULT;
                 }
             }
             break;

	    default:
	        printk("tcc-tsif-module : unrecognized ioctl (0x%X)\n", cmd);
	        ret = -EINVAL;
	        break;
    }
    return ret;
}

static int tcc_tsif_module_init(int tsif_num)
{
	int ret = 0;
    volatile struct tcc_tsif_regs *reg_addr = NULL;
    int port;

    printk("%s: tsif:%d open\n", __func__, tsif_num);

    reg_addr = (volatile struct tcc_tsif_regs *)((tsif_num == 0) ? &(HwTSIF0_BASE) : &(HwTSIF1_BASE));

    memset(&tsif_module_handle, 0, sizeof(struct tcc_tsif_module_handle));

    // reset
	tca_ckc_set_iobus_swreset((tsif_num == 0) ? RB_TSIF0CONTROLLER : RB_TSIF1CONTROLLER, OFF);
	tca_ckc_set_iobus_swreset((tsif_num == 0) ? RB_TSIF0CONTROLLER : RB_TSIF1CONTROLLER, ON);

	tca_ckc_setiobus((tsif_num == 0) ? RB_TSIF0CONTROLLER : RB_TSIF1CONTROLLER, ENABLE);

#ifdef GDMA
	tca_ckc_set_iobus_swreset(RB_GDMA1CONTROLLER, OFF);
	tca_ckc_set_iobus_swreset(RB_GDMA1CONTROLLER, ON);
	tca_ckc_setiobus(RB_GDMA1CONTROLLER, ENABLE);
#else //GDMA
	tca_ckc_set_iobus_swreset((tsif_num == 0) ? RB_GPSBCONTROLLER0 : RB_GPSBCONTROLLER1, OFF);
	tca_ckc_set_iobus_swreset((tsif_num == 0) ? RB_GPSBCONTROLLER0 : RB_GPSBCONTROLLER1, ON);
	tca_ckc_setiobus((tsif_num == 0) ? RB_GPSBCONTROLLER0 : RB_GPSBCONTROLLER1, ENABLE);
#endif //GDMA

    /* GPIO port */
    port = tsif_module_pri.gpio_port;

	if(tca_tsif_module_init(&tsif_module_handle,
					reg_addr,
					tea_alloc_dma_linux,
					tea_free_dma_linux,
					TSIF_DMA_SIZE,
					TSIF_CH,
					TSIF_DMA_CONTROLLER,
					TSIF_DMA_CH,
                    port)) {

        printk(" %s : tcc_tsif_module_init error !!!!!\n", __func__);
		ret = -EBUSY;
		goto err_tsif;
    }

	tsif_module_handle.private_data = (void *)&tsif_module_pri;
	tsif_module_handle.dma_intr_packet_cnt = 1;	//TSIF interrput is occur when device is gather the amount of data until dma_intr_packet_cnt * 188

	tsif_module_handle.msb_first = 0x01;       //1:msb first, 0:lsb first
#ifdef GDMA
    tsif_module_handle.big_endian = 0x01;	     //1:big endian, 0:little endian
#else //GDMA
    tsif_module_handle.big_endian = 0x00;	     //1:big endian, 0:little endian
#endif //GDMA
    //tsif_module_handle.serial_mode = 0x01;     //1:serialmode 0:parallel mode
    tsif_module_handle.serial_mode = 0x00;     //1:serialmode 0:parallel mode
    
    tsif_module_handle.clk_polarity = 0x01;    //1:falling edge 0:rising edge
    tsif_module_handle.valid_polarity = 0x01;  //1:valid high active 0:valid low active
    tsif_module_handle.sync_polarity = 0x00;   //1:sync high active 0:sync low active
    tsif_module_handle.sync_delay = 0x00;

	ret = request_irq(tsif_module_handle.irq, tsif_module_dma_handler, IRQF_SHARED, TSIF_MODULE_DEV_NAME, &tsif_module_handle);
	if(ret) { 
		goto err_irq; 
	}

	return 0;

err_irq:
	free_irq(tsif_module_handle.irq, &tsif_module_handle);

err_tsif:
	tca_tsif_module_clean(&tsif_module_handle);
	tca_ckc_setiobus((tsif_num == 0) ? RB_TSIF0CONTROLLER : RB_TSIF1CONTROLLER, DISABLE);		//ahb tsif clock disable

#ifdef GDMA
	tca_ckc_setiobus(RB_GDMA1CONTROLLER, ENABLE);
#else //GDMA
	tca_ckc_setiobus((tsif_num == 0) ? RB_GPSBCONTROLLER0 : RB_GPSBCONTROLLER1, ENABLE);
#endif //GDMA

	return ret;
}

static void tcc_tsif_module_deinit(int tsif_num)
{
	free_irq(tsif_module_handle.irq, &tsif_module_handle);
	tsif_module_handle.dma_stop(&tsif_module_handle);
    tca_tsif_module_clean(&tsif_module_handle);

	tca_ckc_setiobus((tsif_num == 0) ? RB_TSIF0CONTROLLER : RB_TSIF1CONTROLLER, DISABLE);		//ahb tsif clock disable
	
#ifdef GDMA
	tca_ckc_setiobus(RB_GDMA1CONTROLLER, DISABLE);
#else //GDMA
	tca_ckc_setiobus((tsif_num == 0) ? RB_GPSBCONTROLLER0 : RB_GPSBCONTROLLER1, DISABLE);
#endif //GDMA
}


static int tcc_tsif_module_open(struct inode *inode, struct file *filp)
{
    int ret = 0;
	
    //clk_enable(tsif_clk);
#ifdef GDMA
#else
	clk_enable(gpsb_clk);
#endif
	
    mutex_lock(&(tsif_module_pri.mutex));
	guiPCRPID = 0xFFFF;
	//TSDEMUX_Open();
	
    if(tsif_module_pri.open_cnt == 0) {
        tsif_module_pri.open_cnt++;
/* jhlim		
		ret = tsif_module_init(TSIF_CH);
*/
    } else {
        ret = -EBUSY;
    }

    mutex_unlock(&(tsif_module_pri.mutex));

    return ret;
}


static int tcc_tsif_module_release(struct inode *inode, struct file *filp)
{
    mutex_lock(&(tsif_module_pri.mutex));
	
    if(tsif_module_pri.open_cnt > 0) {
        tsif_module_pri.open_cnt--;
/* jhlim
		tsif_module_deinit(TSIF_CH);
*/
    }

    mutex_unlock(&(tsif_module_pri.mutex));
#ifdef GDMA
#else
	clk_disable(gpsb_clk);
#endif

    return 0;
}


struct file_operations tcc_tsif_module_fops = {
    .owner      = THIS_MODULE,
    .read       = tcc_tsif_module_read,
    .write      = tcc_tsif_module_write,
    .ioctl      = tcc_tsif_module_ioctl,
    .open       = tcc_tsif_module_open,
    .release    = tcc_tsif_module_release,
    .poll       = tcc_tsif_module_poll,
};

static struct class *tsif_module_class; //jhlim

static int __init tsif_module_init(void)
{
    int ret;

	printk(KERN_INFO "tcc_tsif_module: tsif%d init\n", TSIF_CH);

	memset(&tsif_module_pri, 0, sizeof(struct tcc_tsif_module_pri_handle));
	ret = register_chrdev(TSIF_MODULE_DEV_MAJOR, TSIF_MODULE_DEV_NAME, &tcc_tsif_module_fops);
	if(ret < 0) {
		printk(KERN_ERR "[%s:%d] register_chrdev error !!!!!\n", __func__, __LINE__); 
		return ret;
	} 

    init_waitqueue_head(&(tsif_module_pri.wait_q));
    mutex_init(&(tsif_module_pri.mutex));

	ret = platform_driver_probe(&tsif_module_platform_driver, tsif_module_drv_probe);
#ifdef GDMA
#else
	if( 0 == TSIF_CH )
        gpsb_clk = clk_get(NULL, "gpsb0");
    else
        gpsb_clk = clk_get(NULL, "gpsb1");
    if(gpsb_clk == NULL)
    {
        printk("gpsb clock error on tsif: cannot get clock\n");
        return -EINVAL;
    }
#endif
// jhlim	
    tcc_tsif_module_init(TSIF_CH);

    tsif_module_class = class_create(THIS_MODULE, TSIF_MODULE_DEV_NAME);
    device_create(tsif_module_class, NULL, MKDEV(TSIF_MODULE_DEV_MAJOR, 0), NULL, TSIF_MODULE_DEV_NAME);
// jhlim

	guiPCRPID = 0xFFFF;
    return ret;
}


static void __exit tsif_module_exit(void)
{
    tcc_tsif_module_deinit(TSIF_CH);

    unregister_chrdev(TSIF_MODULE_DEV_MAJOR, TSIF_MODULE_DEV_NAME);

	//platform_driver_unregister(&tsif_module_platform_driver);
#ifdef GDMA
#else
	clk_put(gpsb_clk);
#endif

}



module_init(tsif_module_init);
module_exit(tsif_module_exit);

MODULE_AUTHOR("Telechips Inc. linux@telechips.com");
MODULE_DESCRIPTION("Telechips TSIF driver");
MODULE_LICENSE("GPL");

