
/****************************************************************************
 *   FileName    : tca_backlight.c
 *   Description : 
 ****************************************************************************
 *
 *   TCC Version 1.0
 *   Copyright (c) Telechips, Inc.
 *   ALL RIGHTS RESERVED
 *
 ****************************************************************************/
/*****************************************************************************
*
* Header Files Include
*
******************************************************************************/
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/delay.h>
#include <linux/fb.h>
#include <linux/init.h>
#include <linux/dma-mapping.h>
#include <linux/interrupt.h>
#include <linux/workqueue.h>
#include <linux/wait.h>
#include <linux/platform_device.h>
#include <linux/clk.h>

#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/div64.h>
#include <asm/mach/map.h>
#ifdef CONFIG_PM
#include <linux/pm.h>
#endif

#include <mach/bsp.h>
#include <mach/tca_ckc.h>

#include <mach/dev_gpsb.h>
#include <mach/tca_backlight.h>
#include <mach/TCC_LCD_Interface.h>
#include <mach/TCC_LCD_DriverCtrl.h>
#include "../../input/touchscreen/tca_tsc2003tchctl.h"
#include "../../input/touchscreen/tca_tchhwctl.h"

#if 1
#define dprintk(msg...)	 { printk( "Tcc92_bl: " msg); }
#else
#define dprintk(msg...)	 
#endif

static unsigned int bl_value;

/*****************************************************************************
*
* Defines
*
******************************************************************************/
//#define NOT_USE_DIMMING

/*****************************************************************************
*
* Variables
*
******************************************************************************/
static volatile PTIMER pTIMER;

static PLCDC pLCDC;
static char current_lcdc;
static char current_lcdc_FbCh;


static LCDC_CHANNEL LCDC_CH0;
static LCDC_CHANNEL LCDC_CH1;
static LCDC_CHANNEL LCDC_CH2;
static struct clk *lcd_clk;

static LCD_MODULE_FUNC_TYPE LCD_Module_Driver;
/*****************************************************************************
*
* Functions
*
******************************************************************************/


void lcdc_channel_restore(int ch_num, int backup)
{
	PLCDC_CHANNEL pLCDC_CH;

	switch(ch_num)
	{
		case 0:
			if(backup)
				memcpy((char *)&LCDC_CH0, &pLCDC->LI0C, sizeof(LCDC_CHANNEL)); 
			else
				memcpy(&pLCDC->LI0C, (char *)&LCDC_CH0, sizeof(LCDC_CHANNEL)); 				

			break;

		case 1:
			if(backup)
				memcpy((char *)&LCDC_CH1, &pLCDC->LI1C, sizeof(LCDC_CHANNEL)); 
			else
				memcpy(&pLCDC->LI1C, (char *)&LCDC_CH1, sizeof(LCDC_CHANNEL)); 				

			break;

		case 2:
			if(backup)
				memcpy((char *)&LCDC_CH2, &pLCDC->LI2C, sizeof(LCDC_CHANNEL)); 
			else
				memcpy(&pLCDC->LI2C, (char *)&LCDC_CH2, sizeof(LCDC_CHANNEL)); 				

			break;

	}

}

void lcdc_initialize(LCD_MODULE_SPEC_TYPE *lcd_spec)
{
	unsigned int lcd_reg;

	lcd_reg = 0;

	if(lcd_spec->lcd_bus_width == 24)
		lcd_reg |= (0xC<<16);
	else if(lcd_spec->lcd_bus_width == 18)
		lcd_reg |= (0x5<<16);
	else
		lcd_reg |= (0x3<<16);

	if(lcd_spec->SyncSignal_Invert & ID_INVERT_EN)
		lcd_reg |= Hw15;

	if(lcd_spec->SyncSignal_Invert & IV_INVERT_EN)
		lcd_reg |= Hw14;

	if(lcd_spec->SyncSignal_Invert & IH_INVERT_EN)
		lcd_reg |= Hw13;

	if(lcd_spec->SyncSignal_Invert & IP_INVERT_EN)
		lcd_reg |= Hw12;

#ifdef CONFIG_LCD_CPU_INTERFACE
	lcd_reg |= Hw23; // clock gating enable 
#endif//

	lcd_reg |=	Hw8 |			// Non-interlace
				(0x2<<5)|		// TFT-LCD mode: STN(Hw5), TFT(Hw6), TV(Hw7)
				//Hw4 | 		// Master Select for IMG0
				(0x5<<1)|		// OVP[3:1] - 5 : Image2 > Image1 > Image0
				//Hw0 | 		// LCD Controller Enable
				0;			// End Of Value

	pLCDC->LCTRL = lcd_reg;

// Set LCD clock
	pLCDC->LCLKDIV = ( (1 << 16) | lcd_spec->lcdc_clk_div/2); //((PCK_LCD+1)/2)/((LCD_DEVIDE)*2)

// Background color
	pLCDC->LBC = 0x80000000;

//	Horizontal timing
	pLCDC->LHTIME1 = ((lcd_spec->lcd_LPW) << 16) | (lcd_spec->lcd_width - 1);
	pLCDC->LHTIME2 = ((lcd_spec->lcd_LSWC) << 16) | (lcd_spec->lcd_LEWC);

//	Vertical timing
	pLCDC->LVTIME1 = ((lcd_spec->lcd_FPW1) << 16) | (lcd_spec->lcd_height- 1);
	pLCDC->LVTIME2 = ((lcd_spec->lcd_FSWC1) << 16) | (lcd_spec->lcd_FEWC2);
	pLCDC->LVTIME3 = ((lcd_spec->lcd_FPW1) << 16) | (lcd_spec->lcd_height - 1);
	pLCDC->LVTIME4 = ((lcd_spec->lcd_FSWC1) << 16) | (lcd_spec->lcd_FEWC2);

//	Display Size
	pLCDC->LDS = (lcd_spec->lcd_height << 16) | lcd_spec->lcd_width;

//	Interrupt mask
	pLCDC->LIM &= 0xFFFFFFE6;
	pLCDC->LIM |= ((1 << 4) | 1);

//	Display Enhancement
	pLCDC->LENH = 0x20; //default setting value
	
}


#ifdef CONFIG_FB_TCC92XX_WVGA_DX08D11VM0AAA
extern void DX08D11VM0AAA_Module_func(LCD_MODULE_FUNC_TYPE *func);
#elif defined(CONFIG_FB_TCC92XX_WVGA_LMS480KF)
extern void LMS480KF_Module_func(LCD_MODULE_FUNC_TYPE *func);
#elif defined(CONFIG_FB_TCC92XX_HVGA_LMS350DF01)
extern void LMS350DF01_Module_func(LCD_MODULE_FUNC_TYPE *func);
#endif//


void tca_bkl_SetLcdc(char lcdctrl_num)
{
	current_lcdc = lcdctrl_num;

	if(lcdctrl_num)	{
		pLCDC	= (volatile PLCDC)tcc_p2v(HwLCDC1_BASE);
		current_lcdc_FbCh = 2;
	}
	else {
		pLCDC	= (volatile PLCDC)tcc_p2v(HwLCDC0_BASE);
		current_lcdc_FbCh = 0;
	}

}
/*****************************************************************************
* Function Name : tca_bkl_init()
******************************************************************************/
void tca_bkl_init(char lcdctrl_num)
{
	dprintk(KERN_INFO "%s\n", __FUNCTION__);
#ifdef CONFIG_FB_TCC92XX_WVGA_DX08D11VM0AAA
	DX08D11VM0AAA_Module_func(&LCD_Module_Driver);
#elif defined(CONFIG_FB_TCC92XX_WVGA_LMS480KF)
	LMS480KF_Module_func(&LCD_Module_Driver);
#elif defined(CONFIG_FB_TCC92XX_HVGA_LMS350DF01)
	LMS350DF01_Module_func(&LCD_Module_Driver);
#endif//

	LCD_Module_Driver.Lcd_Module_probe();
	tca_bkl_SetLcdc(lcdctrl_num);
}


void tca_lcdc_port_setting(char Nlcdc)
{
	LCD_MODULE_SPEC_TYPE LCDspec;
		
	LCD_Module_Driver.Lcd_GetModuleSpec(&LCDspec);
	LCDC_IO_Set(Nlcdc, LCDspec.lcd_bus_width);
}



/*****************************************************************************
* Function Name : tca_bkl_powerup()
******************************************************************************/
void tca_bkl_powerup(void)
{
	unsigned int lcdctrl;
	LCD_MODULE_SPEC_TYPE LCDspec;
	
	unsigned int lclk, lcd_div;
	
   	dprintk(KERN_INFO "%s bl levelL%d\n", __FUNCTION__, bl_value);

	LCD_Module_Driver.Lcd_GetModuleSpec(&LCDspec);
	
	lcd_div = LCDspec.lcdc_clk_div / 2;
	
	if(current_lcdc)
		lclk  = tca_ckc_getperi(PERI_LCD1);
	else
		lclk  = tca_ckc_getperi(PERI_LCD0);

    dprintk("LCD pclk:%d lcdc:%d div:%d  \n",lclk, LCDspec.lcdc_clk, lcd_div);

	lcdc_initialize(&LCDspec);

	lcdc_channel_restore(current_lcdc_FbCh, 0);

	// lcdc enable
	lcdctrl = pLCDC->LCTRL;
	pLCDC->LCTRL = lcdctrl | (0x1);
    dprintk(" lcdc:0x%x ch2 :0x%x  \n",pLCDC->LCTRL, pLCDC->LI2C);

	msleep(9);

	LCD_Module_Driver.Lcd_on(current_lcdc);
	
}
EXPORT_SYMBOL(tca_bkl_powerup);


/*****************************************************************************
* Function Name : tca_bkl_powerdown()
******************************************************************************/
void tca_bkl_powerdown(void)
{
	unsigned int lcdctrl;
	LCD_MODULE_SPEC_TYPE LCDspec;

	lcdc_channel_restore(current_lcdc_FbCh, 1);

	LCD_Module_Driver.Lcd_off();

	LCD_Module_Driver.Lcd_GetModuleSpec(&LCDspec);

	// lcdc disable
	lcdctrl = pLCDC->LCTRL;
	pLCDC->LCTRL = lcdctrl & (~(0x1));

	LCD_Driver_udelay(1);
	
}
EXPORT_SYMBOL(tca_bkl_powerdown);


/*****************************************************************************
* Function Name : tca_bkl_setpowerval()
******************************************************************************/


void tca_bkl_setpowerval(unsigned int inValue)
{
	dprintk("tca_bkl_setpowerval inValue set level(%d) before level(%d) \n", inValue, bl_value); 
	LCD_Module_Driver.Backligth_ctrl(inValue);
	bl_value = inValue;
}


/*****************************************************************************
* Function Name : tca_bkl_getpowerval()
******************************************************************************/
unsigned int tca_bkl_getpowerval(void)
{
	dprintk("tca_bkl_getpowerval inValue(%d)\n", bl_value); 
	return bl_value;
}


#ifdef __KERNEL__
EXPORT_SYMBOL(tca_bkl_getpowerval);
EXPORT_SYMBOL(tca_bkl_setpowerval);
#endif
