
/****************************************************************************
 *   FileName    : tca_backlight.c
 *   Description : 
 ****************************************************************************
 *
 *   TCC Version 1.0
 *   Copyright (c) Telechips, Inc.
 *   ALL RIGHTS RESERVED
 *
 ****************************************************************************/
/*****************************************************************************
*
* Header Files Include
*
******************************************************************************/
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/delay.h>
#include <linux/fb.h>
#include <linux/init.h>
#include <linux/dma-mapping.h>
#include <linux/interrupt.h>
#include <linux/workqueue.h>
#include <linux/wait.h>
#include <linux/platform_device.h>
#include <linux/clk.h>

#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/div64.h>
#include <asm/mach/map.h>
#ifdef CONFIG_PM
#include <linux/pm.h>
#endif

#include <mach/bsp.h>
#include <mach/tca_ckc.h>

#include <mach/tcc92xxfb_ioctrl.h>
#include <mach/tcc_used_mem.h>
#include <linux/cpufreq.h>
#include <mach/tca_fb_hdmi.h>


#if 0
#define dprintk(msg...)	 { printk( "tca_hdmi: " msg); }
#else
#define dprintk(msg...)	 
#endif

/*****************************************************************************
*
* Defines
*
******************************************************************************/

/*****************************************************************************
*
* structures
*
******************************************************************************/


/*****************************************************************************
*
* Variables
*
******************************************************************************/

/*****************************************************************************
*
* Functions
*
******************************************************************************/

static PLCDC pLCDC_HDMI = NULL;
static char hdmi_lcdc = 0;

int TCC_HDMI_Cable_Detect(void)
{
	PGPIO pGPIO = (PGPIO)tcc_p2v(HwGPIO_BASE);

	//dprintk("%s\n", __func__);

	BITCLR(pGPIO->GPAEN, Hw14);
	BITCLR(pGPIO->GPAFN1, Hw28-Hw24);
	{volatile int ttt;for(ttt=0;ttt<500;ttt++);}

	if(ISSET(pGPIO->GPADAT, Hw14))
	{
		//dprintk("HDMI cable is detected!!\n");
		return true;
	}
	else
	{
		//dprintk("HDMI cable is not detected!!\n");
		return false;
	}
}

void TCC_HDMI_LCDC_Timing(struct lcdc_timimg_parms_t *mode)
{
    volatile unsigned int regl;
	unsigned int lcdc_id;
	
	PDDICONFIG pHwDDICACHE_CONFIG  = (volatile PDDICONFIG)tcc_p2v(HwDDI_CONFIG_BASE);
	PCKC	pCKC = (volatile PCKC *)tcc_p2v(HwCLK_BASE);
	PLCDC	pLCDC;
	
	dprintk(" %s lcdc:[%d]  %d %d %d %d \n", __func__, hdmi_lcdc, mode->id, mode->iv, mode->ih, mode->lpw);


	if (hdmi_lcdc)	// lcdc1
	{
		lcdc_id = 1;
		pLCDC = (volatile PLCDC *)tcc_p2v(HwLCDC1_BASE);
		pLCDC->LCTRL = 0;


		BITSET(pHwDDICACHE_CONFIG->HDMI_CTRL, HwDDIC_HDMI_CTRL_SEL_LCDC1);
		

		// disable lcdc
		regl = readl(&pLCDC->LCTRL);
		writel(regl & ~(0x00000001), &pLCDC->LCTRL);
		
		regl = readl(&pCKC->PCLK_LCD1);
		writel(regl & ~(0x20000000), &pCKC->PCLK_LCD1);	// EN: 0

		// set lcdc1 clock
		regl = readl(&pCKC->PCLK_LCD1);
		regl &= ~(0x1f000fff);
		writel(regl, &pCKC->PCLK_LCD1);	// EN : 0, init.
		regl |= 12<<24;					// SEL: HDMI PCLK(Variable)
		regl |= 0;						// DIV: 0
		writel(regl, &pCKC->PCLK_LCD1);
		regl |= 1 <<29;					// EN : 1
		writel(regl, &pCKC->PCLK_LCD1);
	}
	else		// lcdc0
	{
		lcdc_id = 0;
		pLCDC = (volatile PLCDC *)tcc_p2v(HwLCDC0_BASE);


		pLCDC->LCTRL = 0;

		BITCLR(pHwDDICACHE_CONFIG->HDMI_CTRL, HwDDIC_HDMI_CTRL_SEL_LCDC1);

		// disable lcdc
		regl = readl(&pLCDC->LCTRL);
		writel(regl & ~(0x00000001), &pLCDC->LCTRL);

		regl = readl(&pCKC->PCLK_LCD0);
		writel(regl & ~(0x20000000), &pCKC->PCLK_LCD0);	// EN: 0

		// set lcdc0 clock
		regl = readl(&pCKC->PCLK_LCD0);
		regl &= ~(0x1f000fff);
		writel(regl, &pCKC->PCLK_LCD0);	// EN : 0, init.
		regl |= 12<<24;					// SEL: HDMI PCLK(Variable)
		regl |= 0;						// DIV: 0
		writel(regl, &pCKC->PCLK_LCD0);
		regl |= 1 <<29;					// EN : 1
		writel(regl, &pCKC->PCLK_LCD0);

	}

	//channel disable
	regl = readl(&pLCDC->LI0C);
	writel((regl & (~(0x10000000))) | HwLIC_UPD, &pLCDC->LI0C);
	regl = readl(&pLCDC->LI1C);
	writel((regl & (~(0x10000000))) | HwLIC_UPD, &pLCDC->LI1C);
	regl = readl(&pLCDC->LI2C);
	writel((regl & (~(0x10000000))) | HwLIC_UPD, &pLCDC->LI2C);

	// disable lcdc
	regl = readl(&pLCDC->LCTRL);
	writel(regl & (~(HwLCTRL_LEN)), &pLCDC->LCTRL);


	// set lctrl
	regl = readl(&pLCDC->LCTRL);
	regl &= ~(0x0000f3e0);
	regl |=((mode->id  & 0x1) <<15)
		 | ((mode->iv  & 0x1) <<14)
		 | ((mode->ih  & 0x1) <<13)
		 | ((mode->ip  & 0x1) <<12)
		 | ((mode->dp  & 0x1) << 9)
		 | ((mode->ni  & 0x1) << 8)
		 | ((mode->tv  & 0x1) << 7)
		 | ((mode->tft & 0x1) << 6)
		 | ((mode->stn & 0x1) << 5);
	regl = (regl & ~(0xf<<16)) | ((12)<<16);	//pxdw: 12 - rgb888
	writel(regl, &pLCDC->LCTRL);

	// set clkdiv
	writel(0x80000000, &pLCDC->LCLKDIV);
	
	// set timing
	writel((mode->lpw  <<16) | (mode->lpc  ), &pLCDC->LHTIME1);
	writel((mode->lswc <<16) | (mode->lewc ), &pLCDC->LHTIME2);
	writel((mode->fpw  <<16) | (mode->flc  ), &pLCDC->LVTIME1);
	writel((mode->fswc <<16) | (mode->fewc ), &pLCDC->LVTIME2);
	writel((mode->fpw2 <<16) | (mode->flc2 ), &pLCDC->LVTIME3);
	writel((mode->fswc2<<16) | (mode->fewc2), &pLCDC->LVTIME4);

	regl = readl(&pLCDC->LVTIME1);
	writel(regl | (mode->vdb  <<27) | (mode->vdf  <<22), &pLCDC->LVTIME1);

	// set lcdc.lds
	if( mode->dp == 1)
	{
		regl = ((mode->flc+1 )<<16) | ((mode->lpc+1) >> 1);
	}
	else
	{
		regl = ((mode->flc+1)<<16) | (mode->lpc+1);
	}
	writel(regl, &pLCDC->LDS);

	// set lcdc.img dma interlace mode
	if (mode->ni)	// non-interlace mode (progressive mode)
	{
		regl = readl(&pLCDC->LI0C);
		writel(regl & ~(0x80000000), &pLCDC->LI0C);
		if(lcdc_id)
		{
			regl = readl(&pLCDC->LI1C);
			writel(regl & ~(0x80000000), &pLCDC->LI1C);
			regl = readl(&pLCDC->LI2C);
			writel(regl & ~(0x80000000), &pLCDC->LI2C);
		}
	}
	else			// interlace mode
	{
		regl = readl(&pLCDC->LI0C);
		writel(regl | (0x80000000), &pLCDC->LI0C);
		if(lcdc_id)
		{
			regl = readl(&pLCDC->LI1C);
			writel(regl | (0x80000000), &pLCDC->LI1C);
			regl = readl(&pLCDC->LI2C);
			writel(regl | (0x80000000), &pLCDC->LI2C);
		}
	}

	// enable lcdc
	regl = readl(&pLCDC->LCTRL);
	writel(regl | 0x00000001, &pLCDC->LCTRL);

    return;
}

void TCC_HDMI_DISPLAY_UPDATE(struct tcc_lcdc_image_update *ImageInfo)
{

	unsigned int regl;
	unsigned int lcd_width = 0, lcd_height = 0, lcd_h_pos = 0, lcd_w_pos = 0, scale_x = 0, scale_y = 0;
	unsigned int img_width = 0, img_height = 0;
	unsigned int buffer_width = 0, y2r_option = 0;
	PLCDC	pLCDC;

	PLCDC_CHANNEL pLCDC_channel;
	dprintk(" %s :fmt:%d Fw:%d Fh:%d Iw:%d Ih:%d \n", __func__,
		ImageInfo->fmt,ImageInfo->Frame_width, ImageInfo->Frame_height, ImageInfo->Image_width, ImageInfo->Image_height);
	if((ImageInfo->Lcdc_layer >= 3) || (ImageInfo->fmt >TCC_LCDC_IMG_FMT_MAX))
		return;

	if (hdmi_lcdc)	// lcdc1
		pLCDC = (volatile PLCDC *)tcc_p2v(HwLCDC1_BASE);
	else
		pLCDC = (volatile PLCDC *)tcc_p2v(HwLCDC0_BASE);

	regl = pLCDC->LDS; // get LCD size 

	lcd_width = (regl & 0xFFFF);
	lcd_height = ((regl>>16) & 0xFFFF);
	
	if((!lcd_width) || (!lcd_height))
		return;

	switch(ImageInfo->Lcdc_layer )
	{
		case 0:
			pLCDC_channel = (volatile PLCDC_CHANNEL)&pLCDC->LI0C;
			break;
		case 1:
			pLCDC_channel = (volatile PLCDC_CHANNEL)&pLCDC->LI1C;
			break;
		case 2:
			pLCDC_channel = (volatile PLCDC_CHANNEL)&pLCDC->LI2C;
			break;
	}
	
	img_width = ImageInfo->Frame_width;
	img_height= ImageInfo->Frame_height;

	if(!ImageInfo->enable)	{
		pLCDC_channel->LIC = 0 | HwLIC_UPD;
		return;
	}

	{
		buffer_width = (((ImageInfo->Frame_width + 7)>>3)<<3);

		switch(ImageInfo->fmt)
		{
			case TCC_LCDC_IMG_FMT_RGB444:
			case TCC_LCDC_IMG_FMT_RGB565:
			case TCC_LCDC_IMG_FMT_RGB555:
				pLCDC_channel->LIO = buffer_width *2;
				y2r_option = HwLIC_Y2R;
				break;

			case TCC_LCDC_IMG_FMT_RGB888:
			case TCC_LCDC_IMG_FMT_RGB666:
				pLCDC_channel->LIO = buffer_width * 4;
				y2r_option = HwLIC_Y2R;
				break;
			
			case TCC_LCDC_IMG_FMT_YUV420SP:
			case TCC_LCDC_IMG_FMT_YUV422SP:
				pLCDC_channel->LIO = ((buffer_width / 2)<<16)|(buffer_width);
				y2r_option = HwLIC_Y2R;
				break;

			case TCC_LCDC_IMG_FMT_YUV422SQ:
				pLCDC_channel->LIO = buffer_width * 2;
				y2r_option = HwLIC_Y2R;
				break;

			case TCC_LCDC_IMG_FMT_YUV420ITL0:
			case TCC_LCDC_IMG_FMT_YUV420ITL1:
			case TCC_LCDC_IMG_FMT_YUV422ITL0:
			case TCC_LCDC_IMG_FMT_YUV422ITL1:
				pLCDC_channel->LIO = (buffer_width << 16)| (buffer_width);
				y2r_option = HwLIC_Y2R;
				break;

		}


		scale_x = 0;
		scale_y = 0;
		lcd_h_pos = 0;
		lcd_w_pos = 0;

		if(lcd_width > ImageInfo->Image_width)
			lcd_w_pos = (lcd_width - ImageInfo->Image_width)/2;
		
		if(lcd_height > ImageInfo->Image_height)
			lcd_h_pos = (lcd_height - ImageInfo->Image_height)/2;

	// position
		pLCDC_channel->LIP = ((lcd_h_pos << 16) | (lcd_w_pos));
	// scale
		pLCDC_channel->LISR =((scale_y<<4) | scale_x);
	// size
		pLCDC_channel->LIS =((ImageInfo->Image_height << 16) | (ImageInfo->Image_width));

	// position
		pLCDC_channel->LIP =((ImageInfo->offset_y << 16) | (ImageInfo->offset_x));

	// image address
		pLCDC_channel->LIBA0 = ImageInfo->addr0;
		pLCDC_channel->LIBA1 = ImageInfo->addr1;
		pLCDC_channel->LIBA2 = ImageInfo->addr2;


	//channel control
		pLCDC_channel->LIC = HwLIC_UPD | HwLIC_IEN | y2r_option | ImageInfo->fmt;
	}
}





