

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/delay.h>
#include <linux/fb.h>
#include <linux/init.h>
#include <linux/dma-mapping.h>
#include <linux/interrupt.h>
#include <linux/workqueue.h>
#include <linux/wait.h>
#include <linux/platform_device.h>
#include <linux/clk.h>

#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/div64.h>
#include <asm/mach/map.h>
#ifdef CONFIG_PM
#include <linux/pm.h>
#endif

#include <mach/bsp.h>
#include <mach/tca_ckc.h>
#include <mach/tca_lcdc.h>

/* Debugging stuff */
static int debug = 0;
#define dprintk(msg...)	if (debug) { printk( "tca lcdc: " msg); }
	

void tcc_ddi_cache_setting(void)
{
	PDDICACHE pHwDDICACHE  = (volatile PDDICACHE)tcc_p2v(HwDDICACHE_BASE);

	/* Cacheabe */
	pHwDDICACHE->DDIC0_CFG0 = (HwDDI_LCD0_DMA_0_0) |(HwDDI_LCD0_DMA_0_1 << 4) |(HwDDI_LCD0_DMA_2<<8) |(HwDDI_MSCL0_DMA_0<<12) |(HwDDI_MSCL0_DMA_1<<16)  |(HwDDI_MSCL0_DMA_2<<20);
	/* Non-Cacheable */
	pHwDDICACHE->DDIC0_CFG1 = (HwDDI_LCD0_DMA_0_2) |(HwDDI_LCD0_DMA_1 << 4) |(HwDDI_VIQE_DMA_0_0<<8) |(HwDDI_VIQE_DMA_0_1<<12) |(HwDDI_VIQE_DMA_0_2<<16);
	
	/* Cacheable */
	pHwDDICACHE->DDIC1_CFG0 = (DDIC_LCD1_DMA_0_0) |(DDIC_LCD1_DMA_0_1 << 4) |(DDIC_LCD1_DMA_2<<8) |(DDIC_MSCL1_DMA_0<<12) |(DDIC_MSCL1_DMA_1<<16)  |(DDIC_MSCL1_DMA_2<<20);
	/* Non-Cacheable */
	pHwDDICACHE->DDIC1_CFG1 = (DDIC_LCD1_DMA_0_2) |(DDIC_LCD1_DMA_1 << 4) |(DDIC_VIQE_DMA_1_0<<8) |(DDIC_VIQE_DMA_1_1<<12) |(DDIC_VIQE_DMA_1_2<<16)  |(DDIC_VIQE_DMA_2_0<<20)
								|(DDIC_VIQE_DMA_2_1<<24) |(DDIC_VIQE_DMA_2_2<<28);
	
	pHwDDICACHE->DDIC0_CH0_AMIN = 0x00000000  ;       //  0x020   R/W     0x00000000      DDI_CACHE0 Ch0 Cacheable Address Range (MIN)
	pHwDDICACHE->DDIC0_CH0_AMAX = 0xFFFFFFFF  ;       //  0x024   R/W     0x00000000      DDI_CACHE0 Ch0 Cacheable Address Range (MAX)
	pHwDDICACHE->DDIC0_CH1_AMIN = 0x00000000  ;       //  0x028   R/W     0x00000000      DDI_CACHE0 Ch1 Cacheable Address Range (MIN)
	pHwDDICACHE->DDIC0_CH1_AMAX = 0xFFFFFFFF  ;       //  0x02C   R/W     0x00000000      DDI_CACHE0 Ch1 Cacheable Address Range (MAX)
	pHwDDICACHE->DDIC0_CH2_AMIN = 0x00000000  ;       //  0x030   R/W     0x00000000      DDI_CACHE0 Ch2 Cacheable Address Range (MIN)
	pHwDDICACHE->DDIC0_CH2_AMAX = 0xFFFFFFFF  ;       //  0x034   R/W     0x00000000      DDI_CACHE0 Ch2 Cacheable Address Range (MAX)
	pHwDDICACHE->DDIC0_CH3_AMIN = 0x00000000  ;       //  0x038   R/W     0x00000000      DDI_CACHE0 Ch3 Cacheable Address Range (MIN)
	pHwDDICACHE->DDIC0_CH3_AMAX = 0xFFFFFFFF  ;       //  0x03C   R/W     0x00000000      DDI_CACHE0 Ch3 Cacheable Address Range (MAX)
	pHwDDICACHE->DDIC0_CH4_AMIN = 0x00000000  ;       //  0x040   R/W     0x00000000      DDI_CACHE0 Ch4 Cacheable Address Range (MIN)
	pHwDDICACHE->DDIC0_CH4_AMAX = 0xFFFFFFFF  ;       //  0x044   R/W     0x00000000      DDI_CACHE0 Ch4 Cacheable Address Range (MAX)
	pHwDDICACHE->DDIC0_CH5_AMIN = 0x00000000  ;       //  0x048   R/W     0x00000000      DDI_CACHE0 Ch5 Cacheable Address Range (MIN)
	pHwDDICACHE->DDIC0_CH5_AMAX = 0xFFFFFFFF  ;       //  0x04C   R/W     0x00000000      DDI_CACHE0 Ch5 Cacheable Address Range (MAX)
	pHwDDICACHE->DDIC1_CH0_AMIN = 0x00000000  ;       //  0x050   R/W     0x00000000      DDI_CACHE1 Ch0 Cacheable Address Range (MIN)
	pHwDDICACHE->DDIC1_CH0_AMAX = 0xFFFFFFFF  ;       //  0x054   R/W     0x00000000      DDI_CACHE1 Ch0 Cacheable Address Range (MAX)
	pHwDDICACHE->DDIC1_CH1_AMIN = 0x00000000  ;       //  0x058   R/W     0x00000000      DDI_CACHE1 Ch1 Cacheable Address Range (MIN)
	pHwDDICACHE->DDIC1_CH1_AMAX = 0xFFFFFFFF  ;       //  0x05C   R/W     0x00000000      DDI_CACHE1 Ch1 Cacheable Address Range (MAX)
	pHwDDICACHE->DDIC1_CH2_AMIN = 0x00000000  ;       //  0x060   R/W     0x00000000      DDI_CACHE1 Ch2 Cacheable Address Range (MIN)
	pHwDDICACHE->DDIC1_CH2_AMAX = 0xFFFFFFFF  ;       //  0x064   R/W     0x00000000      DDI_CACHE1 Ch2 Cacheable Address Range (MAX)
	pHwDDICACHE->DDIC1_CH3_AMIN = 0x00000000  ;       //  0x068   R/W     0x00000000      DDI_CACHE1 Ch3 Cacheable Address Range (MIN)
	pHwDDICACHE->DDIC1_CH3_AMAX = 0xFFFFFFFF  ;       //  0x06C   R/W     0x00000000      DDI_CACHE1 Ch3 Cacheable Address Range (MAX)
	pHwDDICACHE->DDIC1_CH4_AMIN = 0x00000000  ;       //  0x070   R/W     0x00000000      DDI_CACHE1 Ch4 Cacheable Address Range (MIN)
	pHwDDICACHE->DDIC1_CH4_AMAX = 0xFFFFFFFF  ;       //  0x074   R/W     0x00000000      DDI_CACHE1 Ch4 Cacheable Address Range (MAX)
	pHwDDICACHE->DDIC1_CH5_AMIN = 0x00000000  ;       //  0x078   R/W     0x00000000      DDI_CACHE1 Ch5 Cacheable Address Range (MIN)
	pHwDDICACHE->DDIC1_CH5_AMAX = 0xFFFFFFFF  ;       //  0x07C   R/W     0x00000000      DDI_CACHE1 Ch5 Cacheable Address Range (MAX)
	
	pHwDDICACHE->DDIC_CTRL = 0x003F003F;
}
EXPORT_SYMBOL(tcc_ddi_cache_setting);

void tca_lcdc_interrupt_onoff(char onoff, char lcdc)
{
	unsigned int LCDC_INTbit;
	PPIC pHwPIC_reg = (volatile PPIC)tcc_p2v(HwPIC_BASE);

	PLCDC pLCDC_reg;
#ifdef TCC_LCDC1_USE
	pLCDC_reg = (volatile PLCDC)tcc_p2v(HwLCDC1_BASE);
#else
	pLCDC_reg = (volatile PLCDC)tcc_p2v(HwLCDC0_BASE);
#endif//	TCC_LCDC1_USE


	if(lcdc)
		LCDC_INTbit = HwINT0_LCD1;
	else
		LCDC_INTbit = HwINT0_LCD0;

	if(onoff == 1)	{
		BITCSET(pLCDC_reg->LIM, 0xFFFFFFFF, HwLIM_VSF);

		BITSET(pHwPIC_reg->CLR0, LCDC_INTbit);			

		BITCLR(pHwPIC_reg->POL0, LCDC_INTbit);
		BITSET(pHwPIC_reg->SEL0, LCDC_INTbit);
		BITCLR(pHwPIC_reg->MODE0, LCDC_INTbit);

		BITSET(pHwPIC_reg->IEN0, LCDC_INTbit);
		BITSET(pHwPIC_reg->INTMSK0, LCDC_INTbit);	
	}
	else	{
		BITCLR(pHwPIC_reg->IEN0, LCDC_INTbit);
		BITCLR(pHwPIC_reg->INTMSK0, LCDC_INTbit);	
	}

}
EXPORT_SYMBOL(tca_lcdc_interrupt_onoff);


unsigned char tcc_LCDC_Wait_signal_disable(char lcdc)
{
	unsigned int loop = 0;
	volatile PLCDC	uiPLCD;
	PDDICONFIG pHwDDICACHE_CONFIG  = (volatile PDDICONFIG)tcc_p2v(HwDDI_CONFIG_BASE);

	if(lcdc == 0)	{
		uiPLCD = (volatile PLCDC)tcc_p2v(HwLCDC0_BASE);

		if(ISSET(pHwDDICACHE_CONFIG->PWDN, HwDDIC_PWDN_LCDC0))
			return FALSE;
	}
	else	{
		uiPLCD = (volatile PLCDC)tcc_p2v(HwLCDC1_BASE);

		if(ISSET(pHwDDICACHE_CONFIG->PWDN, HwDDIC_PWDN_LCDC1))
			return FALSE;
	}

	if( ISSET(uiPLCD->LCTRL, HwLCTRL_LEN) )
	{
		BITSET(uiPLCD->LSTATUS, HwLSTATUS_DD);
		BITCLR(uiPLCD->LCTRL, HwLCTRL_LEN);

		while(TRUE && (loop < 100000)) //check wheather is lcd on
		{
			if((ISZERO(uiPLCD->LSTATUS, HwLSTATUS_BUSY)) && (ISSET(uiPLCD->LSTATUS, HwLSTATUS_DD)))
				break;
			msleep(1);
		}
	}

	return TRUE;
}

unsigned int tcc_LCDC_enable(unsigned lcdc, unsigned onoff)
{
	volatile PLCDC pstLCDC;

	if(lcdc == 0)	{
		pstLCDC = (volatile PLCDC)tcc_p2v(HwLCDC0_BASE);
	}
	else	{
		pstLCDC = (volatile PLCDC)tcc_p2v(HwLCDC1_BASE);
	}

	if(onoff)
	{
		BITSET(pstLCDC->LCTRL, HwLCTRL_LEN);
		BITSET(pstLCDC->LSTATUS, HwLSTATUS_RU);
	}
	else
	{
		BITCLR(pstLCDC->LCTRL, HwLCTRL_LEN);
		BITCLR(pstLCDC->LSTATUS, HwLSTATUS_RU);
	}

	return 1;
}
EXPORT_SYMBOL(tcc_LCDC_enable);


unsigned int tcc_LCDC_ctrl_layer(unsigned lcdc, unsigned layer, unsigned onoff)
{
	unsigned int loop = 0;
	volatile PLCDC pstLCDC;
	volatile PLCDC_CHANNEL pstLCDC_layer;

	if(lcdc == 0)
	{
		pstLCDC = (volatile PLCDC)tcc_p2v(HwLCDC0_BASE);
		pstLCDC_layer = (volatile PLCDC_CHANNEL)tcc_p2v(pstLCDC->LI0C);
	}
	else
	{
		pstLCDC = (volatile PLCDC)tcc_p2v(HwLCDC1_BASE);
		switch(layer)
		{
			case 0:
				pstLCDC_layer = (volatile PLCDC_CHANNEL)tcc_p2v(pstLCDC->LI0C);
				break;
			case 1:
				pstLCDC_layer = (volatile PLCDC_CHANNEL)tcc_p2v(pstLCDC->LI1C);
				break;
			case 2:
				pstLCDC_layer = (volatile PLCDC_CHANNEL)tcc_p2v(pstLCDC->LI2C);
				break;
		}
	}

	if(onoff)
	{
		BITSET(pstLCDC_layer->LIC, (1<<28)); /* channel image enable */
	}
	else
	{
		BITCLR(pstLCDC_layer->LIC, (1<<28)); /* channel image disalbe */
	}

	return 1;
}
EXPORT_SYMBOL(tcc_LCDC_ctrl_layer);


unsigned int tcc_LCDC_set_alpha(unsigned lcdc, unsigned layer, unsigned alpha)
{
	unsigned int loop = 0;
	volatile PLCDC pstLCDC;
	volatile PLCDC_CHANNEL pstLCDC_CH;

	if(lcdc)
		pstLCDC = (volatile PLCDC)tcc_p2v(HwLCDC1_BASE);
	else
		pstLCDC = (volatile PLCDC)tcc_p2v(HwLCDC0_BASE);

	switch(layer)
	{
		case 0:
			pstLCDC_CH = (volatile PLCDC_CHANNEL)&pstLCDC->LI0C;
			break;
		case 1:
			pstLCDC_CH = (volatile PLCDC_CHANNEL)&pstLCDC->LI1C;
			break;
		case 2:
			pstLCDC_CH = (volatile PLCDC_CHANNEL)&pstLCDC->LI2C;
			break;
	}

	alpha &= 0xff;

	BITSET(pstLCDC_CH->LIC, Hw30);						/* AEN bit */
	BITCLR(pstLCDC_CH->LIC, Hw24);						/* ASEL bit */

	BITCSET(pstLCDC_CH->LIA, 0x000000ff, (alpha));		/* A0 bits */
	BITCSET(pstLCDC_CH->LIA, 0x00ff0000, (alpha)<<16);	/* A1 bits */
	
	return 1;
}
EXPORT_SYMBOL(tcc_LCDC_set_alpha);


unsigned int tcc_LCDC_set_bgcolor(unsigned lcdc, unsigned red, unsigned green, unsigned blue)
{
	volatile PLCDC pstLCDC;
	unsigned uRegValue;

	if(lcdc == 0)
		pstLCDC = (volatile PLCDC)tcc_p2v(HwLCDC0_BASE);
	else
		pstLCDC = (volatile PLCDC)tcc_p2v(HwLCDC1_BASE);

	uRegValue = ((blue & 0xff)<<16) | ((green & 0xff)<<8) | (red & 0xff);

	pstLCDC->LBC = uRegValue;
}
EXPORT_SYMBOL(tcc_LCDC_set_bgcolor);

